/* Program to treat IP data file */
#define ProgName "ipbeta"
#define ProgInfo "Calculate Beta-I profile in R-AXIS"
/*                                                         4/24/2002  */
/*         for ANSI C                                                 */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v0.03d_1/15/2007"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#define MAXARG 15
#define MINARG 3
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define DATANUM_LIM 10000
#define FLAT 0
#define CURVE_V 1
#define CURVE_H 2	/* not used */
#define BACK 3		/* not used */

#ifdef _OLDHP
	long int (*readDat)();
	double cosThetaFlat();
	double cosThetaCurveV();
	double getBetaFlat();
	double getBetaCurveV();
	double getBetaLimFlat();
	double getBetaLimCurveV();
	double degfixNormal();
	double degfixRax();
	double (*cosThetaf)();
	double (*getBetaf)();
	double (*getBetaLimf)();
	double (*degfixf)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	double cosThetaFlat(const double *x, const double *z);
	double cosThetaCurveV(const double *x, const double *z);
	double getBetaFlat(const double *x, const double *z);
	double getBetaCurveV(const double *x, const double *z);
	double getBetaLimFlat(const double *cos_t, const double *z);
	double getBetaLimCurveV(const double *cos_t, const double *z);
	double degfixNormal(const double *x);
	double degfixRax(const double *x);
	double (*cosThetaf)(const double *x, const double *z);
	double (*getBetaf)(const double *x, const double *z);
	double (*getBetaLimf)(const double *cos_t, const double *z);
	double (*degfixf)(const double *x);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1, double *arg2);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr;
	long int i, j, ipHeaderSize, startX0, endX0, startX1, endX1,
	     ipHeaderSize2VER1, ipHeaderSize2VER2, temp, dataNum, intensity,
	     readByte, maxZ0, minZ0, maxZ1, minZ1, maxX0, minX0, maxX1, minX1,
	     lines, numbyte, num;
	unsigned long int maxInt;
	char file1[FNAMELEN], file2[FNAMELEN], inputBuffer[20], argNum,
	     confirmBuffer1[8], confirmBuffer2[8], cpuIntelFlag, ipHeaderSwapFlag,
	     ipDataSwapFlag, shiftBit, strangeShiftBitFlag, byte, arg0[4],
	     verboseFlag, camrDoneFlag, xsizDoneFlag, zsizDoneFlag, ofileDoneFlag,
	     drtxDoneFlag, drtzDoneFlag, thetaDoneFlag, stepDoneFlag, debug,
	     noHeadFlag, showInfoFlag, timebuf[20], cameraTypeFlag, cameraTypeDoneFlag,
	     changeValueTypeIndex, noHeadDoneFlag, raxFlag, aveFlag, showNumFlag;
	double arg1, arg2, theta0, theta1, cosTheta0, cosTheta1, fact0x, fact0z,
	       fact1x, fact1z, fact0, fact1, xDist, zDist, beta, betaStep, rato,
	       *outBuf, camr, beta2, *aveBuf;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2;
	time_t timeNow;
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	ipDataSwapFlag = camrDoneFlag = xsizDoneFlag = zsizDoneFlag = drtxDoneFlag
	 = ofileDoneFlag = drtzDoneFlag = thetaDoneFlag = stepDoneFlag = debug
	 = noHeadFlag = showInfoFlag = cameraTypeDoneFlag = noHeadDoneFlag
	 = raxFlag = aveFlag = showNumFlag = 0;
	verboseFlag = 1;
	readByte = 0;
	lines = 0;
	numbyte = 0;
	maxInt = 0x7fffffff;
	cameraTypeFlag = -1;
	dataNum = 360;
	theta0 = theta1 = 0.0;
	betaStep = 1.0;
	num = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-qq")) {
			verboseFlag = -1;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		if (strcmp(argv[argc-1], "-")) {
			strcpy(file2, argv[argc-1]);
			ofileDoneFlag++;
		}
	}
	if (verboseFlag != -1) showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag > 0) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
			if (toupper(confirmBuffer1[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
		
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) ipHeader.zpxl = ipHeader.rnum;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	rato = ipHeader.rato;
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
	}
	if (ipHeader.cyld == 0) cameraTypeFlag = FLAT;
	else if (ipHeader.cyld == 1) cameraTypeFlag = CURVE_V;
	else cameraTypeFlag = -1;
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
		}
	}
	
/* Show Important parameters */
	if (verboseFlag >= 2) {
		fprintf(stdout, " IP size is : %6ld x %6ld.\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, " Centre is : %6.1f, %6.1f.\n", ipHeader.drtx, ipHeader.drtz);
		fprintf(stdout, " PixelSize is : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
		fprintf(stdout, " CameraLength is : %7.3f mm.\n", ipHeader.camr);
	}
	
/* Check the Header and Set Parameters : Interactive Mode */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Do you want to change the value shown above? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			do {
				fprintf(stdout, "  Choice in (1) Centre x     , (2) Centre z     , (3) Pixel Size x ,\n");
				fprintf(stdout, "            (4) Pixel Size z , (5) Camera Length, (0) Return to Previous : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				changeValueTypeIndex = confirmBuffer2[0]-'0';
				switch (changeValueTypeIndex) {
					case 0:
						changeValueTypeIndex = 0;
						break;
					case 1:
						drtxDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Centre x : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.drtx = atof(inputBuffer);
							if ((ipHeader.drtx < 0.0) || (ipHeader.drtx >= ipHeader.rlng/RAXDATAWIDTH)) continue;
							drtxDoneFlag = 1;
						} while (!drtxDoneFlag);
						fprintf(stdout, "Centre x is changed to %6.1f.\n", ipHeader.drtx);
						break;
					case 2:
						drtzDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Centre z : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.drtz = atof(inputBuffer);
							if ((ipHeader.drtz < 0.0) || (ipHeader.drtz >= ipHeader.zpxl)) continue;
							drtzDoneFlag = 1;
						} while (!drtzDoneFlag);
						fprintf(stdout, "Centre z is changed to %6.1f.\n", ipHeader.drtz);
						break;
					case 3:
						xsizDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Pixel Size x : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.xsiz = atof(inputBuffer);
							if (ipHeader.xsiz <= 0.0) continue;
							xsizDoneFlag = 1;
						} while (!xsizDoneFlag);
						fprintf(stdout, "Pixel Size x is changed to %7.5f mm.\n", ipHeader.xsiz);
						break;
					case 4:
						zsizDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Pixel Size z : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.zsiz = atof(inputBuffer);
							if (ipHeader.zsiz <= 0.0) continue;
							zsizDoneFlag = 1;
						} while (!zsizDoneFlag);
						fprintf(stdout, "Pixel Size z is changed to %7.5f mm.\n", ipHeader.zsiz);
						break;
					case 5:
						camrDoneFlag = 0;
						do {
							fprintf(stdout, "    Enter Value for Camera Length : ");
							if (fscanf(stdin, "%s", inputBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
							if (!isdigitEXT(inputBuffer)) continue;
							ipHeader.camr = atof(inputBuffer);
							if (ipHeader.camr <= 0.0) continue;
							camrDoneFlag = 1;
						} while (!camrDoneFlag);
						fprintf(stdout, "Camera Length is changed to %7.3f mm.\n", ipHeader.camr);
						break;
					default:
						fprintf(stdout, "  Enter appropriate value.\n");
				}
			} while (changeValueTypeIndex != 0);
		}
/* Ask Camera Type */
		do {
			fprintf(stdout, "Camera is Flat[F] , Curved[C] , or Other[N] ? : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'F') && (toupper(confirmBuffer1[0]) != 'C') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'F') {
			fprintf(stdout, "  Camera is Flat.\n");
			cameraTypeFlag = FLAT;
		}
		else if (toupper(confirmBuffer1[0]) == 'C') {
			cameraTypeFlag = CURVE_V;
			do {
				fprintf(stdout, "  Camera is Curved. Is Cylinder Axis Vertical on the Screen ? : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
			if (toupper(confirmBuffer2[0]) != 'Y') {
				fprintf(stderr, "  Camera axis must be in Z-direction.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		else {
			fprintf(stdout, "  Camera is Unknwon.\n");
			fprintf(stderr, "  Sorry, only Flat and Curved camera is supported.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
/* Get conversion parameters */
		do {
			fprintf(stdout, "Enter 2Theta of lower scattering angle [deg.] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			theta0 = atof(inputBuffer);
			if (theta0 <= 0.0) continue;
			if (cameraTypeFlag == FLAT && theta0 >= 90.0) continue;
			if (cameraTypeFlag == CURVE_V && theta0 >= 180.0) continue;
			thetaDoneFlag = 1;
		} while (!thetaDoneFlag);
		thetaDoneFlag = 0;
		do {
			fprintf(stdout, "Enter 2Theta of higher scattering angle [deg.] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			theta1 = atof(inputBuffer);
			if (theta1 <= theta0) continue;
			if (cameraTypeFlag == FLAT && theta1 >= 90.0) continue;
			if (cameraTypeFlag == CURVE_V && theta1 >= 180.0) continue;
			thetaDoneFlag = 1;
		} while (!thetaDoneFlag);
		do {
			fprintf(stdout, "Enter Division Number of whole ring(=360deg.) : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			dataNum = atol(inputBuffer);
			if (dataNum < 1 || dataNum > DATANUM_LIM) continue;
			stepDoneFlag = 1;
		} while (!stepDoneFlag);
		do {
			fprintf(stdout, "Output with angle of Rigaku-manner ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') raxFlag = 1;
		do {
			fprintf(stdout, "Output with averaged by pixel number ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') aveFlag = 1;
		do {
			fprintf(stdout, "Also output pixel number ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') showNumFlag = 1;
	}
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1, &arg2);
			if (argNum == 1) {
				if (!strcmp(arg0, "-f")) {
					cameraTypeFlag = FLAT;
					cameraTypeDoneFlag++;
				}
				else if (!strcmp(arg0, "-cv")) {
					ipHeader.cyld = 1;
					cameraTypeFlag = CURVE_V;
					cameraTypeDoneFlag++;
				}
				else if (!strcmp(arg0, "-sh")) {
					noHeadFlag = 1;
					noHeadDoneFlag++;
				}
				else if (!strcmp(arg0, "-sH")) {
					noHeadFlag = 2;
					noHeadDoneFlag++;
				}
				else if (!strcmp(arg0, "-tm")) showInfoFlag++;
				else if (!strcmp(arg0, "-rx")) raxFlag++;
				else if (!strcmp(arg0, "-av")) aveFlag++;
				else if (!strcmp(arg0, "-sn")) showNumFlag++;
#ifdef DEBUG
				else if (!strcmp(arg0, "-d")) debug++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-cl")) {
					if (arg1 <= 0.0) break;
					ipHeader.camr = arg1;
					camrDoneFlag++;
				}
				else if (!strcmp(arg0, "-xc")) {
					if ((arg1 < 0.0) || (arg1 >= ipHeader.xpxl)) break;
					ipHeader.drtx = arg1;
					drtxDoneFlag++;
				}
				else if (!strcmp(arg0, "-zc")) {
					if ((arg1 < 0.0) || (arg1 >= ipHeader.zpxl)) break;
					ipHeader.drtz = arg1;
					drtzDoneFlag++;
				}
				else if (!strcmp(arg0, "-dx")) {
					if (arg1 <= 0.0) break;
					ipHeader.xsiz = arg1;
					xsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-dz")) {
					if (arg1 <= 0.0) break;
					ipHeader.zsiz = arg1;
					zsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-dt")) {
					if (arg1 <= 0.0) break;
					dataNum = (long)floor(360.0 / arg1);
					betaStep = 360.0 / dataNum;
					stepDoneFlag++;
				}
				else if (!strcmp(arg0, "-dn")) {
					if (dToLong(arg1) < 1) break;
					dataNum = dToLong(arg1);
					betaStep = 360.0 / dataNum;
					stepDoneFlag++;
				}
				else break;
			}
			else if (argNum == 3) {
				if (!strcmp(arg0, "-t")) {
					if (arg1 == arg2) break;
					if (arg1 < arg2) {
						theta0 = arg1;
						theta1 = arg2;
					}
					else {
						theta0 = arg2;
						theta1 = arg1;
					}
					if (theta0 <= 0.0) break;
					thetaDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((camrDoneFlag > 1) || (drtxDoneFlag > 1) || (drtzDoneFlag > 1)
		 || (xsizDoneFlag > 1) || (zsizDoneFlag > 1) || (stepDoneFlag > 1)
		 || (cameraTypeDoneFlag > 1) || (stepDoneFlag > 1) || (raxFlag > 1)
		 || (thetaDoneFlag > 1) || (aveFlag > 1) || (showNumFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!thetaDoneFlag) {
			fprintf(stderr, " Lack of Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!stepDoneFlag) {
			dataNum = 360;
			betaStep = 1.0;
		}
	}
	if (ipHeader.xsiz <= 0.0 || ipHeader.zsiz <= 0.0 || ipHeader.camr <= 0) {
		fprintf(stderr, " R-AXIS Image Information is not correct. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (ipHeader.drtx < 0.0 || ipHeader.drtx >= ipHeader.xpxl || ipHeader.drtz < 0.0 || ipHeader.drtz >= ipHeader.zpxl) {
		fprintf(stderr, " Direct Beam Position is out of Image. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	switch (cameraTypeFlag) {
		case FLAT:
			cosThetaf = cosThetaFlat;
			getBetaf = getBetaFlat;
			getBetaLimf = getBetaLimFlat;
			break;
		case CURVE_V:
			cosThetaf = cosThetaCurveV;
			getBetaf = getBetaCurveV;
			getBetaLimf = getBetaLimCurveV;
			break;
		default:
			fprintf(stderr, "Unknown Camera Type. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
			break;
	}
	degfixf = raxFlag ? degfixRax : degfixNormal;
	if (cameraTypeFlag == CURVE_V) {
		if ((ipHeader.xpxl-ipHeader.drtx)*ipHeader.xsiz/ipHeader.camr > M_PI
		 || (              ipHeader.drtx)*ipHeader.xsiz/ipHeader.camr > M_PI) {
			fprintf(stderr, " Image Width of Curved Camera is too large. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (theta1 >= 180.0) {
			fprintf(stderr, " Higher limit of 2Theta is too large. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else if (cameraTypeFlag == FLAT) {
		if (theta1 >= 90.0) {
			fprintf(stderr, " Higher limit of 2Theta is too large. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	if (dataNum > DATANUM_LIM) {
		fprintf(stderr, " Beta Step is too small or Division Number is too Big. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (verboseFlag == 2) {
		if (drtxDoneFlag || drtzDoneFlag) fprintf(stdout, " Centre has changed to : %6.1f, %6.1f.\n", ipHeader.drtx, ipHeader.drtz);
		if (xsizDoneFlag || zsizDoneFlag) fprintf(stdout, " Pixelsize has changed to : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
		if (camrDoneFlag) fprintf(stdout, " CameraLength has changed to : %7.3f mm.\n", ipHeader.camr);
		if (cameraTypeFlag == FLAT) fprintf(stdout, " Camera is Flat.\n");
		else if (cameraTypeFlag == CURVE_V) fprintf(stdout, " Camera is Cylindrical along Z-direction.\n");
		fprintf(stdout, " Beta-I conversion will be done about the range\n");
		fprintf(stdout, "  between %7.3f and %7.3f[deg.] of 2Theta.\n", theta0, theta1);
		fprintf(stdout, " Step of Beta is %7.4f[deg.].\n", betaStep);
		if (aveFlag) fprintf(stdout, " Profile is averaged by area of pixel unit.\n");
	}
	
/* Set Parameters to Create */
	/* some variables for calculation */
	camr = ipHeader.camr;
	fact1z = camr * tan(radEXT(theta1)) / ipHeader.zsiz;
	fact0z = camr * tan(radEXT(theta0)) / ipHeader.zsiz;
	minZ1 = dToLong(ipHeader.drtz - fact1z);
	maxZ1 = dToLong(ipHeader.drtz + fact1z);
	minZ0 = dToLong(ipHeader.drtz - fact0z);
	maxZ0 = dToLong(ipHeader.drtz + fact0z);
	if (cameraTypeFlag == FLAT) {
		fact1x = camr * tan(radEXT(theta1)) / ipHeader.xsiz;
		fact0x = camr * tan(radEXT(theta0)) / ipHeader.xsiz;
	}
	else {
		fact1x = camr * radEXT(theta1) / ipHeader.xsiz;
		fact0x = camr * radEXT(theta0) / ipHeader.xsiz;
	}
	minX1 = dToLong(ipHeader.drtx - fact1x);
	maxX1 = dToLong(ipHeader.drtx + fact1x);
	minX0 = dToLong(ipHeader.drtx - fact0x);
	maxX0 = dToLong(ipHeader.drtx + fact0x);
#ifdef DEBUG
	if (debug) fprintf(stderr, " minZ1, minZ0, maxZ1, maxZ0, minX1, minX0, maxX1, maxX0\n%6ld, %6ld, %6ld, %6ld, %6ld, %6ld, %6ld, %6ld\n", minZ1, minZ0, maxZ1, maxZ0, minX1, minX0, maxX1, maxX0);
#endif
	minZ1 = MAXEXT(0, minZ1);
	minZ0 = MAXEXT(0, minZ0);
	maxZ1 = MINEXT(ipHeader.zpxl, maxZ1);
	maxZ0 = MINEXT(ipHeader.zpxl, maxZ0);
	minX1 = MAXEXT(0, minX1);
	minX0 = MAXEXT(0, minX0);
	maxX1 = MINEXT(ipHeader.xpxl, maxX1);
	maxX0 = MINEXT(ipHeader.xpxl, maxX0);
	dataNum++;
	cosTheta0 = cos(radEXT(theta0));
	cosTheta1 = cos(radEXT(theta1));
	
/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter Output Filename (or \"-\" to Standard Output) : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (strcmp(file2, "-")) ofileDoneFlag = 1;
	}
	if (ofileDoneFlag) {
		while (createFileEXT(file2, &fp2, 1)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
	}
	else fp2 = stdout;
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Suppress Output Header ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer1) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				if (ofileDoneFlag) {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer1[0]) != 'Y') && (toupper(confirmBuffer1[0]) != 'N'));
		if (toupper(confirmBuffer1[0]) == 'Y') {
			noHeadFlag = 1;
			do {
				fprintf(stdout, "Suppress Output Line Title ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
			if (toupper(confirmBuffer2[0]) == 'Y') noHeadFlag = 2;
		}
		else {
			do {
				fprintf(stdout, "Include \"Date\", \"Operator\", and \"Exposure Time\" in Header ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer2) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					if (ofileDoneFlag) {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer2[0]) != 'Y') && (toupper(confirmBuffer2[0]) != 'N'));
			if (toupper(confirmBuffer2[0]) == 'Y') showInfoFlag = 1;
		}
	}
	
/* Make Header */
	if (!noHeadFlag) {
		numbyte += fprintf(fp2, "# Beta-I Intensity Profile for %s.\n", file1);
		numbyte += fprintf(fp2, "# Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		numbyte += fprintf(fp2, "#  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
		numbyte += fprintf(fp2, "# Centre is : %6.1f, %6.1f.\n", ipHeader.drtx, ipHeader.drtz);
		numbyte += fprintf(fp2, "# PixelSize is : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
		numbyte += fprintf(fp2, "# CameraLength is : %7.3f mm.\n", ipHeader.camr);
		numbyte += fprintf(fp2, "# Calculation is done in region : 2Theta in %7.3f - %7.3f [deg.],\n", theta0, theta1);
		numbyte += fprintf(fp2, "# Step of Beta is %7.4f[deg.].\n", betaStep);
		if (aveFlag) numbyte += fprintf(fp2, "# Profile is averaged by area of pixel unit.\n");
		timeNow = time(NULL);
		strftime(timebuf, 20, "%Y-%m-%d %H:%M:%S", localtime(&timeNow));
		numbyte += fprintf(fp2, "# Converted by %s-%s on %s\n", ProgName, VERSION, timebuf);
		lines = 9 + aveFlag;
		if (showInfoFlag) { /* for HiPic treatment */
			numbyte += fprintf(fp2, "# Date : %s\n", ipHeader.date);
			numbyte += fprintf(fp2, "# Time : %s\n", ipHeader.opnm);
			numbyte += fprintf(fp2, "# ExposureTime : %7.3fms\n", ipHeader.fext*60000);
			lines += 3;
		}
	}
	if (verboseFlag > 0) {
		if ((theta1 <  90.0 && (minX1 < 0 || maxX1 >= ipHeader.xpxl || minZ1 < 0 || maxZ1 >= ipHeader.zpxl))
		 || theta1 >= 90.0) {
			fprintf(stderr, "*** Warning : Some of the data are out of screen. Ignored. ***\n");
			if (ofileDoneFlag) {
				numbyte += fprintf(fp2, "# *** Warning : Some of the data are out of screen. Ignored. ***\n");
				lines++;
			}
		}
	}
	
/* Start Calculation */
	if (verboseFlag > 0) fprintf(stdout, " Now, in progress...\n");
	/* start working */
	if (fseek(fp1, ipHeader.rlng*(minZ1+1), SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	if ((outBuf = calloc(dataNum, sizeof(double))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		exit(EXIT_FAILURE);
	}
	if ((aveBuf = calloc(dataNum, sizeof(double))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		free(outBuf);
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		exit(EXIT_FAILURE);
	}
	for (i=minZ1; i<maxZ1; i++) {
		if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(aveBuf);
			free(outBuf);
			fclose(fp1);
			if (ofileDoneFlag) {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		fread(datPtr, ipHeader.rlng, 1, fp1);
		readByte += ipHeader.rlng;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
			free(datPtr);
			free(aveBuf);
			free(outBuf);
			fclose(fp1);
			if (ofileDoneFlag) {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		zDist = (i+0.5-ipHeader.drtz) * ipHeader.zsiz / camr;
		fact1 = ((*getBetaLimf)(&cosTheta1, &zDist)) * camr / ipHeader.xsiz;
		startX1 = MAXEXT(dToLong(ipHeader.drtx - fact1), 0);
		endX1   = MINEXT(dToLong(ipHeader.drtx + fact1), ipHeader.xpxl);
		if (i >= minZ0 && i < maxZ0) {
			fact0 = ((*getBetaLimf)(&cosTheta0, &zDist)) * camr / ipHeader.xsiz;
			startX0 = MAXEXT(dToLong(ipHeader.drtx - fact0), 0);
			endX0   = MINEXT(dToLong(ipHeader.drtx + fact0), ipHeader.xpxl);
		}
		else startX0 = endX0 = endX1;
#ifdef DEBUG
		if (debug) fprintf(stderr, "i=%5ld : startX1, startX0, endX0, endX1 : %ld, %ld, %ld, %ld\n", i, startX1, startX0, endX0, endX1);
#endif
		for (j=startX1; j<startX0; j++) {
			intensity = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
			xDist = (j+0.5-ipHeader.drtx) * ipHeader.xsiz / camr;
			beta = (*getBetaf)(&xDist, &zDist);
			beta2 = (*degfixf)(&beta);
			temp = (long)floor(beta2 / betaStep);
			rato = fmod(beta2, betaStep) / betaStep;
			outBuf[ temp   ] += (intensity*(1.0-rato));
			outBuf[ temp+1 ] += (intensity*rato);
			num++;
			aveBuf[ temp   ] += (1.0-rato);
			aveBuf[ temp+1 ] += rato;
#ifdef DEBUG
			if (debug) fprintf(stderr, "I[%5ld, %5ld] = %ld, beta = %f.\n", j, i, intensity, beta2);
#endif
		}
		for (j=endX0; j<endX1; j++) {
			intensity = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
			xDist = (j+0.5-ipHeader.drtx) * ipHeader.xsiz / camr;
			beta = (*getBetaf)(&xDist, &zDist);
			beta2 = (*degfixf)(&beta);
			temp = (long)floor(beta2 / betaStep);
			rato = fmod(beta2, betaStep) / betaStep;
			outBuf[ temp   ] += (intensity*(1.0-rato));
			outBuf[ temp+1 ] += (intensity*rato);
			num++;
			aveBuf[ temp   ] += (1.0-rato);
			aveBuf[ temp+1 ] += rato;
#ifdef DEBUG
			if (debug) fprintf(stderr, "I[%5ld, %5ld] = %ld, beta = %f.\n", j, i, intensity, beta2);
#endif
		}
		free(datPtr);
	}
	fclose(fp1);
	outBuf[0] += outBuf[dataNum-1];
	aveBuf[0] += aveBuf[dataNum-1];
	outBuf[dataNum-1] = outBuf[0];
	aveBuf[dataNum-1] = aveBuf[0];
	
/* Output */
	if (!num) {
		fprintf(stderr, "There is no effective region !!\n");
		free(outBuf);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		exit(EXIT_FAILURE);
	}
	if (!noHeadFlag) {
		numbyte += fprintf(fp2, "# Number of pixel used is %ld.\n", num);
		lines++;
	}
	if (noHeadFlag != 2) {
		if (showNumFlag) numbyte += fprintf(fp2, "# beta[deg.] , Intensity, area\n");
		else numbyte += fprintf(fp2, "# beta[deg.] , Intensity\n");
		lines++;
	}
	if (aveFlag) {
		for (i=0; i<dataNum; i++) outBuf[i] = aveBuf[i] == 0.0 ? 0.0 : outBuf[i]/aveBuf[i];
	}
	if (showNumFlag) {
		for (i=0; i<dataNum; i++) numbyte += fprintf(fp2, " %9.5f , %15.4f , %11.6f\n", betaStep*i, outBuf[i], aveBuf[i]);
	}
	else {
		for (i=0; i<dataNum; i++) numbyte += fprintf(fp2, " %9.5f , %15.4f\n", betaStep*i, outBuf[i]);
	}
	
/* End */
	free(outBuf);
	free(aveBuf);
	if (ofileDoneFlag) fclose(fp2);
	if (verboseFlag >= 2) ofileDoneFlag ?
	    fprintf(stdout, " Output to %s with %ld lines, %ld bytes.\n", file2, lines+dataNum, numbyte) :
	    fprintf(stdout, " Output %ld lines, %ld bytes.\n", lines+dataNum, numbyte);
	if (verboseFlag >= 2) fprintf(stdout, " Number of pixels used is %ld.\n", num);
	if (verboseFlag > 0) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
	fprintf(fp0, "        %s [-q|-qq|-v] file1 [-f|-cv] -t:VALUE1:VALUE2 [-dt:VALUE|-dn:VALUE]\n", ProgName);
	fprintf(fp0, "          [-cl:VALUE] [-xc:VALUE] [-zc:VALUE] [-dx:VALUE] [-dz:VALUE]\n");
	fprintf(fp0, "          [-rx] [-av] [-sh|-sH] [-sn] [-tm] file2\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program \"Beta-I\" profile about R-AXIS Image File.\n");
	fprintf(fp0, "  !! This program works like \"Beta-I\" in Rigaku R-AXIS.\n");
	fprintf(fp0, "  !!  Calculation of the profile is not so correct, especially\n");
	fprintf(fp0, "  !!  for small scattering angles and for small beta-step.\n");
	fprintf(fp0, "  !!  But in other cases, it seems to be a good approximation.\n");
	fprintf(fp0, "  !! This program is intended to calculate \"Beta-I\" profile\n");
	fprintf(fp0, "  !!  as a routine work, so utilize me within a shell-script.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -qq ......... Very Quiet Mode. Only a few messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   file1 ....... Filename of R-AXIS Image File.\n");
	fprintf(fp0, "   -f .......... Assume CameraType is Flat.\n");
	fprintf(fp0, "   -cv ......... Assume CameraType is Cylindrical, whose axis is vertical\n");
	fprintf(fp0, "                 to the screen.\n");
	fprintf(fp0, "   -t:VALUE1:VALUE2 ...\n");
	fprintf(fp0, "                 Calculate about the region where scattering angle (2Theta) is\n");
	fprintf(fp0, "                 between \"VALUE1\" and \"VALUE2\" [deg.].\n");
	fprintf(fp0, "   -dt:VALUE ... Set step of calculating Beta angle to \"VALUE\"[deg.]\n");
	fprintf(fp0, "   -dn:VALUE ... Set Division Number of whole Beta(=360deg.) to \"VALUE\".\n");
	fprintf(fp0, "                 Above 2 options are exclusive.\n");
	fprintf(fp0, "   -rx ......... Measure Beta-angle from meridian line with CW.\n");
	fprintf(fp0, "                 This is Rigaku-manner. If absent, measure from horizon-right\n");
	fprintf(fp0, "                 with CCW (usual mathematical manner).\n");
	fprintf(fp0, "   -av ......... Average by area of pixel unit.\n");
	fprintf(fp0, "   -sn ......... Also show area of contributing pixel, with pixel number unit.\n");
	fprintf(fp0, "   -cl:VALUE ... Change Camera Langth to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -xc:VALUE ... Change Centre Position X to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -zc:VALUE ... Change Centre Position Z to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -dx:VALUE ... Change Pixel Size in X to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -dz:VALUE ... Change Pixel Size in Z to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -sh ......... Suppress Output Header. If absent, some informations about\n");
	fprintf(fp0, "                 Original File and conversion are print before profile.\n");
	fprintf(fp0, "   -sH ......... Addtion to above, suppress output of title line.\n");
	fprintf(fp0, "   -tm ......... Show about Date, Operator, and Exposure Time in Header.\n");
	fprintf(fp0, "   file2 ....... Output Filename. This must be the last argument.\n");
	fprintf(fp0, "                 If last argument is \"-\", Output to Standard Output.\n");
	return (0);
}

#ifdef _OLDHP
double cosThetaFlat(x, z)
	double *x, *z;
#else
double cosThetaFlat(const double *x, const double *z)
#endif
{
	return (1.0 / sqrt(1.0 + (*x)*(*x) + (*z)*(*z)));
}

#ifdef _OLDHP
double cosThetaCurveV(x, z)
	double *x, *z;
#else
double cosThetaCurveV(const double *x, const double *z)
#endif
{
	return (cos(*x) / sqrt(1.0 + (*z)*(*z)));
}

#ifdef _OLDHP
double getBetaFlat(x, z)
	double *x, *z;
#else
double getBetaFlat(const double *x, const double *z)
#endif
{
	if ((*x) == 0.0 && (*z) == 0.0) return (0.0);
	return (atan2((*z), (*x)));
}

#ifdef _OLDHP
double getBetaCurveV(x, z)
	double *x, *z;
#else
double getBetaCurveV(const double *x, const double *z)
#endif
{
	if (fabs(*x) != M_PI_2) return (atan2((*z), sin(*x)));
	if ((*x) > 0.0) return (atan(*z));
	return (-atan(*z)+M_PI);
}

#ifdef _OLDHP
double degfixNormal(x)
	double *x;
#else
double degfixNormal(const double *x)
#endif
{
	if ((*x) < 0.0) return (degEXT(*x) + 360.0);
	return (degEXT(*x));
}

#ifdef _OLDHP
double degfixRax(x)
	double *x;
#else
double degfixRax(const double *x)
#endif
{
	if ((*x) > M_PI_2) return (450.0 - degEXT(*x));
	return (90.0 - degEXT(*x));
}

#ifdef _OLDHP
double getBetaLimFlat(cos_t, z)
	double *cos_t, *z;
#else
double getBetaLimFlat(const double *cos_t, const double *z)
#endif
{
	return (sqrt(1.0/(*cos_t)/(*cos_t) - 1.0 - (*z)*(*z)));
}

#ifdef _OLDHP
double getBetaLimCurveV(cos_t, z)
	double *cos_t, *z;
#else
double getBetaLimCurveV(const double *cos_t, const double *z)
#endif
{
	return (acos((*cos_t)*dist(1.0, (*z))));
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1, arg2)
	char *argV, *arg0;
	double *arg1, *arg2;
#else
char splitArg(const char *argV, char *arg0, double *arg1, double *arg2)
#endif
{
	char *chPtr1, *chPtr2;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	if ((chPtr2 = strchr(chPtr1+1, ':')) == NULL) return (2);
	if (!isdigitEXT(chPtr2+1)) return (0);
	*arg1 = atof(chPtr1+1);
	*arg2 = atof(chPtr2+1);
	return (3);
}
