/* Program to generate IP data file */
#define ProgName "ipbmp2rax"
#define ProgInfo "Convert Bitmap File to R-AXIS Image File"
/*                                                          9/7/1999  */
/*         for ANSI C                                       9/7/1999  */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.24_1/15/2007"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <time.h>
#ifdef _HASUTIME
#	include <sys/stat.h>
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"
#include "bmpheader.h"
#include "libbmp.h"

#ifdef _HASUTIME
#	define MAXARG 9
#else
#	define MAXARG 8
#endif
#define MINARG 2
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define DEFAULT_SIZE 0.05
#define DEFAULT_LENG 100.0

#ifdef _OLDHP
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr2;
	long int i, j, k, ipHeaderSize, ipHeaderSize2VER2, bmpHeaderSize,
	     bmpInfoHeaderSize, bmpColourHeaderSize, biCount, biCount2,
	     bmpRecordLength, recordLengthCreate, ipHeaderSizeMin, count,
	     readByte, flushWidth, flushWidth0, full_rep, res_bits;
	unsigned long int maxInt;
	char file1[FNAMELEN], file2[FNAMELEN], cpuIntelFlag, reverseIntFlag, expIntFlag,
	     confirmBuffer[8], inputBuffer[20], argNum, arg0[4],
	     verboseFlag, shiftBit, xsizDoneFlag, zsizDoneFlag,
	     removeFlag, setTimeFlag, camrDoneFlag;
	unsigned char *datPtr1;
	double rato, arg1, intensityDbl, expFact;
	struct ipHead ipHeader;
	struct bmpHead bmpHeader;
	struct bmpInfoHead bmpInfoHeader;
	struct bmpColourHead bmpPalette[256];
	time_t timeNow;
	FILE *fp1, *fp2;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	bmpHeaderSize = SIZE_OF_BMPHEADER;
	bmpInfoHeaderSize = SIZE_OF_BMPINFOHEADER;
	bmpColourHeaderSize = SIZE_OF_BMPCOLOURHEADER;
	xsizDoneFlag = zsizDoneFlag = reverseIntFlag = expIntFlag
	 = removeFlag = setTimeFlag = camrDoneFlag = 0;
	verboseFlag = 1;
	readByte = 0;
	rato = 8.0;
	shiftBit = 3;
	maxInt = ((long)(0x7fff)) << shiftBit;
	ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
	flushWidth0 = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckBmp() == 1) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of Bitmap Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Argument */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	if (compilerCheckBmp() == 2) {
		if (verboseFlag) fprintf(stdout, " Size of Bitmap Header mismatch ... skip.\n");
		bmpHeaderSize = SIZE_OF_BMPHEADER_EACH;
	}
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	if (compilerCheckBmp() == 2) {
		fread(&bmpHeader.bfType, 2, 1, fp1);
		if ((ftell(fp1) != 4) && (ftell(fp1) != 2)) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		fseek(fp1, 2, SEEK_SET);
		readByte += 2;
		fread(&bmpHeader.bfSize, 4, 1, fp1);
		readByte += 4;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		fread(&bmpHeader.bfReserved1, 2, 1, fp1);
		readByte += 2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		fread(&bmpHeader.bfReserved2, 2, 1, fp1);
		readByte += 2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		fread(&bmpHeader.bfOffBits, 4, 1, fp1);
		readByte += 4;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		fread(&bmpHeader, bmpHeaderSize, 1, fp1);
		readByte += bmpHeaderSize;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	fread(&bmpInfoHeader, bmpInfoHeaderSize, 1, fp1);
	readByte += bmpInfoHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* Bitmap File Format Check */
	if (bmpMagicCheck(&bmpHeader)) {
		fprintf(stderr, " Magic is incorrect :\"%s\". This File is not Bitmap. Exit.\n", bmpHeader.bfType);
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (!cpuIntelFlag) {
		swapBmpHeader(&bmpHeader);
		swapBmpInfoHeader(&bmpInfoHeader);
	}
	if ((bmpInfoHeader.biSize != 40) || (bmpInfoHeader.biWidth > MAXWIDTH)
	 || (bmpInfoHeader.biWidth < 1) || (bmpInfoHeader.biHeight > MAXHEIGHT)
	 || (bmpInfoHeader.biHeight < 1) || (bmpInfoHeader.biPlanes != 1)
	 || (bmpInfoHeader.biCompression)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		if (verboseFlag >= 2) {
			fprintf(stderr, " biSize : %ld , biWidth : %ld , biHeight : %ld ,\n", bmpInfoHeader.biSize, bmpInfoHeader.biWidth, bmpInfoHeader.biHeight);
			fprintf(stderr, " biPlanes : %d , biCompression : %ld .\n", bmpInfoHeader.biPlanes, bmpInfoHeader.biCompression);
		}
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if ((bmpInfoHeader.biBitCount != 1) && (bmpInfoHeader.biBitCount != 4) &&
		(bmpInfoHeader.biBitCount != 8)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		if (verboseFlag >= 2) fprintf(stderr, " biBitCount : %d .\n", bmpInfoHeader.biBitCount);
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	biCount = (0x0001 << bmpInfoHeader.biBitCount); /* = maximum colour depth */
	if ((bmpInfoHeader.biClrUsed > biCount) || (bmpInfoHeader.biClrUsed < 0)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		if (verboseFlag >= 2) fprintf(stderr, " biClrUsed : %ld .\n", bmpInfoHeader.biClrUsed);
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (!bmpInfoHeader.biClrUsed) bmpInfoHeader.biClrUsed = biCount;
	if (bmpHeader.bfOffBits == bmpHeaderSize + bmpInfoHeader.biSize + biCount * 4) {
		bmpInfoHeader.biClrUsed = biCount;
	}
	else if (bmpHeader.bfOffBits != bmpHeaderSize + bmpInfoHeader.biSize + bmpInfoHeader.biClrUsed * 4) {
		fprintf(stderr, " File Format is Incorrect.\n");
		if (verboseFlag >= 2) fprintf(stderr, " bfOffBits : %ld .\n", bmpHeader.bfOffBits);
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	full_rep = bmpInfoHeader.biWidth*bmpInfoHeader.biBitCount/8;
	res_bits = (bmpInfoHeader.biWidth*bmpInfoHeader.biBitCount) % 8; /* num of res after full-read */
	bmpRecordLength = (long)ceil((res_bits ? full_rep+1 : full_rep)/4.0) * 4;
	if ((bmpInfoHeader.biSizeImage != bmpInfoHeader.biHeight * bmpRecordLength) || !bmpInfoHeader.biSizeImage) {
		fprintf(stderr, " File Format is Incorrect.\n");
		if (verboseFlag >= 2) fprintf(stderr, " biSizeImage : %ld .\n", bmpInfoHeader.biSizeImage);
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if ((bmpHeader.bfSize != bmpHeader.bfOffBits + bmpInfoHeader.biSizeImage)
	 && (bmpHeader.bfSize != dToLong(ceil(bmpHeader.bfOffBits/4.0))*4 + bmpInfoHeader.biSizeImage)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		if (verboseFlag >= 2) fprintf(stderr, " bfSize : %ld .\n", bmpHeader.bfSize);
		fclose(fp1);
		exit(EXIT_FAILURE);
	}

/* Bitmap File Colour Header Read */
	for (i=0; i< bmpInfoHeader.biClrUsed; i++) {
		fread(&bmpPalette[i], bmpColourHeaderSize, 1, fp1);
		readByte += bmpColourHeaderSize;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((bmpPalette[i].rgbBlue - bmpPalette[i].rgbGreen) ||
		    (bmpPalette[i].rgbGreen - bmpPalette[i].rgbRed)) {
			fprintf(stderr, " Input File is not Gray-scale or Monochrome. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Define ipHeader and Initialize */
	memset(&ipHeader, 0, ipHeaderSize);
	strcpy(ipHeader.name, "R-AXIS-DS"); /* Name of the Device */
	strcpy(ipHeader.vers, "1.0-2.0"); /* Version Number */
	strcpy(ipHeader.crnm, "N/A"); /* Crystal Name */
	strcpy(ipHeader.crsy, "cubic"); /* Crystal System */
	ipHeader.alng = 10.0; /* a-axis length in A */
	ipHeader.blng = 10.0; /* b-axis length in A */
	ipHeader.clng = 10.0; /* c-axis length in A */
	ipHeader.alfa = 90.0; /* angle alpha */
	ipHeader.beta = 90.0; /* angle beta */
	ipHeader.gamm = 90.0; /* angle gamma */
	strcpy(ipHeader.spgr, "P23"); /* Space Group */
	ipHeader.mosc = 0.3; /* Mozaic Spread */
	strcpy(ipHeader.opnm, "rigaku"); /* Operator Name */
	strcpy(ipHeader.trgt, "Cu"); /* X-ray Target */
	ipHeader.wlng = 1.5418; /* Wave Length of the x-ray */
	strcpy(ipHeader.mcro, "None"); /* Monochromator Material */
	ipHeader.m2ta = 0.0; /* Angle of the monochromator in 2-theta */
	strcpy(ipHeader.colm, "0.05-0.10"); /* Collimator */
	strcpy(ipHeader.filt, "Ni"); /* Kb filter */
	ipHeader.vltg = 40.0; /* X-ray tube voltage */
	ipHeader.crnt = 100.0; /* X-ray tube current */
	strcpy(ipHeader.focs, "0.3x0.3"); /* X-ray focus */
	strcpy(ipHeader.optc, "Ni-filter"); /* X-ray optics */
	ipHeader.cyld = 0; /* Camera type */
	ipHeader.weis = 0.0; /* weisenberg oscillation */
	strcpy(ipHeader.mnax, "-b*"); /* Mount Axis */
	strcpy(ipHeader.bmax, "-a"); /* Beam Axis */
	ipHeader.phi0 = 0.0; /* phi0 */
	ipHeader.phis = 0.0; /* phi start */
	ipHeader.phie = 0.0; /* phi end */
	ipHeader.oscn = 0; /* Frequency of Oscillation */
	ipHeader.fext = 20.0; /* Exposure Time in min. */
	ipHeader.omga = 0.0; /* omega in theta */
	ipHeader.fkai = 0.0; /* xai */
	ipHeader.thta = 0.000; /* 2-theta */
	ipHeader.mu = 0.000; /* mu */
	ipHeader.ipst = 0; /* Line of start reading */
	ipHeader.ipnm = 1; /* Number of IP */
	ipHeader.rato = rato; /* Ratio of IP in High/Low */
	ipHeader.ft_1 = 0.0; /* fading time 1 */
	ipHeader.ft_2 = 0.0; /* fading time 2 */
	strcpy(ipHeader.host, "IRIS"); /* Type of Host Computer */
	strcpy(ipHeader.ip, "HR-3"); /* Type of IP */
	ipHeader.dr_x = 0; /* Data direction in horizontal */
	ipHeader.dr_z = 0; /* Data direction in vertical */
	ipHeader.drxz = 0; /* Data direction face/back */
	timeNow = time(NULL);
	strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&timeNow));
	strcat(ipHeader.memo, ProgName);
	strcat(ipHeader.memo, "/");
	strcat(ipHeader.memo, ipHeader.date);
#ifdef _HASUTIME
	strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&utime1.modtime));
#endif
	
/* Set ipHeader Information */
	ipHeader.xpxl = bmpInfoHeader.biWidth;
	ipHeader.zpxl = bmpInfoHeader.biHeight;
	ipHeader.camr = DEFAULT_LENG;
	
/* Conversion Parameter */
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels, each pixel has %d bits,\n", bmpInfoHeader.biWidth, bmpInfoHeader.biHeight, bmpInfoHeader.biBitCount);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", bmpRecordLength, bmpInfoHeader.biHeight, bmpRecordLength*bmpInfoHeader.biHeight+bmpHeader.bfOffBits);
	}
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Reverse intensity (i.e. \"White\" corresponds to \"0\") ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') reverseIntFlag = 1;
		do {
			fprintf(stdout, "Intensity is exponential[E] or linear[L] ? : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'E') && (toupper(confirmBuffer[0]) != 'L'));
		if (toupper(confirmBuffer[0]) == 'E') expIntFlag = 1;
		if (bmpInfoHeader.biXPelsPerMeter > 0) ipHeader.xsiz = 1000.0 / bmpInfoHeader.biXPelsPerMeter;
		else ipHeader.xsiz = DEFAULT_SIZE;
		do {
			fprintf(stdout, "PixelSize in X-direction is %.5f [mm]. OK ? [Y/N] : ", ipHeader.xsiz);
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'N') {
			do {
				fprintf(stdout, "  Enter PixelSize in X-direction [mm] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				ipHeader.xsiz = atof(inputBuffer);
				if (ipHeader.xsiz <= 0.0) continue;
				xsizDoneFlag = 1;
			} while (!xsizDoneFlag);
		}
		if (bmpInfoHeader.biYPelsPerMeter > 0) ipHeader.zsiz = 1000.0 / bmpInfoHeader.biYPelsPerMeter;
		else ipHeader.zsiz = DEFAULT_SIZE;
		do {
			fprintf(stdout, "PixelSize in Z-direction is %.5f [mm]. OK ? [Y/N] : ", ipHeader.zsiz);
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'N') {
			do {
				fprintf(stdout, "  Enter PixelSize in Z-direction [mm] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				ipHeader.zsiz = atof(inputBuffer);
				if (ipHeader.zsiz <= 0.0) continue;
				zsizDoneFlag = 1;
			} while (!zsizDoneFlag);
		}
		do {
			fprintf(stdout, "Enter Camera Length [mm] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.camr = atof(inputBuffer);
			if (ipHeader.camr <= 0.0) continue;
			camrDoneFlag = 1;
		} while (!camrDoneFlag);
		do {
			fprintf(stdout, "Delete Original Bitmap File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') removeFlag = 1;
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
	else {
		if (bmpInfoHeader.biXPelsPerMeter > 0) ipHeader.xsiz = 1000.0 / bmpInfoHeader.biXPelsPerMeter;
		else ipHeader.xsiz = DEFAULT_SIZE;
		if (bmpInfoHeader.biYPelsPerMeter > 0) ipHeader.zsiz = 1000.0 / bmpInfoHeader.biYPelsPerMeter;
		else ipHeader.zsiz = DEFAULT_SIZE;
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-r")) reverseIntFlag++;
				else if (!strcmp(arg0, "-e")) expIntFlag++;
				else if (!strcmp(arg0, "-rm")) removeFlag++;
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-dx")) {
					ipHeader.xsiz = arg1;
					if (ipHeader.xsiz <= 0.0) break;
					xsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-dz")) {
					ipHeader.zsiz = arg1;
					if (ipHeader.zsiz <= 0.0) break;
					zsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-cl")) {
					ipHeader.camr = arg1;
					if (ipHeader.camr <= 0.0) break;
					camrDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((reverseIntFlag > 1) || (expIntFlag > 1) || (xsizDoneFlag > 1)
		 || (zsizDoneFlag > 1) || (removeFlag > 1) || (setTimeFlag > 1)
		 || (camrDoneFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " PixelSize is %8.5f [mm] x %8.5f [mm], Camera Length is %8.2f [mm]\n", ipHeader.xsiz, ipHeader.zsiz, ipHeader.camr);
		fprintf(stdout, " Data Conversion is ");
		if (reverseIntFlag) fprintf(stdout, "reverse, ");
		if (expIntFlag) fprintf(stdout, "exponential.\n");
		else fprintf(stdout, "linear.\n");
		if (removeFlag) fprintf(stdout, " Original Bitmap File will be removed !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Set Parameters to Create */
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate =  MAXEXT(ipHeaderSizeMin, flushWidth);
	ipHeader.drtx = ipHeader.xpxl/2.0;
	ipHeader.drtz = ipHeader.zpxl/2.0;
	ipHeader.rlng = recordLengthCreate;
	ipHeader.rnum = ipHeader.zpxl;
	flushWidth0 = recordLengthCreate - flushWidth;
	biCount2 = biCount - 1;
	expFact = 12.4766 / biCount;
	
/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter  Output  Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	while (createFileEXT(file2, &fp2, 0)) {
		if (!verboseFlag) {
			fprintf(stderr, " Output File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Swap Header */
	if (cpuIntelFlag) swapIpHeader(&ipHeader);
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, sizeof(short))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSize, 1, fp2);
	if (fseek(fp1, bmpHeader.bfOffBits, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	if (flushWidth0 == 0) free(datPtr0);
	
/* Swap Header again */
	if (cpuIntelFlag) swapIpHeader(&ipHeader);
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	for (i=0; i<ipHeader.rnum; i++) {
		if ((datPtr1 = malloc(bmpRecordLength)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			if (i) {
				ipHeader.zpxl = ipHeader.rnum = i;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (cpuIntelFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (cpuIntelFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (removeFlag) fprintf(stdout, " Original Bitmap File is not removed.\n");
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		if ((datPtr2 = malloc(flushWidth)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtr1);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			if (i) {
				ipHeader.zpxl = ipHeader.rnum = i;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (cpuIntelFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (cpuIntelFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (removeFlag) fprintf(stdout, " Original Bitmap File is not removed.\n");
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		fread(datPtr1, bmpRecordLength, 1, fp1);
		readByte += bmpRecordLength;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
			free(datPtr2);
			free(datPtr1);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			if (i) {
				ipHeader.zpxl = ipHeader.rnum = i;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (cpuIntelFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (cpuIntelFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (removeFlag) fprintf(stdout, " Original Bitmap File is not removed.\n");
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
#if 0	/* Now, In The Comment */
		count = 0;
		for (j=0; j<bmpRecordLength; j++) {
			for (k=0; k<8/bmpInfoHeader.biBitCount; k++) {
				intensityDbl = bmpPalette[(datPtr1[j] << k*bmpInfoHeader.biBitCount) >> (8-bmpInfoHeader.biBitCount)].rgbBlue;
				if (reverseIntFlag) intensityDbl = biCount - intensityDbl - 1.0;
				if (expIntFlag) intensityDbl = exp(12.4766 * intensityDbl / biCount);
				datPtr2[count] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
				if (count >= bmpInfoHeader.biWidth-1) break;
				count++;
			}
		}
		fwrite(datPtr2, flushWidth, 1, fp2);
		if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
#else	/* End Of The Comment */
		switch (bmpInfoHeader.biBitCount) {
			case 1:
				count = 0;
				for (j=0; j<full_rep; j++) {
					for (k=0; k<8; k++) {
						intensityDbl = bmpPalette[(datPtr1[j] << k) >> 7].rgbBlue;
						if (reverseIntFlag) intensityDbl = biCount2 - intensityDbl;
						if (expIntFlag) intensityDbl = exp(expFact * intensityDbl);
						datPtr2[count++] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
					}
				}
				for (k=0; k<res_bits; k++) {
					intensityDbl = bmpPalette[(datPtr1[j] << k) >> 7].rgbBlue;
					if (reverseIntFlag) intensityDbl = biCount2 - intensityDbl;
					if (expIntFlag) intensityDbl = exp(expFact * intensityDbl);
					datPtr2[count++] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
				}
				break;
			case 4:
				count = 0;
				for (j=0; j<full_rep; j++) {
					intensityDbl = bmpPalette[datPtr1[j] >> 4].rgbBlue;
					if (reverseIntFlag) intensityDbl = biCount2 - intensityDbl;
					if (expIntFlag) intensityDbl = exp(expFact * intensityDbl);
					datPtr2[count++] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
					intensityDbl = bmpPalette[(datPtr1[j] << 4) >> 4].rgbBlue;
					if (reverseIntFlag) intensityDbl = biCount2 - intensityDbl;
					if (expIntFlag) intensityDbl = exp(expFact * intensityDbl);
					datPtr2[count++] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
				}
				if (res_bits) {
					intensityDbl = bmpPalette[datPtr1[full_rep] >> 4].rgbBlue;
					if (reverseIntFlag) intensityDbl = biCount2 - intensityDbl;
					if (expIntFlag) intensityDbl = exp(expFact * intensityDbl);
					datPtr2[count] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
				}
				break;
			case 8:
				for (j=0; j<full_rep; j++) {
					intensityDbl = bmpPalette[datPtr1[j]].rgbBlue;
					if (reverseIntFlag) intensityDbl = biCount2 - intensityDbl;
					if (expIntFlag) intensityDbl = exp(expFact * intensityDbl);
					datPtr2[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
				}
				break;
			default:
				/* NOTREACHED */
				break;
		}
		fwrite(datPtr2, flushWidth, 1, fp2);
		if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
#endif
		free(datPtr1);
		free(datPtr2);
	}
	
/* End */
	if (flushWidth0 > 0) free(datPtr0);
	fclose(fp1);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
	if (removeFlag) {
		if (remove(file1)) fprintf(stderr, " Unable to remove Original Bitmap File.\n");
		else if (verboseFlag) fprintf(stdout, " Original Bitmap File has Removed.\n");
	}
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-r] [-e] [-rm] [-t]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-r] [-e] [-rm]\n", ProgName);
#endif
	fprintf(fp0, "          [-dx:VALUE] [-dz:VALUE] [-cl:VALUE] file2\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Convert 1, 4, or 8-bits gray-scale Bitmap into\n");
	fprintf(fp0, "  R-AXIS Image File. Colour Bitmap, or RLE-encoded Bitmap may cause error.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -r .......... Reverse Intensity. i.e. \"0\" corresponds to White.\n");
	fprintf(fp0, "   -e .......... Intensity is converted with Exponential Function.\n");
	fprintf(fp0, "   -dx:VALUE ... Set PixelSize in X-direction to \"VALUE\" [mm].\n");
	fprintf(fp0, "                 If absent, try to get infomation from Bitmap File,\n");
	fprintf(fp0, "                 and if failed, set to %.5f.\n", DEFAULT_SIZE);
	fprintf(fp0, "   -dz:VALUE ... Set PixelSize in Z-direction to \"VALUE\".\n");
	fprintf(fp0, "                 If absent, same as above.\n");
	fprintf(fp0, "   -cl:VALUE ... Change Camera Langth to \"VALUE\" [mm].\n");
	fprintf(fp0, "                 If absent, set to %.5f.\n", DEFAULT_LENG);
	fprintf(fp0, "   -rm ......... Remove Original Bitmap File after conversion.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t .......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ....... Filename of Original Bitmap File.\n");
	fprintf(fp0, "   file2 ....... Output R-AXIS Image Filename. This must be the last argument.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	double *arg1;
#else
char splitArg(const char *argV, char *arg0, double *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	return (2);
}
