/* Program to treat IP data file */
#define ProgName "ipcentre"
#define ProgInfo "Estimate the Centre of the Image"
/*                                                         1/27/2009  */
/*         for ANSI C                                                 */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v0.03_2/2/2009"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#define MAXARG 8
#define MINARG 1
#define MAXWIDTH 10000		/* these values are important in this program, */
#define MAXHEIGHT 10000		/* in order to prevent overflow */
#define DLONG 4294967296.0
#define EXRATEMIN 0.01
#define EXRATEMAX 0.1
#define EXRATEDEFAULT 0.1

#ifdef _OLDHP
	long int (*readDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#ifndef _HAS_LONGLONG
	char ull_add();
	char ull_mul();
#endif
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1);
#ifndef _HAS_LONGLONG
	char ull_add(unsigned long int *x, unsigned long int *base);
	char ull_mul(const unsigned long int x, const unsigned long int y, unsigned long int *base);
#endif
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr;
	long int i, j, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2,
	         readByte, startX, startZ, endX, endZ, ii, jj, scanX, scanZ,
	         scanX0, scanX1, scanZ0, scanZ1;
	unsigned long int maxInt, intensity, *datPtrL, pos, pos2;
	char file1[FNAMELEN], inputBuffer[20], argNum,
	     confirmBuffer[8], cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag,
	     shiftBit, strangeShiftBitFlag, byte, arg0[4], verboseFlag,
	     widthDoneFlag, heightDoneFlag, drtxFlag, drtzFlag,
	     useHeaderFlag, noHeadFlag, writeFlag, extraSearchFlag, exRateFlag;
	double rato, arg1, width, height, widthMin, heightMin, exRate;
#ifdef _HAS_LONGLONG
	unsigned long long int sumx, sumz, sumI;
#else
	unsigned long int sumx[2], sumz[2], sumI[2], ulbuf[2];
	double dsum;
#endif
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1;
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	ipDataSwapFlag = widthDoneFlag = heightDoneFlag = drtxFlag = drtzFlag
	 = useHeaderFlag = noHeadFlag = writeFlag = extraSearchFlag
	 = exRateFlag = 0;
	verboseFlag = 1;
	readByte = 0;
	maxInt = 0x7fff;
	widthMin = heightMin = 0.0;
	width = height = 0.0;
	startX = startZ = endX = endZ = 0;
	datPtrL = (unsigned long int*)NULL;
	pos = pos2 = 0;
	exRate = EXRATEDEFAULT;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) ipHeader.zpxl = ipHeader.rnum;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	rato = ipHeader.rato;
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
		fprintf(stdout, "  Original Centre (Direct Beam Position : (%7.2f, %7.2f)\n", ipHeader.drtx, ipHeader.drtz);
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
		}
	}
	
/* Get Parameters to Output */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Do you decice Tentative Centre Position to restrict region ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			do {
				fprintf(stdout, " Use (%7.2f, %7.2f) ? [Y/N] : ", ipHeader.drtx, ipHeader.drtz);
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) != 'Y') {
				do {
					fprintf(stdout, "  Enter Tentative Centre Position in X : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					if (!isdigitEXT(inputBuffer)) continue;
					ipHeader.drtx = atof(inputBuffer);
					if ((ipHeader.drtx <= 0.0) || (ipHeader.drtx >= ipHeader.xpxl)) continue;
					drtxFlag = 1;
				} while (!drtxFlag);
				do {
					fprintf(stdout, "  Enter Tentative Centre Position in Z : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					if (!isdigitEXT(inputBuffer)) continue;
					ipHeader.drtz = atof(inputBuffer);
					if ((ipHeader.drtz <= 0.0) || (ipHeader.drtz >= ipHeader.zpxl)) continue;
					drtzFlag = 1;
				} while (!drtzFlag);
			}
		}
		else {
			ipHeader.drtx = ipHeader.xpxl / 2.0;
			ipHeader.drtz = ipHeader.zpxl / 2.0;
		}
		widthMin = floor(ipHeader.drtx < ipHeader.xpxl-ipHeader.drtx ? ipHeader.drtx : ipHeader.xpxl-ipHeader.drtx);
		heightMin = floor(ipHeader.drtz < ipHeader.zpxl-ipHeader.drtz ? ipHeader.drtz : ipHeader.zpxl-ipHeader.drtz);
		do {
			fprintf(stdout, "Do you want to restrict region ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			do {
				fprintf(stdout, " Enter half width [1 - %4ld], or \"0\" to use default : ", (long)floor(widthMin));
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				width = atof(inputBuffer);
				if (width == 0.0) width = widthMin;
				startX = (long)floor(ipHeader.drtx - width);
				endX = (long)ceil(ipHeader.drtx + width);
				if ((startX >= endX) || (startX < 0) || (endX > ipHeader.xpxl)) continue;
				widthDoneFlag = 1;
			} while (!widthDoneFlag);
			do {
				fprintf(stdout, " Enter half height [1 - %4ld], or \"0\" to use default : ", (long)floor(heightMin));
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				height = atof(inputBuffer);
				if (height == 0.0) height = heightMin;
				startZ = (long)floor(ipHeader.drtz - height);
				endZ = (long)ceil(ipHeader.drtz + height);
				if ((startZ >= endZ) || (startZ < 0) || (endZ > ipHeader.zpxl)) continue;
				heightDoneFlag = 1;
			} while (!heightDoneFlag);
		}
		do {
			fprintf(stdout, "Do you want to execute extra search ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			extraSearchFlag = 1;
			do {
				fprintf(stdout, " Search +/- %5.3f of images around centre. OK ? [Y/N] : ", EXRATEDEFAULT);
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) != 'Y') {
				do {
					fprintf(stdout, "  Enter ratio [%5.3f - %5.3f] : ", EXRATEMIN, EXRATEMAX);
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					if (!isdigitEXT(inputBuffer)) continue;
					exRate = atof(inputBuffer);
					if ((exRate < EXRATEMIN) || (exRate > EXRATEMAX)) continue;
					exRateFlag = 1;
				} while (!exRateFlag);
			}
		}
	}
	else {
		for (i=1; i<argc; i++) {
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-sh")) noHeadFlag++;
				else if (!strcmp(arg0, "-w")) writeFlag++;
				else if (!strcmp(arg0, "-d")) useHeaderFlag++;
				else if (!strcmp(arg0, "-e")) extraSearchFlag++;
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-xc")) {
					ipHeader.drtx = arg1;
					if ((ipHeader.drtx <= 0.0) || (ipHeader.drtx >= ipHeader.xpxl)) break;
					drtxFlag++;
				}
				else if (!strcmp(arg0, "-zc")) {
					ipHeader.drtz = arg1;
					if ((ipHeader.drtz <= 0.0) || (ipHeader.drtz >= ipHeader.zpxl)) break;
					drtzFlag++;
				}
				else if (!strcmp(arg0, "-x")) {
					width = arg1;
					if ((width <= 0.0) || (width >= ipHeader.xpxl/2.0)) break;
					widthDoneFlag++;
				}
				else if (!strcmp(arg0, "-z")) {
					height = arg1;
					if ((height <= 0.0) || (height >= ipHeader.zpxl/2.0)) break;
					heightDoneFlag++;
				}
				else if (!strcmp(arg0, "-e")) {
					exRate = arg1;
					if ((exRate < EXRATEMIN) || (exRate > EXRATEMAX)) break;
					extraSearchFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((drtxFlag > 1) || (drtzFlag > 1) || (widthDoneFlag > 1)
		 || (heightDoneFlag > 1) || (writeFlag >1) || (useHeaderFlag > 1)
		 || (noHeadFlag > 1) || (extraSearchFlag > 1)
		 || ((drtxFlag || drtzFlag) && useHeaderFlag)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!useHeaderFlag) {
			if (!drtxFlag) ipHeader.drtx = ipHeader.xpxl / 2.0;
			if (!drtzFlag) ipHeader.drtz = ipHeader.zpxl / 2.0;
		}
		widthMin = floor(ipHeader.drtx < ipHeader.xpxl-ipHeader.drtx ? ipHeader.drtx : ipHeader.xpxl-ipHeader.drtx);
		heightMin = floor(ipHeader.drtz < ipHeader.zpxl-ipHeader.drtz ? ipHeader.drtz : ipHeader.zpxl-ipHeader.drtz);
		if (!widthDoneFlag) width = widthMin;
		if (!heightDoneFlag) height = heightMin;
		startX = (long)floor(ipHeader.drtx - width);
		startZ = (long)floor(ipHeader.drtz - height);
		endX = (long)ceil(ipHeader.drtx + width);
		endZ = (long)ceil(ipHeader.drtz + height);
		if ((startX >= endX) || (startZ >= endZ)
		 || (startX < 0) || (endX > ipHeader.xpxl)
		 || (startZ < 0) || (endZ > ipHeader.zpxl)) {
			fprintf(stderr, " Invalid Value in Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	if (verboseFlag == 2) {
		fprintf(stdout, " Region to use in Calculation :\n");
		fprintf(stdout, "     (%4ld, %4ld) - (%4ld, %4ld).\n", startX, startZ, endX, endZ);
	}
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if (fseek(fp1, ipHeader.rlng*(1+startZ), SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
#ifdef _HAS_LONGLONG
	sumx = sumz = sumI = 0;
#else
	sumx[0] = sumx[1] = sumz[0] = sumz[1] = sumI[0] = sumI[1] = 0;
#endif
	for (i=startZ; i<endZ; i++) {
		if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		fread(datPtr, ipHeader.rlng, 1, fp1);
		readByte += ipHeader.rlng;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
			free(datPtr);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		for (j=startX; j<endX; j++) {
			intensity = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
#ifdef _HAS_LONGLONG
			sumI += intensity;
			sumx += intensity * j;
			sumz += intensity * i;
#else
			ulbuf[0] = 0;
			ulbuf[1] = intensity;
			ull_add(ulbuf, sumI);
			ull_mul(intensity, j, ulbuf);
			ull_add(ulbuf, sumx);
			ull_mul(intensity, i, ulbuf);
			ull_add(ulbuf, sumz);
#endif
		}
		free(datPtr);
	}
	/* output */
#ifdef _HAS_LONGLONG
	ipHeader.drtx = (double)sumx/sumI + 0.5;
	ipHeader.drtz = (double)sumz/sumI + 0.5;
#else
	dsum = sumI[1] + sumI[0]*DLONG;
	ipHeader.drtx = (sumx[1] + sumx[0]*DLONG) / dsum + 0.5;
	ipHeader.drtz = (sumz[1] + sumz[0]*DLONG) / dsum + 0.5;
#endif
	if (noHeadFlag) fprintf(stdout, "%8.2f  %8.2f\n", ipHeader.drtx, ipHeader.drtz);
	else fprintf(stdout, "Centre is estimated to be (%8.2f , %8.2f ).\n", ipHeader.drtx, ipHeader.drtz);
	
	/* deep investigation */
	if (extraSearchFlag) {
		ipHeader.drtx = dToLong(ipHeader.drtx*2.0)/2.0;
		ipHeader.drtz = dToLong(ipHeader.drtz*2.0)/2.0;
		widthMin = floor(ipHeader.drtx < ipHeader.xpxl-ipHeader.drtx ? ipHeader.drtx : ipHeader.xpxl-ipHeader.drtx);
		heightMin = floor(ipHeader.drtz < ipHeader.zpxl-ipHeader.drtz ? ipHeader.drtz : ipHeader.zpxl-ipHeader.drtz);
		startX = (long)floor(ipHeader.drtx - widthMin);
		startZ = (long)floor(ipHeader.drtz - heightMin);
		endX = (long)ceil(ipHeader.drtx + widthMin);
		endZ = (long)ceil(ipHeader.drtz + heightMin);
		if ((startX >= endX) || (startZ >= endZ)
		 || (startX < 0) || (endX > ipHeader.xpxl)
		 || (startZ < 0) || (endZ > ipHeader.zpxl)) {
			fprintf(stderr, "Estimated Centre position is wrong; no further investigation.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		scanX = (long)ceil((endX - startX)*exRate);
		scanZ = (long)ceil((endZ - startZ)*exRate);
		startX += scanX*2;
		startZ += scanZ*2;
		endX -= scanX*2;
		endZ -= scanZ*2;
		scanX0 = dToLong(2.0*ipHeader.drtx) - 2*scanX;	/* offset by 1 to reduce calculation */
		scanX1 = dToLong(2.0*ipHeader.drtx) + 2*scanX - 1;
		scanZ0 = dToLong(2.0*ipHeader.drtz) - 2*scanZ;
		scanZ1 = dToLong(2.0*ipHeader.drtz) + 2*scanZ - 1;
		/*
		 * NOTES:
		 *  Evaluate sum{I(x,z)*I(2*x0-x,2*z0-z)} (sum for x, z)
		 *  about a fixed rectangular region.
		 *  Then move the centre(x0,z0), and find the centre having maximum sum;
		 *  the rectangular region is not changed.
		 *  The position where sum[{I(x,z)-I(2*x0-x,2*z0-z)}^2]/sqrt[sum{I(x,z)^2}*sum{I(2*x0-x,2*z0-z)^2}]
		 *  takes minimum may be better.
		 *  I(x,z)^2 in the numerator does not change,
		 *  I(2*x0-x,2*z0-z)^2 is fixed for convenience,
		 *  the denominator is neglected (or assumed to be fixed).
		 *  In addition, the reference region used as "I(2*x0-x,2*z0-z)"
		 *  outside of the original region have intensities.
		 *  At first, these are set to be 0, which causes the sum{I(2*x0-x,2*z0-z)} smaller
		 *  than necessary.
		 */
		if (verboseFlag == 2) {
			fprintf(stdout, " Region to scan Centre :\n");
			fprintf(stdout, "     (%6.1f, %6.1f) - (%6.1f, %6.1f).\n",
			        (scanX0+1)/2.0, (scanZ0+1)/2.0, (scanX1+1)/2.0, (scanZ1+1)/2.0);
			fprintf(stdout, " Region to use in Extra Calculation :\n");
			fprintf(stdout, "     (%4ld, %4ld) - (%4ld, %4ld).\n", startX, startZ, endX, endZ);
		}
		if ((datPtrL = calloc(ipHeader.xpxl*ipHeader.zpxl, sizeof(unsigned long int))) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (fseek(fp1, ipHeader.rlng, SEEK_SET)) {
			fprintf(stderr, " Input File Read Error.\n");
			free(datPtrL);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		readByte = ftell(fp1);
		pos = 0;
		for (i=0; i<ipHeader.zpxl; i++) {
			if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrL);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fread(datPtr, ipHeader.rlng, 1, fp1);
			readByte += ipHeader.rlng;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtrL);
				free(datPtr);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			
			for (j=0; j<ipHeader.xpxl; j++) {
				datPtrL[pos++] = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
			}
			free(datPtr);
		}
#ifdef _HAS_LONGLONG
		sumx = 0;	/* Max of sumI */
#else
		sumx[0] = sumx[1] = 0;
#endif
		if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
		for (ii=scanZ0; ii<scanZ1; ii++) {
			for (jj=scanX0; jj<scanX1; jj++) {
#ifdef _HAS_LONGLONG
				sumI = 0;
#else
				sumI[0] = sumI[1] = 0;
#endif
				for (i=startZ; i<endZ; i++) {
						/* scan most part of the region twice, for the order of the product
						   is not considered. for the convenience. */
					pos = i*ipHeader.xpxl+startX;
				/*	pos2 = (ii-1)*ipHeader.xpxl+(jj-1) - pos;	*/
					pos2 = ii*ipHeader.xpxl+jj - pos;
					for (j=startX; j<endX; j++) {
#ifdef _HAS_LONGLONG
						sumI += ((unsigned long long int)datPtrL[pos])*datPtrL[pos2];
#else
						ull_mul(datPtrL[pos], datPtrL[pos2], ulbuf);
						ull_add(ulbuf, sumI);
#endif
						pos++;
						pos2--;
					}
				}
#ifdef _HAS_LONGLONG
				if (sumI > sumx) {
					sumx = sumI;
					width = jj;
					height = ii;	/* temporary use */
				}
#else
				if ((sumI[0] > sumx[0])
				 || ((sumI[0] == sumx[0]) && (sumI[1]>sumx[1]))) {
					sumx[0] = sumI[0];
					sumx[1] = sumI[1];
					width = jj;
					height = ii;	/* temporary use */
				}
#endif
			}
			if (verboseFlag == 2) {
				fprintf(stdout, ".");
				fflush(stdout);
			}
		}
		free(datPtrL);
		if (verboseFlag == 2) fprintf(stdout, "\n");
		/* output */
		ipHeader.drtx = width/2.0;
		ipHeader.drtz = height/2.0;
		if (noHeadFlag) fprintf(stdout, "%8.2f  %8.2f\n", ipHeader.drtx, ipHeader.drtz);
		else fprintf(stdout, "Extra Calculation tells Centre : (%8.1f , %8.1f ).\n", ipHeader.drtx, ipHeader.drtz);
	}
	
/* End */
	fclose(fp1);
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Dou you want to record this as new Direct Beam Position ? [Y/N] : \n");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') writeFlag = 1;
	}
	
	if (writeFlag) {
		/* Restore endian on ipHeader */
		if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
		if ((fp1 = fopen(file1, "r+")) == NULL) {
			fprintf(stderr, " Unable to Write to the File.\n");
			exit(EXIT_FAILURE);
		}
		fwrite(&ipHeader, ipHeaderSize, 1, fp1);
		fclose(fp1);
	}
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
	fprintf(fp0, "        %s [-q|-v] file {[-xc:VALUE] [-zc:VALUE]|[-d]}\n", ProgName);
	fprintf(fp0, "          [-x:VALUE] [-z:VALUE] [-sh] [-w] [-e[:VALUE]]\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program to Estimate the Centre of R-AXIS Image File.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   file ........ Filename of R-AXIS Image File.\n");
	fprintf(fp0, "   -xc:VALUE ... Set Tentative Centre Position X to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -zc:VALUE ... Set Tentative Centre Position Z to \"VALUE\" [pixel].\n");
	fprintf(fp0, "                 These options are useful to restrict regions in calculation.\n");
	fprintf(fp0, "                 Use geometrical centre if absent.\n");
	fprintf(fp0, "   -d .......... Use Direct Beam Position stored in the original R-AXIS\n");
	fprintf(fp0, "                 Image File as Tentative Centre Position.\n");
	fprintf(fp0, "                 With this option, available maximum rectangular region\n");
	fprintf(fp0, "                 around Direct Beam Position are used in calculation,\n");
	fprintf(fp0, "                 if other options restrict it.\n");
	fprintf(fp0, "                 This option is exclusive with above 2 options.\n");
	fprintf(fp0, "   -x:VALUE .... Set half width around tentative centre position\n");
	fprintf(fp0, "                 in calculation to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -z:VALUE .... Set half height around tentative centre position\n");
	fprintf(fp0, "                 in calculation to \"VALUE\" [pixel].\n");
	fprintf(fp0, "   -sh ......... Output Coordinates only.\n");
	fprintf(fp0, "   -w .......... Write the result to original R-AXIS File Header\n");
	fprintf(fp0, "                 as the Centre Position.\n");
	fprintf(fp0, "   -e .......... Do extra search after normal calculation.\n");
	fprintf(fp0, "                 Check the balance around the centre,\n");
	fprintf(fp0, "                 assuming axial symmetry around the centre.\n");
	fprintf(fp0, "                 With this option, Centre Position is estimated\n");
	fprintf(fp0, "                 by a resolution of a half pixel.\n");
	fprintf(fp0, "   -e:VALUE .... Just same as above, but decide the region\n");
	fprintf(fp0, "                 to scan centre by setting \"VALUE\" as the ratio\n");
	fprintf(fp0, "                 around the centre.\n");
	fprintf(fp0, "                 Allowed \"VALUE\" is between %5.3f and %5.3f.\n", EXRATEMIN, EXRATEMAX);
	fprintf(fp0, "   file2 ....... Output Filename. This must be the last argument.\n");
	fprintf(fp0, "                 If last argument is \"-\", Output to Standard Output.\n");
	return (0);
}

#ifndef _HAS_LONGLONG
#ifdef _OLDHP
char ull_add(x, base)
	unsigned long int *x, *base;
#else
char ull_add(unsigned long int *x, unsigned long int *base)
#endif
{
	unsigned long int buf;
	/* base[0] is high, base[1] is low */
	buf = base[1];
	if (buf > (base[1] += x[1])) base[0]++;
	base[0] += x[0];
	return (0);
}
#endif

#ifndef _HAS_LONGLONG
#ifdef _OLDHP
char ull_mul(x, y, base)
	unsigned long int x, y, *base;
#else
char ull_mul(const unsigned long int x, const unsigned long int y, unsigned long int *base)
#endif
{
	unsigned long int buf[2], xbuf[2] , ybuf[2], nbuf[2];
	/* base[0] is high, base[1] is low */
	/* init */
	xbuf[0] = x >> 16;
	xbuf[1] = x & 0xffff;
	ybuf[0] = y >> 16;
	ybuf[1] = y & 0xffff;
	base[1] = xbuf[1]*ybuf[1];
	base[0] = xbuf[0]*ybuf[0];
	/* 2^16 */
	buf[0] = 0;
	buf[1] = xbuf[0]*ybuf[1];
	nbuf[0] = 0;
	nbuf[1] = xbuf[1]*ybuf[0];
	ull_add(nbuf, buf);
	/* add to 2^32 and 1^0 */
	nbuf[1] = buf[1] << 16;
	ull_add(nbuf, base);
	base[0] += ((buf[1] >> 16) + (buf[0] << 16));
	return (0);
}
#endif

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	double *arg1;
#else
char splitArg(const char *argV, char *arg0, double *arg1)
#endif
{
	char *chPtr1, *chPtr2;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	if ((chPtr2 = strchr(chPtr1+1, ':')) == NULL) return (2);
	return (3);
}
