/* Program to convert IP data file */
#define ProgName "ipchrato"
#define ProgInfo "R-AXIS Image Hi/Low rato conversion"
/*                                                        12/28/1999  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.17_1/15/2007"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#ifdef _HASUTIME
#	include <sys/stat.h>
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef _HASUTIME
#	define MAXARG 5
#else
#	define MAXARG 4
#endif
#define MINARG 2
#define MAXWIDTH 10000
#define MAXHEIGHT 10000

#ifdef _OLDHP
	long int (*readDat)();
	short int (*writeDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	short int (*writeDat)(const long int *x, const double *rato, const char *shiftBit, const unsigned long int *maxInt, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr;
	long int i, j, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2, recordLength,
	     recordLengthCreate, ipHeaderSizeMin, intensity, readByte,
	     flushWidth, flushWidth0;
	unsigned long int maxInt, maxIntNew;
	char file1[FNAMELEN], file2[FNAMELEN], inputBuffer[20], argNum,
	     confirmBuffer[8], cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag,
	     shiftBit, shiftBitNew, strangeShiftBitFlag, strangeShiftBitFlagCreate,
	     byte, createDosFlag, arg0[4], verboseFlag, ratoDoneFlag, ignoreErrDoneFlag,
	     replaceFlag, setTimeFlag;
#ifdef _HAS_STRANGE_RENAME
	char fnameTmp[FNAMELEN];
#endif
	double rato, ratoNew, arg1;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	verboseFlag = 1;
	readByte = 0;
	ipDataSwapFlag = ratoDoneFlag = ignoreErrDoneFlag = replaceFlag = setTimeFlag = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
		if (!strcmp(argv[argc-1], "-replace")) replaceFlag++;
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
		createDosFlag = 1;
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
		createDosFlag = 0;
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) {
		ipHeader.zpxl = ipHeader.rnum;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height has modified.\n");
	}
	recordLength = ipHeader.rlng;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) {
		ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width has modified.\n");
	}
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (ipHeader.xsiz <= 0.0) {
		ipHeader.xsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in X has modified to 0.05 [mm].\n");
	}
	if (ipHeader.zsiz <= 0.0) {
		ipHeader.zsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in Z has modified to 0.05 [mm].\n");
	}
	rato = ipHeader.rato;
	if (ipHeader.cyld >= 256) {
		if (verboseFlag) fprintf(stdout, " Camera Type Code has changed from %ld to 0.\n", ipHeader.cyld);
		ipHeader.cyld = 0;
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
		fprintf(stdout, "  Hi/Low Rato is now set to %.3f.\n", rato);
	}
	
/* Clean Header */
	cleanIpHeader(&ipHeader);
	createDosFlag ? cleanIpHeader2VER1(&ipHeader2VER1) : cleanIpHeader2VER2(&ipHeader2VER2);
	
/* Logging */
	if (logWrite(&ipHeader, ProgName)) {
		if (verboseFlag) fprintf(stdout, " ipHeader.memo is full, and No Logging.\n");
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato >= MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
		}
	}
	
/* Get Magnification */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Enter New Hi/Low Rato Value : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ratoNew = atof(inputBuffer);
			if ((ratoNew < MINRATO) || (ratoNew > MAXRATO)) continue;
			ratoDoneFlag = 1;
		} while (!ratoDoneFlag);
		do {
			fprintf(stdout, "Replace Original R-AXIS File with Output File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') replaceFlag = 1;
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-f")) ignoreErrDoneFlag++;
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-r")) {
					ratoNew = arg1;
					if ((ratoNew < MINRATO) || (ratoNew > MAXRATO)) break;
					ratoDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!ratoDoneFlag) {
			fprintf(stderr, " Lack of arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((ratoDoneFlag > 1) || (ignoreErrDoneFlag > 1) || (setTimeFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Check New rato value */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	ipHeader.rato = ratoNew;
	strangeShiftBitFlagCreate = 1;
	if (ratoNew == 1.0) {
		maxIntNew = 0x7fff;
		writeDat = ipLtoShortBit0;
		shiftBitNew = 0;
	}
	else {
		for (shiftBitNew=1; shiftBitNew<16; shiftBitNew++) {
			if ((double)(0x0001 << shiftBitNew) == ratoNew) {
				strangeShiftBitFlagCreate = 0;
				maxIntNew = ((long)(0x7fff)) << shiftBitNew;
				writeDat = ipLtoShortBit;
				break;
			}
		}
		if (strangeShiftBitFlagCreate) {
			maxIntNew = dToLong((0x7fff) * ratoNew);
			writeDat = ipLtoShortRato;
		}
	}
	if (strangeShiftBitFlagCreate && !ignoreErrDoneFlag) {
		if (verboseFlag) {
			do {
				fprintf(stdout, " \"rato\" is not a power of 2, this might make file unreadable. OK ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Retry again. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		else {
			fprintf(stdout, " \"rato\" is not a power of 2. Stop.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " Hi/Low Rato is set to %.3f.\n", ratoNew);
		if (replaceFlag) fprintf(stdout, " Original R-AXIS Image File will be replaced !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Set Parameters to Create */
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(ipHeaderSizeMin , flushWidth);
	if (createDosFlag) recordLengthCreate = ((long)(ceil(recordLengthCreate/512.0)))*512;
	if (ipHeader.drtx >= ipHeader.xpxl) ipHeader.drtx = ipHeader.xpxl - 1.0;
	else if (ipHeader.drtx < 0.0) ipHeader.drtx = 0.0;
	if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
	else if (ipHeader.drtz < 0.0) ipHeader.drtz = 0.0;
	ipHeader.rlng = recordLengthCreate;
	flushWidth0 = recordLengthCreate - flushWidth;
	
/* Create Output File */
	if (replaceFlag) {
		if (mkstempEXT(file1, file2, &fp2, 0)) {
			fprintf(stderr, " Temporary File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		if (verboseFlag == 3) {
			fprintf(stdout, "Enter  Output  Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		while (createFileEXT(file2, &fp2, 0)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
	}
	
/* Restore endian on ipHeader */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	createDosFlag ? fwrite(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp2) : fwrite(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSizeMin, 1, fp2);
	if (fseek(fp1, recordLength, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	if (flushWidth0 == 0) free(datPtr0);
	
/* Swap Header again */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	for (i=0; i<ipHeader.rnum; i++) {
		if ((datPtr = malloc(recordLength)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtr);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			if (i) {
				ipHeader.zpxl = ipHeader.rnum = i;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
					}
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		fread(datPtr, recordLength, 1, fp1);
		readByte += recordLength;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
			free(datPtr);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			if (i) {
				ipHeader.zpxl = ipHeader.rnum = i;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
					}
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		for (j=0; j<ipHeader.xpxl; j++) {
			intensity = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
			datPtr[j] = (*writeDat)(&intensity, &ratoNew, &shiftBitNew, &maxIntNew, &ipDataSwapFlag);
		}
		fwrite(datPtr, flushWidth, 1, fp2);
		if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
		free(datPtr);
	}
	
/* End */
	if (flushWidth0 > 0) free(datPtr0);
	fclose(fp1);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
#ifndef _HAS_STRANGE_RENAME
	if (replaceFlag) {
		if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			fprintf(stderr, " Original R-AXIS Image File might be lost... Check It !!\n");
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#else
	if (replaceFlag) {
		if ((fp1 = fopen(file1, "rb+")) == NULL) {
			fprintf(stderr, "Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			replaceFlag = 0;
		}
		else fclose(fp1);
	}
	if (replaceFlag) {
		if (tmpNameEXT(file1, fnameTmp)) strcpy(fnameTmp, tmpnam(NULL));
		if (rename(file1, fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
		}
		else if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (remove(fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#endif
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-t] [-f] -r:VALUE {file2|-replace}\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-f] -r:VALUE {file2|-replace}\n", ProgName);
#endif
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Change Hi/Low Rato of R-AXIS Image File.\n");
	fprintf(fp0, "   -h ......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i ......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q ......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v ......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -r:VALUE ... Set Hi/Low rato to \"VALUE\". This value is usually set\n");
	fprintf(fp0, "                to 8.0 or 32.0, used to gain dynamic range.\n");
	fprintf(fp0, "                This must be in range from %.1f to %.1f\n", MINRATO, MAXRATO);
	fprintf(fp0, "   -f ......... \"rato\" should be a power of 2. If this option is set,\n");
	fprintf(fp0, "                ignore this constraint. This may make the file unreadable...\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t ......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ...... Filename of Original R-AXIS Image File.\n");
	fprintf(fp0, "   file2 ...... Output R-AXIS Image Filename. This must be the last argument.\n");
	fprintf(fp0, "                If last argument is \"-replace\", Original R-AXIS Image File is\n");
	fprintf(fp0, "                replaced by Output File. Be careful using this option.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	double *arg1;
#else
char splitArg(const char *argV, char *arg0, double *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 2) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 2) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	return (2);
}
