/* Program to convert IP data file */
#define ProgName "ipdiv"
#define ProgInfo "Get Divided (or relative) intensities with other Data"
/*                                                         9/26/2001  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.05_1/15/2007"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#ifdef _HASUTIME
#	include <sys/stat.h>
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef _HASUTIME
#	define MAXARG 7
#else
#	define MAXARG 6
#endif
#define MINARG 3
#define MAXWIDTH 10000
#define MAXHEIGHT 10000

#ifdef _OLDHP
	long int (*readDat1)();
	long int (*readDat2)();
	short int (*writeDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	long int (*readDat1)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	long int (*readDat2)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	short int (*writeDat)(const double *x, const double *rato, const unsigned long int *maxInt, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr1, *datPtr2;
	long int i, j, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2, recordLength,
	     ipHeaderSizeMin, readByte1,
	     readByte2, recordLengthCreate, padInt, flushWidth, flushWidth0;
	unsigned long int maxInt1, maxInt2;
	char file1[FNAMELEN], file2[FNAMELEN], file3[FNAMELEN], shiftBit1,
	     shiftBit2, cpuIntelFlag, ipHeaderSwapFlag1, ipHeaderSwapFlag2,
	     strangeShiftBitFlag, ipDataSwapFlag1, ipDataSwapFlag2, byte1, byte2,
	     argNum, arg0[4], createDosFlag, verboseFlag, file1DoneFlag, file2DoneFlag,
	     magDoneFlag, inputBuffer[20], confirmBuffer[8], replaceFlag,
	     setTimeFlag, set0Flag, padDoneFlag;
#ifdef _HAS_STRANGE_RENAME
	char fnameTmp[FNAMELEN];
#endif
	double mag, rato1, rato2, arg1, intensityDbl;
	struct ipHead ipHeaderF1, ipHeaderF2;
	struct ipHead2VER1 ipHeader2VER1F1, ipHeader2VER1F2;
	struct ipHead2VER2 ipHeader2VER2F1, ipHeader2VER2F2;
	FILE *fp1, *fp2, *fp3;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	readByte1 = readByte2 = 0;
	verboseFlag = 1;
	mag = 1.0;
	ipDataSwapFlag1 = ipDataSwapFlag2 = set0Flag = file1DoneFlag = file2DoneFlag =
	magDoneFlag = replaceFlag = setTimeFlag = padDoneFlag = 0;
	padInt = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Argument */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		for (i=0; i<argc-1; i++) {
			if (!strncmp(argv[i], "-f1:", 4)) {
				strcpy(file1, argv[i]+4);
				file1DoneFlag++;
			}
			else if (!strncmp(argv[i], "-f2:", 4)) {
				strcpy(file2, argv[i]+4);
				file2DoneFlag++;
			}
		}
		if (!file1DoneFlag || !file2DoneFlag) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Lack of Argument (File Name).\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if ((file1DoneFlag > 1) || (file2DoneFlag > 1)) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Arguments Duplicated (File Name).\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file3, argv[argc-1]);
		if (!strcmp(argv[argc-1], "-replace")) replaceFlag = 1;
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read 1 */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename 1 : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File 1.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeaderF1, ipHeaderSize, 1, fp1);
	readByte1 += ipHeaderSize;
	if (ftell(fp1) != readByte1) {
		fprintf(stderr, " Input File 1 Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check 1 */
	ipHeaderSwapFlag1 = (ipHeaderF1.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
	   if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag1)) {
		if (verboseFlag) {
			fprintf(stdout, " File 1 is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1F1, ipHeaderSize2VER1, 1, fp1);
		readByte1 += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte1) {
			fprintf(stderr, " Input File 1 Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
		createDosFlag = 1;
	}
	else {
		fread(&ipHeader2VER2F1, ipHeaderSize2VER2, 1, fp1);
		readByte1 += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte1) {
			fprintf(stderr, " Input File 1 Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
		createDosFlag = 0;
	}
	
/* Swap Header 1 */
	if (ipHeaderSwapFlag1) {
		swapIpHeader(&ipHeaderF1);
		if (createDosFlag) swapIpHeader2VER1(&ipHeader2VER1F1);
		else swapIpHeader2VER2(&ipHeader2VER2F1);
	}
	byte1 = createDosFlag ? ipHeader2VER1F1.byte : ipHeader2VER2F1.byte;
	
/* Get ipHeader Information and Check 1 */
	if (XOREXT(byte1, ipHeaderSwapFlag1)) ipDataSwapFlag1 = 1;
	rato1 = ipHeaderF1.rato;
	if (ipHeaderF1.zpxl != ipHeaderF1.rnum) {
		ipHeaderF1.zpxl = ipHeaderF1.rnum;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height in File 1 has modified.\n");
	}
	recordLength = ipHeaderF1.rlng;
	if (ipHeaderF1.xpxl*RAXDATAWIDTH > ipHeaderF1.rlng) {
		ipHeaderF1.xpxl = ipHeaderF1.rlng/RAXDATAWIDTH;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width in File 1 has modified.\n");
	}
	if ((ipHeaderF1.xpxl <= 0) || (ipHeaderF1.xpxl > MAXWIDTH) || (ipHeaderF1.zpxl <= 0) || (ipHeaderF1.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " Format of file1 is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (ipHeaderF1.xsiz <= 0.0) {
		ipHeaderF1.xsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in X in File 1 has modified to 0.05 [mm].\n");
	}
	if (ipHeaderF1.zsiz <= 0.0) {
		ipHeaderF1.zsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in Z in File 1 has modified to 0.05 [mm].\n");
	}
	if (ipHeaderF1.cyld >= 256) {
		if (verboseFlag) fprintf(stdout, " Camera Type Code has changed from %ld to 0.\n", ipHeaderF1.cyld);
		ipHeaderF1.cyld = 0;
	}
	
/* Check bits to shift to gain dynamic-range 1 */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato1 < MINRATO) || (rato1 > MAXRATO)) {
		fprintf(stderr, " Input File 1 Format Error : ipHeader.rato. Stop.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato1 == 1.0) {
		maxInt1 = 0x7fff;
		readDat1 = ipStoLongBit0;
		writeDat = ipDtoShortBit0;
		shiftBit1 = 0;
	}
	else {
		for (shiftBit1=1; shiftBit1<16; shiftBit1++) {
			if ((double)(0x0001 << shiftBit1) == rato1) {
				strangeShiftBitFlag = 0;
				maxInt1 = ((long)(0x7fff)) << shiftBit1;
				readDat1 = ipStoLongBit;
				writeDat = ipDtoShort;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt1 = dToLong((0x7fff) * rato1);
			readDat1 = ipStoLongRato;
			writeDat = ipDtoShort;
		}
	}
	
/* File Read 2 */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter Original Filename 2 : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	if ((fp2 = fopen(file2, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File 2.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	fread(&ipHeaderF2, ipHeaderSize, 1, fp2);
	readByte2 += ipHeaderSize;
	if (ftell(fp2) != readByte2) {
		fprintf(stderr, " Input File 2 Read Error.\n");
		fclose(fp2);
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check 2 */
	ipHeaderSwapFlag2 = (ipHeaderF2.rlng & 0x0000ffff) ? 0 : 1;
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag2)) {
		if (verboseFlag) {
			fprintf(stdout, " File 2 is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp2);
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp2);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		fread(&ipHeader2VER1F2, ipHeaderSize2VER1, 1, fp2);
		readByte2 += ipHeaderSize2VER1;
		if (ftell(fp2) != readByte2) {
			fprintf(stderr, " Input File 2 Read Error.\n");
			fclose(fp2);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		fread(&ipHeader2VER2F2, ipHeaderSize2VER2, 1, fp2);
		readByte2 += ipHeaderSize2VER2;
		if (ftell(fp2) != readByte2) {
			fprintf(stderr, " Input File 2 Read Error.\n");
			fclose(fp2);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Swap Header 2 */
	if (ipHeaderSwapFlag2) {
		swapIpHeader(&ipHeaderF2);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1F2);
		else swapIpHeader2VER2(&ipHeader2VER2F2);
	}
	byte2 = XOREXT(cpuIntelFlag, ipHeaderSwapFlag2) ? ipHeader2VER1F2.byte : ipHeader2VER2F2.byte;
	
/* Get ipHeader Information and Check 2 */
	if (XOREXT(byte2, ipHeaderSwapFlag2)) ipDataSwapFlag2 = 1;
	rato2 = ipHeaderF2.rato;
	if (ipHeaderF2.zpxl != ipHeaderF2.rnum) {
		ipHeaderF2.zpxl = ipHeaderF2.rnum;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height in File 2 has modified.\n");
	}
	if (ipHeaderF2.xpxl*RAXDATAWIDTH > ipHeaderF2.rlng) {
		ipHeaderF2.xpxl = ipHeaderF2.rlng/RAXDATAWIDTH;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width in File 2 has modified.\n");
	}
	if ((ipHeaderF2.xpxl <= 0) || (ipHeaderF2.xpxl > MAXWIDTH) || (ipHeaderF2.zpxl <= 0) || (ipHeaderF2.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " Format of file 2 is Incorrect.\n");
		fclose(fp1);
		fclose(fp2);
		exit(EXIT_FAILURE);
	}
	if (ipHeaderF2.xsiz <= 0.0) {
		ipHeaderF2.xsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in X in File 2 has modified to 0.05 [mm].\n");
	}
	if (ipHeaderF2.zsiz <= 0.0) {
		ipHeaderF2.zsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in Z in File 2 has modified to 0.05 [mm].\n");
	}
	if (ipHeaderF2.cyld >= 256) {
		if (verboseFlag) fprintf(stdout, " Camera Type Code has changed from %ld to 0.\n", ipHeaderF2.cyld);
		ipHeaderF2.cyld = 0;
	}
	
/* Check bits to shift to gain dynamic-range 2 */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato2 < MINRATO) || (rato2 > MAXRATO)) {
		fprintf(stderr, " Input File 2 Format Error : ipHeader.rato. Stop.\n");
		fclose(fp1);
		fclose(fp2);
		exit(EXIT_FAILURE);
	}
	else if (rato2 == 1.0) {
		maxInt2 = 0x7fff;
		readDat2 = ipStoLongBit0;
		writeDat = ipDtoShortBit0;
		shiftBit2 = 0;
	}
	else {
		for (shiftBit2=1; shiftBit2<16; shiftBit2++) {
			if ((double)(0x0001 << shiftBit2) == rato2) {
				strangeShiftBitFlag = 0;
				maxInt2 = ((long)(0x7fff)) << shiftBit2;
				readDat2 = ipStoLongBit;
				writeDat = ipDtoShort;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt2 = dToLong((0x7fff) * rato2);
			readDat2 = ipStoLongRato;
			writeDat = ipDtoShort;
		}
	}
	
/* File Size Check */
	if ((ipHeaderF1.xpxl != ipHeaderF2.xpxl) || (ipHeaderF1.zpxl != ipHeaderF2.zpxl)) {
		fprintf(stderr, " Data size does not match. Exit.\n");
		fclose(fp1);
		fclose(fp2);
		exit(EXIT_FAILURE);
	}
	if ((ipHeaderF1.xsiz != ipHeaderF2.xsiz) || (ipHeaderF1.zsiz != ipHeaderF2.zsiz)) {
		fprintf(stderr, " Pixel size does not match. Exit.\n");
		fclose(fp1);
		fclose(fp2);
		exit(EXIT_FAILURE);
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ipHeaderF1.rlng*(ipHeaderF1.rnum+1));
	}
	if ((ipHeaderSwapFlag1 != ipHeaderSwapFlag2) || (rato1 != rato2)) {
		fprintf(stdout, " File Format is set according to the 1st File.\n");
	}
	
/* Clean Header */
	cleanIpHeader(&ipHeaderF1);
	cleanIpHeader(&ipHeaderF2);
	createDosFlag ? cleanIpHeader2VER1(&ipHeader2VER1F1) : cleanIpHeader2VER2(&ipHeader2VER2F1);
	XOREXT(cpuIntelFlag, ipHeaderSwapFlag2) ? cleanIpHeader2VER1(&ipHeader2VER1F2) : cleanIpHeader2VER2(&ipHeader2VER2F2);
	
/* Logging */
	if (logWrite(&ipHeaderF1, ProgName)) {
		if (verboseFlag) fprintf(stdout, " ipHeader.memo is full, and No Logging.\n");
	}
	
/* Get Parameters or Check Arguments */
	if (verboseFlag == 3) {
	/* Get coefficients for input files */
		do {
			fprintf(stdout, "Enter Magnification for 1st File : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp2);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			mag = atof(inputBuffer);
			if (mag <= 0.0) continue;
			magDoneFlag = 1;
		} while (!magDoneFlag);
	/* Get Constant to Subtract */
		do {
			fprintf(stdout, "Enter Constant Value to subtract temporarily before processing : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp2);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while (!isdigitEXT(inputBuffer));
		padInt = atol(inputBuffer);
	/* 0-treatment check */
		do {
			fprintf(stdout, "Use Max Value instead of 0-inctensity ? [Y/N] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp2);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while (toupper(inputBuffer[0]) != 'Y' && toupper(inputBuffer[0]) != 'N');
		if (toupper(inputBuffer[0]) == 'Y') set0Flag++;
	/* Create File Mode */
		do {
			fprintf(stdout, "Replace Original R-AXIS File with Output File 1 ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp2);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') replaceFlag = 1;
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File 1 ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp2);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
	else {
		for (i=0; i<argc-1; i++) {
			if (!strncmp(argv[i], "-f1:", 4) || !strncmp(argv[i], "-f2:", 4)) continue;
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-x")) set0Flag++;
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-m")) {
					mag = arg1;
					if (mag < 0.0) break;
					magDoneFlag++;
				}
				else if (!strcmp(arg0, "-p")) {
					padInt = arg1;
					padDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp2);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((set0Flag > 1) || (magDoneFlag > 1) || (setTimeFlag > 1)
		 || (padDoneFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp2);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Set Parameters to Create */
	flushWidth = ipHeaderF1.xpxl*RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(ipHeaderSizeMin, flushWidth);
	if (createDosFlag) recordLengthCreate = ((long)(ceil(recordLengthCreate/512.0)))*512;
	if (ipHeaderF1.drtx >= ipHeaderF1.xpxl) ipHeaderF1.drtx = ipHeaderF1.xpxl - 1.0;
	else if (ipHeaderF1.drtx < 0.0) ipHeaderF1.drtx = 0.0;
	if (ipHeaderF1.drtz >= ipHeaderF1.zpxl) ipHeaderF1.drtz = ipHeaderF1.zpxl - 1.0;
	else if (ipHeaderF1.drtz < 0.0) ipHeaderF1.drtz = 0.0;
	ipHeaderF1.rlng = recordLengthCreate;
	flushWidth0 = recordLengthCreate - flushWidth;
	
/* Create Output File */
	if (replaceFlag) {
		if (mkstempEXT(file1, file3, &fp3, 0)) {
			fprintf(stderr, " Temporary File Creation Failure. Exit.\n");
			fclose(fp1);
			fclose(fp2);
			exit(EXIT_FAILURE);
		}
	}
	else {
		if (verboseFlag == 3) {
			fprintf(stdout, "Enter  Output  Filename : ");
			if (fscanf(stdin, "%s", file3) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				fclose(fp2);
				exit(EXIT_FAILURE);
			}
		}
		while (createFileEXT(file3, &fp3, 0)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				fclose(fp2);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file3) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				fclose(fp2);
				exit(EXIT_FAILURE);
			}
		}
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " New File of %s = %s / %s * %6.3f\n", file3, file1, file2, mag);
		if (replaceFlag) fprintf(stdout, " Original R-AXIS Image File 1 will be replaced !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File 1.\n");
#endif
	}
	
/* Restore endian on ipHeader */
	if (ipHeaderSwapFlag1) {
		swapIpHeader(&ipHeaderF1);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1F1) : swapIpHeader2VER2(&ipHeader2VER2F1);
	}
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		fclose(fp3);
		remove(file3);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeaderF1, ipHeaderSize, 1, fp3);
	createDosFlag ? fwrite(&ipHeader2VER1F1, ipHeaderSize2VER1, 1, fp3) : fwrite(&ipHeader2VER2F1, ipHeaderSize2VER2, 1, fp3);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSizeMin, 1, fp3);
	if (fseek(fp1, recordLength, SEEK_SET)) {
		fprintf(stderr, " Input File 1 Read Error.\n");
		free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		fclose(fp3);
		remove(file3);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte1 = ftell(fp1);
	if (flushWidth0 == 0) free(datPtr0);
	
/* Swap Header again */
	if (ipHeaderSwapFlag1) {
		swapIpHeader(&ipHeaderF1);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1F1) : swapIpHeader2VER2(&ipHeader2VER2F1);
	}
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if (fseek(fp2, recordLength, SEEK_SET)) {
		fprintf(stderr, " Input File 2 Read Error.\n");
		if (flushWidth0 > 0) free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		fclose(fp3);
		remove(file3);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte2 = ftell(fp2);
	for (i=0; i<ipHeaderF1.rnum; i++) {
		if ((datPtr1 = malloc(ipHeaderF1.rlng)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			if (i) {
				ipHeaderF1.zpxl = ipHeaderF1.rnum = i;
				if (ipHeaderF1.drtz >= ipHeaderF1.zpxl) ipHeaderF1.drtz = ipHeaderF1.zpxl - 1.0;
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				fseek(fp3, 0, SEEK_SET);
				fwrite(&ipHeaderF1, ipHeaderSize, 1, fp3);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ftell(fp3));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file3);
					}
				}
				fclose(fp3);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file3, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file3, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp3);
				remove(file3);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		if ((datPtr2 = malloc(ipHeaderF2.rlng)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtr1);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			if (i) {
				ipHeaderF1.zpxl = ipHeaderF1.rnum = i;
				if (ipHeaderF1.drtz >= ipHeaderF1.zpxl) ipHeaderF1.drtz = ipHeaderF1.zpxl - 1.0;
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				fseek(fp3, 0, SEEK_SET);
				fwrite(&ipHeaderF1, ipHeaderSize, 1, fp3);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ftell(fp3));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file3);
					}
				}
				fclose(fp3);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file3, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file3, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp3);
				remove(file3);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		fread(datPtr1, ipHeaderF1.rlng, 1, fp1);
		readByte1 += ipHeaderF1.rlng;
		if (ftell(fp1) != readByte1) {
			fprintf(stderr, " Input File 1 Read Error : Input File is Too Short.\n");
			free(datPtr1);
			free(datPtr2);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			if (i) {
				ipHeaderF1.zpxl = ipHeaderF1.rnum = i;
				if (ipHeaderF1.drtz >= ipHeaderF1.zpxl) ipHeaderF1.drtz = ipHeaderF1.zpxl - 1.0;
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				fseek(fp3, 0, SEEK_SET);
				fwrite(&ipHeaderF1, ipHeaderSize, 1, fp3);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ftell(fp3));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file3);
					}
				}
				fclose(fp3);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file3, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file3, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp3);
				remove(file3);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		fread(datPtr2, ipHeaderF2.rlng, 1, fp2);
		readByte2 += ipHeaderF2.rlng;
		if (ftell(fp2) != readByte2) {
			fprintf(stderr, " Input File 2 Read Error : Input File is Too Short.\n");
			free(datPtr1);
			free(datPtr2);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			if (i) {
				ipHeaderF1.zpxl = ipHeaderF1.rnum = i;
				if (ipHeaderF1.drtz >= ipHeaderF1.zpxl) ipHeaderF1.drtz = ipHeaderF1.zpxl - 1.0;
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				fseek(fp3, 0, SEEK_SET);
				fwrite(&ipHeaderF1, ipHeaderSize, 1, fp3);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag1) swapIpHeader(&ipHeaderF1);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ftell(fp3));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file3);
					}
				}
				fclose(fp3);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file3, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file3, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp3);
				remove(file3);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		for (j=0; j<ipHeaderF1.xpxl; j++) {
			if (datPtr2[j] != 0)
			    intensityDbl = ((*readDat1)(&datPtr1[j], &rato1, &shiftBit1, &ipDataSwapFlag1) - padInt) * mag
			                 / (*readDat2)(&datPtr2[j], &rato2, &shiftBit2, &ipDataSwapFlag2) + padInt;
			else {
				if (set0Flag)
				    intensityDbl = (datPtr1[j] - padInt > 0) ? (double)padInt : (double)maxInt1;
				else
				    intensityDbl = ((*readDat1)(&datPtr1[j], &rato1, &shiftBit1, &ipDataSwapFlag1) - padInt) * mag
				                 + padInt;
			}
			datPtr1[j] = (*writeDat)(&intensityDbl, &rato1, &maxInt1, &ipDataSwapFlag1);
		}
		fwrite(datPtr1, flushWidth, 1, fp3);
		if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp3);
		free(datPtr1);
		free(datPtr2);
	}
	
/* End */
	if (flushWidth0 > 0) free(datPtr0);
	fclose(fp1);
	fclose(fp2);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeaderF1.xpxl, ipHeaderF1.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeaderF1.rlng, ipHeaderF1.rnum, ftell(fp3));
	}
	fclose(fp3);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file3, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file3, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
#ifndef _HAS_STRANGE_RENAME
	if (replaceFlag) {
		if (rename(file3, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file3);
			fprintf(stderr, " Original R-AXIS Image File 1 might be lost... Check It !!\n");
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#else
	if (replaceFlag) {
		if ((fp1 = fopen(file1, "rb+")) == NULL) {
			fprintf(stderr, "Unable to replace Original R-AXIS Image File 1.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file3);
			replaceFlag = 0;
		}
		else fclose(fp1);
	}
	if (replaceFlag) {
		if (tmpNameEXT(file1, fnameTmp)) strcpy(fnameTmp, tmpnam(NULL));
		if (rename(file1, fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File 1.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file3);
		}
		else if (rename(file3, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File 1.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file3);
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (remove(fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File 1.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File 1 has Replaced.\n");
	}
#endif
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] [-t] -f1:file1 -f2:file2 [-m:VALUE]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] -f1:file1 -f2:file2 [-m:VALUE]\n", ProgName);
#endif
	fprintf(fp0, "          [-x] [-p:VALUE] {file3|-replace}\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Divide R-AXIS Image with Another Image File. That is:\n");
	fprintf(fp0, "          file3(x, z) = file1(x, z) / file2(x, z) * coefficient\n");
	fprintf(fp0, "  -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "  -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "  -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "  -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "  -f1:file1 ... Set Filename of Original R-AXIS Image File 1 to file1.\n");
	fprintf(fp0, "  -f2:file2 ... Set Filename of Original R-AXIS Image File 2 to file2.\n");
	fprintf(fp0, "  -m:VALUE .... Set Magnification after division to \"VALUE\".\n");
	fprintf(fp0, "                If absent, set to 1.0.\n");
	fprintf(fp0, "  -x .......... Divide with 0 when intensity in file2 is 0.\n");
	fprintf(fp0, "                If absent, divide with 1 instead.\n");
	fprintf(fp0, "  -p:VALUE .... Set VALUE to minus before division.\n");
	fprintf(fp0, "                Same ammount is added at last.\n");
#ifdef _HASUTIME
	fprintf(fp0, "  -t .......... Set Modification time of Output File as Original File 1.\n");
#endif
	fprintf(fp0, "  file3 ....... Output R-AXIS Image Filename. This must be the last argument.\n");
	fprintf(fp0, "                If last argument is \"-replace\", Original R-AXIS Image File 1 is\n");
	fprintf(fp0, "                replaced by Output File. Be careful using this option.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	double *arg1;
#else
char splitArg(const char *argV, char *arg0, double *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0); /* Not argument */
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (10); /* Without ':', and first string is too long */
		strcpy(arg0, argV);
		return (1); /* Without ':', and first string is good */
	}
	if (chPtr1-argV > 3) return (10); /* With ':', and first string is too long */
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (20); /* With ':', and 2nd part is not numeric. */
	*arg1 = atof(chPtr1+1);
	return (2);
}
