/* Program to treat IP data file */
#define ProgName "ipedit"
#define ProgInfo  "Edit R-AXIS Image File informations(Header Part)"
/*                                                         12/9/1999  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.27a_2/3/2006"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#ifdef _HASUTIME
#	include <sys/stat.h>
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef _HASUTIME
#	define MAXARG 61
#else
#	define MAXARG 60
#endif
#define MAXARGX MAXARG-2

struct args {
	char argIndex;
	char *argValue;
};

#ifdef _OLDHP
	char splitArg();
	char showMessage();
	char showUsage();
	char showHelp();
#else
	char splitArg(char *argV, struct args *argArray);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	long int ipHeaderSize, i, j;
	char file1[FNAMELEN], cpuIntelFlag, ipHeaderSwapFlag, confirmBuffer[8], tmpValue,
	     noConfirmFlag, setTimeFlag, errFlag, doneFlag[MAXARGX+10];
	struct ipHead ipHeader;
	struct args argArray[MAXARGX];
	FILE *fp1;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	noConfirmFlag = setTimeFlag = errFlag = 0;
	for (j=10; j<MAXARGX+10; j++) doneFlag[j] = 0;
	tmpValue = 2;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments */
	if (argc==1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (argc>MAXARG) {
		showMessage(stdout, cpuIntelFlag, 1);
		fprintf(stderr, " Too Many Arguments.\n");
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (argc==2) {
		showMessage(stdout, cpuIntelFlag, 1);
		fprintf(stderr, " Too Few Arguments.\n");
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	strcpy(file1, argv[argc-1]);
	argv++;
	argc -= 2;
	for (i=0; i<argc; i++) {
		if (!strcmp(argv[i], "-f")) {
			if (noConfirmFlag) tmpValue = 2;
			else {
				noConfirmFlag++;
				for (j=i; j<argc-1; j++) argv[j] = argv[j+1];
				argc--;
				i--;
				tmpValue = 0;
			}
		}
#ifdef _HASUTIME
		else if (!strcmp(argv[i], "-t")) {
			if (setTimeFlag) tmpValue = 2;
			else {
				setTimeFlag++;
				for (j=i; j<argc-1; j++) argv[j] = argv[j+1];
				argc--;
				i--;
				tmpValue = 0;
			}
		}
#endif
		else {
			tmpValue = splitArg(argv[i], &argArray[i]);
			doneFlag[(int)argArray[i].argIndex] ? tmpValue = 2 : doneFlag[(int)argArray[i].argIndex]++;
		}
		if (tmpValue==1) {
			showMessage(stdout, cpuIntelFlag, 1);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (tmpValue==2) {
			showMessage(stdout, cpuIntelFlag, 1);
			fprintf(stderr, " Arguments Duplicated.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
	}
	showMessage(stdout, cpuIntelFlag, 1);
	
/* File Read */
	if ((fp1 = fopen(file1, "rb+")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	if (ftell(fp1) != ipHeaderSize) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
	   if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		fprintf(stdout, "This File is probably created with OLD MD-DOS(R) Machine.\n");
		do {
			fprintf(stdout, "  Is it true ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'N') {
			fprintf(stderr, " Unable to continue. Stop.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
	
/* Clean Header */
	cleanIpHeader(&ipHeader);
	
/* Edit Header */
	for (i=0; i<argc; i++) {
		switch (argArray[i].argIndex) {
			case 10:
				strncpy(ipHeader.name, argArray[i].argValue, 10);
				break;
			case 11:
				strncpy(ipHeader.vers, argArray[i].argValue, 10);
				break;
			case 12:
				strncpy(ipHeader.crnm, argArray[i].argValue, 20);
				break;
			case 13:
				strncpy(ipHeader.crsy, argArray[i].argValue, 12);
				break;
			case 14:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.alng = atof(argArray[i].argValue);
				break;
			case 15:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.blng = atof(argArray[i].argValue);
				break;
			case 16:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.clng = atof(argArray[i].argValue);
				break;
			case 17:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.alfa = atof(argArray[i].argValue);
				break;
			case 18:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.beta = atof(argArray[i].argValue);
				break;
			case 19:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.gamm = atof(argArray[i].argValue);
				break;
			case 20:
				strncpy(ipHeader.spgr, argArray[i].argValue, 12);
				break;
			case 21:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.mosc = atof(argArray[i].argValue);
				break;
			case 22:
				strncpy(ipHeader.memo, argArray[i].argValue, 80);
				break;
			case 23:
				strncpy(ipHeader.date, argArray[i].argValue, 12);
				break;
			case 24:
				strncpy(ipHeader.opnm, argArray[i].argValue, 20);
				break;
			case 25:
				strncpy(ipHeader.trgt, argArray[i].argValue, 4);
				break;
			case 26:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.wlng = atof(argArray[i].argValue);
				break;
			case 27:
				strncpy(ipHeader.mcro, argArray[i].argValue, 20);
				break;
			case 28:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.m2ta = atof(argArray[i].argValue);
				break;
			case 29:
				strncpy(ipHeader.colm, argArray[i].argValue, 20);
				break;
			case 30:
				strncpy(ipHeader.filt, argArray[i].argValue, 4);
				break;
			case 31:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.camr = atof(argArray[i].argValue);
				break;
			case 32:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.vltg = atof(argArray[i].argValue);
				break;
			case 33:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.crnt = atof(argArray[i].argValue);
				break;
			case 34:
				strncpy(ipHeader.focs, argArray[i].argValue, 12);
				break;
			case 35:
				strncpy(ipHeader.optc, argArray[i].argValue, 80);
				break;
			case 36:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.cyld = atol(argArray[i].argValue);
				break;
			case 37:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.weis = atof(argArray[i].argValue);
				break;
			case 38:
				strncpy(ipHeader.mnax, argArray[i].argValue, 4);
				break;
			case 39:
				strncpy(ipHeader.bmax, argArray[i].argValue, 4);
				break;
			case 40:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.phi0 = atof(argArray[i].argValue);
				break;
			case 41:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.phis = atof(argArray[i].argValue);
				break;
			case 42:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.phie = atof(argArray[i].argValue);
				break;
			case 43:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.oscn = atol(argArray[i].argValue);
				break;
			case 44:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.fext = atof(argArray[i].argValue);
				break;
			case 45:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.drtx = atof(argArray[i].argValue);
				break;
			case 46:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.drtz = atof(argArray[i].argValue);
				break;
			case 47:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.omga = atof(argArray[i].argValue);
				break;
			case 48:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.fkai = atof(argArray[i].argValue);
				break;
			case 49:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.thta = atof(argArray[i].argValue);
				break;
			case 50:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.mu   = atof(argArray[i].argValue);
				break;
			case 51:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.xpxl = atol(argArray[i].argValue);
				break;
			case 52:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.zpxl = atol(argArray[i].argValue);
				break;
			case 53:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.xsiz = atof(argArray[i].argValue);
				break;
			case 54:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.zsiz = atof(argArray[i].argValue);
				break;
			case 55:
				if (!noConfirmFlag) {
					do {
						fprintf(stdout, " This might make the file useless. OK ? [Y/N] : ");
						if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
							fprintf(stderr, "\nError in getting input. Exit.\n");
							fclose(fp1);
							exit(EXIT_FAILURE);
						}
					} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
					if (toupper(confirmBuffer[0]) == 'N') {
						fclose(fp1);
						fprintf(stdout, " Header Edit is in vain. Nothing has been changed.\n");
						exit(EXIT_FAILURE);
					}
				}
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.rlng = atol(argArray[i].argValue);
				break;
			case 56:
				if (!noConfirmFlag) {
					do {
						fprintf(stdout, " This might make the file useless. OK ? [Y/N] : ");
						if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
							fprintf(stderr, "\nError in getting input. Exit.\n");
							fclose(fp1);
							exit(EXIT_FAILURE);
						}
					} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
					if (toupper(confirmBuffer[0]) == 'N') {
						fclose(fp1);
						fprintf(stdout, " Header Edit is in vain. Nothing has been changed.\n");
						exit(EXIT_FAILURE);
					}
				}
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.rnum = atol(argArray[i].argValue);
				break;
			case 57:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.ipst = atol(argArray[i].argValue);
				break;
			case 58:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.ipnm = atol(argArray[i].argValue);
				break;
			case 59:
				if (!noConfirmFlag) {
					do {
						fprintf(stdout, " This might make the file useless. OK ? [Y/N] : ");
						if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
							fprintf(stderr, "\nError in getting input. Exit.\n");
							fclose(fp1);
							exit(EXIT_FAILURE);
						}
					} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
					if (toupper(confirmBuffer[0]) == 'N') {
						fclose(fp1);
						fprintf(stdout, " Header Edit is in vain. Nothing has been changed.\n");
						exit(EXIT_FAILURE);
					}
				}
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.rato = atof(argArray[i].argValue);
				break;
			case 60:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.ft_1 = atof(argArray[i].argValue);
				break;
			case 61:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.ft_2 = atof(argArray[i].argValue);
				break;
			case 62:
				strncpy(ipHeader.host, argArray[i].argValue, 10);
				break;
			case 63:
				strncpy(ipHeader.ip, argArray[i].argValue, 10);
				break;
			case 64:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.dr_x = atol(argArray[i].argValue);
				break;
			case 65:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.dr_z = atol(argArray[i].argValue);
				break;
			case 66:
				if (!isdigitEXT(argArray[i].argValue)) errFlag++;
				else ipHeader.drxz = atol(argArray[i].argValue);
				break;
			default :
				errFlag++;
		}
		if (errFlag) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Check ipHeader */
	if ((ipHeader.zpxl <= 0) || (ipHeader.xpxl <= 0) || (ipHeader.rnum <= 0)
	 || (ipHeader.rlng <= 0) || (ipHeader.xsiz <= 0.0) || (ipHeader.zsiz <= 0.0)
	 || (ipHeader.rato < 1.0) || (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng)) {
		if (noConfirmFlag) fprintf(stdout, " Modified Header has inconsistence.\n");
		else {
			do {
				fprintf(stdout, " Modified Header has inconsistence. Really continue ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fclose(fp1);
				fprintf(stdout, " Header Edit is in vain. Nothing has been changed.\n");
				exit(EXIT_FAILURE);
			}
		}
	}
	if ((ipHeader.zpxl != ipHeader.rnum) || (ipHeader.drtx > ipHeader.xpxl)
	 || (ipHeader.drtx < 0.0) || (ipHeader.drtz > ipHeader.zpxl) || (ipHeader.drtz < 0.0))
	    fprintf(stdout, " Warning : Header has some slight curiousity. This might exist originally.\n");
	
/* Restore Endian on ipHeader */
	if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
	
/* Write Changes and End */
	fseek(fp1, 0, SEEK_SET);
	fwrite(&ipHeader, ipHeaderSize, 1, fp1);
	fclose(fp1);
	fprintf(stdout, " Header Edit is done successfully.\n");
#ifdef _HASUTIME
	if (setTimeFlag) utime(file1, &utime1) ? fprintf(stderr, " Unable to set access time.\n") : fprintf(stdout, " Modification time remains unchanged.\n");
#endif
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag>=2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag>=2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag>=2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s -h\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-f] [-t] -n:value [...] file\n", ProgName);
#else
	fprintf(fp0, "        %s [-f] -n:value [...] file\n", ProgName);
#endif
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Edits R-AXIS file informations stored in the file,\n");
	fprintf(fp0, "  only before %d bytes.\n", SIZE_OF_IPHEADER_EACH);
	fprintf(fp0, "   -h ..... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -f ..... Never check results. Any change may be allowed.\n");
	fprintf(fp0, "            If absent, only serious error is reported.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t ..... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file ... Filename of R-AXIS Image File. This must be the last argument.\n");
	fprintf(fp0, "  \"n\" determines the Header Member to change,\n");
	fprintf(fp0, "  and the value separated by \":\" decides the value to change to.\n");
	fprintf(fp0, "  Be careful not to enter irregal value, because the value is not examined.\n");
	fprintf(fp0, "  Value for n is shown below, with the order of members in Header File\n");
	fprintf(fp0, "  except for \"reserved\". See R-AXIS Manual.\n");
	fprintf(fp0, " 10:Device Name,    11:Version,        12:Crystal Name,   13:Crystal System,\n");
	fprintf(fp0, " 14:a-axis,         15:b-axis,         16:c-axis,         17:alpha,\n");
	fprintf(fp0, " 18:beta,           19:gamma,          20:Space Group,    21:Mosaic Spread,\n");
	fprintf(fp0, " 22:Memo,           23:Date,           24:Operator,       25:Target,\n");
	fprintf(fp0, " 26:Wave Length,    27:Monochrom.Type, 28:Monochrom.Angle, 29:Collimator,\n");
	fprintf(fp0, " 30:Kb filter,      31:CameraLength[mm], 32:Tube Voltage, 33:Tube Current,\n");
	fprintf(fp0, " 34:Focus,          35:Optics,         36:Camera type,    37:Weisenberg Osci.,\n");
	fprintf(fp0, " 38:Mount Axis,     39:Beam Axis,      40:phi0,           41:phi start,\n");
	fprintf(fp0, " 42:phi end,        43:Osci. Freq.,    44:Exposure[min],  45:Direct X,\n");
	fprintf(fp0, " 46:Direct Y,       47:omega ,         48:xai,            49:2-theta,\n");
	fprintf(fp0, " 50:mu,             51:Num.of pxls. X, 52:Num.of pxls. Y, 53:Pxl size X[mm],\n");
	fprintf(fp0, " 54:Pxl size Y[mm], 55:Record Length,  56:Record Number,  57:Start Line,\n");
	fprintf(fp0, " 58:Number of IP,   59:High/Low Ratio, 60:Fading Time 1,  61:Fading Time 2,\n");
	fprintf(fp0, " 62:Host Computer,  63:IP Type,        64:Horiz.Direction, 65:Ver.Direction,\n");
	fprintf(fp0, " 66:Data direction face/back\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, argArray)
	char *argV;
	struct args *argArray;
#else
char splitArg(char *argV, struct args *argArray)
#endif
{
	if (strlen(argV) < 4) return (1);
	if (argV[0] != '-') return (1);
	if (argV[3] != ':') return (1);
	if ((atol(argV) <= -MAXARGX-10) || (atol(argV) > -10)) return (1);
	argArray->argIndex = -atol(argV);
	argArray->argValue = (argV + 4);
	return (0);
}
