/* Program to convert IP data file */
#define ProgName "ipfft"
#define ProgInfo "Fourier Transformation Filter for R-AXIS"
/*                                                         3/20/2002  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */
/* Notice : This program is simple "Image Filter" or "Image Converter"
            so no consideration for "diffraction" image. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v0.09_1/15/2007"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#ifdef _HASUTIME
#	include <sys/stat.h>
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"
#include "libfft.h"

#ifdef _HASUTIME
#	define MAXARG 7
#else
#	define MAXARG 6
#endif
#define MINARG 2
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define OUT_REAL 1
#define OUT_IMAGE 2
#define OUT_AMP 3
#define OUT_PHASE 4
#define OUT_DEFAULT 0

#ifdef _OLDHP
	long int (*readDat)();
	short int (*writeDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
#ifdef USE_DOUBLE
	short int (*writeDat)(const double *x, const double *rato, const unsigned long int *maxInt, const char *ipDataSwapFlag);
#else
	short int (*writeDat)(const float *x, const double *rato, const unsigned long int *maxInt, const char *ipDataSwapFlag);
#endif
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr;
	long int i, j, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2, recordLength,
	     recordLengthCreate, ipHeaderSizeMin, readByte,
	     lineLen, ll0_h, ll1_h, hl0_h, hl1_h, ll0_v, ll1_v, hl0_v, hl1_v, cur,
	     flushWidth, flushWidth0, off_x, off_z, off_x2, off_z2, geta, geta2, geta3;
	unsigned long int maxInt;
	char file1[FNAMELEN], file2[FNAMELEN], inputBuffer[20], argNum,
	     confirmBuffer[8], cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag,
	     shiftBit, strangeShiftBitFlag, byte, createDosFlag, scaleFlag,
	     arg0[4], verboseFlag, highDoneFlag, lowDoneFlag, replaceFlag, setTimeFlag,
	     outModeFlag, outModeDoneFlag, absFlag, padFlag, maxFlag;
#ifdef _HAS_STRANGE_RENAME
	char fnameTmp[FNAMELEN];
#endif
	double rato, arg1, lowLim, highLim, mag, pad;
	fft_float *datPtr_r, *datPtr_i;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	struct fft_dim fft_dim_h, fft_dim_v;
	FILE *fp1, *fp2;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	ipDataSwapFlag = highDoneFlag = lowDoneFlag = replaceFlag = setTimeFlag
	= outModeDoneFlag = scaleFlag = absFlag = padFlag = maxFlag = 0;
	verboseFlag = 1;
	readByte = 0;
	lowLim = 0.0;
	highLim = 1.0;
	flushWidth0 = 0;
	outModeFlag = OUT_DEFAULT;
	mag = 1.0;
	pad = 0.0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
		if (!strcmp(argv[argc-1], "-replace")) replaceFlag = 1;
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
		createDosFlag = 1;
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
		createDosFlag = 0;
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) {
		ipHeader.zpxl = ipHeader.rnum;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height has modified.\n");
	}
	recordLength = ipHeader.rlng;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) {
		ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width has modified.\n");
	}
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (ipHeader.xpxl == 1 || ipHeader.zpxl == 1) {
		fprintf(stderr, " Error : File is too small.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (ipHeader.xsiz <= 0.0) {
		ipHeader.xsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in X has modified to 0.05 [mm].\n");
	}
	if (ipHeader.zsiz <= 0.0) {
		ipHeader.zsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in Z has modified to 0.05 [mm].\n");
	}
	rato = ipHeader.rato;
	if (ipHeader.cyld >= 256) {
		if (verboseFlag) fprintf(stdout, " Camera Type Code has changed from %ld to 0.\n", ipHeader.cyld);
		ipHeader.cyld = 0;
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
	}
	
/* Clean Header */
	cleanIpHeader(&ipHeader);
	createDosFlag ? cleanIpHeader2VER1(&ipHeader2VER1) : cleanIpHeader2VER2(&ipHeader2VER2);
	
/* Logging */
	if (logWrite(&ipHeader, ProgName)) {
		if (verboseFlag) fprintf(stdout, " ipHeader.memo is full, and No Logging.\n");
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
#ifdef USE_DOUBLE
		writeDat = ipDtoShortBit0;
#else
		writeDat = ipFtoShortBit0;
#endif
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
#ifdef USE_DOUBLE
				writeDat = ipDtoShort;
#else
				writeDat = ipFtoShort;
#endif
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
#ifdef USE_DOUBLE
			writeDat = ipDtoShort;
#else
			writeDat = ipFtoShort;
#endif
		}
	}
	
/* Get Magnification */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Enter FFT Type :\n");
			fprintf(stdout, "  (1) Show RealPart , (2) Show ImagenaryPart , (3) Show Amplitude ,\n");
			fprintf(stdout, "  (4) Show Phase    , (0) FFT-filter-FFT    : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			outModeFlag = atol(inputBuffer);
			if (outModeFlag < 0 || outModeFlag > 4) continue;
			outModeDoneFlag = 1;
		} while (!outModeDoneFlag);
		if (outModeFlag == 0) {
			do {
				fprintf(stdout, " Enter Lower Limit Frequency to pass (scaled by the half width) : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				lowLim = atof(inputBuffer);
				if (lowLim < 0.0 || lowLim > 1.0) continue;
				lowDoneFlag = 1;
			} while (!lowDoneFlag);
			do {
				fprintf(stdout, " Enter Higher Limit Frequency to pass (scaled by the half width) : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				highLim = atof(inputBuffer);
				if (highLim <= lowLim || highLim > 1.0) continue;
				highDoneFlag = 1;
			} while (!highDoneFlag);
		}
		else if (outModeFlag != OUT_PHASE) {
			do {
				fprintf(stdout, " Multiply some value to Intensity ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'Y') {
				do {
					fprintf(stdout, "  Enter magnification value : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					if (!isdigitEXT(inputBuffer)) continue;
					mag = atof(inputBuffer);
					if (mag <= 0.0) continue;
					scaleFlag = 1;
				} while (!scaleFlag);
			}
			if (outModeFlag != OUT_AMP) {
				do {
					fprintf(stdout, " Calculated value may be less than 0. To avoid this,\n");
					fprintf(stdout, "  Pad some value[P], Use Absolute[A], or Set to 0[N] ? [P/A/N] : ");
					if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
				} while ((toupper(confirmBuffer[0]) != 'P') && (toupper(confirmBuffer[0]) != 'A') && (toupper(confirmBuffer[0]) != 'N'));
				if (toupper(confirmBuffer[0]) == 'P') {
					do {
						fprintf(stdout, "   Enter padding value : ");
						if (fscanf(stdin, "%s", inputBuffer) == EOF) {
							fprintf(stderr, "\nError in getting input. Exit.\n");
							fclose(fp1);
							exit(EXIT_FAILURE);
						}
						if (!isdigitEXT(inputBuffer)) continue;
						pad = atof(inputBuffer);
						padFlag = 1;
					} while (!padFlag);
				}
				else if (toupper(confirmBuffer[0]) == 'A') absFlag = 1;
			}
		}
		do {
			fprintf(stdout, "Output Full Area of the DFT Image ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') maxFlag = 1;
		do {
			fprintf(stdout, "Replace Original R-AXIS File with Output File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') replaceFlag = 1;
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-rl")) {
					outModeFlag = OUT_REAL;
					outModeDoneFlag++;
				}
				else if (!strcmp(arg0, "-im")) {
					outModeFlag = OUT_IMAGE;
					outModeDoneFlag++;
				}
				else if (!strcmp(arg0, "-am")) {
					outModeFlag = OUT_AMP;
					outModeDoneFlag++;
				}
				else if (!strcmp(arg0, "-ph")) {
					outModeFlag = OUT_PHASE;
					outModeDoneFlag++;
				}
				else if (!strcmp(arg0, "-ab")) absFlag++;
				else if (!strcmp(arg0, "-mx")) maxFlag++;
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-u")) {
					highLim = arg1;
					if (highLim < 0.0 || highLim > 1.0) break;
					highDoneFlag++;
				}
				else if (!strcmp(arg0, "-d")) {
					lowLim = arg1;
					if (lowLim < 0.0 || lowLim > 1.0) break;
					lowDoneFlag++;
				}
				else if (!strcmp(arg0, "-s")) {
					mag = arg1;
					if (mag <= 0.0) break;
					scaleFlag++;
				}
				else if (!strcmp(arg0, "-p")) {
					pad = arg1;
					padFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((lowDoneFlag > 1) || (highDoneFlag > 1) || (setTimeFlag > 1)
		 || (outModeDoneFlag > 1) || (scaleFlag > 1) || (absFlag > 1)
		 || (padFlag > 1) || (maxFlag > 1)
		 || (padFlag && absFlag)
		 || (outModeDoneFlag && (lowDoneFlag || highDoneFlag))) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!outModeDoneFlag) {
			if (lowLim == highLim) {
				fprintf(stderr, " lower limit exceeds higher limit.\n");
				showUsage(stderr);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (lowLim > highLim) {
				arg1 = lowLim;
				lowLim = highLim;
				highLim = arg1;
			}
		}
		if (scaleFlag && (outModeFlag==OUT_DEFAULT || outModeFlag==OUT_PHASE)) {
			fprintf(stdout, "Warning: Multiply option is ignored.\n");
			scaleFlag = 0;
			mag = 1.0;
		}
		if (padFlag && (outModeFlag==OUT_DEFAULT || outModeFlag==OUT_PHASE || outModeFlag==OUT_AMP)) {
			fprintf(stdout, "Warning: Padding option is ignored.\n");
			padFlag = 0;
			pad = 0.0;
		}
		if (absFlag && (outModeFlag==OUT_DEFAULT || outModeFlag==OUT_PHASE || outModeFlag==OUT_AMP)) {
			fprintf(stdout, "Warning: Absolute option is ignored.\n");
			absFlag = 0;
		}
	}
	if (verboseFlag >= 2) {
		switch (outModeFlag) {
			case OUT_DEFAULT:
				fprintf(stdout, " Fourier Bandpass Filter, between %f and %f.\n", lowLim, highLim);
				break;
			case OUT_REAL:
				fprintf(stdout, " Fourier Transformation. Image is Real Part.\n");
				break;
			case OUT_IMAGE:
				fprintf(stdout, " Fourier Transformation. Image is Imagenary Part.\n");
				break;
			case OUT_AMP:
				fprintf(stdout, " Fourier Transformation. Image is Amplitude Image.\n");
				break;
			case OUT_PHASE:
				fprintf(stdout, " Fourier Transformation. Image is Phase Image.\n");
				break;
			default:
				/* NOT REACHED */
				break;
		}
		if (scaleFlag) fprintf(stdout, " Output Image is magnified by %9.2f.\n", mag);
		if (padFlag) fprintf(stdout, " Output Image is padded by %9.1f.\n", pad);
		if (absFlag) fprintf(stdout, " Output Image is absolute value.\n");
		if (maxFlag) fprintf(stdout, " Output Image Size contains Surrounging ghost region.\n");
		if (replaceFlag) fprintf(stdout, " Original R-AXIS Image File will be replaced !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Set Parameters to Create */
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate =  MAXEXT(ipHeaderSizeMin, flushWidth);
	if (createDosFlag) recordLengthCreate = ((long)(ceil(recordLengthCreate/512.0)))*512;
	if (ipHeader.drtx >= ipHeader.xpxl) ipHeader.drtx = ipHeader.xpxl - 1.0;
	else if (ipHeader.drtx < 0.0) ipHeader.drtx = 0.0;
	if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
	else if (ipHeader.drtz < 0.0) ipHeader.drtz = 0.0;
	ipHeader.rlng = recordLengthCreate;
	flushWidth0 = recordLengthCreate - flushWidth;
	for (i=1, j=1; i<32; i++) {
		if (ipHeader.xpxl <= (j <<= 1)) break;
	}
	fft_dim_h.num = i;
	fft_dim_h.len = j;
	fft_dim_h.rev_len_sq = 1.0 / sqrt((double)fft_dim_h.len);
	fft_dim_h.dir = (FFT_DIR_IMG);
	lineLen = fft_dim_h.len * sizeof(fft_float);
	for (i=1, j=1; i<32; i++) {
		if (ipHeader.zpxl <= (j <<= 1)) break;
	}
	fft_dim_v.num = i;
	fft_dim_v.len = j;
	fft_dim_v.rev_len_sq = 1.0 / sqrt((double)fft_dim_v.len);
	fft_dim_v.dir = (FFT_DIR_IMG);
	off_z = (fft_dim_v.len - ipHeader.zpxl)/2; /* low edge: if zpxl is odd, centre is slightly lower */
	off_z2 = ipHeader.zpxl*2 - fft_dim_v.len + off_z;
	off_x = (fft_dim_h.len - ipHeader.xpxl)/2;
	off_x2 = ipHeader.xpxl*2 - fft_dim_h.len + off_x;
	ll0_h = dToLong(fft_dim_h.len / 2.0 * (1.0 - lowLim));
	ll1_h = dToLong(fft_dim_h.len / 2.0 * (1.0 + lowLim));
	ll0_v = dToLong(fft_dim_v.len / 2.0 * (1.0 - lowLim));
	ll1_v = dToLong(fft_dim_v.len / 2.0 * (1.0 + lowLim));
	hl0_h = dToLong(fft_dim_h.len / 2.0 * (1.0 - highLim));
	hl1_h = dToLong(fft_dim_h.len / 2.0 * (1.0 + highLim));
	hl0_v = dToLong(fft_dim_v.len / 2.0 * (1.0 - highLim));
	hl1_v = dToLong(fft_dim_v.len / 2.0 * (1.0 + highLim));
	if (maxFlag && (fft_dim_h.len > MAXWIDTH || fft_dim_v.len > MAXHEIGHT)) {
		fprintf(stderr, "Oops, Output image is too large. Output with Original Image Size.\n");
		maxFlag = 0;
	}
	
/* Create Output File */
	if (replaceFlag) {
		if (mkstempEXT(file1, file2, &fp2, 0)) {
			fprintf(stderr, " Temporary File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		if (verboseFlag == 3) {
			fprintf(stdout, "Enter  Output  Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		while (createFileEXT(file2, &fp2, 0)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
	}
	if (fseek(fp1, recordLength, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if ((datPtr_r = calloc(fft_dim_h.len*fft_dim_v.len, sizeof(fft_float))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	if ((datPtr_i = calloc(fft_dim_h.len*fft_dim_v.len, sizeof(fft_float))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		free(datPtr_r);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	/* read data and generate image */
	geta  = off_z*fft_dim_h.len+off_x;
	geta2 = geta - 1;
	geta3 = geta2 + ipHeader.xpxl*2;
	for (i=0; i<ipHeader.rnum; i++) {
		if ((datPtr = malloc(recordLength)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtr_r);
			free(datPtr_i);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		fread(datPtr, recordLength, 1, fp1);
		readByte += recordLength;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
			free(datPtr);
			free(datPtr_r);
			free(datPtr_i);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		for (j=0; j<off_x; j++) {
			datPtr_r[geta+j] = datPtr_r[geta2-j] = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
		}
		for (j=off_x; j<off_x2; j++) {
			datPtr_r[geta+j] = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
		}
		for (j=off_x2; j<ipHeader.xpxl; j++) {
			datPtr_r[geta+j] = datPtr_r[geta3-j] = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
		}
		free(datPtr);
		geta += fft_dim_h.len;
		geta2 += fft_dim_h.len;
		geta3 += fft_dim_h.len;
	}
	fclose(fp1);
	geta  = off_z * fft_dim_h.len;
	geta2 = geta - fft_dim_h.len;
	for (i=0; i<off_z; i++) {
		memcpy(datPtr_r+geta2, datPtr_r+geta, lineLen);
		geta  += fft_dim_h.len;
		geta2 -= fft_dim_h.len;
	}
	geta  = (off_z2+off_z) * fft_dim_h.len;
	geta2 = (fft_dim_v.len-1)*fft_dim_h.len;
	for (i=off_z2; i<ipHeader.rnum; i++) {
		memcpy(datPtr_r+geta2, datPtr_r+geta, lineLen);
		geta  += fft_dim_h.len;
		geta2 -= fft_dim_h.len;
	}
	/* prepare to output */
	if (maxFlag) {
		ipHeader.xpxl = fft_dim_h.len;
		ipHeader.zpxl = fft_dim_v.len;
		flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
		recordLengthCreate =  MAXEXT(ipHeaderSizeMin, flushWidth);
		ipHeader.rlng = recordLengthCreate;
		flushWidth0 = recordLengthCreate - flushWidth;
		ipHeader.rnum = ipHeader.zpxl;
		ipHeader.drtx += off_x;
		ipHeader.drtz += off_z;
		off_x = 0;
		off_z = 0;
	}
	/* fft */
	if (fft2d(datPtr_r, datPtr_i, &fft_dim_h, &fft_dim_v)) {
		fprintf(stderr, " Failed to allocate memory.\n");
		free(datPtr_r);
		free(datPtr_i);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	/* Restore endian on ipHeader */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
	/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	createDosFlag ? fwrite(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp2) : fwrite(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSizeMin, 1, fp2);
	if (flushWidth0 == 0) free(datPtr0);
	/* Swap Header again */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	switch (outModeFlag) {
		case OUT_REAL:
			if (absFlag) {
				for (i=0, geta=off_z*fft_dim_h.len+off_x; i<ipHeader.rnum; i++, geta+=fft_dim_h.len) {
					for (j=0, cur=geta; j<ipHeader.xpxl; j++, cur++) datPtr_r[cur] = fabs(datPtr_r[cur] * mag);
				}
			}
			else {
				for (i=0, geta=off_z*fft_dim_h.len+off_x; i<ipHeader.rnum; i++, geta+=fft_dim_h.len) {
					for (j=0, cur=geta; j<ipHeader.xpxl; j++, cur++) datPtr_r[cur] = datPtr_r[cur] * mag + pad;
				}
			}
			break;
		case OUT_IMAGE:
			if (absFlag) {
				for (i=0, geta=off_z*fft_dim_h.len+off_x; i<ipHeader.rnum; i++, geta+=fft_dim_h.len) {
					for (j=0, cur=geta; j<ipHeader.xpxl; j++, cur++) datPtr_r[cur] = fabs(datPtr_i[cur] * mag);
				}
			}
			else {
				for (i=0, geta=off_z*fft_dim_h.len+off_x; i<ipHeader.rnum; i++, geta+=fft_dim_h.len) {
					for (j=0, cur=geta; j<ipHeader.xpxl; j++, cur++) datPtr_r[cur] = datPtr_i[cur] * mag + pad;
				}
			}
			break;
		case OUT_AMP:
			for (i=0, geta=off_z*fft_dim_h.len+off_x; i<ipHeader.rnum; i++, geta+=fft_dim_h.len) {
				for (j=0, cur=geta; j<ipHeader.xpxl; j++, cur++)
				    datPtr_r[cur] = dist(datPtr_r[cur], datPtr_i[cur]) * mag;
			}
			break;
		case OUT_PHASE:
			for (i=0, geta=off_z*fft_dim_h.len+off_x; i<ipHeader.rnum; i++, geta+=fft_dim_h.len) {
				for (j=0, cur=geta; j<ipHeader.xpxl; j++, cur++) {
					if (datPtr_i[cur] == 0.0 && datPtr_r[cur] == 0.0) datPtr_r[cur] = 0.0;
					else if ((datPtr_r[cur] = atan2(datPtr_i[cur], datPtr_r[cur])) >= 0.0)
					    datPtr_r[cur] *= RAD_DEG;
					else datPtr_r[cur] = datPtr_r[cur] * RAD_DEG + 360.0;
				}
			}
			break;
		case OUT_DEFAULT:
			/* band-pass filtering */
			for (i=0, geta=0; i<hl0_v; i++, geta+=fft_dim_h.len) {
				for (j=0, cur=geta; j<fft_dim_h.len; j++, cur++) datPtr_r[cur] = datPtr_i[cur] = 0.0;
			}
			for (i=hl0_v; i<ll0_v; i++, geta+=fft_dim_h.len) {
				for (j=0, cur=geta; j<hl0_h; j++, cur++) datPtr_r[cur] = datPtr_i[cur] = 0.0;
				for (j=hl1_h, cur=geta+hl1_h; j<fft_dim_h.len; j++, cur++) datPtr_r[cur] = datPtr_i[cur] = 0.0;
			}
			for (i=ll0_v; i<ll1_v; i++, geta+=fft_dim_h.len) {
				for (j=0, cur=geta; j<hl0_h; j++, cur++) datPtr_r[cur] = datPtr_i[cur] = 0.0;
				for (j=ll0_h, cur=geta+ll0_h; j<ll1_h; j++, cur++) datPtr_r[cur] = datPtr_i[cur] = 0.0;
				for (j=hl1_h, cur=geta+hl1_h; j<fft_dim_h.len; j++, cur++) datPtr_r[cur] = datPtr_i[cur] = 0.0;
			}
			for (i=ll1_v; i<hl1_v; i++, geta+=fft_dim_h.len) {
				for (j=0, cur=geta; j<hl0_h; j++, cur++) datPtr_r[cur] = datPtr_i[cur] = 0.0;
				for (j=hl1_h, cur=geta+hl1_h; j<fft_dim_h.len; j++, cur++) datPtr_r[cur] = datPtr_i[cur] = 0.0;
			}
			for (i=hl1_v; i<fft_dim_v.len; i++, geta+=fft_dim_h.len) {
				for (j=0, cur=geta; j<fft_dim_h.len; j++, cur++) datPtr_r[cur] = datPtr_i[cur] = 0.0;
			}
			/* restore via fft, etc. */
			fft_dim_h.dir = (FFT_DIR_IMG | FFT_DIR_IDFT);
			fft_dim_v.dir = (FFT_DIR_IMG | FFT_DIR_IDFT);
			if (fft2d(datPtr_r, datPtr_i, &fft_dim_h, &fft_dim_v)) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtr_r);
				free(datPtr_i);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			break;
		default:
			/* NOT REACHED */
			break;
	}
	free(datPtr_i);
	for (i=0, geta=off_z*fft_dim_h.len+off_x; i<ipHeader.rnum; i++, geta+=fft_dim_h.len) {
		if ((datPtr = malloc(recordLengthCreate)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtr_r);
			if (flushWidth0 > 0) free(datPtr0);
			if (i) {
				ipHeader.zpxl = ipHeader.rnum = i;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
					}
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		for (j=0; j<ipHeader.xpxl; j++)
		    datPtr[j] = (*writeDat)(&(datPtr_r[geta+j]), &rato, &maxInt, &ipDataSwapFlag);
		fwrite(datPtr, flushWidth, 1, fp2);
		if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
		free(datPtr);
	}
	
/* End */
	free(datPtr_r);
	if (flushWidth0 > 0) free(datPtr0);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
#ifndef _HAS_STRANGE_RENAME
	if (replaceFlag) {
		if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			fprintf(stderr, " Original R-AXIS Image File might be lost... Check It !!\n");
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#else
	if (replaceFlag) {
		if ((fp1 = fopen(file1, "rb+")) == NULL) {
			fprintf(stderr, "Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			replaceFlag = 0;
		}
		else fclose(fp1);
	}
	if (replaceFlag) {
		if (tmpNameEXT(file1, fnameTmp)) strcpy(fnameTmp, tmpnam(NULL));
		if (rename(file1, fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
		}
		else if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (remove(fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#endif
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-t] [-d:VALUE] [-u:VALUE] {file2|-replace}\n", ProgName);
	fprintf(fp0, "        %s [-q|-v] file1 [-t] [-rl|-im|-am|-ph] [-s:VALUE] [-p:VALUE|-ab]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-d:VALUE] [-u:VALUE] {file2|-replace}\n", ProgName);
	fprintf(fp0, "        %s [-q|-v] file1 [-rl|-im|-am|-ph] [-s:VALUE] [-p:VALUE|-ab]\n", ProgName);
#endif
	fprintf(fp0, "          [-mx] {file2|-replace}\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program execute Fourier Filtering for R-AXIS Image File.\n");
	fprintf(fp0, "  Notice : This program is simple \"Image Filter\" or \"Image Converter\",\n");
	fprintf(fp0, "           so no consideration for \"diffraction\" image.\n");
	fprintf(fp0, "   -h ......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i ......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q ......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v ......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -d:VALUE ... Set lower limit of frequency to pass to \"VALUE\".\n");
	fprintf(fp0, "   -u:VALUE ... Set higher limit of frequency to pass to \"VALUE\".\n");
	fprintf(fp0, "                These values determine the frequency to pass.\n");
	fprintf(fp0, "                Value must be between 0 and 1, relative to the Image Size.\n");
	fprintf(fp0, "                Image is processed via DFT, filtered, IDFT.\n");
	fprintf(fp0, "   -rl ........ Stop after DFT, and show \"real part\" image.\n");
	fprintf(fp0, "   -im ........ Stop after DFT, and show \"imagenary part\" image.\n");
	fprintf(fp0, "   -am ........ Stop after DFT, and show \"amplitude\" image.\n");
	fprintf(fp0, "   -ph ........ Stop after DFT, and show \"phase\" image.\n");
	fprintf(fp0, "                Output is between 0 and 360.\n");
	fprintf(fp0, "   -s:VALUE ... Multiply \"VALUE\" just before save image.\n");
	fprintf(fp0, "                Valid only with \"-rl\", \"-im\", \"-am\".\n");
	fprintf(fp0, "   -p:VALUE ... Pad \"VALUE\" before save image and after multiplied.\n");
	fprintf(fp0, "                Valid only with \"-rl\", \"-im\".\n");
	fprintf(fp0, "   -ab ........ Use Absolute Value. Exclusive with \"-p\" option.\n");
	fprintf(fp0, "                Valid only with \"-rl\", \"-im\".\n");
	fprintf(fp0, "   -mx ........ Output Full Area of the image of DFT. If this is not set,\n");
	fprintf(fp0, "                output with the same image size with original image.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t ......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ...... Filename of Original R-AXIS Image File.\n");
	fprintf(fp0, "   file2 ...... Output R-AXIS Image Filename. This must be the last argument.\n");
	fprintf(fp0, "                If last argument is \"-replace\", Original R-AXIS Image File is\n");
	fprintf(fp0, "                replaced by Output File. Be careful using this option.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	double *arg1;
#else
char splitArg(const char *argV, char *arg0, double *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	return (2);
}
