/* Program to generate IP data file */
#define ProgName "ipgen"
#define ProgInfo "Dummy Data Generator"
/*                                                         5/24/1999  */
/*                        for ANSI C                       5/24/1999  */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v2.17a_5/8/2007"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#define MAXARG 6
#define MINARG 3
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define LAMBDA_DEFAULT 1.54178
#define CAMERA_DEFAULT 50.0
#define XSIZ_DEFAULT 0.05
#define ZSIZ_DEFAULT 0.05

#ifdef _OLDHP
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, long int *arg1);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr, tmpInt;
	long int i, j, ipHeaderSize, ipHeaderSize2VER2, recordLengthCreate,
	     ipHeaderSizeMin, padInt, arg1, intDummy, readByte,
	     flushWidth, flushWidth0;
	unsigned long int maxInt;
	char file1[FNAMELEN], inputBuffer[20], confirmBuffer[8], argNum,
	     cpuIntelFlag, ipDataSwapFlag, shiftBit, arg0[4], verboseFlag,
	     newWidthDoneFlag, newHeightDoneFlag,  padDoneFlag,
	    noDummyFlag, file3[FNAMELEN], useRefFlag, ipHeaderSwapFlag;
	double rato;
	struct ipHead ipHeader;
	time_t timeNow;
	FILE *fp1, *fp3;

/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	newWidthDoneFlag = newHeightDoneFlag = padDoneFlag = noDummyFlag
	 = useRefFlag = 0;
	verboseFlag = 1;
	ipHeaderSwapFlag = 0;
	readByte = 0;
	flushWidth0 = 0;
	flushWidth = 0;
	rato = 8.0;
	shiftBit = 3;
	maxInt = ((long)(0x7fff)) << shiftBit;
	ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
	intDummy = 0x0000ffff;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag) {
		ipDataSwapFlag = 1;
		intDummy = swapLong(intDummy);
	}
	else ipDataSwapFlag = 0;
	
/* Check Argument */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strncpy(file1, argv[argc-1], FNAMELEN-1);
		for (i=0; i<argc-1; i++) {
			if (!strncmp(argv[i], "-r:", 3)) {
				if (useRefFlag) {
					fprintf(stderr, "Too much REFFILEs.\n");
					exit(EXIT_FAILURE);
				}
				if (strlen(argv[i]) < 4) {
					fprintf(stderr, "\"REFFILE\" must be provided.\n");
					exit(EXIT_FAILURE);
				}
				strncpy(file3, argv[i]+3, FNAMELEN-1);
				useRefFlag++;
			}
		}
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* Interactive ? */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Do you like to use Reference File to get parameters for New File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			useRefFlag = 1;
			fprintf(stdout, "Enter Reference Filename : ");
			if (fscanf(stdin, "%s", file3) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				exit(EXIT_FAILURE);
			}
		}
	}
	
/* Define ipHeader and Initialize */
	if (useRefFlag) {	/* Read REFFILE */
		if ((fp3 = fopen(file3, "rb")) == NULL) {
			fprintf(stderr, " Reference File Open Error.\n");
			exit(EXIT_FAILURE);
		}
		fread(&ipHeader, ipHeaderSize, 1, fp3);
		if (ftell(fp3) != ipHeaderSize) {
			fprintf(stderr, " Reference File Read Error.\n");
			fclose(fp3);
			exit(EXIT_FAILURE);
		}
		fclose(fp3);
		/* File Format Check */
		ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
		/* ipHeader.rlng is assumed to be less than 65536;
		   if each data has 16-bits, width of IP should less than 32768 pixels. */
		if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
			if (verboseFlag) {
				fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
				do {
					fprintf(stdout, "  Is it true ? [Y/N] : ");
					if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						exit(EXIT_FAILURE);
					}
				} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
				if (toupper(confirmBuffer[0]) == 'N') {
					fprintf(stderr, " Unable to continue. Stop.\n");
					exit(EXIT_FAILURE);
				}
				fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
			}
		}
		/* Swap Header */
		if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
		/* Get ipHeader Information and Check */
		if (ipHeader.zpxl != ipHeader.rnum) {
			ipHeader.zpxl = ipHeader.rnum;
			if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height in Reference File has been modified.\n");
		}
		if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) {
			ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
			if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width in Reference File has been modified.\n");
		}
		if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH)
		 || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
			fprintf(stderr, " Reference File Format is Incorrect.\n");
			exit(EXIT_FAILURE);
		}
		if ((ipHeader.drtx < 0.0) || (ipHeader.drtx > ipHeader.xpxl)) {
			ipHeader.drtx = ipHeader.xpxl / 2.0;
			if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Center in X of Reference File has been reset.\n");
		}
		if ((ipHeader.drtz < 0.0) || (ipHeader.drtz > ipHeader.zpxl)) {
			ipHeader.drtz = ipHeader.zpxl / 2.0;
			if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Center in Z of Reference File has been reset.\n");
		}
		if (ipHeader.xsiz <= 0.0) {
			ipHeader.drtx = XSIZ_DEFAULT;
			if (verboseFlag >= 2) fprintf(stdout, " Incorrect PixelSize in X of Reference File has been reset.\n");
		}
		if (ipHeader.zsiz <= 0.0) {
			ipHeader.drtz = ZSIZ_DEFAULT;
			if (verboseFlag >= 2) fprintf(stdout, " Incorrect PixelSize in Z of Reference File has been reset.\n");
		}
		if (ipHeader.camr <= 0.0) {
			ipHeader.camr = CAMERA_DEFAULT;
			if (verboseFlag >= 2) fprintf(stdout, " Incorrect Camera Length of Reference File has been reset.\n");
		}
		if (ipHeader.wlng <= 0.0) {
			ipHeader.wlng = LAMBDA_DEFAULT;
			if (verboseFlag >= 2) fprintf(stdout, " Incorrect WaveLength of Reference File has been reset.\n");
		}
		/* Clean Header */
		cleanIpHeader(&ipHeader);
		ipHeader.rato = rato;
		timeNow = time(NULL);
		strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&timeNow));
		strcpy(ipHeader.memo, ProgName);
		strcat(ipHeader.memo, "/");
		strcat(ipHeader.memo, ipHeader.date);
	}
	else {	/* Do not use REFFILE */
		memset(&ipHeader, 0, ipHeaderSize);
		strcpy(ipHeader.name, "R-AXIS-DS"); /* Name of the Device */
		strcpy(ipHeader.vers, "1.0-2.0"); /* Version Number */
		strcpy(ipHeader.crnm, "N/A"); /* Crystal Name */
		strcpy(ipHeader.crsy, "cubic"); /* Crystal System */
		ipHeader.alng = 10.0; /* a-axis length in A */
		ipHeader.blng = 10.0; /* b-axis length in A */
		ipHeader.clng = 10.0; /* c-axis length in A */
		ipHeader.alfa = 90.0; /* angle alpha */
		ipHeader.beta = 90.0; /* angle beta */
		ipHeader.gamm = 90.0; /* angle gamma */
		strcpy(ipHeader.spgr, "P23"); /* Space Group */
		ipHeader.mosc = 0.3; /* Mozaic Spread */
		strcpy(ipHeader.opnm, "rigaku"); /* Operator Name */
		strcpy(ipHeader.trgt, "Cu"); /* X-ray Target */
		ipHeader.wlng = LAMBDA_DEFAULT; /* Wave Length of the x-ray */
		strcpy(ipHeader.mcro, "None"); /* Monochromator Material */
		ipHeader.m2ta = 0.0; /* Angle of the monochromator in 2-theta */
		strcpy(ipHeader.colm, "0.05-0.10"); /* Collimator */
		strcpy(ipHeader.filt, "Ni"); /* Kb filter */
		ipHeader.camr = CAMERA_DEFAULT; /* Camera length in mm */
		ipHeader.vltg = 40.0; /* X-ray tube voltage */
		ipHeader.crnt = 100.0; /* X-ray tube current */
		strcpy(ipHeader.focs, "0.3x0.3"); /* X-ray focus */
		strcpy(ipHeader.optc, "Ni-filter"); /* X-ray optics */
		ipHeader.cyld = 0; /* Camera type */
		ipHeader.weis = 0.0; /* weisenberg oscillation */
		strcpy(ipHeader.mnax, "-b*"); /* Mount Axis */
		strcpy(ipHeader.bmax, "-a"); /* Beam Axis */
		ipHeader.phi0 = 0.0; /* phi0 */
		ipHeader.phis = 0.0; /* phi start */
		ipHeader.phie = 0.0; /* phi end */
		ipHeader.oscn = 0; /* Frequency of Oscillation */
		ipHeader.fext = 20.0; /* Exposure Time in min. */
		ipHeader.omga = 0.0; /* omega in theta */
		ipHeader.fkai = 0.0; /* xai */
		ipHeader.thta = 0.000; /* 2-theta */
		ipHeader.mu = 0.000; /* mu */
		ipHeader.xsiz = XSIZ_DEFAULT; /* Pixel size in X direction in mm */
		ipHeader.zsiz = ZSIZ_DEFAULT; /* Pixel size in Y direction in mm */
		ipHeader.ipst = 0; /* Line of start reading */
		ipHeader.ipnm = 1; /* Number of IP */
		ipHeader.rato = rato; /* Ratio of IP in High/Low */
		ipHeader.ft_1 = 0.0; /* fading time 1 */
		ipHeader.ft_2 = 0.0; /* fading time 2 */
		strcpy(ipHeader.host, "IRIS"); /* Type of Host Computer */
		strcpy(ipHeader.ip, "HR-3"); /* Type of IP */
		ipHeader.dr_x = 0; /* Data direction in horizontal */
		ipHeader.dr_z = 0; /* Data direction in vertical */
		ipHeader.drxz = 0; /* Data direction face/back */
		timeNow = time(NULL);
		strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&timeNow));
		strcpy(ipHeader.memo, "Generated by ");
		strcat(ipHeader.memo, ProgName);
		strcat(ipHeader.memo, " on ");
		strcat(ipHeader.memo, ipHeader.date);
	}
	
/* Get values */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		if (useRefFlag) {
			fprintf(stdout, "Reference File has following Values :\n");
			fprintf(stdout, " ImageSize is %ld [pixel] x %ld [pixel]\n", ipHeader.xpxl, ipHeader.zpxl);
		}
		do {
			fprintf(stdout, "Enter Image Width to generate [pixel]%s : ", useRefFlag ? " or \"D\" to unchange" : "");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				exit(EXIT_FAILURE);
			}
			if (useRefFlag && !strcmp(inputBuffer, "D")) {
				newWidthDoneFlag = 1;
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.xpxl = atol(inputBuffer);
			if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH)) continue;
			ipHeader.drtx = ipHeader.xpxl/2.0;
			newWidthDoneFlag = 1;
		} while (!newWidthDoneFlag);
		do {
			fprintf(stdout, "Enter Image Height to generate [pixel]%s : ", useRefFlag ? " or \"D\" to unchange" : "");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				exit(EXIT_FAILURE);
			}
			if (useRefFlag && !strcmp(inputBuffer, "D")) {
				newHeightDoneFlag = 1;
				break;
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.zpxl = atol(inputBuffer);
			if ((ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) continue;
			ipHeader.drtz = ipHeader.zpxl/2.0;
			newHeightDoneFlag = 1;
		} while (!newHeightDoneFlag);
		do {
			fprintf(stdout, "Enter Intensity to generate : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			padInt = atol(inputBuffer);
			if ((padInt < 0) || ((unsigned long)padInt > maxInt)) continue;
			padDoneFlag = 1;
		} while (!padDoneFlag);
		do {
			fprintf(stdout, "Skip setting Dummy Data for (0, 0) and (1, 0) ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'N') noDummyFlag++;
	}
	else {
		for (i=0; i<argc-1; i++) {
			if (!strncmp(argv[i], "-r:", 3)) continue;
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-nd")) noDummyFlag++;
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-x")) {
					ipHeader.xpxl = arg1;
					if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH)) break;
					ipHeader.drtx = ipHeader.xpxl/2.0;
					newWidthDoneFlag++;
				}
				else if (!strcmp(arg0, "-z")) {
					ipHeader.zpxl = arg1;
					if ((ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) break;
					ipHeader.drtz = ipHeader.zpxl/2.0;
					newHeightDoneFlag++;
				}
				else if (!strcmp(arg0, "-p")) {
					padInt = arg1;
					if ((padInt < 0) || ((unsigned long)padInt > maxInt)) break;
					padDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (((!newWidthDoneFlag || !newHeightDoneFlag) && !useRefFlag)
		 || !padDoneFlag) {
			fprintf(stderr, " Lack of Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if ((newWidthDoneFlag > 1) || (newHeightDoneFlag > 1) || (padDoneFlag > 1) || (noDummyFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
	}
	
/* Set Parameters to Create */
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(flushWidth, ipHeaderSizeMin);
	ipHeader.rlng = recordLengthCreate;
	ipHeader.rnum = ipHeader.zpxl;
	flushWidth0 = recordLengthCreate/RAXDATAWIDTH;
	
/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter  Output  Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	while (createFileEXT(file1, &fp1, 0)) {
		if (!verboseFlag) {
			fprintf(stderr, " Output File Creation Failure. Exit.\n");
			exit(EXIT_FAILURE);
		}
		fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	
/* Swap Header */
	if (cpuIntelFlag) swapIpHeader(&ipHeader);
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		remove(file1);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp1);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSize, 1, fp1);
	free(datPtr0);
	
/* Swap Header again */
	if (cpuIntelFlag) swapIpHeader(&ipHeader);
	
/* Write Data */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if ((datPtr = malloc(recordLengthCreate)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		remove(file1);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	tmpInt = ipLtoShortBit(&padInt, &rato, &shiftBit, &maxInt, &ipDataSwapFlag);
	for (j=0; j<ipHeader.xpxl; j++) datPtr[j] = tmpInt;
	for (j=ipHeader.xpxl; j<flushWidth0; j++) datPtr[j] = 0;
	for (i=0; i<ipHeader.rnum; i++) fwrite(datPtr, recordLengthCreate, 1, fp1);
	if (!noDummyFlag) {
		fseek(fp1, recordLengthCreate, SEEK_SET);
		fwrite(&intDummy, 4, 1, fp1);
	}
	free(datPtr);
	
/* End */
	if (verboseFlag >= 2) {
		if (ipDataSwapFlag) tmpInt = swapShort(tmpInt);
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp1));
		noDummyFlag ? fprintf(stdout, "  Intensity is %d. Dummy Intensity is not set.\n", tmpInt) : fprintf(stdout, "  Intensity is %d. Dummy Intensity : I(0, 0)=0, I(1, 0)=%lu.\n", tmpInt, maxInt);
	}
	fclose(fp1);
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
	fprintf(fp0, "        %s [-q|-v] -x:VALUE -z:VALUE -p:VALUE [-nd] file\n", ProgName);
	fprintf(fp0, "        %s [-q|-v] -r:REFFILE -p:VALUE [-nd] file\n", ProgName);
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Generate Dummy R-AXIS Image File with uniform intensity.\n");
	fprintf(fp0, "   -h ......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i ......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q ......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v ......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -x:VALUE ... Set Width of Image to \"VALUE\".\n");
	fprintf(fp0, "   -z:VALUE ... Set Height of Image to \"VALUE\".\n");
	fprintf(fp0, "   -r:REFFILE\n");
	fprintf(fp0, "            ... Instead of setting file size in above options,\n");
	fprintf(fp0, "                use parameters just same as \"REFFILE\".\n");
	fprintf(fp0, "                \"REFFILE\" must be valid R-AXIS Image File.\n");
	fprintf(fp0, "   -p:VALUE ... Set uniform intensity to \"VALUE\".\n");
	fprintf(fp0, "   -nd ........ No generation of dummy data.\n");
	fprintf(fp0, "                 If absent, set I(0, 0) = 0 and I(1, 0) = 262136.\n");
	fprintf(fp0, "   file ....... Filename to generate. This must be the last argument.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	long int *arg1;
#else
char splitArg(const char *argV, char *arg0, long int *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atol(chPtr1+1);
	return (2);
}
