/* Program to treat IP data file */
#define ProgName "ipgetint"
#define ProgInfo "Print Intensity of specified pixel or a region"
/*                                                        12/27/1999  */
/*         for ANSI C                                                 */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.16e_5/22/2007"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#define MAXARG 5
#define MINARG 1
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define MAXLINE 100000

#ifdef _OLDHP
	long int (*readDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, long int *arg1, long int *arg2);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr;
	long int i, j, ipHeaderSize, arg1, arg2, startX, endX, startZ, endZ,
	     width, numbyte, ipHeaderSize2VER1, ipHeaderSize2VER2, readByte, memsize;
	char file1[FNAMELEN], file2[FNAMELEN], inputBuffer[20], argNum,
	     confirmBuffer[8], cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag,
	     shiftBit, strangeShiftBitFlag, byte, arg0[4], verboseFlag, xDoneFlag,
	     zDoneFlag, showHexFlag, ofileDoneFlag, formatFlag;
	double rato;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2;
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	ipDataSwapFlag = xDoneFlag = zDoneFlag = showHexFlag = ofileDoneFlag = formatFlag = 0;
	verboseFlag = 1;
	numbyte = 0;
	readByte = 0;
	startX = startZ = endX = endZ = width = memsize = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
		
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) ipHeader.zpxl = ipHeader.rnum;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	rato = ipHeader.rato;
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		readDat = ipStoLongBit0;
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				readDat = ipStoLongBit;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			readDat = ipStoLongRato;
		}
	}
	
/* Get Parameters to Output */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Enter X-coordinate of Start Pixel to Print : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			startX = atol(inputBuffer);
			if ((startX < 0) || (startX >= ipHeader.xpxl)) continue;
			xDoneFlag = 1;
		} while (!xDoneFlag);
		xDoneFlag = 0;
		do {
			fprintf(stdout, "Enter X-coordinate of End Pixel to Print : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			endX = atol(inputBuffer);
			width = endX - startX + 1;
			if ((endX >= ipHeader.xpxl) || (width <= 0)) continue;
			xDoneFlag = 1;
		} while (!xDoneFlag);
		do {
			fprintf(stdout, "Enter Z-coordinate of Start Pixel to Print : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			startZ = atol(inputBuffer);
			if ((startZ < 0) || (startZ >= ipHeader.zpxl)) continue;
			zDoneFlag = 1;
		} while (!zDoneFlag);
		zDoneFlag = 0;
		do {
			fprintf(stdout, "Enter Z-coordinate of End Pixel to Print : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			endZ = atol(inputBuffer);
			if ((endZ >= ipHeader.zpxl) || (endZ < startZ)) continue;
			zDoneFlag = 1;
		} while (!zDoneFlag);
		do {
			fprintf(stdout, "Do you want to print also with Hexadecimal Format ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') showHexFlag = 1;
		else {
			do {
				fprintf(stdout, "Do you want to print also with long-line- Format ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'Y') formatFlag = 1;
		}
	}
	else {
		for (i=1; i<argc; i++) {
			if (!strcmp(argv[i], "-hex")) {
				showHexFlag++;
			}
			else if (!strcmp(argv[i], "-lf")) {
				formatFlag++;
			}
			else if (!strncmp(argv[i], "-o:", 3)) {
				strcpy(file2, argv[i]+3);
				ofileDoneFlag++;
			}
			else {
				argNum = splitArg(argv[i], arg0, &arg1, &arg2);
				if (argNum == 2) {
					if (!strcmp(arg0, "-x")) {
						startX = endX = arg1;
						width = 1;
						if ((startX < 0) || (startX >= ipHeader.xpxl)) break;
						xDoneFlag++;
					}
					else if (!strcmp(arg0, "-z")) {
						startZ = endZ = arg1;
						if ((startZ < 0) || (startZ >= ipHeader.zpxl)) break;
						zDoneFlag++;
					}
					else break;
				}
				else if (argNum == 3) {
					if (!strcmp(arg0, "-x")) {
						startX = arg1;
						endX = arg2;
						width = endX - startX + 1;
						if ((startX < 0) || (endX >= ipHeader.xpxl) || (endX < startX)) break;
						xDoneFlag++;
					}
					else if (!strcmp(arg0, "-z")) {
						startZ = arg1;
						endZ = arg2;
						if ((startZ < 0) || (endZ >= ipHeader.zpxl) || (endZ < startZ)) break;
						zDoneFlag++;
					}
					else break;
				}
				else break;
			}
		}
		if (i != argc) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((xDoneFlag != 1) || (zDoneFlag != 1) || (showHexFlag > 1)
		 || (ofileDoneFlag > 1) || (formatFlag > 1)
		 || (showHexFlag && formatFlag)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Set Parameters to Create */
	if (width * (endZ - startZ) > MAXLINE) {
		if (verboseFlag) {
			do {
				fprintf(stdout, " Output Data Exceeds %d. Do you want to continue ? [Y/N] : ", MAXLINE);
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		else {
			fprintf(stderr, " Output Lines Exceeds %d. Unable to continue. Stop.", MAXLINE);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	if (verboseFlag && formatFlag) {
		fprintf(stdout, "Print from (%5ld, %5ld) to (%5ld, %5ld) along with X-coodinates.\n", startX, startZ, endX, endZ);
	}
	memsize = width * RAXDATAWIDTH;
	
/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter Output Filename (or \"-\" to Standard Output) : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (strcmp(file2, "-")) ofileDoneFlag = 1;
	}
	if (ofileDoneFlag) {
		while (createFileEXT(file2, &fp2, 1)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
	}
	else fp2 = stdout;
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if (fseek(fp1, ipHeader.rlng*(startZ+1)+startX*RAXDATAWIDTH, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	for (i=startZ; i<=endZ; i++) {
		if ((datPtr = malloc(memsize)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			fclose(fp1);
			if (i > startZ) {
				if (verboseFlag) {
					if (ofileDoneFlag) {
						fprintf(stdout, " Output File is saved using available data.\n");
						fprintf(stdout, " Output to %s with %ld lines, %ld bytes.\n", file2, width*(i-startZ), numbyte);
					}
					else fprintf(stdout, " Output %ld lines, %ld bytes.\n", width*(i-startZ), numbyte);
				}
				if (ofileDoneFlag) fclose(fp2);
			}
			else {
				if (ofileDoneFlag) {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				else fprintf(stderr, " No data to print.\n");
			}
			exit(EXIT_FAILURE);
		}
		fread(datPtr, memsize, 1, fp1);
		readByte += memsize;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
			free(datPtr);
			fclose(fp1);
			if (i > startZ) {
				if (verboseFlag) {
					if (ofileDoneFlag) {
						fprintf(stdout, " Output File is saved using available data.\n");
						fprintf(stdout, " Output to %s with %ld lines, %ld bytes.\n", file2, width*(i-startZ), numbyte);
					}
					else fprintf(stdout, " Output %ld lines, %ld bytes.\n", width*(i-startZ), numbyte);
				}
				if (ofileDoneFlag) fclose(fp2);
			}
			else {
				if (ofileDoneFlag) {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				else fprintf(stderr, " No data to print.\n");
			}
			exit(EXIT_FAILURE);
		}
		for (j=0; j<width; j++) {
			if (showHexFlag) {
				if (ipDataSwapFlag) {
					numbyte += fprintf(fp2, " %5ld, %5ld, %10ld, 0x %.2X %.2X\n",
					                   j+startX, i, (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag),
					                   (unsigned char)((datPtr[j])&(0x00ff)), (unsigned char)((datPtr[j]>>8)&(0x00ff)));
				}
				else {
					numbyte += fprintf(fp2, " %5ld, %5ld, %10ld, 0x %.2X %.2X\n",
					                   j+startX, i, (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag),
					                   (unsigned char)((datPtr[j]>>8)&(0x00ff)), (unsigned char)((datPtr[j])&(0x00ff)));
				}
			}
			else if (formatFlag) {
				if (j<width-1) numbyte += fprintf(fp2, "%10ld, ",
				    (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag));
				else numbyte += fprintf(fp2, "%10ld\n",
				    (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag));
			}
			else numbyte += fprintf(fp2, " %5ld, %5ld, %10ld\n",
			                j+startX, i, (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag));
		}
		free(datPtr);
		if (i < endZ) {
			if (fseek(fp1, ipHeader.rlng-memsize, SEEK_CUR)) {
				fprintf(stderr, " Input File Read Error.\n");
				fclose(fp1);
				if (verboseFlag) {
					if (ofileDoneFlag) {
						fprintf(stdout, " Output File is saved using available data.\n");
						fprintf(stdout, " Output to %s with %ld lines, %ld bytes.\n", file2, width*(i-startZ+1), numbyte);
					}
					else fprintf(stdout, " Output %ld lines, %ld bytes.\n", width*(i-startZ+1), numbyte);
				}
				if (ofileDoneFlag) fclose(fp2);
				exit(EXIT_FAILURE);
			}
			readByte = ftell(fp1);
		}
	}
	
/* End */
	fclose(fp1);
	if (ofileDoneFlag) fclose(fp2);
	if (verboseFlag >= 2) {
		if (ofileDoneFlag)
		    fprintf(stdout, " Output to %s with %ld lines, %ld bytes.\n", file2, width*(endZ-startZ+1), numbyte);
		else
		    fprintf(stdout, " Output %ld lines, %ld bytes.\n", width*(endZ-startZ+1), numbyte);
	}
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
	fprintf(fp0, "        %s [-q|-v] file [-hex] [-lf] -x:VALUE[:VALUE]\n", ProgName);
	fprintf(fp0, "          -z:VALUE[:VALUE] [-o:file2]\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Print Intensity Data in the specified region\n");
	fprintf(fp0, "  of R-AXIS Image File. Output Format is\n");
	fprintf(fp0, "    X-coordinate, Z-coordinate, Intensity, InternalFormatIntensity(optional)\n");
	fprintf(fp0, "  Be careful not to point TOO LARGE REGION, for this might print\n");
	fprintf(fp0, "  ENORMOUS data to hurt your system.\n");
	fprintf(fp0, "   -h ..... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i ..... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q ..... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v ..... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   file ... Filename of R-AXIS Image File.\n");
	fprintf(fp0, "   -hex ... Add Information by Internal Format of 2bytes integer.\n");
	fprintf(fp0, "   -lf .... Show with long-line-format. Exclusive with \"-hex\".\n");
	fprintf(fp0, "   -x:VALUE1[:VALUE2]\n");
	fprintf(fp0, "        ... Set X-coordinate of the pixel to print intensity to \"VALUE1\".\n");
	fprintf(fp0, "            If \"VALUE2\" is set, print between \"VALUE1\" and \"VALUE2\".\n");
	fprintf(fp0, "   -z:VALUE1[:VALUE2]\n");
	fprintf(fp0, "        ... Set Z-coordinate of the pixel to print intensity to \"VALUE1\".\n");
	fprintf(fp0, "            If \"VALUE2\" is set, print between \"VALUE1\" and \"VALUE2\".\n");
	fprintf(fp0, "   -o:file2 ... Output to \"file2\". If absent, print to Standard Output.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1, arg2)
	char *argV, *arg0;
	long int *arg1, *arg2;
#else
char splitArg(const char *argV, char *arg0, long int *arg1, long int *arg2)
#endif
{
	char *chPtr1, *chPtr2;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 2) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 2) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atol(chPtr1+1);
	if ((chPtr2 = strchr(chPtr1+1, ':')) == NULL) return (2);
	if (!isdigitEXT(chPtr2+1)) return (0);
	*arg1 = atol(chPtr1+1);
	*arg2 = atol(chPtr2+1);
	return (3);
}
