/* Program to treat IP data file */
#define ProgName "iphist"
#define ProgInfo "Print Intensity Distribution Histogram"
/*                                                          6/5/2000  */
/*         for ANSI C                                                 */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.05_1/15/2007"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#define MAXARG 7
#define MINARG 3
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define MAXDIV 1024

#ifdef _OLDHP
	long int (*readDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr;
	long int i, j, k, ipHeaderSize, divNum,
	     numbyte, ipHeaderSize2VER1, ipHeaderSize2VER2, readByte,
	     *histPtr, lines, posL, posH, lbuf;
	unsigned long int maxInt, lowLimit, highLimit, window, intensity;
	char file1[FNAMELEN], file2[FNAMELEN], inputBuffer[20], argNum,
	     confirmBuffer[8], cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag,
	     shiftBit, strangeShiftBitFlag, byte, arg0[4], verboseFlag,
	     lowLimitDoneFlag, highLimitDoneFlag, ofileDoneFlag, stepDoneFlag,
	     divNumDoneFlag, noHeadFlag, csvFlag, timebuf[20];
	double rato, step, arg1;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2;
	time_t timeNow;
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	ipDataSwapFlag = lowLimitDoneFlag = highLimitDoneFlag = divNumDoneFlag
	 = ofileDoneFlag = stepDoneFlag = noHeadFlag = csvFlag = 0;
	verboseFlag = 1;
	numbyte = 0;
	readByte = 0;
	lines = 0;
	maxInt = 0x7fff;
	lowLimit = highLimit = 0;
	divNum = 1;
	step = 1.0;
	lbuf = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		if (strcmp(argv[argc-1], "-")) {
			strcpy(file2, argv[argc-1]);
			ofileDoneFlag++;
		}
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) ipHeader.zpxl = ipHeader.rnum;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	rato = ipHeader.rato;
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
		}
	}
	
/* Get Parameters to Output */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Enter Lower Limit Intensity for calculation. Enter 0 for No Limit : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			lbuf = atol(inputBuffer);
			if ((lbuf < 0) || (lowLimit=lbuf) >= maxInt) continue;
			lowLimitDoneFlag = 1;
		} while (!lowLimitDoneFlag);
		do {
			fprintf(stdout, "Enter Upper Limit Intensity for calculation. Enter 0 for No Limit : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			lbuf = atol(inputBuffer);
			if ((lbuf < 0) || ((highLimit=lbuf) <= lowLimit) || (highLimit > maxInt)) continue;
			highLimitDoneFlag = 1;
		} while (!highLimitDoneFlag);
		if (highLimit == 0) highLimit = maxInt;
		window = highLimit - lowLimit;
		do {
			fprintf(stdout, "Which type of division do you want to use ?\n");
			fprintf(stdout, " (1) Division Number , (2) Division Step  : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != '1') && (toupper(confirmBuffer[0]) != '2'));
		if (toupper(confirmBuffer[0]) == '1') {
			do {
				fprintf(stdout, "  Enter Division Number : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				divNum = atol(inputBuffer);
				if (((unsigned long)divNum > MINEXT((unsigned long)MAXDIV, window-1)) || (divNum < 0)) continue;
				stepDoneFlag = 1;
			} while (!stepDoneFlag);
			step = (float)window / (divNum + 1.0);
		}
		else {
			do {
				fprintf(stdout, "  Enter Division Step : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				step = atof(inputBuffer);
				if ((step > window) || (step < 1.0) || ((step > 1.0) && (window/step > MAXDIV))) continue;
				stepDoneFlag = 1;
			} while (!stepDoneFlag);
		}
	}
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-sh")) noHeadFlag++;
				else if (!strcmp(arg0, "-c")) csvFlag++;
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-up")) {
					if ((arg1 <= 0.0) || ((highLimit=dToULong(arg1)) > maxInt)) break;
					highLimitDoneFlag++;
				}
				else if (!strcmp(arg0, "-dn")) {
					if ((arg1 < 0.0) || ((lowLimit=dToULong(arg1)) >= maxInt)) break;
					lowLimitDoneFlag++;
				}
				else if (!strcmp(arg0, "-n")) {
					divNum = dToLong(arg1);
					if ((divNum > MAXDIV) || (divNum < 0)) break;
					divNumDoneFlag++;
				}
				else if (!strcmp(arg0, "-st")) {
					step = arg1;
					if (step < 1.0) break;
					stepDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!highLimitDoneFlag && !lowLimitDoneFlag && !divNumDoneFlag && !stepDoneFlag) {
			fprintf(stderr, " Lack of Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((highLimitDoneFlag > 1) || (lowLimitDoneFlag > 1) || (divNumDoneFlag > 1)
		 || (stepDoneFlag > 1) || (stepDoneFlag && divNumDoneFlag) || (csvFlag > 1)
		 || (ofileDoneFlag > 1) || (noHeadFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!highLimitDoneFlag) highLimit = maxInt;
		if (!lowLimitDoneFlag) lowLimit = 0;
		window = highLimit - lowLimit;
		if (!divNumDoneFlag && !stepDoneFlag) {
			step = window;
			divNum = 0;
		}
		if ((window <= 0)
		 || (stepDoneFlag && ((step > window) || ((step > 1.0) && (window/step > MAXDIV))))
		 || (divNumDoneFlag && ((unsigned long)divNum >= window))) {
			fprintf(stderr, " Invalid Value in Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (divNumDoneFlag) step = (float)window / (divNum + 1.0);
		if (stepDoneFlag) divNum = (long)ceil(window / step) - 1;
	}
	if (verboseFlag == 2) {
		fprintf(stdout, " Intensity Distribution Histogram is calculated\n");
		fprintf(stdout, "   between %lu and %lu, with %4.1f step.\n", lowLimit, highLimit, step);
	}
	
/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter Output Filename (or \"-\" to Standard Output) : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (strcmp(file2, "-")) ofileDoneFlag = 1;
		do {
			fprintf(stdout, "Do you want the output be CSV Style ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') csvFlag = 1;
	}
	if (ofileDoneFlag) {
		while (createFileEXT(file2, &fp2, 1)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
	}
	else fp2 = stdout;
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Suppress Output Header ? [Y/N] :\n");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				if (ofileDoneFlag) {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') noHeadFlag = 1;
	}
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if ((verboseFlag >= 2) && noHeadFlag) fprintf(stdout, " Output Information is suppressed.\n");
	if (!noHeadFlag) {
		numbyte += fprintf(fp2, "# Intensity Distribution Histogram for %s.\n", file1);
		numbyte += fprintf(fp2, "# Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		numbyte += fprintf(fp2, "#  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
		numbyte += fprintf(fp2, "# Calculation is done between %7lu and %7lu, with %10.1f step.\n", lowLimit, highLimit, step);
		timeNow = time(NULL);
		strftime(timebuf, 20, "%Y-%m-%d %H:%M:%S", localtime(&timeNow));
		numbyte += fprintf(fp2, "# Calculated by %s on %s\n", ProgName, timebuf);
		lines += 5;
	}
	if (fseek(fp1, ipHeader.rlng, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	if ((histPtr = malloc((divNum+3)*sizeof(long))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		if (ofileDoneFlag) {
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
		}
		exit(EXIT_FAILURE);
	}
	memset(histPtr, 0, (divNum+3)*sizeof(long));
	posL = divNum + 1;
	posH = divNum + 2;
	for (i=0; i<ipHeader.rnum; i++) {
		if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			if (i) {
				if (verboseFlag) {
					fprintf(stdout, " Output File is saved using available data of %ld lines.\n", i);
					if (ofileDoneFlag) {
						fprintf(fp2, " Output File is saved using available data of %ld lines.\n", i);
						lines++;
					}
				}
				j = -1;
				break;
			}
			else {
				if (ofileDoneFlag) {
					fclose(fp1);
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				else fprintf(stderr, " No data to print.\n");
				exit(EXIT_FAILURE);
			}
		}
		fread(datPtr, ipHeader.rlng, 1, fp1);
		readByte += ipHeader.rlng;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
			free(datPtr);
			if (i) {
				if (verboseFlag) {
					fprintf(stdout, " Output File is saved using available data of %ld lines.\n", i);
					if (ofileDoneFlag) {
						fprintf(fp2, " Output File is saved using available data of %ld lines.\n", i);
						lines++;
					}
				}
				j = -1;
				break;
			}
			else {
				if (ofileDoneFlag) {
					fclose(fp1);
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				else fprintf(stderr, " No data to print.\n");
				exit(EXIT_FAILURE);
			}
		}
		for (j=0; j<ipHeader.xpxl; j++) {
			intensity = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
			if (intensity < lowLimit) histPtr[posL]++;
			else if (intensity >= highLimit) histPtr[posH]++;
			else histPtr[(long)floor((intensity-lowLimit)/step)]++;
		}
		free(datPtr);
	}
	
/* End */
	fclose(fp1);
	i *= ipHeader.xpxl;
	if (csvFlag) {
		numbyte += fprintf(fp2, "  RANGE  ,  Number  ,  Ratio [%%]\n");
		if (lowLimit > 0) numbyte += fprintf(fp2, " %7d , %8ld , %9.5f\n", 0, histPtr[posL], 100.0*histPtr[posL]/i);
		for (k=0; k<divNum; k++)
		    numbyte += fprintf(fp2, " %7ld , %8ld , %9.5f\n", (long)floor(lowLimit+step*k), histPtr[k], 100.0*histPtr[k]/i);
		numbyte += fprintf(fp2, " %7ld , %8ld , %9.5f\n", (long)floor(lowLimit+step*divNum), histPtr[divNum], 100.0*histPtr[divNum]/i);
		numbyte += fprintf(fp2, " %7ld , %8ld , %9.5f\n", highLimit, histPtr[posH], 100.0*histPtr[posH]/i);
	}
	else {
		numbyte += fprintf(fp2, "       RANGE       :  Number  (   Ratio [%%] )\n");
		if (lowLimit > 0) numbyte += fprintf(fp2, " %7d - %7ld : %8ld ( %9.5f %% )\n", 0, lowLimit-1, histPtr[posL], 100.0*histPtr[posL]/i);
		for (k=0; k<divNum; k++)
		    numbyte += fprintf(fp2, " %7ld - %7ld : %8ld ( %9.5f %% )\n", (long)floor(lowLimit+step*k), (long)floor(lowLimit+step*(k+1)-1), histPtr[k], 100.0*histPtr[k]/i);
		numbyte += fprintf(fp2, " %7ld - %7ld : %8ld ( %9.5f %% )\n", (long)floor(lowLimit+step*divNum), highLimit-1, histPtr[divNum], 100.0*histPtr[divNum]/i);
		numbyte += fprintf(fp2, " %7ld - %7ld : %8ld ( %9.5f %% )\n", highLimit, maxInt, histPtr[posH], 100.0*histPtr[posH]/i);
	}
	free(histPtr);
	if (ofileDoneFlag) fclose(fp2);
	if (verboseFlag >= 2) ofileDoneFlag ?
	    fprintf(stdout, " Output to %s with %ld lines, %ld bytes.\n", file2, lines+divNum+4, numbyte) :
	    fprintf(stdout, " Output %ld lines, %ld bytes.\n", lines+divNum+4, numbyte);
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
	fprintf(fp0, "        %s [-q|-v] file [-up:VALUE] [-dn:VALUE]\n", ProgName);
	fprintf(fp0, "          {-n:VALUE|-st:VALUE} [-sh] [-c] file2\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Print Intensity Distribution Histogram of R-AXIS Image File.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   file ........ Filename of R-AXIS Image File.\n");
	fprintf(fp0, "   -up:VALUE ... Higher Limit of interest region. If absent, no limit.\n");
	fprintf(fp0, "   -dn:VALUE ... Lower Limit of interest region. If absent, no limit.\n");
	fprintf(fp0, "   -n:VALUE .... Number of division of the region. Exclusive with\n");
	fprintf(fp0, "                 the just below.\n");
	fprintf(fp0, "   -st:VALUE ... Intensity Step of the division. Exclusive with\n");
	fprintf(fp0, "                 the just above.\n");
	fprintf(fp0, "   -sh ......... Suppress Output Header. If absent, some informations on\n");
	fprintf(fp0, "                 Original File and conversion are print before profile.\n");
	fprintf(fp0, "   -c .......... Output with CSV File Style: Intensity, Number, and Ratio.\n");
	fprintf(fp0, "   file2 ....... Output Filename. This must be the last argument.\n");
	fprintf(fp0, "                 If last argument is \"-\", Output to Standard Output.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	double *arg1;
#else
char splitArg(const char *argV, char *arg0, double *arg1)
#endif
{
	char *chPtr1, *chPtr2;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	if ((chPtr2 = strchr(chPtr1+1, ':')) == NULL) return (2);
	return (3);
}
