/* Program to convert IP data file */
#define ProgName "ipiifix"
#define ProgInfo "R-AXIS Image Correction for IICCD"
/*                                                         1/30/2002  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

/* Caution !! This program contains some bugs, that pixel far from centre
   is not correctly converted because pixel is no longer square.
   But I release this program, for a net correctness of the pattern is
   very important than to see the distorted image. */

/* The way of correction is very important in this program.
   Method adopted in this program is based on the assumption that,
   1. Only the shape of the surface of II, which is assumed to be spherical,
      distorts image.
   2. Electorical lens is ideal.
   3. No exudation occures in the surface of II.
   Radius of II is the core of this correction.    v1.06, 12/24/2003 */

/* Some very "ad-hoc" fix in small camera length with specific II is
   prepared. This is not correct with other cases.
   Be careful to use "-cs" option.                 v1.08, 12/25/2003 */

/* Above option is developed to work using arbitrary coefficient
   with parabolic function.                        v1.09,   1/5/2004 */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v0.12_1/15/2007"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#ifdef _HASUTIME
#	include <sys/stat.h>
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef _HASUTIME
#	define MAXARG 9
#else
#	define MAXARG 8
#endif
#define MINARG 3
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define SPHEREMODE 0
#define APPROXMODE 1
#define ESPHERMODE 2

#ifdef _OLDHP
	long int (*readDat)();
	short int (*writeDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
	double funcSphere();
	double funcApprox();
	double (*funcHiPic)();
	double funcSphereAdHoc();
	double lim_funcSphereAdHocN();
	double lim_funcSphereAdHocP();
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	short int (*writeDat)(const float *x, const double *rato, const unsigned long int *maxInt, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1);
	double funcSphere(double x, double z);
	double funcApprox(double x, double z);
	double (*funcHiPic)(double x, double z);
	double funcSphereAdHoc(double x, double z);
	double lim_funcSphereAdHocN(double r);
	double lim_funcSphereAdHocP(double r);
#endif

double pf0, pf1, pf2, pf3, pf4, xsiz2, zsiz2, rlim2, rfactor;

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr;
	long int i, j, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2, recordLength,
	     recordLengthCreate, ipHeaderSizeMin, readByte, flushWidth, flushWidth0,
	     offx, offz, centrex, centrez, oldxpxl, oldzpxl, base, minx, minz,
	     newWidth, newHeight, num, limx, limz, x0, x1, x0p, x1p, newWidthP,
	     zBuf, zBufP, cOffX, cOffZ, newx0n, newx1n, newz0n, newz1n;
	unsigned long int maxInt;
	char file1[FNAMELEN], file2[FNAMELEN], inputBuffer[20], argNum,
	     confirmBuffer[8], cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag,
	     shiftBit, strangeShiftBitFlag, byte, createDosFlag, useAll,
	     arg0[4], verboseFlag, replaceFlag, setTimeFlag, camrDoneFlag,
	     drtxDoneFlag, drtzDoneFlag, smallFlag, calibFlag, calibMode,
	     extraCalibFlag;
#ifdef _HAS_STRANGE_RENAME
	char fnameTmp[FNAMELEN];
#endif
	double rato, arg1, intensityDbl, ratox, ratoz, newx0, newz0, calib,
	       *dBuf0, *dBuf1, newx1, newz1, newz00, newz01, newz10, newz11,
	       buf0, buf1, rlim, camr;
	float *datPtrF;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	ipDataSwapFlag = camrDoneFlag = replaceFlag = setTimeFlag
	 = drtxDoneFlag = drtzDoneFlag = smallFlag = calibFlag = extraCalibFlag = 0;
	calibMode = SPHEREMODE;
	useAll = 0;
	verboseFlag = 1;
	readByte = 0;
	flushWidth = 0;
	flushWidth0 = 0;
	calib = 0.0;
	buf0 = buf1 = 0.0;
	datPtr = (short*)NULL;
	minx = minz = 0;
	limx = limz = 0;
	rlim = 0.0;
	rfactor = 0.0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
		if (!strcmp(argv[argc-1], "-replace")) replaceFlag = 1;
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
		createDosFlag = 1;
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
		createDosFlag = 0;
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) {
		ipHeader.zpxl = ipHeader.rnum;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height has modified.\n");
	}
	recordLength = ipHeader.rlng;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) {
		ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width has modified.\n");
	}
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (ipHeader.xsiz <= 0.0 || ipHeader.zsiz <= 0.0) {
		fprintf(stderr, " PixelSize is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	rato = ipHeader.rato;
	if (ipHeader.cyld >= 256) {
		if (verboseFlag) fprintf(stdout, " Camera Type Code has changed from %ld to 0.\n", ipHeader.cyld);
		ipHeader.cyld = 0;
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
		fprintf(stdout, "  Camera Length is %10.4f mm.\n", ipHeader.camr);
		fprintf(stdout, " PixelSize is : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
	}
	
/* Clean Header */
	cleanIpHeader(&ipHeader);
	createDosFlag ? cleanIpHeader2VER1(&ipHeader2VER1) : cleanIpHeader2VER2(&ipHeader2VER2);
	
/* Logging */
	if (logWrite(&ipHeader, ProgName)) {
		if (verboseFlag) fprintf(stdout, " ipHeader.memo is full, and No Logging.\n");
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
		writeDat = ipFtoShortBit0;
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
				writeDat = ipFtoShort;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
			writeDat = ipFtoShort;
		}
	}
	
/* Get Magnification */
	camr = ipHeader.camr;
	centrex = ipHeader.xpxl / 2;
	centrez = ipHeader.zpxl / 2;
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Use [D]efalut mode or [M]anual mode in correction ? : [D/M] ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'D') && (toupper(confirmBuffer[0]) != 'M'));
		if (toupper(confirmBuffer[0]) == 'M') {
			do {
				fprintf(stdout, "  Enter pre-factor value (caution!! keep it small !!) : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				calib = atof(inputBuffer);
				if (calib <= 0.0) continue;
				calibFlag = 1;
			} while (!calibFlag);
			calibMode = APPROXMODE;
		}
		else {
			fprintf(stdout, " OK, use default mode.\n");
			if (camr <= 0.0) {
				fprintf(stdout, " Camera Length is not valid.\n");
				confirmBuffer[0] = 'Y';
			}
			else {
				do {
					fprintf(stdout, "Do you want to fix camera length ? [Y/N] : ");
					if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
				} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			}
			if (toupper(confirmBuffer[0]) == 'Y') {
				do {
					fprintf(stdout, "  Enter camera length to use [mm] : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					if (!isdigitEXT(inputBuffer)) continue;
					camr = atof(inputBuffer);
					if (camr <= 0.0) continue;
					camrDoneFlag = 1;
				} while (!camrDoneFlag);
			}
			do {
				fprintf(stdout, "  Enter pre-factor value (tpically, radius of II[mm]) : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				calib = atof(inputBuffer);
				if (calib <= 0.0) continue;
				calibFlag = 1;
			} while (!calibFlag);
			calibMode = SPHEREMODE;
			do {
				fprintf(stdout, " Do you want to use extra term with parabolic function ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'Y') {
				do {
					fprintf(stdout, "  Enter coefficient for parabolic term : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					if (!isdigitEXT(inputBuffer)) continue;
					rfactor = atof(inputBuffer);
					extraCalibFlag = 1;
				} while (!extraCalibFlag);
				calibMode = ESPHERMODE;
			}
		}
		do {
			fprintf(stdout, "Do you want to use another centre of image in correction ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			do {
				fprintf(stdout, " Enter centre in X [pixel] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				centrex = atol(inputBuffer);
				if (centrex <= 0 || centrex >= ipHeader.xpxl) continue;
				drtxDoneFlag = 1;
			} while (!drtxDoneFlag);
			do {
				fprintf(stdout, " Enter centre in Z [pixel] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				centrez = atol(inputBuffer);
				if (centrez <= 0 || centrez >= ipHeader.zpxl) continue;
				drtzDoneFlag = 1;
			} while (!drtzDoneFlag);
		}
		do {
			fprintf(stdout, "Do you want to ignore region which might be \"white\" ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') smallFlag = 1;
		do {
			fprintf(stdout, "Replace Original R-AXIS File with Output File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') replaceFlag = 1;
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-s")) smallFlag++;
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-xc")) {
					centrex = dToLong(arg1);
					if (centrex <= 0 || centrex >= ipHeader.xpxl) break;
					drtxDoneFlag++;
				}
				else if (!strcmp(arg0, "-zc")) {
					centrez = dToLong(arg1);
					if (centrez <= 0 || centrez >= ipHeader.zpxl) break;
					drtzDoneFlag++;
				}
				else if (!strcmp(arg0, "-cl")) {
					camr = arg1;
					if (camr <= 0.0) break;
					camrDoneFlag++;
				}
				else if (!strcmp(arg0, "-cr")) {
					calib = arg1;
					if (calib <= 0.0) break;
					calibMode = SPHEREMODE;
					calibFlag++;
				}
				else if (!strcmp(arg0, "-cf")) {
					calib = arg1;
					if (calib <= 0.0) break;
					calibMode = APPROXMODE;
					calibFlag++;
				}
				else if (!strcmp(arg0, "-cs")) {
					rfactor = arg1;
					extraCalibFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((drtxDoneFlag > 1) || (drtzDoneFlag > 1) || (camrDoneFlag > 1)
		 || (setTimeFlag > 1) || (smallFlag > 1) || (calibFlag > 1)
		 || (extraCalibFlag > 1)
		 || (extraCalibFlag && calibMode != SPHEREMODE)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!calibFlag) {
			fprintf(stderr, " Lack of arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (extraCalibFlag) calibMode = ESPHERMODE;
		if (camr <= 0.0 && calibMode == SPHEREMODE) {
			fprintf(stderr, " Camera Length of %10.4f is Incorrect. Exit.\n", camr);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	if (verboseFlag >= 2) {
		if ((calibMode == SPHEREMODE || calibMode == ESPHERMODE) && camrDoneFlag) fprintf(stdout, " Camera Length is changed to %10.4f.\n", camr);
		if (calibMode == SPHEREMODE) fprintf(stdout, " Calculate using standard method.\n");
		else if (calibMode == ESPHERMODE) fprintf(stdout, " Calculate using standard method, with some extra term.\n");
		else fprintf(stdout, " Calculate with parabolic graph with even function.\n");
		if (replaceFlag) fprintf(stdout, " Original R-AXIS Image File will be replaced !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Set Parameters to Create */
	if (ipHeader.drtx >= ipHeader.xpxl) ipHeader.drtx = ipHeader.xpxl - 1.0;
	else if (ipHeader.drtx < 0.0) ipHeader.drtx = 0.0;
	if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
	else if (ipHeader.drtz < 0.0) ipHeader.drtz = 0.0;
	oldxpxl = ipHeader.xpxl;
	oldzpxl = ipHeader.zpxl;
	xsiz2 = ipHeader.xsiz * ipHeader.xsiz;
	zsiz2 = ipHeader.zsiz * ipHeader.zsiz;
	if (calibMode == SPHEREMODE) {
		funcHiPic = funcSphere;
		/* = pf0 / (pf1*sqrt(pf2+r*pf3) - pf2), r=r*r */
		pf0 = calib / camr;
		pf1 = pf0 + 1.0;
		pf2 = pf0 * pf0;
		pf3 = 1.0/camr/camr;
		rlim2 = camr*camr*(pf0*2.0+1.0);
	}
	else if (calibMode == APPROXMODE) {
		funcHiPic = funcApprox;
		pf0 = calib;
		rlim2 = 1.0 / 3.0 / calib;
	}
	else if (calibMode == ESPHERMODE) {
		funcHiPic = funcSphereAdHoc;
		/* = pf0 / (pf1*sqrt(pf2+r*pf3) - pf2), r=r*r */
		pf0 = calib / camr;
		pf1 = pf0 + 1.0;
		pf2 = pf0 * pf0;
		pf3 = 1.0/camr/camr;
		pf4 = rfactor/camr/camr;
		arg1 = pf0*2.0+1.0;
		if (rfactor < 0.0) {
			rlim2 = MINEXT(ipHeader.xsiz, ipHeader.xsiz)/camr/100.0; /* "100.0" has no means. "rlim2" is reused. */
			if ((rlim = bisect(lim_funcSphereAdHocN, pf0, pf1, rlim2)) == 0.0) {
				fprintf(stderr, "Error in getting limit of radius... Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			rlim -= rlim2;
			rlim2 = (rlim*rlim - pf2)*camr*camr;
		}
		else if (rfactor <= 1.0/arg1) rlim2 = camr*camr*(arg1);
		else if (rfactor < pf1/pf2/2.0) {
			rlim2 = MINEXT(ipHeader.xsiz, ipHeader.xsiz)/camr/100.0; /* "100.0" has no means. "rlim2" is reused. */
			if ((rlim = bisect(lim_funcSphereAdHocP, pf0+rlim2, pf1, rlim2)) == 0.0) {
				fprintf(stderr, "Error in getting limit of radius... Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			rlim -= rlim2;
			rlim2 = (rlim*rlim - pf2)*camr*camr;
		}
		else {
			fprintf(stderr, "Factor for parabola is too large. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	if (rlim2 <= 0.0) {
		fprintf(stderr, "Unknown error while calculating image size. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	rlim = sqrt(rlim2);
	buf0 = centrex*centrex*xsiz2 + centrez*centrez*zsiz2;
	if ((buf1 = centrex*centrex*xsiz2 + (ipHeader.zpxl-centrez)*(ipHeader.zpxl-centrez)*zsiz2) > buf0) buf0 = buf1;
	if ((buf1 = (ipHeader.xpxl-centrex)*(ipHeader.xpxl-centrex)*xsiz2 + centrez*centrez*zsiz2) > buf0) buf0 = buf1;
	if ((buf1 = (ipHeader.xpxl-centrex)*(ipHeader.xpxl-centrex)*xsiz2 + (ipHeader.zpxl-centrez)*(ipHeader.zpxl-centrez)*zsiz2) > buf0) buf0 = buf1;
	if (buf0 < rlim2) useAll = 1;
	if (useAll) {
		offx = (long)floor(centrex * (1.0 - (funcHiPic(centrex, 1.0) + funcHiPic(centrex, 0.0)) / 2.0));
		newWidth = (long)ceil(centrex + (oldxpxl-centrex) * ((funcHiPic(oldxpxl-centrex, 1.0) + funcHiPic(oldxpxl-centrex, 0.0)) / 2.0)) - offx;
		offz = (long)floor(centrez * (1.0 - (funcHiPic(1.0, centrez) + funcHiPic(0.0, centrez)) / 2.0));
		newHeight = (long)ceil(centrez + (oldzpxl-centrez) * ((funcHiPic(1.0, oldzpxl-centrez) + funcHiPic(0.0, oldzpxl-centrez)) / 2.0)) - offz;
		if (smallFlag) {
			minz = centrex*2 < oldxpxl ? (long)ceil(centrez + funcHiPic(oldxpxl-1.0-centrex, 1.0-centrez) * (1.0-centrez))
			                           : (long)ceil(centrez + funcHiPic(1.0-centrex, 1.0-centrez) * (1.0-centrez));
			ipHeader.zpxl = centrex*2 < oldxpxl ? (long)floor(centrez + funcHiPic(oldxpxl-1.0-centrex, oldzpxl-1.0-centrez) * (oldzpxl-1.0-centrez)) - minz
			                                    : (long)floor(centrez + funcHiPic(1.0-centrex, oldzpxl-1.0-centrez) * (oldzpxl-1.0-centrez)) - minz;
			minx = centrez*2 < oldzpxl ? (long)ceil(centrex + funcHiPic(1.0-centrex, oldzpxl-1.0-centrez) * (1.0-centrex))
			                           : (long)ceil(centrex + funcHiPic(1.0-centrex, 1.0-centrez) * (1.0-centrex));
			ipHeader.xpxl = centrez*2 < oldzpxl ? (long)floor(centrex + funcHiPic(oldxpxl-1.0-centrex, oldzpxl-1.0-centrez) * (oldxpxl-1.0-centrex)) - minx
			                                    : (long)floor(centrex + funcHiPic(oldxpxl-1.0-centrex, 1.0-centrez) * (oldxpxl-1.0-centrex)) - minx;
			arg1 = centrex + funcHiPic(ipHeader.drtx-centrex, ipHeader.drtz-centrez) * (ipHeader.drtx-centrex) - minx;
			ipHeader.drtz = centrez + funcHiPic(ipHeader.drtx-centrex, ipHeader.drtz-centrez) * (ipHeader.drtz-centrez) - minz;
			ipHeader.drtx = arg1;
		}
		else {
			ipHeader.xpxl = newWidth;
			ipHeader.zpxl = newHeight;
			arg1 = centrex + funcHiPic(ipHeader.drtx-centrex, ipHeader.drtz-centrez) * (ipHeader.drtx-centrex) - offx;
			ipHeader.drtz = centrez + funcHiPic(ipHeader.drtx-centrex, ipHeader.drtz-centrez) * (ipHeader.drtz-centrez) - offz;
			ipHeader.drtx = arg1;
		}
		newWidth += 1; /* ad-hoc fix, but important and meaningful for treating buffers */
	}
	else {
		limx = (long)floor(rlim / ipHeader.xsiz);
		limz = (long)floor(rlim / ipHeader.zsiz);
		offx = (long)ceil(centrex * (1.0 - funcHiPic(MINEXT(limx, centrex), 0.0)));
		newWidth = (long)floor(centrex + (oldxpxl-centrex) * funcHiPic(MINEXT(limx, oldxpxl-centrex), 0.0)) - offx;
		offz = (long)ceil(centrez * (1.0 - funcHiPic(0.0, MINEXT(limz, centrez))));
		newHeight = (long)floor(centrez + (oldzpxl-centrez) * funcHiPic(0.0, MINEXT(limz, oldzpxl-centrez))) - offz;
		ipHeader.xpxl = newWidth;
		ipHeader.zpxl = newHeight;
		arg1 = centrex + funcHiPic(ipHeader.drtx-centrex, ipHeader.drtz-centrez) * (ipHeader.drtx-centrex) - offx;
		ipHeader.drtz = centrez + funcHiPic(ipHeader.drtx-centrex, ipHeader.drtz-centrez) * (ipHeader.drtz-centrez) - offz;
		ipHeader.drtx = arg1;
		newWidth += 2; /* ad-hoc fix, but important and meaningful for treating buffers */
	}
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(flushWidth, ipHeaderSizeMin);
	if (createDosFlag) recordLengthCreate = ((long)(ceil(recordLengthCreate/512.0)))*512;
	ipHeader.rlng = recordLengthCreate;
	ipHeader.rnum = ipHeader.zpxl;
	flushWidth0 = recordLengthCreate - flushWidth;
	newWidthP = newWidth - 1;
	if (verboseFlag >= 2) {
		if (calibMode == SPHEREMODE) fprintf(stdout, " Image is corrected using Sphere-Correction Method of R = %10.5E.\n", calib);
		else if (calibMode == ESPHERMODE) {
			fprintf(stdout, " Image is corrected using Sphere-Correction Method of R = %10.5E,\n", calib);
			fprintf(stdout, "  with additional term of %10.5E * r^2.\n", rfactor);
		}
		else fprintf(stdout, " Image is corrected using r* = (1.0 - %10.5E * r^2)*r.\n", calib);
		if (drtxDoneFlag || drtzDoneFlag) fprintf(stdout, " Image centre for correction is ( %ld, %ld ).\n", centrex, centrez);
		if (!useAll) fprintf(stdout, " Image contains some areas which can not be corrected with this program.\n");
		else {
			if (smallFlag) {
				fprintf(stdout, " Shrink of image is not valid.\n");
				smallFlag = 0;
			}
		}
	}
	
/* Create Output File */
	if (replaceFlag) {
		if (mkstempEXT(file1, file2, &fp2, 0)) {
			fprintf(stderr, " Temporary File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		if (verboseFlag == 3) {
			fprintf(stdout, "Enter  Output  Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		while (createFileEXT(file2, &fp2, 0)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
	}
	
/* Restore endian on ipHeader */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	createDosFlag ? fwrite(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp2) : fwrite(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSizeMin, 1, fp2);
	if (fseek(fp1, recordLength, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	if (flushWidth0 == 0) free(datPtr0);
	
/* Swap Header again */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if ((dBuf0 = malloc((oldxpxl+1)*sizeof(double))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		if (flushWidth0 > 0) free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	if ((dBuf1 = malloc((oldxpxl+1)*sizeof(double))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		if (flushWidth0 > 0) free(datPtr0);
		free(dBuf0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	/* bottom half */
	if ((datPtrF = calloc(newWidth*(centrez-offz+2), sizeof(float))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		if (flushWidth0 > 0) free(datPtr0);
		free(dBuf0);
		free(dBuf1);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	cOffX = centrex - offx;
	cOffZ = centrez - offz;
	if (useAll) {
		for (j=0; j<=oldxpxl; j++) dBuf1[j] = funcHiPic(j-centrex, centrez) * (j-centrex) + centrex - offx;
		for (i=0; i<centrez; i++) {
			if ((datPtr = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				if (flushWidth0 > 0) free(datPtr0);
				free(datPtrF);
				free(dBuf0);
				free(dBuf1);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			fread(datPtr, recordLength, 1, fp1);
			readByte += recordLength;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtr);
				if (flushWidth0 > 0) free(datPtr0);
				free(datPtrF);
				free(dBuf0);
				free(dBuf1);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			zBuf = i - centrez;
			zBufP = zBuf + 1;
			for (j=0; j<=oldxpxl; j++) {
				dBuf0[j] = dBuf1[j];
				dBuf1[j] = funcHiPic(j-centrex, zBufP) * (j-centrex) + cOffX;
			}
			newz10 = funcHiPic(centrex, zBuf) * (zBuf) + cOffZ;
			newz11 = funcHiPic(centrex, zBufP) * (zBufP) + cOffZ;
			for (j=0; j<oldxpxl; j++) {
				intensityDbl = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
				newx0 = (dBuf0[  j]+dBuf1[  j])/2.0;
				newx1 = (dBuf0[j+1]+dBuf1[j+1])/2.0;
				newx0n = (long)floor(newx0);
				newx1n = (long)floor(newx1);
				ratox = (newx0n == newx1n) ? 1.0 : (newx1-newx1n) / (newx1-newx0);
				newz00 = newz10;
				newz01 = newz11;
				newz10 = funcHiPic(j+1-centrex, zBuf) * (zBuf) + cOffZ;
				newz11 = funcHiPic(j+1-centrex, zBufP) * (zBufP) + cOffZ;
				newz0 = (newz00+newz10)/2.0;
				newz1 = (newz01+newz11)/2.0;
				newz0n = (long)floor(newz0);
				newz1n = (long)floor(newz1);
				if (newz0n == newz1n) {
					base = newz1n*newWidth + newx1n - 1;
					datPtrF[base++] += intensityDbl * (1.0-ratox);
					datPtrF[base  ] += intensityDbl * ratox;
				}
				else {
					ratoz = (newz1-newz1n) / (newz1-newz0);
					base = newz0n*newWidth + newx1n - 1;
					datPtrF[base++] += intensityDbl * (1.0-ratox) * (1.0-ratoz);
					datPtrF[base  ] += intensityDbl * ratox * (1.0-ratoz);
					base += newWidthP;
					datPtrF[base++] += intensityDbl * (1.0-ratox) * ratoz;
					datPtrF[base  ] += intensityDbl * ratox * ratoz;
				}
			}
			free(datPtr);
		}
	}
	else {
		if (limz < centrez) {
			if (fseek(fp1, (centrez-limz)*recordLength, SEEK_CUR)) {
				fprintf(stderr, " Input File Read Error.\n");
				if (flushWidth0 > 0) free(datPtr0);
				free(datPtrF);
				free(dBuf0);
				free(dBuf1);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			readByte += recordLength*(centrez-limz);
		}
		x0 = centrex;
		x1 = centrex - 1;
		for (i=MAXEXT(0, centrez-limz); i<centrez; i++) {
			if ((datPtr = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				if (flushWidth0 > 0) free(datPtr0);
				free(datPtrF);
				free(dBuf0);
				free(dBuf1);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			fread(datPtr, recordLength, 1, fp1);
			readByte += recordLength;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtr);
				if (flushWidth0 > 0) free(datPtr0);
				free(datPtrF);
				free(dBuf0);
				free(dBuf1);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			zBuf = i - centrez;
			zBufP = zBuf + 1;
			x0p = x0;
			x1p = x1;
			buf0 = sqrt(rlim2 - (zBuf)*(zBuf)*zsiz2) / ipHeader.xsiz; /* low, left or right */
			x0 = centrex - (long)floor(buf0);
			if (x0 < 0) x0 = 0;
			x1 = centrex + (long)floor(buf0);
			if (x1 > oldxpxl) x1 = oldxpxl;
			for (j=x0; j<x0p; j++) {
				dBuf0[j] = funcHiPic(j-centrex, zBuf) * (j-centrex) + cOffX;
				dBuf1[j] = funcHiPic(j-centrex, zBufP) * (j-centrex) + cOffX;
			}
			for (j=x0p; j<=x1p; j++) {
				dBuf0[j] = dBuf1[j];
				dBuf1[j] = funcHiPic(j-centrex, zBufP) * (j-centrex) + cOffX;
			}
			for (j=x1p+1; j<=x1; j++) {
				dBuf0[j] = funcHiPic(j-centrex, zBuf) * (j-centrex) + cOffX;
				dBuf1[j] = funcHiPic(j-centrex, zBufP) * (j-centrex) + cOffX;
			}
			newz10 = funcHiPic(x0-centrex, zBuf) * (zBuf) + cOffZ;
			newz11 = funcHiPic(x0-centrex, zBufP) * (zBufP) + cOffZ;
			for (j=x0; j<x1; j++) {
				intensityDbl = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
				newx0 = (dBuf0[  j]+dBuf1[  j])/2.0;
				newx1 = (dBuf0[j+1]+dBuf1[j+1])/2.0;
				newx0n = (long)floor(newx0);
				newx1n = (long)floor(newx1);
				ratox = (newx0n == newx1n) ? 1.0 : (newx1-newx1n) / (newx1-newx0);
				newz00 = newz10;
				newz01 = newz11;
				newz10 = funcHiPic(j+1-centrex, zBuf) * (zBuf) + cOffZ;
				newz11 = funcHiPic(j+1-centrex, zBufP) * (zBufP) + cOffZ;
				newz0 = (newz00+newz10)/2.0;
				newz1 = (newz01+newz11)/2.0;
				newz0n = (long)floor(newz0);
				newz1n = (long)floor(newz1);
				if (newz0n == newz1n) {
					base = (newz1n+1)*newWidth + newx1n;
					datPtrF[base++] += intensityDbl * (1.0-ratox);
					datPtrF[base  ] += intensityDbl * ratox;
				}
				else {
					base = newz1n*newWidth + newx1n;
					ratoz = (newz1-newz1n) / (newz1-newz0);
					datPtrF[base++] += intensityDbl * (1.0-ratox) * (1.0-ratoz);
					datPtrF[base  ] += intensityDbl * ratox * (1.0-ratoz);
					base += newWidthP;
					datPtrF[base++] += intensityDbl * (1.0-ratox) * ratoz;
					datPtrF[base  ] += intensityDbl * ratox * ratoz;
				}
			}
			free(datPtr);
		}
	}
	num = smallFlag ? centrez-minz : centrez-offz;
	base = smallFlag ? minx-offx+newWidth*(minz-offz) : (useAll ? 0 : newWidth+1);
	for (i=0; i<num; i++) {
		if ((datPtr = malloc(recordLengthCreate)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			if (flushWidth0 > 0) free(datPtr0);
			free(datPtrF);
			free(dBuf0);
			free(dBuf1);
			fclose(fp1);
			if (i) {
				ipHeader.zpxl = ipHeader.rnum = i;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
					}
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		for (j=0; j<ipHeader.xpxl; j++) datPtr[j] = (*writeDat)(&datPtrF[base+j], &rato, &maxInt, &ipDataSwapFlag);
		fwrite(datPtr, flushWidth, 1, fp2);
		if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
		free(datPtr);
		base += newWidth;
	}
	free(datPtrF);
	/* top half */
	if ((datPtrF = calloc(newWidth*(newHeight-centrez+offz+2), sizeof(float))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		if (flushWidth0 > 0) free(datPtr0);
		free(dBuf0);
		free(dBuf1);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	if (useAll) {
		for (i=centrez; i<oldzpxl; i++) {
			if ((datPtr = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				if (flushWidth0 > 0) free(datPtr0);
				free(datPtrF);
				free(dBuf0);
				free(dBuf1);
				fclose(fp1);
				ipHeader.zpxl = ipHeader.rnum = smallFlag?centrez-minz:centrez-offz;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
					}
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				exit(EXIT_FAILURE);
			}
			fread(datPtr, recordLength, 1, fp1);
			readByte += recordLength;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtr);
				if (flushWidth0 > 0) free(datPtr0);
				free(datPtrF);
				free(dBuf0);
				free(dBuf1);
				fclose(fp1);
				ipHeader.zpxl = ipHeader.rnum = smallFlag?centrez-minz:centrez-offz;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
					}
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				exit(EXIT_FAILURE);
			}
			zBuf = i - centrez;
			zBufP = zBuf + 1;
			for (j=0; j<=oldxpxl; j++) {
				dBuf0[j] = dBuf1[j];
				dBuf1[j] = funcHiPic(j-centrex, zBufP) * (j-centrex) + cOffX;
			}
			newz10 = funcHiPic(centrex, zBuf) * (zBuf);
			newz11 = funcHiPic(centrex, zBufP) * (zBufP);
			for (j=0; j<oldxpxl; j++) {
				intensityDbl = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
				newx0 = (dBuf0[  j]+dBuf1[  j])/2.0;
				newx1 = (dBuf0[j+1]+dBuf1[j+1])/2.0;
				newx0n = (long)floor(newx0);
				newx1n = (long)floor(newx1);
				ratox = (newx0n == newx1n) ? 1.0 : (newx1-newx1n) / (newx1-newx0);
				newz00 = newz10;
				newz01 = newz11;
				newz10 = funcHiPic(j+1-centrex, zBuf) * (zBuf);
				newz11 = funcHiPic(j+1-centrex, zBufP) * (zBufP);
				newz0 = (newz00+newz10)/2.0;
				newz1 = (newz01+newz11)/2.0;
				newz0n = (long)floor(newz0);
				newz1n = (long)floor(newz1);
				if (newz0n == newz1n) {
					base = newz1n*newWidth + newx1n - 1;
					datPtrF[base++] += intensityDbl * (1.0-ratox);
					datPtrF[base  ] += intensityDbl * ratox;
				}
				else {
					ratoz = (newz1-newz1n) / (newz1-newz0);
					base = newz0n*newWidth + newx1n - 1;
					datPtrF[base++] += intensityDbl * (1.0-ratox) * (1.0-ratoz);
					datPtrF[base  ] += intensityDbl * ratox * (1.0-ratoz);
					base += newWidthP;
					datPtrF[base++] += intensityDbl * (1.0-ratox) * ratoz;
					datPtrF[base  ] += intensityDbl * ratox * ratoz;
				}
			}
			free(datPtr);
		}
	}
	else {
		x1p = MINEXT(centrez+limz, oldzpxl); /* usage of x1p has no means */
		for (i=centrez; i<x1p; i++) {
			if ((datPtr = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				if (flushWidth0 > 0) free(datPtr0);
				free(datPtrF);
				free(dBuf0);
				free(dBuf1);
				fclose(fp1);
				ipHeader.zpxl = ipHeader.rnum = smallFlag?centrez-minz:centrez-offz;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
					}
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				exit(EXIT_FAILURE);
			}
			fread(datPtr, recordLength, 1, fp1);
			readByte += recordLength;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtr);
				if (flushWidth0 > 0) free(datPtr0);
				free(datPtrF);
				free(dBuf0);
				free(dBuf1);
				fclose(fp1);
				ipHeader.zpxl = ipHeader.rnum = smallFlag?centrez-minz:centrez-offz;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
					}
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				exit(EXIT_FAILURE);
			}
			zBuf = i - centrez;
			zBufP = zBuf + 1;
			buf0 = sqrt(rlim2 - (zBufP)*(zBufP)*zsiz2) / ipHeader.xsiz; /* high, left or right */
			x0 = centrex - (long)floor(buf0);
			if (x0 < 0) x0 = 0;
			x1 = centrex + (long)floor(buf0);
			if (x1 > oldxpxl) x1 = oldxpxl;
			for (j=x0; j<=x1; j++) {
				dBuf0[j] = dBuf1[j];
				dBuf1[j] = funcHiPic(j-centrex, zBufP) * (j-centrex) + cOffX;
			}
			newz10 = funcHiPic(x0-centrex, zBuf) * (zBuf);
			newz11 = funcHiPic(x0-centrex, zBufP) * (zBufP);
			for (j=x0; j<x1; j++) {
				intensityDbl = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
				newx0 = (dBuf0[  j]+dBuf1[  j])/2.0;
				newx1 = (dBuf0[j+1]+dBuf1[j+1])/2.0;
				newx0n = (long)floor(newx0);
				newx1n = (long)floor(newx1);
				ratox = (newx0n == newx1n) ? 1.0 : (newx1-newx1n) / (newx1-newx0);
				newz00 = newz10;
				newz01 = newz11;
				newz10 = funcHiPic(j+1-centrex, zBuf) * (zBuf);
				newz11 = funcHiPic(j+1-centrex, zBufP) * (zBufP);
				newz0 = (newz00+newz10)/2.0;
				newz1 = (newz01+newz11)/2.0;
				newz0n = (long)floor(newz0);
				newz1n = (long)floor(newz1);
				if (newz0n == newz1n) {
					base = (newz1n+1)*newWidth + newx1n;
					datPtrF[base++] += intensityDbl * (1.0-ratox);
					datPtrF[base  ] += intensityDbl * ratox;
				}
				else {
					base = newz1n*newWidth + newx1n;
					ratoz = (newz1-newz1n) / (newz1-newz0);
					datPtrF[base++] += intensityDbl * (1.0-ratox) * (1.0-ratoz);
					datPtrF[base  ] += intensityDbl * ratox * (1.0-ratoz);
					base += newWidthP;
					datPtrF[base++] += intensityDbl * (1.0-ratox) * ratoz;
					datPtrF[base  ] += intensityDbl * ratox * ratoz;
				}
			}
			free(datPtr);
		}
	}
	num = smallFlag ? ipHeader.zpxl-centrez+minz : ipHeader.zpxl-centrez+offz;
	base = smallFlag ? minx-offx : (useAll ? 0 : newWidth+1);
	for (i=0; i<num; i++) {
		if ((datPtr = malloc(recordLengthCreate)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			if (flushWidth0 > 0) free(datPtr0);
			free(datPtrF);
			free(dBuf0);
			free(dBuf1);
			fclose(fp1);
			ipHeader.zpxl = ipHeader.rnum = (smallFlag ? centrez-minz+i : centrez-offz+i);
			if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
			if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
			fseek(fp2, 0, SEEK_SET);
			fwrite(&ipHeader, ipHeaderSize, 1, fp2);
			fprintf(stdout, " Output File is saved using available data.\n");
			if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
			if (verboseFlag) {
				fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
				fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
				if (replaceFlag) {
					fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
					fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
				}
			}
			fclose(fp2);
#ifdef _HASUTIME
			if (setTimeFlag) {
				if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
			}
			if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			exit(EXIT_FAILURE);
		}
		for (j=0; j<ipHeader.xpxl; j++) datPtr[j] = (*writeDat)(&datPtrF[base+j], &rato, &maxInt, &ipDataSwapFlag);
		fwrite(datPtr, flushWidth, 1, fp2);
		if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
		free(datPtr);
		base += newWidth;
	}
	free(datPtrF);
	
/* End */
	if (flushWidth0 > 0) free(datPtr0);
	free(dBuf0);
	free(dBuf1);
	fclose(fp1);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
#ifndef _HAS_STRANGE_RENAME
	if (replaceFlag) {
		if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			fprintf(stderr, " Original R-AXIS Image File might be lost... Check It !!\n");
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#else
	if (replaceFlag) {
		if ((fp1 = fopen(file1, "rb+")) == NULL) {
			fprintf(stderr, "Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			replaceFlag = 0;
		}
		else fclose(fp1);
	}
	if (replaceFlag) {
		if (tmpNameEXT(file1, fnameTmp)) strcpy(fnameTmp, tmpnam(NULL));
		if (rename(file1, fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
		}
		else if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (remove(fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#endif
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
	fprintf(fp0, "        %s [-q|-v] file1 [-xc:VALUE] [-zc:VALUE] [-s] [-cl:VALUE]\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "         {-cr:VALUE|-cf:VALUE} [-cs:VALUE] [-t] {file2|-replace}\n");
#else
	fprintf(fp0, "         {-cr:VALUE|-cf:VALUE} [-cs:VALUE] {file2|-replace}\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program execute correction for IICCD in R-AXIS Image File.\n");
	fprintf(fp0, " =======================================================================\n");
	fprintf(fp0, " Caution !! This program contains some bugs, that pixel far from centre\n");
	fprintf(fp0, "  is not correctly converted because pixel is no longer square.\n");
	fprintf(fp0, "  But I release this program, for a net correctness of the pattern is\n");
	fprintf(fp0, "  very important than to see the distorted image.\n");
	fprintf(fp0, " =======================================================================\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -xc:VALUE ... Set centre of II in X to \"VALUE\".\n");
	fprintf(fp0, "                 If absent, set to the true centre of image.\n");
	fprintf(fp0, "   -zc:VALUE ... Set centre of II in Z to \"VALUE\".\n");
	fprintf(fp0, "                 If absent, set to the true centre of image.\n");
	fprintf(fp0, "                 These values are important for conversion.\n");
	fprintf(fp0, "   -cl:VALUE ... Set Camera Length to \"VALUE\".\n");
	fprintf(fp0, "                 If absent, use value saved in original file.\n");
	fprintf(fp0, "   -cr:VALUE ... Set radius of II to \"VALUE\"[mm].\n");
	fprintf(fp0, "                 This value is used while calculating correction factor.\n");
	fprintf(fp0, "   -cs:VALUE ... Same as above, but this contains ad-hoc fix with additional\n");
	fprintf(fp0, "                 terms of parabolic function having coefficient of \"VALUE\".\n");
	fprintf(fp0, "                 Use this when above option is not sufficient for correction.\n");
	fprintf(fp0, "                 This option should be used with above option.\n");
	fprintf(fp0, "                 Some unexpected trouble may be caused.\n");
	fprintf(fp0, "   -cf:VALUE ... Set Pre-Factor of x^2 in Calculation to \"VALUE\".\n");
	fprintf(fp0, "                 This approximation is fairly when camera length is large,\n");
	fprintf(fp0, "                 but not so good for small camera length or large image.\n");
	fprintf(fp0, "                 One of above 2 options is necessary.\n");
	fprintf(fp0, "   -s .......... Ganerate image with no \"white\" region.\n");
	fprintf(fp0, "                 This option is ignored when image is too large to treat\n");
	fprintf(fp0, "                 with this program. Without this option, such region is\n");
	fprintf(fp0, "                 just ignored.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t .......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ....... Filename of Original R-AXIS Image File.\n");
	fprintf(fp0, "   file2 ....... Output R-AXIS Image Filename. This must be the last argument.\n");
	fprintf(fp0, "                 If last argument is \"-replace\", Original R-AXIS Image File is\n");
	fprintf(fp0, "                 replaced by Output File. Be careful using this option.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	double *arg1;
#else
char splitArg(const char *argV, char *arg0, double *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	return (2);
}

#ifdef _OLDHP
double funcSphere(x, z)
	double x, z;
#else
double funcSphere(double x, double z)
#endif
{
	return (pf0 / (pf1 * sqrt(pf2+(x*x*xsiz2 + z*z*zsiz2)*pf3) - pf2));
}

#ifdef _OLDHP
double funcApprox(x, z)
	double x, z;
#else
double funcApprox(double x, double z)
#endif
{
	/* pf0 = calib; */
	return (1.0 - pf0*(x*x*xsiz2 + z*z*zsiz2));
}

#ifdef _OLDHP
double funcSphereAdHoc(x, z)
	double x, z;
#else
double funcSphereAdHoc(double x, double z)
#endif
{
	double r;
	r = (x*x*xsiz2 + z*z*zsiz2);
	/*
		pf0 = calib / camr;
		pf1 = pf0 + 1.0;
		pf2 = pf0 * pf0;
		pf3 = 1.0/camr/camr;
		pf4 = rfactor/camr/camr;
		"rfactor" is defined arbitrarily by user.
	*/
	return (pf0 / (pf1 * sqrt(pf2+r*pf3) - pf2) + pf4*r);
}

#ifdef _OLDHP
double lim_funcSphereAdHocN(r)
	double r;
#else
double lim_funcSphereAdHocN(double r)
#endif
{
	/* Get limit when pf4 in above function is negative.
	   Used by bisection method. */
	return (pf0*pf2*(pf1 - r) + 3.0*rfactor*(r*r-pf2)*(pf1*r-pf2)*(pf1*r-pf2)*r);
}

#ifdef _OLDHP
double lim_funcSphereAdHocP(r)
	double r;
#else
double lim_funcSphereAdHocP(double r)
#endif
{
	/* Get limit when pf4 in above function is positive.
	   Used by bisection method. */
	return (pf0*pf2*(pf1 - r) + (3.0*rfactor*(r*r-pf2)-1.0)*(pf1*r-pf2)*(pf1*r-pf2)*r);
}
