/* Program to treat IP data file */
#define ProgName "ipinfo"
#define ProgInfo  "Show IP file informations"
/*                                                         12/6/1999  */
/*         for ANSI C                                                 */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.39b_2/17/2006"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef WITH_PAUSE
#define MAXARG 5
#else
#define MAXARG 4
#endif
#define MINARG 1
#define NORMAL_STYLE 0
#define LONG_STYLE 1
#define ONE_STYLE 2
#define NORMAL_FORMAT 0
#define FREE_FORMAT 1
#define EXP_FORMAT 2
#define ONE_FORMAT 3

#ifdef _OLDHP
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, long int *arg1);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	long int i, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2, readByte, arg1;
	char file1[FNAMELEN], cpuIntelFlag, ipHeaderSwapFlag,
	     styleDoneFlag, ipDataSwapFlag, byte, createDosFlag, forceFlag,
	     argNum, arg0[4], verboseFlag, formatDoneFlag, nopauseFlag,
	     styleMode, formatMode, item;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1;
	union {
		long int dint;
		short int dints[2];
	} ipHeaderDint;
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	ipDataSwapFlag = styleDoneFlag = forceFlag = formatDoneFlag = nopauseFlag = 0;
	readByte = 0;
	verboseFlag = 1;
	styleMode = NORMAL_STYLE;
	formatMode = NORMAL_FORMAT;
	item = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 1);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-q")) {
		verboseFlag = 0;
		argv++;
		argc--;
	}
	argv++;
	argc--;
	if (argc > MAXARG) {
		showMessage(stdout, cpuIntelFlag, 1);
		fprintf(stderr, " Too Many Arguments.\n");
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (argc < MINARG) {
		showMessage(stdout, cpuIntelFlag, 1);
		fprintf(stderr, " Invalid Arguments .. Too Short.\n");
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	strcpy(file1, argv[argc-1]);
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* Get Options */
	if (argc > 1) {
		for (i=0; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-a")) {
					styleMode = LONG_STYLE;
					styleDoneFlag++;
				}
				else if (!strcmp(arg0, "-l")) {
					formatMode = FREE_FORMAT;
					formatDoneFlag++;
				}
				else if (!strcmp(arg0, "-e")) {
					formatMode = EXP_FORMAT;
					formatDoneFlag++;
				}
				else if (!strcmp(arg0, "-aa")) { /* for compatibility */
					styleMode = LONG_STYLE;
					styleDoneFlag++;
					formatMode = FREE_FORMAT;
					formatDoneFlag++;
				}
				else if (!strcmp(arg0, "-f")) forceFlag++;
#ifdef WITH_PAUSE
				else if (!strcmp(arg0, "-p")) nopauseFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-n")) {
					styleMode = ONE_STYLE;
					formatMode = ONE_FORMAT;
					item = arg1;
					if (item < 10 || item > 66) break;
					styleDoneFlag++;
					formatDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
#ifdef WITH_PAUSE
			fprintf(stdout, "Press return key to exit : ");
			fgetc(stdin);
#endif
			exit(EXIT_FAILURE);
		}
		if ((styleDoneFlag > 1) || (forceFlag > 1) || (formatDoneFlag > 1)
		 || (nopauseFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
#ifdef WITH_PAUSE
			fprintf(stdout, "Press return key to exit : ");
			fgetc(stdin);
#endif
			exit(EXIT_FAILURE);
		}
	}
	
/* File Read */
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
#ifdef WITH_PAUSE
		if (!nopauseFlag) {
			fprintf(stdout, "Press return key to exit : ");
			fgetc(stdin);
		}
#endif
		exit(EXIT_FAILURE);
	}
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
#ifdef WITH_PAUSE
		if (!nopauseFlag) {
			fprintf(stdout, "Press return key to exit : ");
			fgetc(stdin);
		}
#endif
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	if (ipMagicCheck(&ipHeader)) {
		if (forceFlag) {
			fprintf(stdout, " This might not be a correct R-AXIS File...\n");
		}
		else {
			fprintf(stderr, " This might not be a correct R-AXIS File...\n");
			fprintf(stderr, " Confirm File Format again, and try with \"-f\".\n");
			fclose(fp1);
#ifdef WITH_PAUSE
			if (!nopauseFlag) {
				fprintf(stdout, "Press return key to exit : ");
				fgetc(stdin);
			}
#endif
			exit(EXIT_FAILURE);
		}
	}
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
	   if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
#ifdef WITH_PAUSE
			if (!nopauseFlag) {
				fprintf(stdout, "Press return key to exit : ");
				fgetc(stdin);
			}
#endif
			exit(EXIT_FAILURE);
		}
		ipHeaderDint.dints[0] = ipHeader2VER1.dint1;
		ipHeaderDint.dints[1] = ipHeader2VER1.dint2;
		createDosFlag = 1;
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
#ifdef WITH_PAUSE
			if (!nopauseFlag) {
				fprintf(stdout, "Press return key to exit : ");
				fgetc(stdin);
			}
#endif
			exit(EXIT_FAILURE);
		}
		createDosFlag = 0;
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (createDosFlag) {
			swapIpHeader2VER1(&ipHeader2VER1);
			ipHeaderDint.dint = swapLong(ipHeaderDint.dint);
		}
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Clean Header */
	cleanIpHeader(&ipHeader);
	createDosFlag ? cleanIpHeader2VER1(&ipHeader2VER1) : cleanIpHeader2VER2(&ipHeader2VER2);
	
/* Get ipHeader Information */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	
/* Show Header */
	if (styleMode == LONG_STYLE) {
		fprintf(stdout, "\nIP File Information about %s.\n", file1);
		if (createDosFlag) fprintf(stdout, "This File is probably created with OLD MD-DOS(R) Machine.\n");
		switch (formatMode) {
			case NORMAL_FORMAT:
				fprintf(stdout, " Device Name           : %-.*s\n", (int)sizeof(ipHeader.name), ipHeader.name); /* 10 */
				fprintf(stdout, " Versions              : %-.*s\n", (int)sizeof(ipHeader.vers), ipHeader.vers); /* 10 */
				fprintf(stdout, " Crystal Name          : %-.*s\n", (int)sizeof(ipHeader.crnm), ipHeader.crnm); /* 20 */
				fprintf(stdout, " Crystal System        : %-.*s\n", (int)sizeof(ipHeader.crsy), ipHeader.crsy); /* 12 */
				fprintf(stdout, " Cell Parameter a      : %-8.4f\n", ipHeader.alng);
				fprintf(stdout, " Cell Parameter b      : %-8.4f\n", ipHeader.blng);
				fprintf(stdout, " Cell Parameter c      : %-8.4f\n", ipHeader.clng);
				fprintf(stdout, " Cell Parameter alpha  : %-9.4f\n", ipHeader.alfa);
				fprintf(stdout, " Cell Parameter beta   : %-9.4f\n", ipHeader.beta);
				fprintf(stdout, " Cell Parameter gamma  : %-9.4f\n", ipHeader.gamm);
				fprintf(stdout, " Space Group           : %-.*s\n", (int)sizeof(ipHeader.spgr), ipHeader.spgr); /* 12 */
				fprintf(stdout, " Mosaic Spread         : %-7.3f\n", ipHeader.mosc);
				fprintf(stdout, " Memo                  : %-.*s\n", (int)sizeof(ipHeader.memo), ipHeader.memo); /* 80 */
				fprintf(stdout, " Date                  : %-.*s\n", (int)sizeof(ipHeader.date), ipHeader.date); /* 12 */
				fprintf(stdout, " Operator Name         : %-.*s\n", (int)sizeof(ipHeader.opnm), ipHeader.opnm); /* 20 */
				fprintf(stdout, " X-ray Target          : %-.*s\n", (int)sizeof(ipHeader.trgt), ipHeader.trgt); /* 4 */
				fprintf(stdout, " X-ray Wavelength      : %-8.5f\n", ipHeader.wlng);
				fprintf(stdout, " Monochromator         : %-.*s\n", (int)sizeof(ipHeader.mcro), ipHeader.mcro); /* 20 */
				fprintf(stdout, " Monochromator 2theta  : %-9.4f\n", ipHeader.m2ta);
				fprintf(stdout, " Collimator            : %-.*s\n", (int)sizeof(ipHeader.colm), ipHeader.colm); /* 20 */
				fprintf(stdout, " Filter                : %-.*s\n", (int)sizeof(ipHeader.filt), ipHeader.filt); /* 4 */
				fprintf(stdout, " Camera Length (mm)    : %-7.2f\n", ipHeader.camr);
				fprintf(stdout, " Tube Voltage (kV)     : %-6.1f\n", ipHeader.vltg);
				fprintf(stdout, " Tube Current (mA)     : %-6.1f\n", ipHeader.crnt);
				fprintf(stdout, " X-ray Focus           : %-.*s\n", (int)sizeof(ipHeader.focs), ipHeader.focs); /* 12 */
				fprintf(stdout, " X-ray Optics          : %-.*s\n", (int)sizeof(ipHeader.optc), ipHeader.optc); /* 80 */
				fprintf(stdout, " Detector Shape Code   : %-5ld\n", ipHeader.cyld);
				fprintf(stdout, " Weissenberg Coupling  : %-5.2f\n", ipHeader.weis);
				fprintf(stdout, " Mount Axis            : %-.*s\n", (int)sizeof(ipHeader.mnax), ipHeader.mnax); /* 4 */
				fprintf(stdout, " X-ray Beam Axis       : %-.*s\n", (int)sizeof(ipHeader.bmax), ipHeader.bmax); /* 4 */
				fprintf(stdout, " Phi 0                 : %-7.3f\n", ipHeader.phi0);
				fprintf(stdout, " Phi Start             : %-7.3f\n", ipHeader.phis);
				fprintf(stdout, " Phi End               : %-7.3f\n", ipHeader.phie);
				fprintf(stdout, " Oscillation number    : %-5ld\n", ipHeader.oscn);
				fprintf(stdout, " Exposure Time (min)   : %-9.3f\n", ipHeader.fext);
				fprintf(stdout, " Direct Position X     : %-8.2f\n", ipHeader.drtx);
				fprintf(stdout, " Direct Position Z     : %-8.2f\n", ipHeader.drtz);
				fprintf(stdout, " Omega                 : %-7.3f\n", ipHeader.omga);
				fprintf(stdout, " Chi                   : %-7.3f\n", ipHeader.fkai);
				fprintf(stdout, " Theta                 : %-7.3f\n", ipHeader.thta);
				fprintf(stdout, " Mu                    : %-7.3f\n", ipHeader.mu);
				fprintf(stdout, " Number of Pixel in X  : %-5ld\n", ipHeader.xpxl);
				fprintf(stdout, " Number of Pixel in Z  : %-5ld\n", ipHeader.zpxl);
				fprintf(stdout, " Pixel Size X (mm)     : %-7.5f\n", ipHeader.xsiz);
				fprintf(stdout, " Pixel Sixe Z (mm)     : %-7.5f\n", ipHeader.zsiz);
				fprintf(stdout, " Record Length (bytes) : %-5ld\n", ipHeader.rlng);
				fprintf(stdout, " Number of Records     : %-5ld\n", ipHeader.rnum);
				fprintf(stdout, " Start Line            : %-5ld\n", ipHeader.ipst);
				fprintf(stdout, " IP Number             : %-5ld\n", ipHeader.ipnm);
				fprintf(stdout, " Ratio                 : %-6.2f\n", ipHeader.rato);
				fprintf(stdout, " Fading Function No.1  : %-7.2f\n", ipHeader.ft_1);
				fprintf(stdout, " Fading Function No.2  : %-7.2f\n", ipHeader.ft_2);
				fprintf(stdout, " Host Type             : %-.*s\n", (int)sizeof(ipHeader.host), ipHeader.host); /* 10 */
				fprintf(stdout, " IP Type               : %-.*s\n", (int)sizeof(ipHeader.ip), ipHeader.ip); /* 10 */
				fprintf(stdout, " Image Direction in X  : %-5ld\n", ipHeader.dr_x);
				fprintf(stdout, " Image Direction in Z  : %-5ld\n", ipHeader.dr_z);
				fprintf(stdout, " Image Direction in XZ : %-5ld\n", ipHeader.drxz);
				fprintf(stdout, "---- Optional Header (Over 1024 bytes) ----\n");
				if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
					fprintf(stdout, " \"file\"                : %-.*s\n", (int)sizeof(ipHeader2VER1.file), ipHeader2VER1.file); /* 16 */
					fprintf(stdout, " \"iext\"                : %-10d\n", ipHeader2VER1.iext);
					fprintf(stdout, " \"reso\"                : %-10d\n", ipHeader2VER1.reso);
					fprintf(stdout, " \"save\"                : %-10d\n", ipHeader2VER1.save);
					fprintf(stdout, " \"dint\"                : %-10ld\n", ipHeaderDint.dint);
					fprintf(stdout, " \"byte\"                : %-10d\n", ipHeader2VER1.byte);
				}
				else {
					fprintf(stdout, " \"file\"                : %-.*s\n", (int)sizeof(ipHeader2VER2.file), ipHeader2VER2.file); /* 16 */
					fprintf(stdout, " \"cmnt\"                : %-.*s\n", (int)sizeof(ipHeader2VER2.cmnt), ipHeader2VER2.cmnt); /* 20 */
					fprintf(stdout, " \"smpl\"                : %-.*s\n", (int)sizeof(ipHeader2VER2.smpl), ipHeader2VER2.smpl); /* 20 */
					fprintf(stdout, " \"iext\"                : %-10ld\n", ipHeader2VER2.iext);
					fprintf(stdout, " \"reso\"                : %-10ld\n", ipHeader2VER2.reso);
					fprintf(stdout, " \"save\"                : %-10ld\n", ipHeader2VER2.save);
					fprintf(stdout, " \"dint\"                : %-10ld\n", ipHeader2VER2.dint);
					fprintf(stdout, " \"byte\"                : %-10ld\n", ipHeader2VER2.byte);
					fprintf(stdout, " \"init\"                : %-10ld\n", ipHeader2VER2.init);
					fprintf(stdout, " \"ipus\"                : %-10ld\n", ipHeader2VER2.ipus);
					fprintf(stdout, " \"dexp\"                : %-10ld\n", ipHeader2VER2.dexp);
					fprintf(stdout, " \"expn\"                : %-10ld\n", ipHeader2VER2.expn);
					for (i=0; i<20; i++) fprintf(stdout, " \"posx[%2ld]\"            : %-10ld\n", i, ipHeader2VER2.posx[i]);
					for (i=0; i<20; i++) fprintf(stdout, " \"posy[%2ld]\"            : %-10ld\n", i, ipHeader2VER2.posy[i]);
					fprintf(stdout, " \"xray\"                : %-10ld\n", ipHeader2VER2.xray);
				}
				break;
			case FREE_FORMAT:
				fprintf(stdout, " Device Name           : %-.*s\n", (int)sizeof(ipHeader.name), ipHeader.name); /* 10 */
				fprintf(stdout, " Versions              : %-.*s\n", (int)sizeof(ipHeader.vers), ipHeader.vers); /* 10 */
				fprintf(stdout, " Crystal Name          : %-.*s\n", (int)sizeof(ipHeader.crnm), ipHeader.crnm); /* 20 */
				fprintf(stdout, " Crystal System        : %-.*s\n", (int)sizeof(ipHeader.crsy), ipHeader.crsy); /* 12 */
				fprintf(stdout, " Cell Parameter a      : %-f\n", ipHeader.alng);
				fprintf(stdout, " Cell Parameter b      : %-f\n", ipHeader.blng);
				fprintf(stdout, " Cell Parameter c      : %-f\n", ipHeader.clng);
				fprintf(stdout, " Cell Parameter alpha  : %-f\n", ipHeader.alfa);
				fprintf(stdout, " Cell Parameter beta   : %-f\n", ipHeader.beta);
				fprintf(stdout, " Cell Parameter gamma  : %-f\n", ipHeader.gamm);
				fprintf(stdout, " Space Group           : %-.*s\n", (int)sizeof(ipHeader.spgr), ipHeader.spgr); /* 12 */
				fprintf(stdout, " Mosaic Spread         : %-f\n", ipHeader.mosc);
				fprintf(stdout, " Memo                  : %-.*s\n", (int)sizeof(ipHeader.memo), ipHeader.memo); /* 80 */
				fprintf(stdout, " Date                  : %-.*s\n", (int)sizeof(ipHeader.date), ipHeader.date); /* 12 */
				fprintf(stdout, " Operator Name         : %-.*s\n", (int)sizeof(ipHeader.opnm), ipHeader.opnm); /* 20 */
				fprintf(stdout, " X-ray Target          : %-.*s\n", (int)sizeof(ipHeader.trgt), ipHeader.trgt); /* 4 */
				fprintf(stdout, " X-ray Wavelength      : %-f\n", ipHeader.wlng);
				fprintf(stdout, " Monochromator         : %-.*s\n", (int)sizeof(ipHeader.mcro), ipHeader.mcro); /* 20 */
				fprintf(stdout, " Monochromator 2theta  : %-f\n", ipHeader.m2ta);
				fprintf(stdout, " Collimator            : %-.*s\n", (int)sizeof(ipHeader.colm), ipHeader.colm); /* 20 */
				fprintf(stdout, " Filter                : %-.*s\n", (int)sizeof(ipHeader.filt), ipHeader.filt); /* 4 */
				fprintf(stdout, " Camera Length (mm)    : %-f\n", ipHeader.camr);
				fprintf(stdout, " Tube Voltage (kV)     : %-f\n", ipHeader.vltg);
				fprintf(stdout, " Tube Current (mA)     : %-f\n", ipHeader.crnt);
				fprintf(stdout, " X-ray Focus           : %-.*s\n", (int)sizeof(ipHeader.focs), ipHeader.focs); /* 12 */
				fprintf(stdout, " X-ray Optics          : %-.*s\n", (int)sizeof(ipHeader.optc), ipHeader.optc); /* 80 */
				fprintf(stdout, " Detector Shape Code   : %-ld\n", ipHeader.cyld);
				fprintf(stdout, " Weissenberg Coupling  : %-f\n", ipHeader.weis);
				fprintf(stdout, " Mount Axis            : %-.*s\n", (int)sizeof(ipHeader.mnax), ipHeader.mnax); /* 4 */
				fprintf(stdout, " X-ray Beam Axis       : %-.*s\n", (int)sizeof(ipHeader.bmax), ipHeader.bmax); /* 4 */
				fprintf(stdout, " Phi 0                 : %-f\n", ipHeader.phi0);
				fprintf(stdout, " Phi Start             : %-f\n", ipHeader.phis);
				fprintf(stdout, " Phi End               : %-f\n", ipHeader.phie);
				fprintf(stdout, " Oscillation number    : %-ld\n", ipHeader.oscn);
				fprintf(stdout, " Exposure Time (min)   : %-f\n", ipHeader.fext);
				fprintf(stdout, " Direct Position X     : %-f\n", ipHeader.drtx);
				fprintf(stdout, " Direct Position Z     : %-f\n", ipHeader.drtz);
				fprintf(stdout, " Omega                 : %-f\n", ipHeader.omga);
				fprintf(stdout, " Chi                   : %-f\n", ipHeader.fkai);
				fprintf(stdout, " Theta                 : %-f\n", ipHeader.thta);
				fprintf(stdout, " Mu                    : %-f\n", ipHeader.mu);
				fprintf(stdout, " Number of Pixel in X  : %-ld\n", ipHeader.xpxl);
				fprintf(stdout, " Number of Pixel in Z  : %-ld\n", ipHeader.zpxl);
				fprintf(stdout, " Pixel Size X (mm)     : %-f\n", ipHeader.xsiz);
				fprintf(stdout, " Pixel Sixe Z (mm)     : %-f\n", ipHeader.zsiz);
				fprintf(stdout, " Record Length (bytes) : %-ld\n", ipHeader.rlng);
				fprintf(stdout, " Number of Records     : %-ld\n", ipHeader.rnum);
				fprintf(stdout, " Start Line            : %-ld\n", ipHeader.ipst);
				fprintf(stdout, " IP Number             : %-ld\n", ipHeader.ipnm);
				fprintf(stdout, " Ratio                 : %-f\n", ipHeader.rato);
				fprintf(stdout, " Fading Function No.1  : %-f\n", ipHeader.ft_1);
				fprintf(stdout, " Fading Function No.2  : %-f\n", ipHeader.ft_2);
				fprintf(stdout, " Host Type             : %-.*s\n", (int)sizeof(ipHeader.host), ipHeader.host); /* 10 */
				fprintf(stdout, " IP Type               : %-.*s\n", (int)sizeof(ipHeader.ip), ipHeader.ip); /* 10 */
				fprintf(stdout, " Image Direction in X  : %-ld\n", ipHeader.dr_x);
				fprintf(stdout, " Image Direction in Z  : %-ld\n", ipHeader.dr_z);
				fprintf(stdout, " Image Direction in XZ : %-ld\n", ipHeader.drxz);
				fprintf(stdout, "---- Optional Header (Over 1024 bytes) ----\n");
				if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
					fprintf(stdout, " \"file\"                : %-.*s\n", (int)sizeof(ipHeader2VER1.file), ipHeader2VER1.file); /* 16 */
					fprintf(stdout, " \"iext\"                : %-d\n", ipHeader2VER1.iext);
					fprintf(stdout, " \"reso\"                : %-d\n", ipHeader2VER1.reso);
					fprintf(stdout, " \"save\"                : %-d\n", ipHeader2VER1.save);
					fprintf(stdout, " \"dint\"                : %-ld\n", ipHeaderDint.dint);
					fprintf(stdout, " \"byte\"                : %-d\n", ipHeader2VER1.byte);
				}
				else {
					fprintf(stdout, " \"file\"                : %-.*s\n", (int)sizeof(ipHeader2VER2.file), ipHeader2VER2.file); /* 16 */
					fprintf(stdout, " \"cmnt\"                : %-.*s\n", (int)sizeof(ipHeader2VER2.cmnt), ipHeader2VER2.cmnt); /* 20 */
					fprintf(stdout, " \"smpl\"                : %-.*s\n", (int)sizeof(ipHeader2VER2.smpl), ipHeader2VER2.smpl); /* 20 */
					fprintf(stdout, " \"iext\"                : %-ld\n", ipHeader2VER2.iext);
					fprintf(stdout, " \"reso\"                : %-ld\n", ipHeader2VER2.reso);
					fprintf(stdout, " \"save\"                : %-ld\n", ipHeader2VER2.save);
					fprintf(stdout, " \"dint\"                : %-ld\n", ipHeader2VER2.dint);
					fprintf(stdout, " \"byte\"                : %-ld\n", ipHeader2VER2.byte);
					fprintf(stdout, " \"init\"                : %-ld\n", ipHeader2VER2.init);
					fprintf(stdout, " \"ipus\"                : %-ld\n", ipHeader2VER2.ipus);
					fprintf(stdout, " \"dexp\"                : %-ld\n", ipHeader2VER2.dexp);
					fprintf(stdout, " \"expn\"                : %-ld\n", ipHeader2VER2.expn);
					for (i=0; i<20; i++) fprintf(stdout, " \"posx[%2ld]\"            : %-ld\n", i, ipHeader2VER2.posx[i]);
					for (i=0; i<20; i++) fprintf(stdout, " \"posy[%2ld]\"            : %-ld\n", i, ipHeader2VER2.posy[i]);
					fprintf(stdout, " \"xray\"                : %-ld\n", ipHeader2VER2.xray);
				}
				break;
			case EXP_FORMAT:
				fprintf(stdout, " Device Name           : %-.*s\n", (int)sizeof(ipHeader.name), ipHeader.name); /* 10 */
				fprintf(stdout, " Versions              : %-.*s\n", (int)sizeof(ipHeader.vers), ipHeader.vers); /* 10 */
				fprintf(stdout, " Crystal Name          : %-.*s\n", (int)sizeof(ipHeader.crnm), ipHeader.crnm); /* 20 */
				fprintf(stdout, " Crystal System        : %-.*s\n", (int)sizeof(ipHeader.crsy), ipHeader.crsy); /* 12 */
				fprintf(stdout, " Cell Parameter a      : %-E\n", ipHeader.alng);
				fprintf(stdout, " Cell Parameter b      : %-E\n", ipHeader.blng);
				fprintf(stdout, " Cell Parameter c      : %-E\n", ipHeader.clng);
				fprintf(stdout, " Cell Parameter alpha  : %-E\n", ipHeader.alfa);
				fprintf(stdout, " Cell Parameter beta   : %-E\n", ipHeader.beta);
				fprintf(stdout, " Cell Parameter gamma  : %-E\n", ipHeader.gamm);
				fprintf(stdout, " Space Group           : %-.*s\n", (int)sizeof(ipHeader.spgr), ipHeader.spgr); /* 12 */
				fprintf(stdout, " Mosaic Spread         : %-E\n", ipHeader.mosc);
				fprintf(stdout, " Memo                  : %-.*s\n", (int)sizeof(ipHeader.memo), ipHeader.memo); /* 80 */
				fprintf(stdout, " Date                  : %-.*s\n", (int)sizeof(ipHeader.date), ipHeader.date); /* 12 */
				fprintf(stdout, " Operator Name         : %-.*s\n", (int)sizeof(ipHeader.opnm), ipHeader.opnm); /* 20 */
				fprintf(stdout, " X-ray Target          : %-.*s\n", (int)sizeof(ipHeader.trgt), ipHeader.trgt); /* 4 */
				fprintf(stdout, " X-ray Wavelength      : %-E\n", ipHeader.wlng);
				fprintf(stdout, " Monochromator         : %-.*s\n", (int)sizeof(ipHeader.mcro), ipHeader.mcro); /* 20 */
				fprintf(stdout, " Monochromator 2theta  : %-E\n", ipHeader.m2ta);
				fprintf(stdout, " Collimator            : %-.*s\n", (int)sizeof(ipHeader.colm), ipHeader.colm); /* 20 */
				fprintf(stdout, " Filter                : %-.*s\n", (int)sizeof(ipHeader.filt), ipHeader.filt); /* 4 */
				fprintf(stdout, " Camera Length (mm)    : %-E\n", ipHeader.camr);
				fprintf(stdout, " Tube Voltage (kV)     : %-E\n", ipHeader.vltg);
				fprintf(stdout, " Tube Current (mA)     : %-E\n", ipHeader.crnt);
				fprintf(stdout, " X-ray Focus           : %-.*s\n", (int)sizeof(ipHeader.focs), ipHeader.focs); /* 12 */
				fprintf(stdout, " X-ray Optics          : %-.*s\n", (int)sizeof(ipHeader.optc), ipHeader.optc); /* 80 */
				fprintf(stdout, " Detector Shape Code   : %-ld\n", ipHeader.cyld);
				fprintf(stdout, " Weissenberg Coupling  : %-E\n", ipHeader.weis);
				fprintf(stdout, " Mount Axis            : %-.*s\n", (int)sizeof(ipHeader.mnax), ipHeader.mnax); /* 4 */
				fprintf(stdout, " X-ray Beam Axis       : %-.*s\n", (int)sizeof(ipHeader.bmax), ipHeader.bmax); /* 4 */
				fprintf(stdout, " Phi 0                 : %-E\n", ipHeader.phi0);
				fprintf(stdout, " Phi Start             : %-E\n", ipHeader.phis);
				fprintf(stdout, " Phi End               : %-E\n", ipHeader.phie);
				fprintf(stdout, " Oscillation number    : %-ld\n", ipHeader.oscn);
				fprintf(stdout, " Exposure Time (min)   : %-E\n", ipHeader.fext);
				fprintf(stdout, " Direct Position X     : %-E\n", ipHeader.drtx);
				fprintf(stdout, " Direct Position Z     : %-E\n", ipHeader.drtz);
				fprintf(stdout, " Omega                 : %-E\n", ipHeader.omga);
				fprintf(stdout, " Chi                   : %-E\n", ipHeader.fkai);
				fprintf(stdout, " Theta                 : %-E\n", ipHeader.thta);
				fprintf(stdout, " Mu                    : %-E\n", ipHeader.mu);
				fprintf(stdout, " Number of Pixel in X  : %-ld\n", ipHeader.xpxl);
				fprintf(stdout, " Number of Pixel in Z  : %-ld\n", ipHeader.zpxl);
				fprintf(stdout, " Pixel Size X (mm)     : %-E\n", ipHeader.xsiz);
				fprintf(stdout, " Pixel Sixe Z (mm)     : %-E\n", ipHeader.zsiz);
				fprintf(stdout, " Record Length (bytes) : %-ld\n", ipHeader.rlng);
				fprintf(stdout, " Number of Records     : %-ld\n", ipHeader.rnum);
				fprintf(stdout, " Start Line            : %-ld\n", ipHeader.ipst);
				fprintf(stdout, " IP Number             : %-ld\n", ipHeader.ipnm);
				fprintf(stdout, " Ratio                 : %-E\n", ipHeader.rato);
				fprintf(stdout, " Fading Function No.1  : %-E\n", ipHeader.ft_1);
				fprintf(stdout, " Fading Function No.2  : %-E\n", ipHeader.ft_2);
				fprintf(stdout, " Host Type             : %-.*s\n", (int)sizeof(ipHeader.host), ipHeader.host); /* 10 */
				fprintf(stdout, " IP Type               : %-.*s\n", (int)sizeof(ipHeader.ip), ipHeader.ip); /* 10 */
				fprintf(stdout, " Image Direction in X  : %-ld\n", ipHeader.dr_x);
				fprintf(stdout, " Image Direction in Z  : %-ld\n", ipHeader.dr_z);
				fprintf(stdout, " Image Direction in XZ : %-ld\n", ipHeader.drxz);
				fprintf(stdout, "---- Optional Header (Over 1024 bytes) ----\n");
				if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
					fprintf(stdout, " \"file\"                : %-.*s\n", (int)sizeof(ipHeader2VER1.file), ipHeader2VER1.file); /* 16 */
					fprintf(stdout, " \"iext\"                : %-d\n", ipHeader2VER1.iext);
					fprintf(stdout, " \"reso\"                : %-d\n", ipHeader2VER1.reso);
					fprintf(stdout, " \"save\"                : %-d\n", ipHeader2VER1.save);
					fprintf(stdout, " \"dint\"                : %-ld\n", ipHeaderDint.dint);
					fprintf(stdout, " \"byte\"                : %-d\n", ipHeader2VER1.byte);
				}
				else {
					fprintf(stdout, " \"file\"                : %-.*s\n", (int)sizeof(ipHeader2VER2.file), ipHeader2VER2.file); /* 16 */
					fprintf(stdout, " \"cmnt\"                : %-.*s\n", (int)sizeof(ipHeader2VER2.cmnt), ipHeader2VER2.cmnt); /* 20 */
					fprintf(stdout, " \"smpl\"                : %-.*s\n", (int)sizeof(ipHeader2VER2.smpl), ipHeader2VER2.smpl); /* 20 */
					fprintf(stdout, " \"iext\"                : %-ld\n", ipHeader2VER2.iext);
					fprintf(stdout, " \"reso\"                : %-ld\n", ipHeader2VER2.reso);
					fprintf(stdout, " \"save\"                : %-ld\n", ipHeader2VER2.save);
					fprintf(stdout, " \"dint\"                : %-ld\n", ipHeader2VER2.dint);
					fprintf(stdout, " \"byte\"                : %-ld\n", ipHeader2VER2.byte);
					fprintf(stdout, " \"init\"                : %-ld\n", ipHeader2VER2.init);
					fprintf(stdout, " \"ipus\"                : %-ld\n", ipHeader2VER2.ipus);
					fprintf(stdout, " \"dexp\"                : %-ld\n", ipHeader2VER2.dexp);
					fprintf(stdout, " \"expn\"                : %-ld\n", ipHeader2VER2.expn);
					for (i=0; i<20; i++) fprintf(stdout, " \"posx[%2ld]\"            : %-ld\n", i, ipHeader2VER2.posx[i]);
					for (i=0; i<20; i++) fprintf(stdout, " \"posy[%2ld]\"            : %-ld\n", i, ipHeader2VER2.posy[i]);
					fprintf(stdout, " \"xray\"                : %-ld\n", ipHeader2VER2.xray);
				}
				break;
			default:
				break;
				/* NOT REACHED */
		}
	}
	else if (styleMode == ONE_STYLE) {
#ifdef WITH_PAUSE
		nopauseFlag = 1;
#endif
		switch (formatMode) {
			case ONE_FORMAT:
				if (item == 10)      fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.name), ipHeader.name); /* 10 */
				else if (item == 11) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.vers), ipHeader.vers); /* 10 */
				else if (item == 12) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.crnm), ipHeader.crnm); /* 20 */
				else if (item == 13) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.crsy), ipHeader.crsy); /* 12 */
				else if (item == 14) fprintf(stdout, "%-f\n", ipHeader.alng);
				else if (item == 15) fprintf(stdout, "%-f\n", ipHeader.blng);
				else if (item == 16) fprintf(stdout, "%-f\n", ipHeader.clng);
				else if (item == 17) fprintf(stdout, "%-f\n", ipHeader.alfa);
				else if (item == 18) fprintf(stdout, "%-f\n", ipHeader.beta);
				else if (item == 19) fprintf(stdout, "%-f\n", ipHeader.gamm);
				else if (item == 20) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.spgr), ipHeader.spgr); /* 12 */
				else if (item == 21) fprintf(stdout, "%-f\n", ipHeader.mosc);
				else if (item == 22) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.memo), ipHeader.memo); /* 80 */
				else if (item == 23) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.date), ipHeader.date); /* 12 */
				else if (item == 24) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.opnm), ipHeader.opnm); /* 20 */
				else if (item == 25) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.trgt), ipHeader.trgt); /* 4 */
				else if (item == 26) fprintf(stdout, "%-f\n", ipHeader.wlng);
				else if (item == 27) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.mcro), ipHeader.mcro); /* 20 */
				else if (item == 28) fprintf(stdout, "%-f\n", ipHeader.m2ta);
				else if (item == 29) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.colm), ipHeader.colm); /* 20 */
				else if (item == 30) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.filt), ipHeader.filt); /* 4 */
				else if (item == 31) fprintf(stdout, "%-f\n", ipHeader.camr);
				else if (item == 32) fprintf(stdout, "%-f\n", ipHeader.vltg);
				else if (item == 33) fprintf(stdout, "%-f\n", ipHeader.crnt);
				else if (item == 34) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.focs), ipHeader.focs); /* 12 */
				else if (item == 35) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.optc), ipHeader.optc); /* 80 */
				else if (item == 36) fprintf(stdout, "%-ld\n", ipHeader.cyld);
				else if (item == 37) fprintf(stdout, "%-f\n", ipHeader.weis);
				else if (item == 38) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.mnax), ipHeader.mnax); /* 4 */
				else if (item == 39) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.bmax), ipHeader.bmax); /* 4 */
				else if (item == 40) fprintf(stdout, "%-f\n", ipHeader.phi0);
				else if (item == 41) fprintf(stdout, "%-f\n", ipHeader.phis);
				else if (item == 42) fprintf(stdout, "%-f\n", ipHeader.phie);
				else if (item == 43) fprintf(stdout, "%-ld\n", ipHeader.oscn);
				else if (item == 44) fprintf(stdout, "%-f\n", ipHeader.fext);
				else if (item == 45) fprintf(stdout, "%-f\n", ipHeader.drtx);
				else if (item == 46) fprintf(stdout, "%-f\n", ipHeader.drtz);
				else if (item == 47) fprintf(stdout, "%-f\n", ipHeader.omga);
				else if (item == 48) fprintf(stdout, "%-f\n", ipHeader.fkai);
				else if (item == 49) fprintf(stdout, "%-f\n", ipHeader.thta);
				else if (item == 50) fprintf(stdout, "%-f\n", ipHeader.mu);
				else if (item == 51) fprintf(stdout, "%-ld\n", ipHeader.xpxl);
				else if (item == 52) fprintf(stdout, "%-ld\n", ipHeader.zpxl);
				else if (item == 53) fprintf(stdout, "%-f\n", ipHeader.xsiz);
				else if (item == 54) fprintf(stdout, "%-f\n", ipHeader.zsiz);
				else if (item == 55) fprintf(stdout, "%-ld\n", ipHeader.rlng);
				else if (item == 56) fprintf(stdout, "%-ld\n", ipHeader.rnum);
				else if (item == 57) fprintf(stdout, "%-ld\n", ipHeader.ipst);
				else if (item == 58) fprintf(stdout, "%-ld\n", ipHeader.ipnm);
				else if (item == 59) fprintf(stdout, "%-f\n", ipHeader.rato);
				else if (item == 60) fprintf(stdout, "%-f\n", ipHeader.ft_1);
				else if (item == 61) fprintf(stdout, "%-f\n", ipHeader.ft_2);
				else if (item == 62) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.host), ipHeader.host); /* 10 */
				else if (item == 63) fprintf(stdout, "%-.*s\n", (int)sizeof(ipHeader.ip), ipHeader.ip); /* 10 */
				else if (item == 64) fprintf(stdout, "%-ld\n", ipHeader.dr_x);
				else if (item == 65) fprintf(stdout, "%-ld\n", ipHeader.dr_z);
				else if (item == 66) fprintf(stdout, "%-ld\n", ipHeader.drxz);
				break;
			default:
				break;
				/* NOT REACHED */
		}
	}
	else {
		fprintf(stdout, "\nIP File Information about %s.\n", file1);
		if (createDosFlag) fprintf(stdout, "This File is probably created with OLD MD-DOS(R) Machine.\n");
		switch (formatMode) {
			case NORMAL_FORMAT:
				fprintf(stdout, " Device Name, Versions : %-.*s ver.%-.*s\n", (int)sizeof(ipHeader.name), ipHeader.name, (int)sizeof(ipHeader.vers), ipHeader.vers);
				fprintf(stdout, " Crystal Name   : %-.*s\n", (int)sizeof(ipHeader.crnm), ipHeader.crnm);
				fprintf(stdout, " Crystal System : %-.*s\n", (int)sizeof(ipHeader.crsy), ipHeader.crsy);
				fprintf(stdout, " Date : %-.*s / Operator : %-.*s\n", (int)sizeof(ipHeader.date), ipHeader.date, (int)sizeof(ipHeader.opnm), ipHeader.opnm);
				fprintf(stdout, " X-ray Target, Wavelength : %-.*s, %-8.5f\n", (int)sizeof(ipHeader.trgt), ipHeader.trgt, ipHeader.wlng);
				fprintf(stdout, " Monochromator and its 2theta : %-.*s, %9.4f degrees\n", (int)sizeof(ipHeader.mcro), ipHeader.mcro, ipHeader.m2ta);
				fprintf(stdout, " Collimator : %-.*s / Filter :  %-.*s\n", (int)sizeof(ipHeader.colm), ipHeader.colm, (int)sizeof(ipHeader.filt), ipHeader.filt);
				fprintf(stdout, " Camera Length          : %7.2f mm\n", ipHeader.camr);
				fprintf(stdout, " Tube Voltage, Current  : %6.1f kV, %6.1f mA\n", ipHeader.vltg, ipHeader.crnt);
				fprintf(stdout, " Exposure Time          : %9.3f min.\n", ipHeader.fext);
				fprintf(stdout, " Direct Position X, Z    : ( %8.2f , %8.2f )\n", ipHeader.drtx, ipHeader.drtz);
				fprintf(stdout, " Number of Pixel in X, Z : %5ld , %5ld\n", ipHeader.xpxl, ipHeader.zpxl);
				fprintf(stdout, " Pixel Size X, Z         : %7.5f mm, %7.5f mm\n", ipHeader.xsiz, ipHeader.zsiz);
				fprintf(stdout, " Record Length          : %5ld bytes\n", ipHeader.rlng);
				fprintf(stdout, " Number of Records      : %5ld\n", ipHeader.rnum);
				fprintf(stdout, " Ratio                  : %-6.2f\n", ipHeader.rato);
				fprintf(stdout, " Host Type : %-.*s / IP Type : %-.*s / Camera Type Code : %4ld\n", (int)sizeof(ipHeader.host), ipHeader.host, (int)sizeof(ipHeader.ip), ipHeader.ip, ipHeader.cyld);
				fprintf(stdout, " Memo : %-.*s\n", (int)sizeof(ipHeader.memo), ipHeader.memo);
				break;
			case FREE_FORMAT:
				fprintf(stdout, " Device Name, Versions : %-.*s ver.%-.*s\n", (int)sizeof(ipHeader.name), ipHeader.name, (int)sizeof(ipHeader.vers), ipHeader.vers);
				fprintf(stdout, " Crystal Name   : %-.*s\n", (int)sizeof(ipHeader.crnm), ipHeader.crnm);
				fprintf(stdout, " Crystal System : %-.*s\n", (int)sizeof(ipHeader.crsy), ipHeader.crsy);
				fprintf(stdout, " Date : %-.*s / Operator : %-.*s\n", (int)sizeof(ipHeader.date), ipHeader.date, (int)sizeof(ipHeader.opnm), ipHeader.opnm);
				fprintf(stdout, " X-ray Target, Wavelength : %-.*s, %-f\n", (int)sizeof(ipHeader.trgt), ipHeader.trgt, ipHeader.wlng);
				fprintf(stdout, " Monochromator and its 2theta : %-.*s, %f degrees\n", (int)sizeof(ipHeader.mcro), ipHeader.mcro, ipHeader.m2ta);
				fprintf(stdout, " Collimator : %-.*s / Filter :  %-.*s\n", (int)sizeof(ipHeader.colm), ipHeader.colm, (int)sizeof(ipHeader.filt), ipHeader.filt);
				fprintf(stdout, " Camera Length          : %f mm\n", ipHeader.camr);
				fprintf(stdout, " Tube Voltage, Current  : %f kV, %f mA\n", ipHeader.vltg, ipHeader.crnt);
				fprintf(stdout, " Exposure Time          : %f min.\n", ipHeader.fext);
				fprintf(stdout, " Direct Position X, Z    : ( %f , %f )\n", ipHeader.drtx, ipHeader.drtz);
				fprintf(stdout, " Number of Pixel in X, Z : %ld , %ld\n", ipHeader.xpxl, ipHeader.zpxl);
				fprintf(stdout, " Pixel Size X, Z         : %f mm, %f mm\n", ipHeader.xsiz, ipHeader.zsiz);
				fprintf(stdout, " Record Length          : %ld bytes\n", ipHeader.rlng);
				fprintf(stdout, " Number of Records      : %ld\n", ipHeader.rnum);
				fprintf(stdout, " Ratio                  : %-f\n", ipHeader.rato);
				fprintf(stdout, " Host Type : %-.*s / IP Type : %-.*s / Camera Type Code : %ld\n", (int)sizeof(ipHeader.host), ipHeader.host, (int)sizeof(ipHeader.ip), ipHeader.ip, ipHeader.cyld);
				fprintf(stdout, " Memo : %-.*s\n", (int)sizeof(ipHeader.memo), ipHeader.memo);
				break;
			case EXP_FORMAT:
				fprintf(stdout, " Device Name, Versions : %-.*s ver.%-.*s\n", (int)sizeof(ipHeader.name), ipHeader.name, (int)sizeof(ipHeader.vers), ipHeader.vers);
				fprintf(stdout, " Crystal Name   : %-.*s\n", (int)sizeof(ipHeader.crnm), ipHeader.crnm);
				fprintf(stdout, " Crystal System : %-.*s\n", (int)sizeof(ipHeader.crsy), ipHeader.crsy);
				fprintf(stdout, " Date : %-.*s / Operator : %-.*s\n", (int)sizeof(ipHeader.date), ipHeader.date, (int)sizeof(ipHeader.opnm), ipHeader.opnm);
				fprintf(stdout, " X-ray Target, Wavelength : %-.*s, %-E\n", (int)sizeof(ipHeader.trgt), ipHeader.trgt, ipHeader.wlng);
				fprintf(stdout, " Monochromator and its 2theta : %-.*s, %E degrees\n", (int)sizeof(ipHeader.mcro), ipHeader.mcro, ipHeader.m2ta);
				fprintf(stdout, " Collimator : %-.*s / Filter :  %-.*s\n", (int)sizeof(ipHeader.colm), ipHeader.colm, (int)sizeof(ipHeader.filt), ipHeader.filt);
				fprintf(stdout, " Camera Length          : %E mm\n", ipHeader.camr);
				fprintf(stdout, " Tube Voltage, Current  : %E kV, %E mA\n", ipHeader.vltg, ipHeader.crnt);
				fprintf(stdout, " Exposure Time          : %E min.\n", ipHeader.fext);
				fprintf(stdout, " Direct Position X, Z    : ( %E , %E )\n", ipHeader.drtx, ipHeader.drtz);
				fprintf(stdout, " Number of Pixel in X, Z : %ld , %ld\n", ipHeader.xpxl, ipHeader.zpxl);
				fprintf(stdout, " Pixel Size X, Z         : %E mm, %E mm\n", ipHeader.xsiz, ipHeader.zsiz);
				fprintf(stdout, " Record Length          : %ld bytes\n", ipHeader.rlng);
				fprintf(stdout, " Number of Records      : %ld\n", ipHeader.rnum);
				fprintf(stdout, " Ratio                  : %-E\n", ipHeader.rato);
				fprintf(stdout, " Host Type : %-.*s / IP Type : %-.*s / Camera Type Code : %ld\n", (int)sizeof(ipHeader.host), ipHeader.host, (int)sizeof(ipHeader.ip), ipHeader.ip, ipHeader.cyld);
				fprintf(stdout, " Memo : %-.*s\n", (int)sizeof(ipHeader.memo), ipHeader.memo);
				break;
			default:
				break;
				/* NOT REACHED */
		}
		if (XOREXT(cpuIntelFlag, ipDataSwapFlag)) fprintf(stdout, "Caution : Data is stored with Little Endian.\n");
	}
	fclose(fp1);
#ifdef WITH_PAUSE
	if (!nopauseFlag) {
		fprintf(stdout, "Press return key to exit : ");
		fgetc(stdin);
	}
#endif
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	if (!verboseFlag) return (0);
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s -h\n", ProgName);
#ifdef WITH_PAUSE
	fprintf(fp0, "        %s [-q] [-a] [-e|-l] [-f] [-p] file\n", ProgName);
#else
	fprintf(fp0, "        %s [-q] [-a] [-e|-l] [-f] file\n", ProgName);
#endif
	fprintf(fp0, "        %s [-q] -n:VALUE [-f] file\n", ProgName);
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Shows R-AXIS File informations stored in the File.\n");
	fprintf(fp0, "   -q ..... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -a ..... Show All Members of Header Part of the File.\n");
	fprintf(fp0, "            Without \"-a\", show only a part of them.\n");
	fprintf(fp0, "   -l ..... Not formatted output.\n");
	fprintf(fp0, "   -e ..... Scientific-style output about floating-point values.\n");
	fprintf(fp0, "   -f ..... Try to show File informations if file might be broken.\n");
	fprintf(fp0, "            This might confuse your console output...\n");
	fprintf(fp0, "   -aa .... Same as \"-a -l\". Remain for backward compatibility.\n");
#ifdef WITH_PAUSE
	fprintf(fp0, "   -p ..... Do not stop program after showing information.\n");
	fprintf(fp0, "            If absent, pause before finish, and it is usefull\n");
	fprintf(fp0, "            when use as Windows Shotcut for Drug & Drop of R-AXIS Image File.\n");
#endif
	fprintf(fp0, "   -n:VALUE\n");
	fprintf(fp0, "        ... Show only one value of File Information pointed by the\n");
	fprintf(fp0, "            following \"VALUE\". See help of \"ipedit\".\n");
	fprintf(fp0, "   -h ..... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   file ... Filename of R-AXIS Image File. This must be the last argument.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	long int *arg1;
#else
char splitArg(const char *argV, char *arg0, long int *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atol(chPtr1+1);
	return (2);
}
