/* Program to generate IP data file */
#define ProgName "ipitex2rax"
#define ProgInfo "Convert ITEX Image File to R-AXIS Image File"
/*                                                         5/24/2000  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

/* 180-deg. rotation of image with "-hp" is now not carried out.
   To do it, use "-rt" option together.             v1.08, 12/24/2003 */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.13_11/5/2010"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <time.h>
#ifdef _HASUTIME
#	include <sys/stat.h>
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"
#include "itexheader.h"
#include "libitex.h"

#ifdef _HASUTIME
#	define MAXARG 13
#else
#	define MAXARG 12
#endif
#define MINARG 2
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define DEFAULT_SIZE 0.05
#define DEFAULT_LENG 100.0
#define MAXCHARBUF 4096

#ifdef _OLDHP
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr1;
	long int i, j, ipHeaderSize, ipHeaderSize2VER2, itexHeaderSize,
	     depthNum, depth, recordLengthCreate, recordLength, recordLengthBW,
	     ipHeaderSizeMin, readByte, flushWidth0, flushWidth, cur, depthNumP, widthP;
	unsigned long int maxInt;
	char file1[FNAMELEN], file2[FNAMELEN], cpuIntelFlag, reverseIntFlag,
	     confirmBuffer[8], inputBuffer[20], argNum, arg0[4],
	     verboseFlag, shiftBit, xsizDoneFlag, zsizDoneFlag, magFlag,
	     removeFlag, setTimeFlag, camrDoneFlag, hipicFlag, *charBuf,
	     *charPtr, *chptr, saveTimeFlag, rotateFlag, mirrorFlag, direcFlag;
	unsigned char *datPtrC;
	short int *datPtrS;
	long int *datPtrL;
	double rato, arg1, exposure, mag, intensityDbl;
	struct ipHead ipHeader;
	struct itexHead itexHeader;
	time_t timeNow;
	FILE *fp1, *fp2;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	itexHeaderSize = SIZE_OF_ITEXHEADER;
	xsizDoneFlag = zsizDoneFlag = reverseIntFlag = rotateFlag
	 = removeFlag = setTimeFlag = camrDoneFlag = hipicFlag = saveTimeFlag
	 = mirrorFlag = direcFlag = magFlag = 0;
	verboseFlag = 1;
	readByte = 0;
	flushWidth0 = 0;
	flushWidth = 0;
	rato = 8.0;
	mag = 1.0;
	shiftBit = 3;
	maxInt = ((long)(0x7fff)) << shiftBit;
	ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
	exposure = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckItex() == 1) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of ITEX Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Argument */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&itexHeader, itexHeaderSize, 1, fp1);
	readByte += itexHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* ITEX File Format Check */
	if (itexMagicCheck(&itexHeader)) {
		fprintf(stderr, " Magic is incorrect :\"%s\". This File is not ITEX File. Exit.\n", itexHeader.ident);
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (!cpuIntelFlag) swapItexHeader(&itexHeader);
	if ((itexHeader.commentLength < 0) || (itexHeader.width > MAXWIDTH)
	 || (itexHeader.width < 1) || (itexHeader.height > MAXHEIGHT)
	 || (itexHeader.height < 1)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		if (verboseFlag >= 2) fprintf(stderr, " commentLength : %d , width : %d , height : %d ,\n", itexHeader.commentLength, itexHeader.width, itexHeader.height);
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (itexHeader.fType == 0) {	/* 8bits, unsighed */
		depth = 8;
		depthNum = 256;
	}
	else if (itexHeader.fType == 2) {	/* 16bits, sighed */
		depth = 16;
		depthNum = 0x8000;
	}
	else if (itexHeader.fType == 3) {	/* 32bits, sighed */
		depth = 32;
		depthNum = 0x80000000;
	}
	else if (itexHeader.fType == 1) {
		fprintf(stderr, " This File Format is Not Supported.\n");
		if (verboseFlag >= 2) fprintf(stderr, " fType : %d .\n", itexHeader.fType);
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else {
		fprintf(stderr, " File Format is Incorrect.\n");
		if (verboseFlag >= 2) fprintf(stderr, " fType : %d .\n", itexHeader.fType);
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* Define ipHeader and Initialize */
	memset(&ipHeader, 0, ipHeaderSize);
	strcpy(ipHeader.name, "R-AXIS-DS"); /* Name of the Device */
	strcpy(ipHeader.vers, "1.0-2.0"); /* Version Number */
	strcpy(ipHeader.crnm, "N/A"); /* Crystal Name */
	strcpy(ipHeader.crsy, "cubic"); /* Crystal System */
	ipHeader.alng = 10.0; /* a-axis length in A */
	ipHeader.blng = 10.0; /* b-axis length in A */
	ipHeader.clng = 10.0; /* c-axis length in A */
	ipHeader.alfa = 90.0; /* angle alpha */
	ipHeader.beta = 90.0; /* angle beta */
	ipHeader.gamm = 90.0; /* angle gamma */
	strcpy(ipHeader.spgr, "P23"); /* Space Group */
	ipHeader.mosc = 0.3; /* Mozaic Spread */
	strcpy(ipHeader.opnm, "rigaku"); /* Operator Name */
	strcpy(ipHeader.trgt, "Cu"); /* X-ray Target */
	ipHeader.wlng = 1.5418; /* Wave Length of the x-ray */
	strcpy(ipHeader.mcro, "None"); /* Monochromator Material */
	ipHeader.m2ta = 0.0; /* Angle of the monochromator in 2-theta */
	strcpy(ipHeader.colm, "0.05-0.10"); /* Collimator */
	strcpy(ipHeader.filt, "Ni"); /* Kb filter */
	ipHeader.vltg = 40.0; /* X-ray tube voltage */
	ipHeader.crnt = 100.0; /* X-ray tube current */
	strcpy(ipHeader.focs, "0.3x0.3"); /* X-ray focus */
	strcpy(ipHeader.optc, "Ni-filter"); /* X-ray optics */
	ipHeader.cyld = 0; /* Camera type */
	ipHeader.weis = 0.0; /* weisenberg oscillation */
	strcpy(ipHeader.mnax, "-b*"); /* Mount Axis */
	strcpy(ipHeader.bmax, "-a"); /* Beam Axis */
	ipHeader.phi0 = 0.0; /* phi0 */
	ipHeader.phis = 0.0; /* phi start */
	ipHeader.phie = 0.0; /* phi end */
	ipHeader.oscn = 0; /* Frequency of Oscillation */
	ipHeader.fext = 20.0; /* Exposure Time in min. */
	ipHeader.omga = 0.0; /* omega in theta */
	ipHeader.fkai = 0.0; /* xai */
	ipHeader.thta = 0.000; /* 2-theta */
	ipHeader.mu = 0.000; /* mu */
	ipHeader.ipst = 0; /* Line of start reading */
	ipHeader.ipnm = 1; /* Number of IP */
	ipHeader.rato = rato; /* Ratio of IP in High/Low */
	ipHeader.ft_1 = 0.0; /* fading time 1 */
	ipHeader.ft_2 = 0.0; /* fading time 2 */
	strcpy(ipHeader.host, "IRIS"); /* Type of Host Computer */
	strcpy(ipHeader.ip, "HR-3"); /* Type of IP */
	ipHeader.dr_x = 0; /* Data direction in horizontal */
	ipHeader.dr_z = 0; /* Data direction in vertical */
	ipHeader.drxz = 0; /* Data direction face/back */
	timeNow = time(NULL);
	strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&timeNow));
	strcat(ipHeader.memo, ProgName);
	strcat(ipHeader.memo, "/");
	strcat(ipHeader.memo, ipHeader.date);
#ifdef _HASUTIME
	strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&utime1.modtime));
#endif
	
/* Set ipHeader Information */
	ipHeader.xsiz = ipHeader.zsiz = DEFAULT_SIZE;
	ipHeader.camr = DEFAULT_LENG;
	ipHeader.xpxl = itexHeader.width;
	ipHeader.zpxl = itexHeader.height;
	recordLength = itexHeader.width*(depth/8);
	
/* Conversion Parameter */
	if ((verboseFlag >= 1) && (itexHeader.offsetX*itexHeader.offsetY != 0)) fprintf(stdout, " Offset Value in original file is ignored.\n");
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %d x %d ) pixels, each pixel has %ld bits,\n", itexHeader.width, itexHeader.height, depth);
		fprintf(stdout, "  total %ld bytes.\n", recordLength*itexHeader.height+itexHeader.commentLength+itexHeaderSize);
	}
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Reverse intensity ? (for negative-film) [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') reverseIntFlag = 1;
		do {
			fprintf(stdout, "Enter PixelSize in X-direction [mm] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.xsiz = atof(inputBuffer);
			if (ipHeader.xsiz <= 0.0) continue;
			xsizDoneFlag = 1;
		} while (!xsizDoneFlag);
		do {
			fprintf(stdout, "Enter PixelSize in Z-direction [mm] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.zsiz = atof(inputBuffer);
			if (ipHeader.zsiz <= 0.0) continue;
			zsizDoneFlag = 1;
		} while (!zsizDoneFlag);
		do {
			fprintf(stdout, "Enter Camera Length [mm] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.camr = atof(inputBuffer);
			if (ipHeader.camr <= 0.0) continue;
			camrDoneFlag = 1;
		} while (!camrDoneFlag);
		do {
			fprintf(stdout, "Use \"HiPic\" File Mode Header ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			hipicFlag = 1;
			do {
				fprintf(stdout, " Do you want to set acquisition \"Time\" information as\n");
				fprintf(stdout, "  operator name in IP file ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'Y') saveTimeFlag = 1;
		}
		do {
			fprintf(stdout, "Rotate Image by 180-deg. ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') rotateFlag = 1;
		do {
			fprintf(stdout, "Vertical Mirror Image ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') mirrorFlag = 1;
		do {
			fprintf(stdout, "Delete Original ITEX File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') removeFlag = 1;
		do {
			fprintf(stdout, "Do you want to multiply some constans to intensities ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) != 'Y') {
			do {
				fprintf(stdout, " Enter constants [must be positive] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				mag = atof(inputBuffer);
				if (mag <= 0.0) continue;
				magFlag = 1;
			} while (!magFlag);
		}
			
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-r")) reverseIntFlag++;
				else if (!strcmp(arg0, "-rm")) removeFlag++;
				else if (!strcmp(arg0, "-hp")) hipicFlag++;
				else if (!strcmp(arg0, "-rt")) rotateFlag++;
				else if (!strcmp(arg0, "-rs")) mirrorFlag++;
				else if (!strcmp(arg0, "-tm")) saveTimeFlag++;
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-dx")) {
					ipHeader.xsiz = arg1;
					if (ipHeader.xsiz <= 0.0) break;
					xsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-dz")) {
					ipHeader.zsiz = arg1;
					if (ipHeader.zsiz <= 0.0) break;
					zsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-cl")) {
					ipHeader.camr = arg1;
					if (ipHeader.camr <= 0.0) break;
					camrDoneFlag++;
				}
				else if (!strcmp(arg0, "-m")) {
					mag = arg1;
					if (mag <= 0.0) break;
					magFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((reverseIntFlag > 1) || (xsizDoneFlag > 1)
		 || (zsizDoneFlag > 1) || (camrDoneFlag > 1) || (hipicFlag > 1)
		 || (removeFlag > 1) || (setTimeFlag > 1) || (saveTimeFlag > 1)
		 || (rotateFlag > 1) || (mirrorFlag > 1) || (magFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " PixelSize is %8.5f [mm] x %8.5f [mm], Camera Length is %8.2f [mm]\n", ipHeader.xsiz, ipHeader.zsiz, ipHeader.camr);
		if (reverseIntFlag) fprintf(stdout, " Intensity is reversed.");
		if (magFlag) fprintf(stdout, " Intensity is multiplied by %8.5f\n", mag);
		if (hipicFlag) fprintf(stdout, "Try to use HiPic Header.\n");
		if (rotateFlag) fprintf(stdout, "Rotate image by 180 deg.\n");
		if (mirrorFlag) fprintf(stdout, "Vertical Mirror image.\n");
		if (removeFlag) fprintf(stdout, " Original ITEX File will be removed !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	direcFlag = XOREXT(rotateFlag, mirrorFlag);
	
/* Treatment of Original Header */
	if (hipicFlag) {
		if (itexHeader.commentLength > MAXCHARBUF) fprintf(stderr, " HiPic Header is too long, and ignore it !!\n");
		else {
			if ((charBuf = malloc(itexHeader.commentLength)) == NULL)
			    fprintf(stderr, " Buffer Allocation Error in reading HiPic Header. Ignore it.\n");
			else {
				fread(charBuf, itexHeader.commentLength, 1, fp1);
				readByte += itexHeader.commentLength;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (charBuf[0] == '[') {
					strncpy(ipHeader.date, strstr(charBuf, "Date")+6, 10);
					if (verboseFlag >= 2) fprintf(stdout, " Date is set to %s.\n", ipHeader.date);
					if (saveTimeFlag) {
						strncpy(ipHeader.opnm, strstr(charBuf, "Time")+6, 8);
						if (verboseFlag >= 2) fprintf(stdout, " Time is set to %s in OPNM field.\n", ipHeader.opnm);
					}
					exposure = strtod(strstr(charBuf, "ExposureTime")+13, &chptr);
					if (!strncmp(chptr, "s", 1)||!strncmp(chptr, " s", 2)) exposure /= 60.0;
					else if (!strncmp(chptr, "ms", 2)||!strncmp(chptr, " ms", 3)) exposure /= 60000.0;
					else if (strncmp(chptr, "m", 1)&&strncmp(chptr, " m", 2)) exposure = 0.0;
					ipHeader.fext = exposure;
					if (verboseFlag >= 2) fprintf(stdout, " Exposure Time is set to %10.7f.\n", ipHeader.fext);
				}
				else {
					if (strtok(charBuf, ", ") == NULL) fprintf(stderr, " HiPic Header Format Error. Ignore it.\n");
					else {
						for (i=1; i<31; i++) {
							if ((charPtr = strtok(NULL, ", ")) == NULL) break;
							if (i == 11) {
								strncpy(ipHeader.date, charPtr, 10);
								if (verboseFlag >= 2) fprintf(stdout, " Date is set to %s.\n", ipHeader.date);
							}
							else if (saveTimeFlag && i == 12) {
								strncpy(ipHeader.opnm, charPtr, 8);
								if (verboseFlag >= 2) fprintf(stdout, " Time is set to %s in OPNM field.\n", ipHeader.opnm);
							}
							else if (i == 30) {
								exposure = strtod(charPtr, &chptr);
								if (strncmp(chptr, "m", 1)&&strncmp(chptr, " m", 2)) continue;
								else if (!strncmp(chptr, "s", 1)||!strncmp(chptr, " s", 2)) exposure /= 60.0;
								else if (!strncmp(chptr, "ms", 2)||!strncmp(chptr, " ms", 3)) exposure /= 60000.0;
								else break;
							}
						}
						if (i == 31) {
							ipHeader.fext = exposure;
							if (verboseFlag >= 2) fprintf(stdout, " Exposure Time is set to %10.7f.\n", ipHeader.fext);
						}
						else fprintf(stderr, " HiPic Header Format Error. Ignore it.\n");
					}
				}
				free(charBuf);
			}
		}
	}
	
/* Set Parameters to Create */
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(ipHeaderSizeMin, flushWidth);
	ipHeader.drtx = ipHeader.xpxl/2.0;
	ipHeader.drtz = ipHeader.zpxl/2.0;
	ipHeader.rlng = recordLengthCreate;
	ipHeader.rnum = ipHeader.zpxl;
	flushWidth0 = recordLengthCreate - flushWidth;
	recordLengthBW = recordLength*(-2);
	depthNumP = depthNum - 1;
	widthP = itexHeader.width - 1;
	
/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter  Output  Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	while (createFileEXT(file2, &fp2, 0)) {
		if (!verboseFlag) {
			fprintf(stderr, " Output File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Swap Header */
	if (cpuIntelFlag) swapIpHeader(&ipHeader);
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSize, 1, fp2);
	if (fseek(fp1, itexHeader.commentLength+itexHeaderSize, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	if (flushWidth0 == 0) free(datPtr0);
	
/* Swap Header again */
	if (cpuIntelFlag) swapIpHeader(&ipHeader);
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if (!rotateFlag) {
		if (fseek(fp1, recordLength*(ipHeader.rnum-1), SEEK_CUR)) {
			fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		readByte += recordLength*(ipHeader.rnum-1);
	}
	if (depth == 8) {
		for (i=0; i<ipHeader.rnum; i++) {
			if ((datPtrC = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (removeFlag) fprintf(stdout, " Original ITEX File is not removed.\n");
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			if ((datPtr1 = malloc(flushWidth)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrC);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (removeFlag) fprintf(stdout, " Original ITEX File is not removed.\n");
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			fread(datPtrC, recordLength, 1, fp1);
			readByte += recordLength;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtr1);
				free(datPtrC);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (removeFlag) fprintf(stdout, " Original ITEX File is not removed.\n");
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			if (direcFlag) {
				if (reverseIntFlag) {
					for (j=0, cur=widthP; j<itexHeader.width; j++) {
						intensityDbl = (depthNumP - datPtrC[j])*mag;
						datPtr1[cur--] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
					}
				}
				else {
					for (j=0, cur=widthP; j<itexHeader.width; j++) {
						intensityDbl = datPtrC[j]*mag;
						datPtr1[cur--] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
					}
				}
			}
			else {
				if (reverseIntFlag) {
					for (j=0; j<itexHeader.width; j++) {
						intensityDbl = (depthNumP - datPtrC[j])*mag;
						datPtr1[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
					}
				}
				else {
					for (j=0; j<itexHeader.width; j++) {
						intensityDbl = datPtrC[j]*mag;
						datPtr1[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
					}
				}
			}
			fwrite(datPtr1, flushWidth, 1, fp2);
			if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
			free(datPtrC);
			free(datPtr1);
			if (!rotateFlag) {
				fseek(fp1, recordLengthBW, SEEK_CUR);
				readByte += recordLengthBW;
			}
		}
	}
	else if (depth == 16) {
		for (i=0; i<ipHeader.rnum; i++) {
			if ((datPtrS = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (removeFlag) fprintf(stdout, " Original ITEX File is not removed.\n");
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			if ((datPtr1 = malloc(flushWidth)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrS);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (removeFlag) fprintf(stdout, " Original ITEX File is not removed.\n");
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			fread(datPtrS, recordLength, 1, fp1);
			readByte += recordLength;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtr1);
				free(datPtrS);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (removeFlag) fprintf(stdout, " Original ITEX File is not removed.\n");
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			if (direcFlag) {
				if (reverseIntFlag) {
					for (j=0, cur=widthP; j<itexHeader.width; j++) {
						intensityDbl = (depthNumP - (cpuIntelFlag ? datPtrS[j] : swapShort(datPtrS[j])))*mag;
						datPtr1[cur--] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
					}
				}
				else {
					for (j=0, cur=widthP; j<itexHeader.width; j++) {
						intensityDbl = (cpuIntelFlag ? datPtrS[j] : swapShort(datPtrS[j]))*mag;
						datPtr1[cur--] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
					}
				}
			}
			else {
				if (reverseIntFlag) {
					for (j=0; j<itexHeader.width; j++) {
						intensityDbl = (depthNumP - (cpuIntelFlag ? datPtrS[j] : swapShort(datPtrS[j])))*mag;
						datPtr1[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
					}
				}
				else {
					for (j=0; j<itexHeader.width; j++) {
						intensityDbl = (cpuIntelFlag ? datPtrS[j] : swapShort(datPtrS[j]))*mag;
						datPtr1[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
					}
				}
			}
			fwrite(datPtr1, flushWidth, 1, fp2);
			if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
			free(datPtrS);
			free(datPtr1);
			if (!rotateFlag) {
				fseek(fp1, recordLengthBW, SEEK_CUR);
				readByte += recordLengthBW;
			}
		}
	}
	else {	/* depth == 32 */
		for (i=0; i<ipHeader.rnum; i++) {
			if ((datPtrL = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (removeFlag) fprintf(stdout, " Original ITEX File is not removed.\n");
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			if ((datPtr1 = malloc(flushWidth)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrL);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (removeFlag) fprintf(stdout, " Original ITEX File is not removed.\n");
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			fread(datPtrL, recordLength, 1, fp1);
			readByte += recordLength;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtr1);
				free(datPtrL);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (cpuIntelFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (removeFlag) fprintf(stdout, " Original ITEX File is not removed.\n");
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			if (direcFlag) {
				if (reverseIntFlag) {
					for (j=0, cur=widthP; j<itexHeader.width; j++) {
						intensityDbl = (depthNumP - (cpuIntelFlag ? datPtrL[j] : swapLong(datPtrL[j])))*mag;
						datPtr1[cur--] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
					}
				}
				else {
					for (j=0, cur=widthP; j<itexHeader.width; j++) {
						intensityDbl = (cpuIntelFlag ? datPtrL[j] : swapLong(datPtrL[j]))*mag;
						datPtr1[cur--] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
					}
				}
			}
			else {
				if (reverseIntFlag) {
					for (j=0; j<itexHeader.width; j++) {
						intensityDbl = (depthNumP - (cpuIntelFlag ? datPtrL[j] : swapLong(datPtrL[j])))*mag;
						datPtr1[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
					}
				}
				else {
					for (j=0; j<itexHeader.width; j++) {
						intensityDbl = (cpuIntelFlag ? datPtrL[j] : swapLong(datPtrL[j]))*mag;
						datPtr1[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
					}
				}
			}
			fwrite(datPtr1, flushWidth, 1, fp2);
			if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
			free(datPtrL);
			free(datPtr1);
			if (!rotateFlag) {
				fseek(fp1, recordLengthBW, SEEK_CUR);
				readByte += recordLengthBW;
			}
		}
	}
	
/* End */
	if (flushWidth0 > 0) free(datPtr0);
	fclose(fp1);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
	if (removeFlag) {
		if (remove(file1)) fprintf(stderr, " Unable to remove Original ITEX File.\n");
		else if (verboseFlag) fprintf(stdout, " Original ITEX File has Removed.\n");
	}
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-r] [-tm] [-rm] [-hp] [-m] [-t]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-r] [-tm] [-rm] [-hp] [-m]\n", ProgName);
#endif
	fprintf(fp0, "          [-dx:VALUE] [-dz:VALUE] [-cl:VALUE] [-rt] [-rs] file2\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Convert 8 or 16-bits ITEX Image File into\n");
	fprintf(fp0, "  R-AXIS Image File. Compressed ITEX File may cause error.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -r .......... Reverse Intensity. i.e. \"0\" corresponds to White.\n");
	fprintf(fp0, "   -hp ......... Try to use Header Information of ITEX File taken by\n");
	fprintf(fp0, "                 Hamamatsu Photonics \"HiPic\".\n");
	fprintf(fp0, "   -rt ......... Rotate 180-deg. of image while conversion.\n");
	fprintf(fp0, "   -rs ......... Vertical Mirror of image while conversion.\n");
	fprintf(fp0, "   -dx:VALUE ... Set PixelSize in X-direction to \"VALUE\" [mm].\n");
	fprintf(fp0, "                 If absent, set to %.5f.\n", DEFAULT_SIZE);
	fprintf(fp0, "   -dz:VALUE ... Set PixelSize in Z-direction to \"VALUE\".\n");
	fprintf(fp0, "                 If absent, same as above.\n");
	fprintf(fp0, "   -cl:VALUE ... Change Camera Langth to \"VALUE\" [mm].\n");
	fprintf(fp0, "                 If absent, set to %.5f.\n", DEFAULT_LENG);
	fprintf(fp0, "   -m:VALUE .... Multiply \"VALUE\" to original intensity.\n");
	fprintf(fp0, "                 Useful for 32bits ITEX file.\n");
	fprintf(fp0, "   -tm ......... Save DataAcquisitionTime in original file as\n");
	fprintf(fp0, "                 \"operator name\" field. Valid only with \"-hp\".\n");
	fprintf(fp0, "   -rm ......... Remove Original ITEX File after conversion.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t .......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ....... Filename of Original ITEX File.\n");
	fprintf(fp0, "   file2 ....... Output R-AXIS Image Filename. This must be the last argument.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	double *arg1;
#else
char splitArg(const char *argV, char *arg0, double *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	return (2);
}
