/* Program to convert IP data file */
#define ProgName "ippng2rax"
#define ProgInfo "Convert Portable Network Graphic File to R-AXIS Format."
/*                                                         6/20/2000  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

/* This Program depends on "zlib" library.
   You should already have installed zlib shared library and
   header file, or compile and link it together.
   Compilng with HP-UX 9.0x Standard cc compiler might cause ill result,
   for zlib cannot be compiled correctly with the compiler.*/

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.12_1/15/2007"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <time.h>
#ifdef _HASZLIB
#	include <zlib.h>
#endif
#ifdef _HASUTIME
#	include <sys/stat.h>
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"
#include "libpng.h"

#ifdef _HASUTIME
#	define MAXARG 9
#else
#	define MAXARG 8
#endif
#define MINARG 2
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define DEFAULT_SIZE 0.05
#define DEFAULT_LENG 100.0
#define READBUFSIZE 16384

#ifdef _OLDHP
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	unsigned short int *datPtr0, *datPtr, *shortBuf, year;
	long int i, recordLengthCreate, ipHeaderSize, ipHeaderSize2VER2,
	     ipHeaderSizeMin, readByte, pngRecordLength, chunkLen,
	     crc, crc2, idatStartPos, idatNum, zStatus, dataEnd, lineNum,
	     limInt, intensity, lenTMP, flushWidth, flushWidth0;
	unsigned long int xSize, zSize, maxInt;
	char file1[FNAMELEN], file2[FNAMELEN], cpuIntelFlag, inputBuffer[20], confirmBuffer[8],
	     argNum, arg0[4], verboseFlag, charBuf[8], chunkType[4], unit,
	     removeFlag, setTimeFlag, reverseIntFlag, sizeFlag,
	     xsizDoneFlag, zsizDoneFlag, camrDoneFlag, shiftBit, *ipHeaderTMP,
	     zBufFullFlag, useHeadFlag, replacedHeaderFlag;
	unsigned char *readBuf, *writeBuf, *outputBuf, *outputBufPrev, month, day,
	     hour, minute, second, *textBuf, textType[80];
	double rato, arg1;
	struct ipHead ipHeader;
	struct pngIHDR pngHeader;
	struct tm timeRec;
	time_t timeNow;
	FILE *fp1, *fp2;
#ifdef _HASZLIB
	z_stream z;
#endif
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	readByte = lineNum = idatNum = idatStartPos = 0;
	verboseFlag = 1;
	xsizDoneFlag = zsizDoneFlag = camrDoneFlag =removeFlag = setTimeFlag =
	reverseIntFlag = useHeadFlag = replacedHeaderFlag = 0;
	sizeFlag = 2;
	timeNow = (time_t)(-1);
	rato = 8.0;
	shiftBit = 3;
	maxInt = ((long)(0x7fff)) << shiftBit;   /* =8*32767 = 8*(2^15-1) */
	ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
	memset(&ipHeader, 0, ipHeaderSize);
	ipHeaderTMP = NULL;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check zlib */
#ifndef _HASZLIB
	showMessage(stdout, cpuIntelFlag, 2);
	showUsage(stderr);
	fprintf(stderr, "This program does not work without \"zlib\".\n");
	fprintf(stderr, " Re-compile after install zlib and define \"_HASZLIB\".\n");
	fprintf(stderr, " On HP-UX 9.0x, standard \"cc\" compiler cannot make zlib correctly!!\n");
	exit(EXIT_FAILURE);
#else
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
		for (i=1; i<argc-1; i++) if (!strcmp(argv[i], "-u")) useHeadFlag++;
		if (useHeadFlag > 1) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	
/* Read PNG File */
	/* Signature Check */
	fread(charBuf, 8, 1, fp1);
	if (ftell(fp1) != 8) {
		fprintf(stderr, " File Read Error. File is too short. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (pngSignatureCheck(charBuf)) {
		fprintf(stderr, " Signature is incorrect. This File is not PNG. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	/* 1st Chunk must be IHDR */
	crc = 0x00000000L;
	if (pngReadChunk1(chunkType, &chunkLen, fp1, cpuIntelFlag)) {
		fprintf(stderr, " File Format Error : Invalid Chunk. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (strncmp(chunkType, "IHDR", 4)) {
		fprintf(stderr, " Chuck Layout is incorrect. This File is not PNG. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	crc = crc32(crc, chunkType, 4);
	fread(&pngHeader, 13, 1, fp1);
	readByte += 13;
	if (ftell(fp1) < readByte) {
		fprintf(stderr, " File Read Error. File is too short. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (ftell(fp1) > readByte) fseek(fp1, readByte+13, SEEK_SET);
	crc = crc32(crc, (char*)(&pngHeader), 13);
	fread(&crc2, 4, 1, fp1);
	readByte += 4;
	if (ftell(fp1) < readByte) {
		fprintf(stderr, " File Read Error. File is too short. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag) crc = swapLong(crc);
	if (crc2 != crc) {
		fprintf(stderr, " IHDR Chunk CRC Error. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag) {
		pngHeader.width = swapLong(pngHeader.width);
		pngHeader.height = swapLong(pngHeader.height);
	}
	if ((pngHeader.width <= 0) || (pngHeader.height <= 0)
	 || (pngHeader.compressMethod != 0) || (pngHeader.filterMethod != 0)) {
		fprintf(stderr, " File Format Error : Invalid IHDR Chunk. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if ((pngHeader.width > MAXWIDTH) || (pngHeader.height > MAXHEIGHT)
	 || (pngHeader.colorType != 0) || (pngHeader.interlaceMethod != 0)
	 || ((pngHeader.bitDepth != 1) && (pngHeader.bitDepth != 2) && (pngHeader.bitDepth != 4)
	   && (pngHeader.bitDepth != 8) && (pngHeader.bitDepth != 16))) {
		fprintf(stderr, " File Format Error : PNG FIle Type is not the expected. Exit.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	/* Check necessary chunk */
	for (;;) {
		if (pngReadChunk1(chunkType, &chunkLen, fp1, cpuIntelFlag)) {
			fprintf(stderr, " File Format Error : Invalid Chunk. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!strncmp(chunkType, "pHYs", 4)) {
			if (chunkLen != 9 || idatStartPos != 0) {
				fprintf(stderr, " File Format Error : Invalid pHYs Chunk. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			readByte = ftell(fp1);
			fread(&xSize, 4, 1, fp1);
			fread(&zSize, 4, 1, fp1);
			fread(&sizeFlag, 1, 1, fp1);
			if (ftell(fp1) < readByte + 9) {
				fprintf(stderr, " File Read Error. File is too short. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fseek(fp1, readByte+9-ftell(fp1), SEEK_CUR);
			readByte = ftell(fp1);
			fread(&crc2, 4, 1, fp1);
			if (ftell(fp1) < readByte + 4) {
				fprintf(stderr, " File Read Error. File is too short. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			crc = 0x00000000L;
			crc = crc32(crc, chunkType, 4);
			crc = crc32(crc, (char*)&xSize, 4);
			crc = crc32(crc, (char*)&zSize, 4);
			crc = crc32(crc, &sizeFlag, 1);
			if (cpuIntelFlag) crc = swapLong(crc);
			if (crc2 != crc) {
				fprintf(stderr, " pHYs Chunk CRC Error. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (cpuIntelFlag) {
				xSize = swapLong(xSize);
				zSize = swapLong(zSize);
			}
			if (xSize <= 0 || zSize <= 0) {
				fprintf(stderr, " PixelSize stored in the File is incorrect. Ignore it.\n");
				sizeFlag = 2;
			}
			continue;
		}
		if (!strncmp(chunkType, "tIME", 4)) {
			if (chunkLen != 7) {
				fprintf(stderr, " File Format Error : Invalid tIME Chunk. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			readByte = ftell(fp1);
			fread(&year, 2, 1, fp1);
			fread(&month, 1, 1, fp1);
			fread(&day, 1, 1, fp1);
			fread(&hour, 1, 1, fp1);
			fread(&minute, 1, 1, fp1);
			fread(&second, 1, 1, fp1);
			if (ftell(fp1) < readByte + 7) {
				fprintf(stderr, " File Read Error. File is too short. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fseek(fp1, readByte+7-ftell(fp1), SEEK_CUR);
			readByte = ftell(fp1);
			fread(&crc2, 4, 1, fp1);
			if (ftell(fp1) < readByte + 4) {
				fprintf(stderr, " File Read Error. File is too short. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			crc = 0x00000000L;
			crc = crc32(crc, chunkType, 4);
			crc = crc32(crc, (char*)&year, 2);
			crc = crc32(crc, &month, 1);
			crc = crc32(crc, &day, 1);
			crc = crc32(crc, &hour, 1);
			crc = crc32(crc, &minute, 1);
			crc = crc32(crc, &second, 1);
			if (cpuIntelFlag) crc = swapLong(crc);
			if (crc2 != crc) {
				fprintf(stderr, " tIME Chunk CRC Error. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (cpuIntelFlag) year = swapShort(year);
			timeRec.tm_sec = second;
			timeRec.tm_min = minute;
			timeRec.tm_hour = hour;
			timeRec.tm_mday = day;
			timeRec.tm_mon = month;
			timeRec.tm_year = year - 1900;
			timeNow = mktime(&timeRec);
			continue;
		}
		if (!strncmp(chunkType, "tEXt", 4)) {
			if (chunkLen <= 0) {
				fprintf(stderr, " File Format Error : Invalid tEXt Chunk. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			readByte = ftell(fp1);
			if ((textBuf = malloc(chunkLen)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				fprintf(stderr, " Ignore this tEXt chunk.\n");
				continue;
			}
			fread(textBuf, chunkLen, 1, fp1);
			if (ftell(fp1) < readByte + chunkLen) {
				fprintf(stderr, " File Read Error. File is too short. Exit.\n");
				fclose(fp1);
				free(textBuf);
				exit(EXIT_FAILURE);
			}
			fseek(fp1, readByte+chunkLen-ftell(fp1), SEEK_CUR);
			readByte = ftell(fp1);
			fread(&crc2, 4, 1, fp1);
			if (ftell(fp1) < readByte + 4) {
				fprintf(stderr, " File Read Error. File is too short. Exit.\n");
				free(textBuf);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			crc = 0x00000000L;
			crc = crc32(crc, chunkType, 4);
			crc = crc32(crc, textBuf, chunkLen);
			if (cpuIntelFlag) crc = swapLong(crc);
			if (crc2 != crc) {
				fprintf(stderr, " tEXt Chunk CRC Error. Exit.\n");
				free(textBuf);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			sscanf(textBuf, "%s", textType);
			if (!strcmp(textType, "Author")) strncpy(ipHeader.opnm, textBuf+strlen(textType)+1, 19);
			else if (!strcmp(textType, "xRAXISHEAD")) {
				if (ipHeaderTMP != NULL) {
					fprintf(stderr, " Stored IP Header Duplicate. Ignore it.\n");
					free(textBuf);
					continue;
				}
				if (chunkLen-(long)strlen(textType)-1 > (long)ceil(ipHeaderSize/3.0)*4) {
					fprintf(stderr, " Stored IP Header is broken. Ignore it.\n");
					free(textBuf);
					continue;
				}
				if ((ipHeaderTMP = malloc(ipHeaderSize)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					fprintf(stderr, " Ignore Stored IP Header.\n");
					free(textBuf);
					continue;
				}
				base64de(textBuf+strlen(textType)+1, ipHeaderTMP, chunkLen-strlen(textType)-1, &lenTMP);
				if (lenTMP != ipHeaderSize) {
					fprintf(stderr, " Stored IP Header is broken. Ignore it.\n");
					free(ipHeaderTMP);
					ipHeaderTMP = NULL;
				}
				else if (verboseFlag == 3) {
					do {
						fprintf(stdout, "Use Original Information Stored in PNG File ? [Y/N] : ");
						if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
							fprintf(stderr, "\nError in getting input. Exit.\n");
							free(textBuf);
							fclose(fp1);
							exit(EXIT_FAILURE);
						}
					} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
					if (toupper(confirmBuffer[0]) == 'Y') useHeadFlag = 1;
				}
			}
			free(textBuf);
			continue;
		}
		if (!strncmp(chunkType, "zTXt", 4)) {
			if (chunkLen <= 0) {
				fprintf(stderr, " File Format Error : Invalid zTXt Chunk. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			readByte = ftell(fp1);
			if ((textBuf = malloc(chunkLen)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				fprintf(stderr, " Ignore this zTXt chunk.\n");
				continue;
			}
			fread(textBuf, chunkLen, 1, fp1);
			if (ftell(fp1) < readByte + chunkLen) {
				fprintf(stderr, " File Read Error. File is too short. Exit.\n");
				free(textBuf);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fseek(fp1, readByte+chunkLen-ftell(fp1), SEEK_CUR);
			readByte = ftell(fp1);
			fread(&crc2, 4, 1, fp1);
			if (ftell(fp1) < readByte + 4) {
				fprintf(stderr, " File Read Error. File is too short. Exit.\n");
				free(textBuf);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			crc = 0x00000000L;
			crc = crc32(crc, chunkType, 4);
			crc = crc32(crc, textBuf, chunkLen);
			if (cpuIntelFlag) crc = swapLong(crc);
			if (crc2 != crc) {
				fprintf(stderr, " zTXt Chunk CRC Error. Exit.\n");
				free(textBuf);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			sscanf(textBuf, "%s", textType);
			if (!strcmp(textType, "Author") || !strcmp(textType, "xRAXISHEAD")) {
				if (textBuf[strlen(textType)+1] != 0) {
					fprintf(stderr, " Compression Format Error in zTXt Chunk. Ignore this Chunk.\n");
					free(textBuf);
					continue;
				}
				if ((writeBuf = malloc((long)ceil(ipHeaderSize/3.0)*4)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					fprintf(stderr, " Ignore this zTXt chunk.\n");
					free(textBuf);
					continue;
				}
				zBufFullFlag = 0;
				z.zalloc = Z_NULL;
				z.zfree = Z_NULL;
				z.opaque = Z_NULL;
				z.next_in = Z_NULL;
				z.avail_in = 0;
				if (inflateInit(&z) != Z_OK) {
					fprintf(stderr, " zTXt Chunk Inflate Error. Ignore this Chunk.\n");
					free(writeBuf);
					free(textBuf);
					continue;
				}
				z.next_out = writeBuf;
				z.avail_out = (long)ceil(ipHeaderSize/3.0)*4;
				z.next_in = textBuf + strlen(textType) + 2;
				z.avail_in = chunkLen - strlen(textType) - 2;
				zStatus = Z_OK;
				while (zStatus != Z_STREAM_END) {
					zStatus = inflate(&z, Z_NO_FLUSH);
					if (zStatus == Z_STREAM_END) break;
					if (zStatus != Z_OK) {
						fprintf(stderr, " zTXt Chunk Inflate Error. Ignore this Chunk.\n");
						zBufFullFlag += 10;
						break;
					}
					if (z.avail_out == 0) {
						if (zBufFullFlag && z.avail_out == 0) {
							fprintf(stderr, " zTXt Chunk Inflate Error. Ignore this Chunk.\n");
							zBufFullFlag += 10;
							break;
						}
						z.next_out = inputBuffer;
						z.avail_out = 10;
						zBufFullFlag++;
					}
				}
				if (zBufFullFlag > 1) {
					free(writeBuf);
					free(textBuf);
					continue;
				}
				if (zBufFullFlag != 1 && z.avail_out != 0) {
					fprintf(stderr, " zTXt Chunk Inflate Error. Ignore this Chunk.\n");
					free(writeBuf);
					free(textBuf);
					continue;
				}
				if (inflateEnd(&z) != Z_OK) {
					fprintf(stderr, " zTXt Chunk Inflate Error. Ignore this Chunk.\n");
					free(writeBuf);
					free(textBuf);
					continue;
				}
				if (!strcmp(textType, "Author")) strncpy(ipHeader.opnm, writeBuf, 19);
				else {
					if (ipHeaderTMP != NULL) {
						fprintf(stderr, " Stored IP Header Duplicate. Ignore it.\n");
						free(writeBuf);
						free(textBuf);
						continue;
					}
					if ((ipHeaderTMP = malloc(ipHeaderSize)) == NULL) {
						fprintf(stderr, " Failed to allocate memory.\n");
						fprintf(stderr, " Ignore Stored IP Header.\n");
						free(writeBuf);
						free(textBuf);
						continue;
					}
					base64de(writeBuf, ipHeaderTMP, (long)ceil(ipHeaderSize/3.0)*4, &lenTMP);
					if (lenTMP != ipHeaderSize) {
						fprintf(stderr, " Stored IP Header is broken. Ignore it.\n");
						free(ipHeaderTMP);
						ipHeaderTMP = NULL;
					}
					else if (verboseFlag == 3) {
						do {
							fprintf(stdout, "Use Original Information Stored in PNG File ? [Y/N] : ");
							if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
								fprintf(stderr, "\nError in getting input. Exit.\n");
								free(textBuf);
								free(writeBuf);
								free(ipHeaderTMP);
								fclose(fp1);
								exit(EXIT_FAILURE);
							}
						} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
						if (toupper(confirmBuffer[0]) == 'Y') useHeadFlag = 1;
					}
				}
				free(writeBuf);
			}
			free(textBuf);
			continue;
		}
		if (!strncmp(chunkType, "IDAT", 4)) {
			if (idatStartPos != 0) {
				fprintf(stderr, " File Format Error. IDAT chunk Duplicated. Exit.\n");
				fclose(fp1);
				if (useHeadFlag && ipHeaderTMP != NULL) free(ipHeaderTMP);
				exit(EXIT_FAILURE);
			}
			idatStartPos = ftell(fp1) - 8;
			if (fseek(fp1, chunkLen+4, SEEK_CUR)) {
				fprintf(stderr, " File Read Error. File is too short. Exit.\n");
				if (useHeadFlag && ipHeaderTMP != NULL) free(ipHeaderTMP);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			for (;;) {
				if (pngReadChunk1(chunkType, &chunkLen, fp1, cpuIntelFlag)) {
					fprintf(stderr, " File Format Error : Invalid Chunk. Exit.\n");
					if (useHeadFlag && ipHeaderTMP != NULL) free(ipHeaderTMP);
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				idatNum++;
				if (!strncmp(chunkType, "IDAT", 4)) {
					if (fseek(fp1, chunkLen+4, SEEK_CUR)) {
						fprintf(stderr, " File Read Error. File is too short. Exit.\n");
						if (useHeadFlag && ipHeaderTMP != NULL) free(ipHeaderTMP);
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
				}
				else break;
			}
			fseek(fp1, -8, SEEK_CUR);
			continue;
		}
		if (!strncmp(chunkType, "IEND", 4)) {
			if (chunkLen != 0) {
				fprintf(stderr, " File Format Error : Invalid IEND Chunk. Exit.\n");
				if (useHeadFlag && ipHeaderTMP != NULL) free(ipHeaderTMP);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			readByte = ftell(fp1);
			fread(&crc2, 4, 1, fp1);
			if (ftell(fp1) < readByte + 4 || idatStartPos == 0) {
				fprintf(stderr, " File Read Error. File is too short. Exit.\n");
				if (useHeadFlag && ipHeaderTMP != NULL) free(ipHeaderTMP);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			crc = 0x00000000L;
			crc = crc32(crc, chunkType, 4);
			if (cpuIntelFlag) crc = swapLong(crc);
			if (crc2 != crc) {
				fprintf(stderr, " IEND Chunk CRC Error. Exit.\n");
				if (useHeadFlag && ipHeaderTMP != NULL) free(ipHeaderTMP);
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			break;
		}
		if (fseek(fp1, chunkLen+4, SEEK_CUR)) {
			fprintf(stderr, " File Read Error. File is too short. Exit.\n");
			if (useHeadFlag && ipHeaderTMP != NULL) free(ipHeaderTMP);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Define ipHeader and Initialize */
	if (useHeadFlag && ipHeaderTMP != NULL) {
		memcpy(&ipHeader, ipHeaderTMP, ipHeaderSize);
		if (verboseFlag) fprintf(stdout, " Stored IP Header is recovered successfully.\n");
		if (cpuIntelFlag) swapIpHeader(&ipHeader);
		replacedHeaderFlag++;
	}
	else {
		strcpy(ipHeader.name, "R-AXIS-DS"); /* Name of the Device */
		strcpy(ipHeader.vers, "1.0-2.0"); /* Version Number */
		strcpy(ipHeader.crnm, "N/A"); /* Crystal Name */
		strcpy(ipHeader.crsy, "cubic"); /* Crystal System */
		ipHeader.alng = 10.0; /* a-axis length in A */
		ipHeader.blng = 10.0; /* b-axis length in A */
		ipHeader.clng = 10.0; /* c-axis length in A */
		ipHeader.alfa = 90.0; /* angle alpha */
		ipHeader.beta = 90.0; /* angle beta */
		ipHeader.gamm = 90.0; /* angle gamma */
		strcpy(ipHeader.spgr, "P23"); /* Space Group */
		ipHeader.mosc = 0.3; /* Mozaic Spread */
		if (ipHeader.opnm[0] == 0) strcpy(ipHeader.opnm, "rigaku"); /* Operator Name */
		strcpy(ipHeader.trgt, "Cu"); /* X-ray Target */
		ipHeader.wlng = 1.5418; /* Wave Length of the x-ray */
		strcpy(ipHeader.mcro, "None"); /* Monochromator Material */
		ipHeader.m2ta = 0.0; /* Angle of the monochromator in 2-theta */
		strcpy(ipHeader.colm, "0.05-0.10"); /* Collimator */
		strcpy(ipHeader.filt, "Ni"); /* Kb filter */
		ipHeader.camr = 50.0; /* Camera length in mm */
		ipHeader.vltg = 40.0; /* X-ray tube voltage */
		ipHeader.crnt = 100.0; /* X-ray tube current */
		strcpy(ipHeader.focs, "0.3x0.3"); /* X-ray focus */
		strcpy(ipHeader.optc, "Ni-filter"); /* X-ray optics */
		ipHeader.cyld = 0; /* Camera type */
		ipHeader.weis = 0.0; /* weisenberg oscillation */
		strcpy(ipHeader.mnax, "-b*"); /* Mount Axis */
		strcpy(ipHeader.bmax, "-a"); /* Beam Axis */
		ipHeader.phi0 = 0.0; /* phi0 */
		ipHeader.phis = 0.0; /* phi start */
		ipHeader.phie = 0.0; /* phi end */
		ipHeader.oscn = 0; /* Frequency of Oscillation */
		ipHeader.fext = 20.0; /* Exposure Time in min. */
		ipHeader.omga = 0.0; /* omega in theta */
		ipHeader.fkai = 0.0; /* xai */
		ipHeader.thta = 0.000; /* 2-theta */
		ipHeader.mu = 0.000; /* mu */
		ipHeader.ipst = 0; /* Line of start reading */
		ipHeader.ipnm = 1; /* Number of IP */
		ipHeader.rato = rato; /* Ratio of IP in High/Low */
		ipHeader.ft_1 = 0.0; /* fading time 1 */
		ipHeader.ft_2 = 0.0; /* fading time 2 */
		strcpy(ipHeader.host, "IRIS"); /* Type of Host Computer */
		strcpy(ipHeader.ip, "HR-3"); /* Type of IP */
		ipHeader.dr_x = 0; /* Data direction in horizontal */
		ipHeader.dr_z = 0; /* Data direction in vertical */
		ipHeader.drxz = 0; /* Data direction face/back */
#ifdef _HASUTIME
		if (timeNow != (time_t)(-1)) strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&timeNow));
		else strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&utime1.modtime));
#endif
	}
	timeNow = time(NULL);
	strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&timeNow));
	strcat(ipHeader.memo, ProgName);
	strcat(ipHeader.memo, "/");
	strcat(ipHeader.memo, ipHeader.date);
	if (useHeadFlag) free(ipHeaderTMP);
	
/* Set ipHeader Information */
	ipHeader.xpxl = pngHeader.width;
	ipHeader.zpxl = pngHeader.height;
	if (!replacedHeaderFlag) ipHeader.camr = DEFAULT_LENG;
	unit = (pngHeader.bitDepth == 16) ? 2 : 1;
	pngRecordLength = (long)ceil(pngHeader.width * pngHeader.bitDepth / 8.0);
	limInt = ((0x00000001L) << pngHeader.bitDepth) - 1;
	
/* Conversion Parameter */
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels, each pixel has %d bits.\n", pngHeader.width, pngHeader.height, pngHeader.bitDepth);
	}
	if (!replacedHeaderFlag) {
		if (sizeFlag == 1 || sizeFlag == 0) {
			ipHeader.xsiz = 1000.0 / xSize;
			ipHeader.zsiz = 1000.0 / zSize;
		}
		else ipHeader.xsiz = ipHeader.zsiz = DEFAULT_SIZE;
	}
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Reverse intensity (i.e. \"White\" corresponds to \"0\") ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') reverseIntFlag = 1;
		if (useHeadFlag == 0) {
			do {
				fprintf(stdout, "PixelSize in X-direction is %.5f [mm]. OK ? [Y/N] : ", ipHeader.xsiz);
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				do {
					fprintf(stdout, "  Enter PixelSize in X-direction [mm] : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					if (!isdigitEXT(inputBuffer)) continue;
					ipHeader.xsiz = atof(inputBuffer);
					if (ipHeader.xsiz <= 0.0) continue;
					xsizDoneFlag = 1;
				} while (!xsizDoneFlag);
			}
			if (sizeFlag == 0) ipHeader.zsiz = ipHeader.xsiz * xSize / zSize;
			do {
				fprintf(stdout, "PixelSize in Z-direction is %.5f [mm]. OK ? [Y/N] : ", ipHeader.zsiz);
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				do {
					fprintf(stdout, "  Enter PixelSize in Z-direction [mm] : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					if (!isdigitEXT(inputBuffer)) continue;
					ipHeader.zsiz = atof(inputBuffer);
					if (ipHeader.zsiz <= 0.0) continue;
					zsizDoneFlag = 1;
				} while (!zsizDoneFlag);
			}
			do {
				fprintf(stdout, "Enter Camera Length [mm] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				ipHeader.camr = atof(inputBuffer);
				if (ipHeader.camr <= 0.0) continue;
				camrDoneFlag = 1;
			} while (!camrDoneFlag);
		}
		do {
			fprintf(stdout, "Delete Original PNG File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') removeFlag = 1;
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-r")) reverseIntFlag++;
				else if (!strcmp(arg0, "-rm")) removeFlag++;
				else if (!strcmp(arg0, "-u")) continue;
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-dx")) {
					ipHeader.xsiz = arg1;
					if (ipHeader.xsiz <= 0.0) break;
					xsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-dz")) {
					ipHeader.zsiz = arg1;
					if (ipHeader.zsiz <= 0.0) break;
					zsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-cl")) {
					ipHeader.camr = arg1;
					if (ipHeader.camr <= 0.0) break;
					camrDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((reverseIntFlag > 1) || (xsizDoneFlag > 1) || (zsizDoneFlag > 1)
		 || (removeFlag > 1) || (setTimeFlag > 1) || (camrDoneFlag > 1)
		 || (useHeadFlag && (xsizDoneFlag || zsizDoneFlag || camrDoneFlag))) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!replacedHeaderFlag && sizeFlag == 0) {
			if (xsizDoneFlag && !zsizDoneFlag) ipHeader.zsiz = ipHeader.xsiz * xSize / zSize;
			if (!xsizDoneFlag && zsizDoneFlag) ipHeader.xsiz = ipHeader.zsiz * zSize / xSize;
		}
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " PixelSize is %8.5f [mm] x %8.5f [mm], Camera Length is %8.2f [mm]\n", ipHeader.xsiz, ipHeader.zsiz, ipHeader.camr);
		if (reverseIntFlag) fprintf(stdout, " Reverse Data Conversion.\n");
		if (removeFlag) fprintf(stdout, " Original PNG File will be removed !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Set Parameters to Create */
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(ipHeaderSizeMin, flushWidth);
	if (!replacedHeaderFlag) {
		ipHeader.drtx = ipHeader.xpxl/2.0;
		ipHeader.drtz = ipHeader.zpxl/2.0;
	}
	else {
		if (ipHeader.drtx >= ipHeader.xpxl) ipHeader.drtx = ipHeader.xpxl/2.0;
		if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl/2.0;
	}
	ipHeader.rlng = recordLengthCreate;
	ipHeader.rnum = ipHeader.zpxl;
	flushWidth0 = recordLengthCreate - flushWidth;
	
/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter  Output  Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	while (createFileEXT(file2, &fp2, 0)) {
		if (!verboseFlag) {
			fprintf(stderr, " Output File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Swap Header */
	if (cpuIntelFlag) swapIpHeader(&ipHeader);
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSize, 1, fp2);
	if (fseek(fp1, idatStartPos, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	
/* Swap Header again */
	if (cpuIntelFlag) swapIpHeader(&ipHeader);
	
/* Write Dummy Data */
	for (i=0; i<ipHeader.rnum; i++) fwrite(datPtr0, recordLengthCreate, 1, fp2);
	fseek(fp2, -recordLengthCreate, SEEK_CUR);
	if (flushWidth0 == 0) free(datPtr0);
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	z.zalloc = Z_NULL;
	z.zfree = Z_NULL;
	z.opaque = Z_NULL;
	z.next_in = Z_NULL;
	z.avail_in = 0;
	if ((readBuf = malloc(READBUFSIZE)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		if (flushWidth > 0) free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	if ((writeBuf = malloc(pngRecordLength+1)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		if (flushWidth > 0) free(datPtr0);
		free(readBuf);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	if ((outputBuf = malloc(pngRecordLength)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		if (flushWidth > 0) free(datPtr0);
		free(readBuf);
		free(writeBuf);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	if ((outputBufPrev = malloc(pngRecordLength)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		if (flushWidth > 0) free(datPtr0);
		free(readBuf);
		free(writeBuf);
		free(outputBuf);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	if ((shortBuf = malloc(ipHeader.xpxl*sizeof(short))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		if (flushWidth > 0) free(datPtr0);
		free(readBuf);
		free(writeBuf);
		free(outputBuf);
		free(outputBufPrev);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	if ((datPtr = malloc(ipHeader.xpxl*sizeof(short))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		if (flushWidth > 0) free(datPtr0);
		free(readBuf);
		free(writeBuf);
		free(outputBuf);
		free(outputBufPrev);
		free(shortBuf);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	if (inflateInit(&z) != Z_OK) {
		fprintf(stderr, " InflateInit Error. : %s\n", (z.msg) ? z.msg : "unknown");
		if (flushWidth > 0) free(datPtr0);
		free(readBuf);
		free(writeBuf);
		free(outputBuf);
		free(outputBufPrev);
		free(shortBuf);
		free(datPtr);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	memset(outputBufPrev, 0, pngRecordLength);
	z.next_out = writeBuf;
	z.avail_out = pngRecordLength+1;
	zStatus = Z_OK;
	do {
		crc = 0x00000000L;
		pngReadChunk1(chunkType, &chunkLen, fp1, cpuIntelFlag);
		readByte += 8;
		dataEnd = readByte + chunkLen;
		crc = crc32(crc, chunkType, 4);
		while (zStatus != Z_STREAM_END) {
			if (z.avail_in == 0) {
				z.next_in = readBuf;
				if ((z.avail_in = MINEXT(dataEnd-ftell(fp1), READBUFSIZE)) <= 0) {
					fprintf(stderr, " File Read Error or Inflate Error. NO Z_STREAM_END.\n");
					if (flushWidth > 0) free(datPtr0);
					free(readBuf);
					free(writeBuf);
					free(outputBuf);
					free(outputBufPrev);
					free(shortBuf);
					free(datPtr);
					fclose(fp1);
					fclose(fp2);
					fprintf(stderr, " Output File is not properly created. It might a \"0\" sea...\n");
					exit(EXIT_FAILURE);
				}
				fread(readBuf, z.avail_in, 1, fp1);
/*				if (ftell(fp1) != readByte + z.avail_in) {
					fprintf(stderr, " File Read Error.\n");
					if (flushWidth > 0) free(datPtr0);
					free(readBuf);
					free(writeBuf);
					free(outputBuf);
					free(outputBufPrev);
					free(shortBuf);
					free(datPtr);
					fclose(fp1);
					fclose(fp2);
					fprintf(stderr, " Output File is not properly created. It might a \"0\" sea...\n");
					exit(EXIT_FAILURE);
				}*/
				readByte += z.avail_in;
				crc = crc32(crc, readBuf, z.avail_in);
			}
			zStatus = inflate(&z, Z_NO_FLUSH);
			if (zStatus == Z_STREAM_END) break;
			if (zStatus != Z_OK) {
				fprintf(stderr, " Inflate Error : %s\n", (z.msg) ? z.msg : "unknown");
				if (flushWidth > 0) free(datPtr0);
				free(readBuf);
				free(writeBuf);
				free(outputBuf);
				free(outputBufPrev);
				free(shortBuf);
				free(datPtr);
				fclose(fp1);
				fclose(fp2);
				fprintf(stderr, " Output File is not properly created. It might a \"0\" sea...\n");
				exit(EXIT_FAILURE);
			}
			if (z.avail_out == 0) {
				if (++lineNum > ipHeader.rnum) {
					fprintf(stderr, " File Read Error or Inflate Error. NO Z_STREAM_END.\n");
					if (flushWidth > 0) free(datPtr0);
					free(readBuf);
					free(writeBuf);
					free(outputBuf);
					free(outputBufPrev);
					free(shortBuf);
					free(datPtr);
					fclose(fp1);
					fclose(fp2);
					fprintf(stderr, " Output File is not properly created. It might a \"0\" sea...\n");
					exit(EXIT_FAILURE);
				}
				pngDefilter(writeBuf, outputBuf, outputBufPrev, unit, pngRecordLength);
				free(outputBufPrev);
				pngResolveScanline(outputBuf, shortBuf, pngHeader.bitDepth, ipHeader.xpxl, cpuIntelFlag);
				for (i=0; i<ipHeader.xpxl; i++) {
					intensity = reverseIntFlag ? limInt - shortBuf[i] : shortBuf[i];
					datPtr[i] = ipLtoShortBit(&intensity, &rato, &shiftBit, &maxInt, &cpuIntelFlag);
				}
				fwrite(datPtr, flushWidth, 1, fp2);
				if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
				fseek(fp2, -2*recordLengthCreate, SEEK_CUR);
				if (lineNum < ipHeader.rnum) {
					outputBufPrev = outputBuf;
					if ((outputBuf = malloc(pngRecordLength)) == NULL) {
						fprintf(stderr, " Failed to allocate memory.\n");
						if (flushWidth > 0) free(datPtr0);
						free(readBuf);
						free(writeBuf);
						free(outputBufPrev);
						free(shortBuf);
						free(datPtr);
						fclose(fp1);
						fclose(fp2);
						fprintf(stderr, " Output File is not properly created. It might a \"0\" sea...\n");
						exit(EXIT_FAILURE);
					}
				}
				z.next_out = writeBuf;
				z.avail_out = pngRecordLength+1;
			}
		}
		fread(&crc2, 4, 1, fp1);
		readByte += 4;
/*		if (ftell(fp1) != readByte) {
			fprintf(stderr, " File Read Error. File is too short. Exit.\n");
			if (flushWidth > 0) free(datPtr0);
			free(readBuf);
			free(writeBuf);
			free(outputBuf);
			free(outputBufPrev);
			free(shortBuf);
			free(datPtr);
			fclose(fp1);
			fclose(fp2);
			fprintf(stderr, " Output File is not properly created. It might a \"0\" sea...\n");
			exit(EXIT_FAILURE);
		}*/
		if (cpuIntelFlag) crc = swapLong(crc);
		if (crc2 != crc) {
			fprintf(stderr, " IDAT Chunk CRC Error. Exit.\n");
			if (flushWidth > 0) free(datPtr0);
			free(readBuf);
			free(writeBuf);
			free(outputBuf);
			free(outputBufPrev);
			free(shortBuf);
			free(datPtr);
			fclose(fp1);
			fclose(fp2);
			fprintf(stderr, " Output File is not properly created. It might a \"0\" sea...\n");
			exit(EXIT_FAILURE);
		}
		if (zStatus == Z_STREAM_END) break;
	} while (--idatNum > 0);
	free(readBuf);
	fclose(fp1);
	if (z.avail_out != (unsigned long)(pngRecordLength+1) && lineNum == ipHeader.rnum) {
		fprintf(stderr, " File Read Error or Inflate Error. Data is too large than is expected.\n");
		if (flushWidth > 0) free(datPtr0);
		free(writeBuf);
		free(outputBuf);
		free(outputBufPrev);
		free(shortBuf);
		free(datPtr);
		fclose(fp2);
		fprintf(stderr, " Output File is not properly created. It might a \"0\" sea...\n");
		exit(EXIT_FAILURE);
	}
	if (lineNum < ipHeader.rnum - 1 || (lineNum == ipHeader.rnum - 1 && z.avail_out != 0)) {
		fprintf(stderr, " File Read Error or Inflate Error. Data is too few than is expected. Exit.\n");
		if (flushWidth > 0) free(datPtr0);
		free(writeBuf);
		free(outputBuf);
		free(outputBufPrev);
		free(shortBuf);
		free(datPtr);
		fclose(fp2);
		fprintf(stderr, " Output File is not properly created. It might a \"0\" sea...\n");
		exit(EXIT_FAILURE);
	}
	if (z.avail_out == 0) {
		pngDefilter(writeBuf, outputBuf, outputBufPrev, unit, pngRecordLength);
		free(writeBuf);
		free(outputBufPrev);
		pngResolveScanline(outputBuf, shortBuf, pngHeader.bitDepth, ipHeader.xpxl, cpuIntelFlag);
		free(outputBuf);
		for (i=0; i<ipHeader.xpxl; i++) {
			intensity = reverseIntFlag ? limInt - shortBuf[i] : shortBuf[i];
			datPtr[i] = ipLtoShortBit(&intensity, &rato, &shiftBit, &maxInt, &cpuIntelFlag);
		}
		free(shortBuf);
		fwrite(datPtr, flushWidth, 1, fp2);
		free(datPtr);
		if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
		free(datPtr0);
	}
	else {
		free(writeBuf);
		free(outputBufPrev);
		free(outputBuf);
		free(shortBuf);
		free(datPtr);
		if (flushWidth > 0) free(datPtr0);
	}
	if (inflateEnd(&z) != Z_OK) {
		fprintf(stderr, " InflateEnd Error : %s\n", (z.msg) ? z.msg : "unknown");
		fclose(fp2);
		fprintf(stderr, " Output File is not properly created. It might a \"0\" sea...\n");
		exit(EXIT_FAILURE);
	}
	
/* End */
	fseek(fp2, 0, SEEK_END);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
	if (removeFlag) {
		if (remove(file1)) fprintf(stderr, " Unable to remove Original Raw Data File.\n");
		else if (verboseFlag) fprintf(stdout, " Original Raw Data File has Removed.\n");
	}
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
#endif /* _HASZLIB */
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-r] [-rm] [-u] [-t]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-r] [-rm] [-u]\n", ProgName);
#endif
	fprintf(fp0, "          [-dx:VALUE] [-dz:VALUE] [-cl:VALUE] file2\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
#ifdef _HASZLIB
	fprintf(fp0, " This Program Convert PNG (Portable Network Graphic) File\n");
	fprintf(fp0, "  with 1, 2, 4, 8 or 16-bits depth Grayscale into R-AXIS Format.\n");
	fprintf(fp0, "  Gamma Value, Alpha Channel, etc. are ignored.\n");
	fprintf(fp0, "   This program does not work without \"zlib\".\n");
	fprintf(fp0, "    Re-compile after install zlib and define \"_HASZLIB\".\n");
	fprintf(fp0, "    On HP-UX 9.0x, standard \"cc\" compiler cannot make zlib correctly!!\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -r .......... Reverse Intensity. i.e. \"0\" corresponds to White.\n");
	fprintf(fp0, "   -dx:VALUE ... Set PixelSize in X-direction to \"VALUE\" [mm].\n");
	fprintf(fp0, "                 If absent, try to get infomation from PNG File,\n");
	fprintf(fp0, "                 and if failed, set to %.5f.\n", DEFAULT_SIZE);
	fprintf(fp0, "   -dz:VALUE ... Set PixelSize in Z-direction to \"VALUE\".\n");
	fprintf(fp0, "                 If absent, same as above.\n");
	fprintf(fp0, "   -cl:VALUE ... Change Camera Langth to \"VALUE\" [mm].\n");
	fprintf(fp0, "                 If absent, set to %.5f.\n", DEFAULT_LENG);
	fprintf(fp0, "   -u .......... Use Information of PNG File. This is valid for a PNG File\n");
	fprintf(fp0, "                 converted from an R-AXIS File through the program like this.\n");
	fprintf(fp0, "                 Original Header is stored after Base64 coding preceding to\n");
	fprintf(fp0, "                 LZ77 deflation. This may not work well...\n");
	fprintf(fp0, "   -rm ......... Remove Original Raw Data File after conversion.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t .......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ....... Filename of Original Data File.\n");
	fprintf(fp0, "   file2 ....... Output R-AXIS Image Filename. This must be the last argument.\n");
#endif /* _HASZLIB */
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	double *arg1;
#else
char splitArg(const char *argV, char *arg0, double *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	return (2);
}
