/* Program to convert IP data file */
#define ProgName "ipraw2rax"
#define ProgInfo "Convert Raw Data File to R-AXIS Format."
/*                                                         5/18/1999  */
/*         for ANSI C                                                 */
/*         Raw data to Rigaku format.                      5/18/1999  */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v2.24_1/15/2007"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <time.h>
#ifdef _HASUTIME
#	include <sys/stat.h>
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef _HASUTIME
#	define MAXARG 12
#else
#	define MAXARG 11
#endif
#define MINARG 2
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define WIDTH1 3760
#define HEIGHT1 3000
#define XSIZ1 0.025
#define ZSIZ1 0.025

#ifdef _OLDHP
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtrS;
	unsigned short int *datPtrUS;
	long int i, j, recordLengthCreate, recordLength, ipHeaderSize, ipHeaderSize2VER2,
	     ipHeaderSizeMin, intensity, skipBytes, readByte,
	     flushWidth0, flushWidth, *datPtrL;
	unsigned long int *datPtrUL, maxInt;
	char file1[FNAMELEN], file2[FNAMELEN], cpuIntelFlag, inputBuffer[20], confirmBuffer[8],
	     argNum, ipDataSwapFlag, arg0[4], verboseFlag, *datPtrC,
	     newWidthDoneFlag, newHeightDoneFlag, xsizDoneFlag, zsizDoneFlag,
	     inputLittleFlag, dataTypeDoneFlag, basDoneFlag, skipDoneFlag,
	     dataBits, shiftBit, removeFlag, setTimeFlag, magFlag, signFlag;
	unsigned char *datPtrUC;
	double rato, intensityDbl, arg1, pf1, pf2, mag;
	struct ipHead ipHeader;
	time_t timeNow;
	FILE *fp1, *fp2;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	readByte = 0;
	verboseFlag = 1;
	newWidthDoneFlag = newHeightDoneFlag = xsizDoneFlag = zsizDoneFlag = dataTypeDoneFlag
	 = basDoneFlag = skipDoneFlag = removeFlag = setTimeFlag = inputLittleFlag
	 = ipDataSwapFlag = magFlag = 0;
	skipBytes = 0;
	dataBits = 16;
	signFlag = 1;
	rato = 8.0;
	shiftBit = 3;
	maxInt = ((long)(0x7fff)) << shiftBit;   /* =8*32767 = 8*(2^15-1) */
	ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
	pf1 = pf2 = 1.0;
	mag = 1.0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif

/* Define ipHeader and Initialize */
	memset(&ipHeader, 0, ipHeaderSize);
	strcpy(ipHeader.name, "R-AXIS-DS"); /* Name of the Device */
	strcpy(ipHeader.vers, "1.0-2.0"); /* Version Number */
	strcpy(ipHeader.crnm, "N/A"); /* Crystal Name */
	strcpy(ipHeader.crsy, "cubic"); /* Crystal System */
	ipHeader.alng = 10.0; /* a-axis length in A */
	ipHeader.blng = 10.0; /* b-axis length in A */
	ipHeader.clng = 10.0; /* c-axis length in A */
	ipHeader.alfa = 90.0; /* angle alpha */
	ipHeader.beta = 90.0; /* angle beta */
	ipHeader.gamm = 90.0; /* angle gamma */
	strcpy(ipHeader.spgr, "P23"); /* Space Group */
	ipHeader.mosc = 0.3; /* Mozaic Spread */
	strcpy(ipHeader.opnm, "rigaku"); /* Operator Name */
	strcpy(ipHeader.trgt, "Cu"); /* X-ray Target */
	ipHeader.wlng = 1.5418; /* Wave Length of the x-ray */
	strcpy(ipHeader.mcro, "None"); /* Monochromator Material */
	ipHeader.m2ta = 0.0; /* Angle of the monochromator in 2-theta */
	strcpy(ipHeader.colm, "0.05-0.10"); /* Collimator */
	strcpy(ipHeader.filt, "Ni"); /* Kb filter */
	ipHeader.camr = 50.0; /* Camera length in mm */
	ipHeader.vltg = 40.0; /* X-ray tube voltage */
	ipHeader.crnt = 100.0; /* X-ray tube current */
	strcpy(ipHeader.focs, "0.3x0.3"); /* X-ray focus */
	strcpy(ipHeader.optc, "Ni-filter"); /* X-ray optics */
	ipHeader.cyld = 0; /* Camera type */
	ipHeader.weis = 0.0; /* weisenberg oscillation */
	strcpy(ipHeader.mnax, "-b*"); /* Mount Axis */
	strcpy(ipHeader.bmax, "-a"); /* Beam Axis */
	ipHeader.phi0 = 0.0; /* phi0 */
	ipHeader.phis = 0.0; /* phi start */
	ipHeader.phie = 0.0; /* phi end */
	ipHeader.oscn = 0; /* Frequency of Oscillation */
	ipHeader.fext = 20.0; /* Exposure Time in min. */
	ipHeader.omga = 0.0; /* omega in theta */
	ipHeader.fkai = 0.0; /* xai */
	ipHeader.thta = 0.000; /* 2-theta */
	ipHeader.mu = 0.000; /* mu */
	ipHeader.xpxl = WIDTH1; /* Pixel Number in X direction */
	ipHeader.zpxl = HEIGHT1; /* Pixel Number in Z direction */
	ipHeader.xsiz = XSIZ1; /* Pixel size in X direction in mm */
	ipHeader.zsiz = ZSIZ1; /* Pixel size in Y direction in mm */
	ipHeader.ipst = 0; /* Line of start reading */
	ipHeader.ipnm = 1; /* Number of IP */
	ipHeader.rato = rato; /* Ratio of IP in High/Low */
	ipHeader.ft_1 = 0.0; /* fading time 1 */
	ipHeader.ft_2 = 0.0; /* fading time 2 */
	strcpy(ipHeader.host, "IRIS"); /* Type of Host Computer */
	strcpy(ipHeader.ip, "HR-3"); /* Type of IP */
	ipHeader.dr_x = 0; /* Data direction in horizontal */
	ipHeader.dr_z = 0; /* Data direction in vertical */
	ipHeader.drxz = 0; /* Data direction face/back */
	timeNow = time(NULL);
	strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&timeNow));
	strcat(ipHeader.memo, ProgName);
	strcat(ipHeader.memo, "/");
	strcat(ipHeader.memo, ipHeader.date);
#ifdef _HASUTIME
	strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&utime1.modtime));
#endif
	
/* Get information about Orignal File */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Image Size is X :%5d x Z :%5d ? [Y/N] : ", WIDTH1, HEIGHT1);
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'N') {
			do {
				fprintf(stdout, "  Enter Image Width  : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				ipHeader.xpxl = atol(inputBuffer);
				if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH)) continue;
				newWidthDoneFlag = 1;
			} while (!newWidthDoneFlag);
			do {
				fprintf(stdout, "  Enter Image Height : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				ipHeader.zpxl = atol(inputBuffer);
				if ((ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) continue;
				newHeightDoneFlag = 1;
			} while (!newHeightDoneFlag);
		}
		if (verboseFlag) fprintf(stdout, "Image Size is set to X :%5ld x Z :%5ld .\n", ipHeader.xpxl, ipHeader.zpxl);
		do {
			fprintf(stdout, "PixelSize is X : %7.5f [mm] x Z : %7.5f [mm] ? [Y/N] : ", ipHeader.xsiz, ipHeader.zsiz);
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'N') {
			do {
				fprintf(stdout, "  Enter PixelSize in X [mm] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				ipHeader.xsiz = atof(inputBuffer);
				if (ipHeader.xsiz <= 0) continue;
				xsizDoneFlag = 1;
			} while (!xsizDoneFlag);
			do {
				fprintf(stdout, "  Enter PixelSize in Z [mm] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				ipHeader.zsiz = atof(inputBuffer);
				if (ipHeader.zsiz <= 0) continue;
				zsizDoneFlag = 1;
			} while (!zsizDoneFlag);
		}
		if (verboseFlag) fprintf(stdout, "Image Size is set to X : %7.5f [mm] x Z : %7.5f [mm].\n", ipHeader.xsiz, ipHeader.zsiz);
		do {
			fprintf(stdout, "Is each data 8-bits, 16-bits or 32-bits ? [8/16/32] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while (strcmp(confirmBuffer, "8") && strcmp(confirmBuffer, "16") && strcmp(confirmBuffer, "32"));
		if (!strcmp(confirmBuffer, "16") || !strcmp(confirmBuffer, "32")) {
			dataBits = (!strcmp(confirmBuffer, "16")) ? 16 : 32;
			do {
				fprintf(stdout, "Byte Sequence is assumed to be Big Endian (cf:Mac). OK ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) != 'N') inputLittleFlag = 1;
		}
		do {
			fprintf(stdout, " Assume unsigned value ? [Y/N] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') signFlag = 0;
		do {
			fprintf(stdout, "Enter Skip Bytes before Data [bytes] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			skipBytes = atol(inputBuffer);
			if (skipBytes < 0) continue;
			skipDoneFlag = 1;
		} while (!skipDoneFlag);
		do {
			fprintf(stdout, "Delete Original Raw File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') removeFlag = 1;
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
	else {
		for (i=1; i<argc-1; i++) {
			if (!strcmp(argv[i], "-bas")) {
				inputLittleFlag = 0;
				dataBits = 16;
				signFlag = 0;
				skipBytes = 0;
				pf1 = 7.615E-4;
				pf2 = 0;
				basDoneFlag++;
			}
			else if (!strcmp(argv[i], "-bas0")) {
				inputLittleFlag = 0;
				dataBits = 16;
				signFlag = 0;
				skipBytes = 0;
				pf1 = 6.073E-4;
				pf2 = 3.55;
				basDoneFlag++;
			}
			else if (!strcmp(argv[i], "-bas1")) {
				inputLittleFlag = 0;
				dataBits = 16;
				signFlag = 0;
				skipBytes = 0;
				pf1 = log(10.0)/0x0fff;
				pf2 = log(100.0/256.0);
				basDoneFlag++;
			}
			else if (!strcmp(argv[i], "-bas2")) {
				inputLittleFlag = 0;
				dataBits = 16;
				signFlag = 0;
				skipBytes = 0;
				pf1 = log(10.0)/0x0fff;
				pf2 = 6.0*log(2.0);
				basDoneFlag++;
			}
			else {
				argNum = splitArg(argv[i], arg0, &arg1);
				if (argNum == 1) {
					if (!strcmp(arg0, "-rm")) removeFlag++;
#ifdef _HASUTIME
					else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
					else break;
				}
				else if (argNum == 2) {
					if (!strcmp(arg0, "-b")) {
						dataBits = dToLong(arg1);
						if (dataBits == 16) {
							if (*(argv[i]+5) == 'L') inputLittleFlag = 1;
							else if (*(argv[i]+5) == 'B') inputLittleFlag = 0;
							else break;
							if (*(argv[i]+6) == 'u') signFlag = 0;
						}
						else if (dataBits == 32) {
							if (*(argv[i]+5) == 'L') inputLittleFlag = 1;
							else if (*(argv[i]+5) == 'B') inputLittleFlag = 0;
							else break;
							if (*(argv[i]+6) == 'u') signFlag = 0;
						}
						else if (dataBits == 8) {
							if (*(argv[i]+4) == 'u') signFlag = 0;
						}
						else break;
						dataTypeDoneFlag++;
					}
					else if (!strcmp(arg0, "-x")) {
						ipHeader.xpxl = dToLong(arg1);
						if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH)) break;
						newWidthDoneFlag++;
					}
					else if (!strcmp(arg0, "-z")) {
						ipHeader.zpxl = dToLong(arg1);
						if ((ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) break;
						newHeightDoneFlag++;
					}
					else if (!strcmp(arg0, "-dx")) {
						ipHeader.xsiz = arg1;
						if (ipHeader.xsiz <= 0) break;
						xsizDoneFlag++;
					}
					else if (!strcmp(arg0, "-dz")) {
						ipHeader.zsiz = arg1;
						if (ipHeader.zsiz <= 0) break;
						zsizDoneFlag++;
					}
					else if (!strcmp(arg0, "-of")) {
						skipBytes = dToLong(arg1);
						if (skipBytes < 0) break;
						skipDoneFlag++;
					}
					else if (!strcmp(arg0, "-m")) {
						mag = arg1;
						if (mag <= 0.0) break;
						magFlag++;
					}
					else break;
				}
				else break;
			}
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((basDoneFlag > 1) || (dataTypeDoneFlag > 1) || (newWidthDoneFlag > 1)
		 || (newHeightDoneFlag > 1) || (xsizDoneFlag > 1) || (zsizDoneFlag > 1)
		 || (skipDoneFlag > 1) || (removeFlag > 1) || (setTimeFlag > 1)
		 || (magFlag > 1)
		 || (!basDoneFlag && magFlag)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!basDoneFlag && (!dataTypeDoneFlag || !newWidthDoneFlag
		 || !newHeightDoneFlag || !xsizDoneFlag || !zsizDoneFlag)) {
			fprintf(stderr, " Lack of Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (basDoneFlag) {
			if (dataBits != 16 || signFlag) {
				dataBits = 16;
				signFlag = 0;
				if (verboseFlag) fprintf(stdout, " conflicted options with \"-bas\" : data size is set to unsigned 16bits/pixel.\n");
			}
		}
	}
	if (verboseFlag >= 2) {
		if (dataBits == 8) fprintf(stdout, " Input File is %d byte/data Raw File.\n", dataBits/8);
		else if (inputLittleFlag) fprintf(stdout, " Input File is %d bytes/data Raw File with Little Endian.\n", dataBits/8);
		else fprintf(stdout, " Input File is %d bytes/data Raw File with Big Endian.\n", dataBits/8);
		if (basDoneFlag) {
			fprintf(stdout, " Data Conversion is BAS type.\n");
			fprintf(stdout, "  I_new = exp( (%f) x I_orig + (%f) ) * %f.\n", pf1, pf2, mag);
		}
		fprintf(stdout, " PixelSize is set to %8.5f [mm] x %8.5f [mm].\n", ipHeader.xsiz, ipHeader.zsiz);
		fprintf(stdout, " %ld bytes Header in Original File is assumed.\n", skipBytes);
		if (removeFlag) fprintf(stdout, " Original Raw File will be removed !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Set Parameters to Create */
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(ipHeaderSizeMin, flushWidth);
	recordLength = ipHeader.xpxl*dataBits/8;
	flushWidth0 = recordLengthCreate - flushWidth;
	ipHeader.drtx = ipHeader.xpxl/2.0;
	ipHeader.drtz = ipHeader.zpxl/2.0;
	ipHeader.rlng = recordLengthCreate;
	ipHeader.rnum = ipHeader.zpxl;
	if (XOREXT(inputLittleFlag, cpuIntelFlag)) ipDataSwapFlag = 1;
	
/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter  Output  Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	while (createFileEXT(file2, &fp2, 0)) {
		if (!verboseFlag) {
			fprintf(stderr, " Output File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Swap Header */
	if (cpuIntelFlag) swapIpHeader(&ipHeader);
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSize, 1, fp2);
	if (skipBytes) {
		if (fseek(fp1, skipBytes, SEEK_SET)) {
			fprintf(stderr, " Input File Read Error.\n");
			free(datPtr0);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		readByte = ftell(fp1);
	}
	if (flushWidth0 == 0) free(datPtr0);
	
/* Swap Header again */
	if (cpuIntelFlag) swapIpHeader(&ipHeader);
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if (dataBits == 16) {
		if (basDoneFlag) {	/* unsigned short */
			for (i=0; i<ipHeader.rnum; i++) {
				if ((datPtrUS = malloc(recordLength)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				if ((datPtrS = malloc(recordLengthCreate)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					if (flushWidth0 > 0) free(datPtr0);
					free(datPtrUS);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtrUS, recordLength, 1, fp1);
				readByte += recordLength;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtrUS);
					free(datPtrS);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					if (ipDataSwapFlag) datPtrUS[j] = swapUShort(datPtrUS[j]);
					intensityDbl = exp(pf1 * datPtrUS[j] + pf2) * mag;
					datPtrS[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
				}
				fwrite(datPtrS, flushWidth, 1, fp2);
				if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
				free(datPtrUS);
				free(datPtrS);
			}
		}
		else if (!signFlag) {	/* unsigned short */
			for (i=0; i<ipHeader.rnum; i++) {
				if ((datPtrUS = malloc(recordLength)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				if ((datPtrS = malloc(recordLengthCreate)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrUS);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtrUS, recordLength, 1, fp1);
				readByte += recordLength;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtrUS);
					free(datPtrS);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					intensity = ipDataSwapFlag ? swapUShort(datPtrUS[j]) : datPtrUS[j];
					datPtrS[j] = ipLtoShortBit(&intensity, &rato, &shiftBit, &maxInt, &cpuIntelFlag);
				}
				fwrite(datPtrS, flushWidth, 1, fp2);
				if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
				free(datPtrUS);
				free(datPtrS);
			}
		}
		else {	/* signed short */
			for (i=0; i<ipHeader.rnum; i++) {
				if ((datPtrS = malloc(recordLengthCreate)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtrS, recordLength, 1, fp1);
				readByte += recordLength;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtrS);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					intensity = ipDataSwapFlag ? swapShort(datPtrS[j]) : datPtrS[j];
					datPtrS[j] = ipLtoShortBit(&intensity, &rato, &shiftBit, &maxInt, &cpuIntelFlag);
				}
				fwrite(datPtrS, flushWidth, 1, fp2);
				if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
				free(datPtrS);
			}
		}
	}
	else if (dataBits == 32) {
		if (signFlag) {	/* signed long */
			for (i=0; i<ipHeader.rnum; i++) {
				if ((datPtrL = malloc(recordLength)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				if ((datPtrS = malloc(recordLengthCreate)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrL);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtrL, recordLength, 1, fp1);
				readByte += recordLength;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtrL);
					free(datPtrS);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					if (ipDataSwapFlag) datPtrL[j] = swapLong(datPtrL[j]);
					datPtrS[j] = ipLtoShortBit(&(datPtrL[j]), &rato, &shiftBit, &maxInt, &cpuIntelFlag);
				}
				fwrite(datPtrS, flushWidth, 1, fp2);
				if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
				free(datPtrL);
				free(datPtrS);
			}
		}
		else {	/* unsigned long */
			for (i=0; i<ipHeader.rnum; i++) {
				if ((datPtrUL = malloc(recordLength)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				if ((datPtrS = malloc(recordLengthCreate)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrUL);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtrUL, recordLength, 1, fp1);
				readByte += recordLength;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtrUL);
					free(datPtrS);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					if (ipDataSwapFlag) datPtrUL[j] = swapULong(datPtrUL[j]);
					datPtrS[j] = ipULtoShortBit(&(datPtrUL[j]), &rato, &shiftBit, &maxInt, &cpuIntelFlag);
				}
				fwrite(datPtrS, flushWidth, 1, fp2);
				if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
				free(datPtrUL);
				free(datPtrS);
			}
		}
	}
	else if (dataBits == 8) {
		if (signFlag) {	/* signed char */
			for (i=0; i<ipHeader.rnum; i++) {
				if ((datPtrC = malloc(recordLength)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				if ((datPtrS = malloc(recordLengthCreate)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrC);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtrC, recordLength, 1, fp1);
				readByte += recordLength;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtrC);
					free(datPtrS);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					intensity = datPtrC[j];
					datPtrS[j] = ipLtoShortBit(&intensity, &rato, &shiftBit, &maxInt, &cpuIntelFlag);
				}
				fwrite(datPtrS, flushWidth, 1, fp2);
				if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
				free(datPtrC);
				free(datPtrS);
			}
		}
		else {	/* unsigned char */
			for (i=0; i<ipHeader.rnum; i++) {
				if ((datPtrUC = malloc(recordLength)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				if ((datPtrS = malloc(recordLengthCreate)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtrUC);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtrUC, recordLength, 1, fp1);
				readByte += recordLength;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtrUC);
					free(datPtrS);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					if (i) {
						ipHeader.zpxl = ipHeader.rnum = i;
						if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						fseek(fp2, 0, SEEK_SET);
						fwrite(&ipHeader, ipHeaderSize, 1, fp2);
						fprintf(stdout, " Output File is saved using available data.\n");
						if (cpuIntelFlag) swapIpHeader(&ipHeader);
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
							fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original Raw Data File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					intensity = datPtrUC[j];
					datPtrS[j] = ipLtoShortBit(&intensity, &rato, &shiftBit, &maxInt, &cpuIntelFlag);
				}
				fwrite(datPtrS, flushWidth, 1, fp2);
				if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
				free(datPtrUC);
				free(datPtrS);
			}
		}
	}
	
/* End */
	if (flushWidth0 > 0) free(datPtr0);
	fclose(fp1);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
	if (removeFlag) {
		if (remove(file1)) fprintf(stderr, " Unable to remove Original Raw Data File.\n");
		else if (verboseFlag) fprintf(stdout, " Original Raw Data File has Removed.\n");
	}
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-rm] [-t] -b:{8|16L|16B|32L|32B}[u]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-rm] -b:{8|16L|16B|32L|32B}[u]\n", ProgName);
#endif
	fprintf(fp0, "          -x:VALUE -z:VALUE -dx:VALUE -dz:VALUE [-of:VALUE] file2\n");
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-rm] [-t] [-bas|-bas0|-bas1|-bas2]\n", ProgName);
	fprintf(fp0, "          [-m:VALUE] file2\n");
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-rm] [-bas|-bas0|-bas1|-bas2] [-m:VALUE] file2\n", ProgName);
#endif
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Convert Raw Data Sequence File into R-AXIS Format.\n");
	fprintf(fp0, "  Original Data may be simple 8bits/16bits RawData, with Little/Big Endians.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -bas ........ Assume Original File to be taken in special circumstance.\n");
	fprintf(fp0, "                 Next 5 options may set to default value, if absent.\n");
	fprintf(fp0, "                 (-b:16B, -x:3760, -z:3000, -dx:0.025, -dz:0.025).\n");
	fprintf(fp0, "                 Intensity = exp(0.0007615 I);maximum conversion.\n");
	fprintf(fp0, "                 This case, set to treat data with Kobayashi\'s Lab in KUICR.\n");
	fprintf(fp0, "   -bas0 ....... Same as above, but use other function:\n");
	fprintf(fp0, "                 Intensity = exp(0.0006073 I + 3.55); empirical conversion.\n");
	fprintf(fp0, "   -bas1 ....... Same as above, but use yet other function:\n");
	fprintf(fp0, "                 Intensity = exp(0.0005621 I - 0.94); true conversion by Fuji.\n");
	fprintf(fp0, "   -bas2 ....... Same as above, but use yet other function:\n");
	fprintf(fp0, "                 Intensity = exp(0.0005621 I + 4.16); fitting for Fuji.\n");
	fprintf(fp0, "   -m:VALUE .... Multiply \"VALUE\" after intensity conversion.\n");
	fprintf(fp0, "   -b:8 ........ Assume Original File with 8-bits/pixel.\n");
	fprintf(fp0, "      16L ...... Assume Original File with 16-bits/pixel, little endian\n");
	fprintf(fp0, "      16B ...... Assume Original File with 16-bits/pixel, big endian\n");
	fprintf(fp0, "      32L ...... Assume Original File with 32-bits/pixel, little endian\n");
	fprintf(fp0, "      32B ...... Assume Original File with 32-bits/pixel, big endian\n");
	fprintf(fp0, "                 Following \"u\" means \"unsigned\" values in each case.\n");
	fprintf(fp0, "   -x:VALUE .... Set Width of Image to \"VALUE\".\n");
	fprintf(fp0, "   -z:VALUE .... Set Height of Image to \"VALUE\".\n");
	fprintf(fp0, "   -dx:VALUE ... Set PixelSize in X-direction to \"VALUE\".\n");
	fprintf(fp0, "   -dz:VALUE ... Set PixelSize in Z-direction to \"VALUE\".\n");
	fprintf(fp0, "   -of:VALUE ... Skip \"VALUE\" bytes of Input File before precess.\n");
	fprintf(fp0, "                 If absent, no skip.\n");
	fprintf(fp0, "   -rm ......... Remove Original Raw Data File after conversion.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t .......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ....... Filename of Original Data File.\n");
	fprintf(fp0, "   file2 ....... Output R-AXIS Image Filename. This must be the last argument.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	double *arg1;
#else
char splitArg(const char *argV, char *arg0, double *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	return (2);
}
