/* Program to convert IP data file */
#define ProgName "iprax2bmp"
#define ProgInfo "Convert R-AXIS Data File to Bitmap File."
/*                                                         6/12/2000  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.09_1/15/2007"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#ifdef _HASUTIME
#	include <sys/stat.h>
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"
#include "bmpheader.h"
#include "libbmp.h"

#ifdef _HASUTIME
#	define MAXARG 8
#else
#	define MAXARG 7
#endif
#define MINARG 2
#define MAXWIDTH 10000
#define MAXHEIGHT 10000

#ifdef _OLDHP
	long int (*readDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, long int *arg1);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr;
	long int i, j, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2,
	     ipHeaderSizeMin, maxRange, arg1, readByte, bmpHeaderSize,
	     bmpInfoHeaderSize, bmpColourHeaderSize, bmpRecordLength, lbuf;
	unsigned long int maxInt, highLimit, lowLimit, window;
	char file1[FNAMELEN], file2[FNAMELEN], inputBuffer[20], argNum,
	     confirmBuffer[8], cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag,
	     shiftBit, strangeShiftBitFlag, byte, removeFlag, setTimeFlag,
	     arg0[4], verboseFlag, lowLimitDoneFlag, highLimitDoneFlag,
	     expFlag, reverseFlag;
	unsigned char *datPtrC;
	double rato, intensityDbl, expFactor;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	struct bmpHead bmpHeader;
	struct bmpInfoHead bmpInfoHeader;
	struct bmpColourHead bmpPalette[256];
	FILE *fp1, *fp2;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	bmpHeaderSize = SIZE_OF_BMPHEADER;
	bmpInfoHeaderSize = SIZE_OF_BMPINFOHEADER;
	bmpColourHeaderSize = SIZE_OF_BMPCOLOURHEADER;
	verboseFlag = 1;
	readByte = 0;
	ipDataSwapFlag = lowLimitDoneFlag = highLimitDoneFlag = expFlag = reverseFlag
	 = removeFlag = setTimeFlag = 0;
	maxRange = 255;
	maxInt = 0x7fff;
	highLimit = lowLimit = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckBmp() == 1) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of Bitmap Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	if (compilerCheckBmp() == 2) {
		if (verboseFlag) fprintf(stdout, " Size of Bitmap Header mismatch ... skip.\n");
		bmpHeaderSize = SIZE_OF_BMPHEADER_EACH;
	}
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) ipHeader.zpxl = ipHeader.rnum;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	rato = ipHeader.rato;
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop. \n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
		}
	}
	
/* Get information about Output File */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Do you want to change range of 0 - %5ld ? [Y/N] : ", maxRange);
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			do {
				fprintf(stdout, "  Enter Lower Intensity to set to 0 : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				lbuf = atol(inputBuffer);
				if ((lbuf < 0) || ((lowLimit=lbuf) > maxInt)) continue;
				lowLimitDoneFlag = 1;
			} while (!lowLimitDoneFlag);
			do {
				fprintf(stdout, "  Enter Higher Intensity to set to %5ld : ", maxRange);
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				lbuf = atol(inputBuffer);
				if ((lbuf < 0) || ((highLimit=lbuf) <= lowLimit) || (highLimit > maxInt)) continue;
				highLimitDoneFlag = 1;
			} while (!highLimitDoneFlag);
		}
		else {
			lowLimit = 0;
			highLimit = maxRange;
		}
		do {
			fprintf(stdout, "Reverse Intensity (i.e. \"0\" corresponds to WHITE) ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') reverseFlag = 1;
		do {
			fprintf(stdout, "Convert with Logarithmic Function ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') expFlag = 1;
		do {
			fprintf(stdout, "Delete Original R-AXIS File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') removeFlag = 1;
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-rm")) removeFlag++;
				else if (!strcmp(arg0, "-e")) expFlag++;
				else if (!strcmp(arg0, "-r")) reverseFlag++;
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-rl")) {
					if ((arg1 < 0.0) || ((lowLimit=dToULong(arg1)) > maxInt)) break;
					lowLimitDoneFlag++;
				}
				else if (!strcmp(arg0, "-rh")) {
					if ((arg1 <= 0.0) || ((highLimit=dToULong(arg1)) > maxInt)) break;
					highLimitDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((expFlag > 1) || (reverseFlag > 1) || (lowLimitDoneFlag > 1)
		 || (highLimitDoneFlag > 1) || (removeFlag > 1) || (setTimeFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!lowLimitDoneFlag) lowLimit = 0;
		if (!highLimitDoneFlag) highLimit = maxRange;
		if (highLimit <= lowLimit) {
			fprintf(stderr, " Arguments out of range.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	if (verboseFlag >= 2) {
		if (reverseFlag) fprintf(stdout, " Convert %ld - %ld in Original R-AXIS File to White - Black.\n", lowLimit, highLimit);
		else fprintf(stdout, " Convert %ld - %ld in Original R-AXIS File to Black - White.\n", lowLimit, highLimit);
		if (expFlag) fprintf(stdout, " Data is converted with Logarithmic Function.\n");
		if (removeFlag) fprintf(stdout, " Original R-AXIS Image File will be removed !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Set Parameters */
	bmpRecordLength = dToLong(ceil(ipHeader.xpxl / 4.0)) * 4;
	strcpy(bmpHeader.bfType, "BM");	/* FileType for Bitmap */
	bmpHeader.bfReserved1 = 0;	/* Reserved */
	bmpHeader.bfReserved2 = 0;	/* Reserved */
	bmpInfoHeader.biWidth = ipHeader.xpxl;	/* Width of Image */
	bmpInfoHeader.biHeight = ipHeader.zpxl;	/* Height of Image */
	bmpInfoHeader.biPlanes = 1;	/* Number of Planes */
	bmpInfoHeader.biBitCount = 8;	/* Number of Bits par Pixel */
	bmpInfoHeader.biCompression = 0;	/* Compression Type */
	bmpInfoHeader.biXPelsPerMeter = 1000/ipHeader.xsiz;	/* Horizontal Resolution */
	bmpInfoHeader.biYPelsPerMeter = 1000/ipHeader.zsiz;	/* Vertical Resolution */
	bmpInfoHeader.biClrUsed = 256;	/* Number of Colour */
	bmpInfoHeader.biClrImportant = 0;	/* Number of Important Colour */
	bmpInfoHeader.biSize = bmpInfoHeaderSize;		/* Size of Information Header */
	bmpInfoHeader.biSizeImage = bmpInfoHeader.biHeight * bmpRecordLength;	/* Size of Image */
	bmpHeader.bfOffBits = bmpHeaderSize + bmpInfoHeaderSize + 256 * bmpColourHeaderSize;	/* Size of File Header */
	bmpHeader.bfSize = bmpHeader.bfOffBits + bmpInfoHeader.biSizeImage;		/* Size of Bitmap */
	for (i=0; i<256; i++) {
		bmpPalette[i].rgbBlue = bmpPalette[i].rgbGreen = bmpPalette[i].rgbRed = i;
		bmpPalette[i].rgbReserved = 0;
	}
	if (!cpuIntelFlag) {
		swapBmpHeader(&bmpHeader);
		swapBmpInfoHeader(&bmpInfoHeader);
	}
	window = highLimit - lowLimit;
	expFactor = maxRange / log(window);
	
/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter  Output  Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	while (createFileEXT(file2, &fp2, 0)) {
		if (!verboseFlag) {
			fprintf(stderr, " Output File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	if (compilerCheckBmp() == 2) {
		fwrite(&bmpHeader.bfType, 2, 1, fp2);
		fseek(fp2, 2, SEEK_SET);
		fwrite(&bmpHeader.bfSize, 4, 1, fp2);
		fwrite(&bmpHeader.bfReserved1, 2, 1, fp2);
		fwrite(&bmpHeader.bfReserved2, 2, 1, fp2);
		fwrite(&bmpHeader.bfOffBits, 4, 1, fp2);
		if (ftell(fp2) != bmpHeaderSize) {
			fprintf(stderr, " Output File Header Error.\n");
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
	}
	else fwrite(&bmpHeader, bmpHeaderSize, 1, fp2);
	fwrite(&bmpInfoHeader, bmpInfoHeaderSize, 1, fp2);
	for (i=0; i<256; i++) fwrite(&bmpPalette[i], bmpColourHeaderSize, 1, fp2);
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if (fseek(fp1, ipHeader.rlng, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	for (i=0; i<ipHeader.rnum; i++) {
		if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		if ((datPtrC = malloc(bmpRecordLength)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtr);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		fread(datPtr, ipHeader.rlng, 1, fp1);
		readByte += ipHeader.rlng;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
			free(datPtr);
			free(datPtrC);
			fclose(fp1);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		for (j=0; j<ipHeader.xpxl; j++) {
			intensityDbl = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
			if (intensityDbl >= highLimit) intensityDbl = maxRange;
			else if (intensityDbl <= lowLimit) intensityDbl = 0;
			else if (expFlag) intensityDbl = log(intensityDbl - lowLimit + 1.0) * expFactor;
			else intensityDbl = (intensityDbl - lowLimit) * maxRange / window;
			if (reverseFlag) intensityDbl = maxRange - intensityDbl;
			datPtrC[j] = dToShort(intensityDbl);
		}
		for (j=ipHeader.xpxl; j<bmpRecordLength; j++) datPtrC[j] = 0;
		fwrite(datPtrC, bmpRecordLength, 1, fp2);
		free(datPtr);
		free(datPtrC);
	}
	
/* End */
	fclose(fp1);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels, each pixel has %d bits,\n", bmpInfoHeader.biWidth, bmpInfoHeader.biHeight, bmpInfoHeader.biBitCount);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", bmpRecordLength, bmpInfoHeader.biHeight, ftell(fp2));
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
	if (removeFlag) {
		if (remove(file1)) fprintf(stderr, " Unable to remove Original R-AXIS Image File.\n");
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Removed.\n");
	}
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-rm] [-t] [-r] [-e]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-rm] [-r] [-e]\n", ProgName);
#endif
	fprintf(fp0, "          [-rl:VALUE] [-rh:VALUE2] file2\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Convert R-AXIS Image File to 8-bits gray-scale Bitmap File.\n");
	fprintf(fp0, "  Note that R-AXIS File has about 18bits-depth, which is much greater than\n");
	fprintf(fp0, "  Bitmap File can hold. You must specify the way.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -r .......... Reverse Intensity. i.e. \"0\" corresponds to White.\n");
	fprintf(fp0, "   -e .......... Intensity is converted with Logarithmic Function.\n");
	fprintf(fp0, "   -rl:VALUE ... Set Lower Critical Intensity for convertion to 0 to \"VALUE\".\n");
	fprintf(fp0, "                 If absent, set to 0.\n");
	fprintf(fp0, "   -rh:VALUE ... Set Higher Critical Intensity for convertion to 255\n");
	fprintf(fp0, "                 to \"VALUE\". If absent, set to 255.\n");
	fprintf(fp0, "   -rm ......... Remove Original R-AXIS Image File after conversion.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t .......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ....... Filename of R-AXIS Image Data File.\n");
	fprintf(fp0, "   file2 ....... Output Bitmap Filename. This must be the last argument.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	long int *arg1;
#else
char splitArg(const char *argV, char *arg0, long int *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atol(chPtr1+1);
	return (2);
}
