/* Program to convert IP data file */
#define ProgName "iprax2raw"
#define ProgInfo "Conver R-AXIS File to Raw Data File."
/*                                                        12/27/1999  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.19_1/15/2007"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#ifdef _HASUTIME
#	include <sys/stat.h>
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef _HASUTIME
#	define MAXARG 8
#else
#	define MAXARG 7
#endif
#define MINARG 2
#define MAXWIDTH 10000
#define MAXHEIGHT 10000

#ifdef _OLDHP
	long int (*readDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, long int *arg1);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr, *datPtrCreateS;
	unsigned short int *datPtrCreateUS;
	long int i, j, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2,
	     ipHeaderSizeMin, sub, lbuf,
	     arg1, readByte, recordLengthCreate, *datPtrCreateL;
	unsigned long int maxRange, *datPtrCreateUL, maxInt, highLimit, lowLimit;
	char file1[FNAMELEN], file2[FNAMELEN], inputBuffer[20], argNum,
	     confirmBuffer[8], cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag,
	     shiftBit, strangeShiftBitFlag, byte, createDosFlag, dataBits, signFlag,
	     arg0[4], verboseFlag, dataTypeDoneFlag, lowLimitDoneFlag, highLimitDoneFlag,
	     basDoneFlag, outputLittleFlag, ipDataSwapFlagCreate, basFlag,
	     removeFlag, setTimeFlag, *datPtrCreateC;
	unsigned char *datPtrCreateUC;
	double rato, intensityDbl, coef;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	verboseFlag = 1;
	readByte = 0;
	ipDataSwapFlag = outputLittleFlag = lowLimitDoneFlag = highLimitDoneFlag
	 = basDoneFlag = dataTypeDoneFlag = basFlag = removeFlag = signFlag = setTimeFlag = 0;
	maxInt = 0x7fff;
	highLimit = lowLimit = 0;
	dataBits = 2;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
		createDosFlag = 1;
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
		createDosFlag = 0;
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) ipHeader.zpxl = ipHeader.rnum;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	rato = ipHeader.rato;
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop. \n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
		}
	}
	
/* Get information about Output File */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Is output raw data size/pixel, 8-bits, 16-bits or 32-bits ? [8/16/32] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while (strcmp(confirmBuffer, "8") && strcmp(confirmBuffer, "16") && strcmp(confirmBuffer, "32"));
		if (!strcmp(confirmBuffer, "32")) dataBits = 32;
		else if (!strcmp(confirmBuffer, "16")) dataBits = 16;
		else dataBits = 8;
		do {
			fprintf(stdout, "  Signed or Unsigned ? [s/u] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'S') && (toupper(confirmBuffer[0]) != 'U'));
		if (toupper(confirmBuffer[0]) == 'S') signFlag = 1;
		else signFlag = 0;
		if (dataBits == 32) {
			if (signFlag) maxRange = 0x7fffffff;
			else maxRange = 0xffffffff;
		}
		else maxRange = (0x00000001<<(dataBits-signFlag)) - 1;
		if (!strcmp(confirmBuffer, "16") || !strcmp(confirmBuffer, "32")) {
			do {
				fprintf(stdout, "Byte Sequence is assumed to be Big Endian (cf:Mac). OK ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) != 'N') outputLittleFlag = 1;
		}
		do {
			fprintf(stdout, "Do you want to change range from 0-%10ld ? [Y/N] : ", maxRange);
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) != 'Y') {
			do {
				fprintf(stdout, "  Enter Lower Intensity to set to 0 : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				lbuf = atol(inputBuffer);
				if ((lbuf < 0) || ((lowLimit=lbuf) > maxInt)) continue;
				lowLimitDoneFlag = 1;
			} while (!lowLimitDoneFlag);
			do {
				fprintf(stdout, "  Enter Higher Intensity to set to %5ld : ", maxRange);
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				lbuf = atol(inputBuffer);
				if ((lbuf < 0) || ((highLimit=lbuf) <= lowLimit) || (highLimit > maxInt)) continue;
				highLimitDoneFlag = 1;
			} while (!highLimitDoneFlag);
		}
		else {
			lowLimit = 0;
			highLimit = maxRange;
		}
		do {
			fprintf(stdout, "Delete Original R-AXIS File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') removeFlag = 1;
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
	else {
		for (i=1; i<argc-1; i++) {
			if (!strcmp(argv[i], "-bas")) {
				outputLittleFlag = 0;
				dataBits = 16;
				signFlag = 1;
				basFlag = 1;
				basDoneFlag++;
			}
			else if (!strcmp(argv[i], "-bas0")) {
				outputLittleFlag = 0;
				dataBits = 16;
				signFlag = 1;
				basFlag = 2;
				basDoneFlag++;
			}
			else {
				argNum = splitArg(argv[i], arg0, &arg1);
				if (argNum == 1) {
					if (!strcmp(arg0, "-rm")) removeFlag++;
#ifdef _HASUTIME
					else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
					else break;
				}
				else if (argNum == 2) {
					if (!strcmp(arg0, "-b")) {
						dataBits = arg1;
						if (dataBits == 16) {
							if (*(argv[i]+5) == 'L') outputLittleFlag = 1;
							else if (*(argv[i]+5) == 'B') outputLittleFlag = 0;
							else break;
							if (*(argv[i]+6) == 's') signFlag = 1;
							else if (*(argv[i]+6) == 'u') signFlag = 0;
							else break;
						}
						else if (dataBits == 32) {
							if (*(argv[i]+5) == 'L') outputLittleFlag = 1;
							else if (*(argv[i]+5) == 'B') outputLittleFlag = 0;
							else break;
							if (*(argv[i]+6) == 's') signFlag = 1;
							else if (*(argv[i]+6) == 'u') signFlag = 0;
							else break;
						}
						else if (dataBits == 8) {
							if (*(argv[i]+5) == 's') signFlag = 1;
							else if (*(argv[i]+5) == 'u') signFlag = 0;
							else break;
						}
						else break;
						dataTypeDoneFlag++;
					}
					else if (!strcmp(arg0, "-rl")) {
						if ((arg1 < 0.0) || ((lowLimit=dToULong(arg1)) > maxInt)) break;
						lowLimitDoneFlag++;
					}
					else if (!strcmp(arg0, "-rh")) {
						if ((arg1 <= 0.0) || ((highLimit=dToULong(arg1)) > maxInt)) break;
						highLimitDoneFlag++;
					}
					else break;
				}
				else break;
			}
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((dataTypeDoneFlag > 1) || (basDoneFlag > 1) || (lowLimitDoneFlag > 1)
		 || (highLimitDoneFlag > 1) || (removeFlag > 1) || (setTimeFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!basDoneFlag && !dataTypeDoneFlag) {
			fprintf(stderr, " Lack of Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!lowLimitDoneFlag) lowLimit = 0;
		if (!highLimitDoneFlag) highLimit = maxInt;
		if (highLimit <= lowLimit) {
			fprintf(stderr, " Arguments out of range.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (dataBits == 32) {
			if (signFlag) maxRange = 0x7fffffff;
			else maxRange = 0xffffffff;
		}
		else maxRange = (0x00000001<<(dataBits-signFlag)) - 1;
		if (basFlag) {
			if (dataBits != 16 || signFlag) {
				dataBits = 16;
				signFlag = 0;
				if (verboseFlag) fprintf(stdout, " conflicted options with \"-bas\" : data size is set to unsigned 16bits/pixel.\n");
			}
			maxRange = ((0x0001)<<14) - 1;
			if (lowLimit == 0) lowLimit = 1;
			if (basFlag == 2) {
				highLimit = dToLong(exp(6.073E-4 * maxRange + 3.55));
				lowLimit = 1;
				if (verboseFlag) {
					if (lowLimitDoneFlag) fprintf(stdout, " \"-rl\" is ignored: set to %ld.\n", lowLimit);
					if (highLimitDoneFlag) fprintf(stdout, " \"-rh\" is ignored: set to %ld.\n", highLimit);
				}
			}
		}
	}
	if (verboseFlag >= 2) {
		if ((dataBits == 8) && signFlag) fprintf(stdout, " Output File is signed %d byte/data Raw File.\n", dataBits/8);
		else if ((dataBits == 8) && !signFlag) fprintf(stdout, " Output File is unsigned %d byte/data Raw File.\n", dataBits/8);
		else if (outputLittleFlag && signFlag) fprintf(stdout, " Output File is signed %d bytes/data Raw File with Little Endian.\n", dataBits/8);
		else if (outputLittleFlag && !signFlag) fprintf(stdout, " Output File is unsigned %d bytes/data Raw File with Little Endian.\n", dataBits/8);
		else if (signFlag) fprintf(stdout, " Output File is signed %d bytes/data Raw File with Big Endian.\n", dataBits/8);
		else fprintf(stdout, " Output File is unsigned %d bytes/data Raw File with Big Endian.\n", dataBits/8);
		if (basFlag == 1) fprintf(stdout, " Data Conversion is BAS-maximum type.\n");
		else if (basFlag == 2) fprintf(stdout, " Data Conversion is BAS-empirical type.\n");
		fprintf(stdout, " Intensities in %ld - %ld range have effective scale.\n", lowLimit, highLimit);
		if (removeFlag) fprintf(stdout, " Original R-AXIS Image File will be removed !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Set Parameters */
	ipDataSwapFlagCreate = XOREXT(cpuIntelFlag, outputLittleFlag);
	if (basFlag == 1) {
		coef = maxRange / log(highLimit/lowLimit);
		sub = log(lowLimit);
	}
	else if (basFlag == 2) {
		coef = 1.0 / (6.073E-4);
		sub = 3.55;
	}
	else {
		coef = maxRange / (highLimit - lowLimit);
		sub = lowLimit;
	}
	recordLengthCreate = ipHeader.xpxl*dataBits/8;
	
/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter  Output  Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	while (createFileEXT(file2, &fp2, 0)) {
		if (!verboseFlag) {
			fprintf(stderr, " Output File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if (fseek(fp1, ipHeader.rlng, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	if (dataBits == 16) {
		if (basFlag) {	/* signed short, bas1/2 */
			for (i=0; i<ipHeader.rnum; i++) {
				if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				if ((datPtrCreateS = malloc(recordLengthCreate)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtr);
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtr, ipHeader.rlng, 1, fp1);
				readByte += ipHeader.rlng;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtr);
					free(datPtrCreateS);
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					intensityDbl = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
					if (intensityDbl >= highLimit) intensityDbl = maxRange;
					else if (intensityDbl <= lowLimit) intensityDbl = 0;
					else intensityDbl = (log(intensityDbl) - sub) * coef;
					datPtrCreateS[j] = dToUShort(intensityDbl);
					if (ipDataSwapFlagCreate) datPtrCreateS[j] = swapShort(datPtrCreateS[j]);
				}
				fwrite(datPtrCreateS, recordLengthCreate, 1, fp2);
				free(datPtr);
				free(datPtrCreateS);
			}
		}
		else if (signFlag) {	/* signed short */
			for (i=0; i<ipHeader.rnum; i++) {
				if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				if ((datPtrCreateS = malloc(recordLengthCreate)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtr);
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtr, ipHeader.rlng, 1, fp1);
				readByte += ipHeader.rlng;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtr);
					free(datPtrCreateS);
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					intensityDbl = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
					if (intensityDbl >= highLimit) intensityDbl = highLimit;
					else if (intensityDbl <= lowLimit) intensityDbl = lowLimit;
					else intensityDbl = (intensityDbl - sub) * coef;
					datPtrCreateS[j] = dToShort(intensityDbl);
					if (ipDataSwapFlagCreate) datPtrCreateS[j] = swapShort(datPtrCreateS[j]);
				}
				fwrite(datPtrCreateS, recordLengthCreate, 1, fp2);
				free(datPtr);
				free(datPtrCreateS);
			}
		}
		else {	/* unsigned short */
			for (i=0; i<ipHeader.rnum; i++) {
				if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				if ((datPtrCreateUS = malloc(recordLengthCreate)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtr);
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtr, ipHeader.rlng, 1, fp1);
				readByte += ipHeader.rlng;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtr);
					free(datPtrCreateUS);
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					intensityDbl = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
					if (intensityDbl >= highLimit) intensityDbl = highLimit;
					else if (intensityDbl <= lowLimit) intensityDbl = lowLimit;
					else intensityDbl = (intensityDbl - sub) * coef;
					datPtrCreateUS[j] = dToUShort(intensityDbl);
					if (ipDataSwapFlagCreate) datPtrCreateUS[j] = swapUShort(datPtrCreateUS[j]);
				}
				fwrite(datPtrCreateUS, recordLengthCreate, 1, fp2);
				free(datPtr);
				free(datPtrCreateUS);
			}
		}
	}
	else if (dataBits == 32) {
		if (signFlag) {	/* signed long */
			for (i=0; i<ipHeader.rnum; i++) {
				if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				if ((datPtrCreateL = malloc(recordLengthCreate)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtr);
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtr, ipHeader.rlng, 1, fp1);
				readByte += ipHeader.rlng;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtr);
					free(datPtrCreateL);
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					intensityDbl = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
					if (intensityDbl >= highLimit) intensityDbl = highLimit;
					else if (intensityDbl <= lowLimit) intensityDbl = lowLimit;
					else intensityDbl = (intensityDbl - lowLimit) * coef;
					datPtrCreateL[j] = dToLong(intensityDbl);
					if (ipDataSwapFlagCreate) datPtrCreateL[j] = swapLong(datPtrCreateL[j]);
				}
				fwrite(datPtrCreateL, recordLengthCreate, 1, fp2);
				free(datPtr);
				free(datPtrCreateL);
			}
		}
		else {	/* unsigned long */
			for (i=0; i<ipHeader.rnum; i++) {
				if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				if ((datPtrCreateUL = malloc(recordLengthCreate)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtr);
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtr, ipHeader.rlng, 1, fp1);
				readByte += ipHeader.rlng;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtr);
					free(datPtrCreateUL);
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					intensityDbl = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
					if (intensityDbl >= highLimit) intensityDbl = highLimit;
					else if (intensityDbl <= lowLimit) intensityDbl = lowLimit;
					else intensityDbl = (intensityDbl - lowLimit) * coef;
					datPtrCreateUL[j] = dToLong(intensityDbl);
					if (ipDataSwapFlagCreate) datPtrCreateUL[j] = swapULong(datPtrCreateUL[j]);
				}
				fwrite(datPtrCreateUL, recordLengthCreate, 1, fp2);
				free(datPtr);
				free(datPtrCreateUL);
			}
		}
	}
	else {
		if (signFlag) {	/* signed char */
			for (i=0; i<ipHeader.rnum; i++) {
				if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				if ((datPtrCreateC = malloc(recordLengthCreate)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtr);
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtr, ipHeader.rlng, 1, fp1);
				readByte += ipHeader.rlng;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtr);
					free(datPtrCreateC);
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					intensityDbl = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
					if (intensityDbl >= highLimit) intensityDbl = highLimit;
					else if (intensityDbl <= lowLimit) intensityDbl = lowLimit;
					else intensityDbl = (intensityDbl - lowLimit) * coef;
					datPtrCreateC[j] = dToShort(intensityDbl);
				}
				fwrite(datPtrCreateC, recordLengthCreate, 1, fp2);
				free(datPtr);
				free(datPtrCreateC);
			}
		}
		else {	/* unsigned char */
			for (i=0; i<ipHeader.rnum; i++) {
				if ((datPtr = malloc(ipHeader.rlng)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				if ((datPtrCreateUC = malloc(recordLengthCreate)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					free(datPtr);
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				fread(datPtr, ipHeader.rlng, 1, fp1);
				readByte += ipHeader.rlng;
				if (ftell(fp1) != readByte) {
					fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
					free(datPtr);
					free(datPtrCreateUC);
					fclose(fp1);
					if (i) {
						fprintf(stdout, " Output File is saved using available data.\n");
						if (verboseFlag) {
							fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, i, ftell(fp2));
							if (removeFlag) fprintf(stdout, " Original R-AXIS Image File is not removed.\n");
						}
						fclose(fp2);
#ifdef _HASUTIME
						if (setTimeFlag) {
							if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
						}
						if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
					}
					else {
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
					}
					exit(EXIT_FAILURE);
				}
				for (j=0; j<ipHeader.xpxl; j++) {
					intensityDbl = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
					if (intensityDbl >= highLimit) intensityDbl = highLimit;
					else if (intensityDbl <= lowLimit) intensityDbl = lowLimit;
					else intensityDbl = (intensityDbl - lowLimit) * coef;
					datPtrCreateUC[j] = dToUShort(intensityDbl);
				}
				fwrite(datPtrCreateUC, recordLengthCreate, 1, fp2);
				free(datPtr);
				free(datPtrCreateUC);
			}
		}
	}
	
/* End */
	fclose(fp1);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels, total %ld bytes.\n", ipHeader.xpxl, ipHeader.zpxl, ftell(fp2));
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
	if (removeFlag) {
		if (remove(file1)) fprintf(stderr, " Unable to remove Original R-AXIS Image File.\n");
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Removed.\n");
	}
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-rm] [-t] [-bas|-bas0]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-rm] [-bas|-bas0]\n", ProgName);
#endif
	fprintf(fp0, "          -b:{8|16L|16B|32L|32B}{u|s} [-rl:VALUE] [-rh:VALUE2] file2\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Convert R-AXIS Image File to Raw Data Sequence File.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -bas ........ Convert to Data File useful in special circumstance.\n");
	fprintf(fp0, "                 Next 3 options may set to default value, if absent.\n");
	fprintf(fp0, "                 Output Data must be unsigned 16-bits/pixel.\n");
	fprintf(fp0, "                 Intensity = log(I)/0.0007615; maximum conversion.\n");
	fprintf(fp0, "                 This case, set to treat data with Kobayashi\'s Lab in KUICR.\n");
	fprintf(fp0, "   -bas0 ....... Same as above, but use other function:\n");
	fprintf(fp0, "                 Intensity = (log(I)-3.55)/0.0006073; empirical conversion.\n");
	fprintf(fp0, "   -b:8u ....... Output Data with signed 8-bits/pixel.\n");
	fprintf(fp0, "      16Lu ..... Output Data with signed 16-bits/pixel, little endian\n");
	fprintf(fp0, "      16Bu ..... Output Data with signed 16-bits/pixel, big endian\n");
	fprintf(fp0, "      32Lu ..... Output Data with signed 32-bits/pixel, little endian\n");
	fprintf(fp0, "      32Bu ..... Output Data with signed 32-bits/pixel, big endian\n");
	fprintf(fp0, "      8s ....... Output Data with unsigned 8-bits/pixel.\n");
	fprintf(fp0, "      16Ls ..... Output Data with unsigned 16-bits/pixel, little endian\n");
	fprintf(fp0, "      16Bs ..... Output Data with unsigned 16-bits/pixel, big endian\n");
	fprintf(fp0, "      32Ls ..... Output Data with unsigned 32-bits/pixel, little endian\n");
	fprintf(fp0, "      32Bs ..... Output Data with unsigned 32-bits/pixel, big endian\n");
	fprintf(fp0, "   -rl:VALUE ... Set Lower Critical Intensity for convertion to 0 to \"VALUE\".\n");
	fprintf(fp0, "                 If absent, set to 0 (or 1 at \"-bas\" cases).\n");
	fprintf(fp0, "   -rh:VALUE ... Set Higher Critical Intensity for convertion to (2^n)-1\n");
	fprintf(fp0, "                 to \"VALUE\". If absent, set to (2^n)-1, where n is bits/pixel.\n");
	fprintf(fp0, "   -rm ......... Remove Original R-AXIS Image File after conversion.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t .......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ....... Filename of R-AXIS Image Data File.\n");
	fprintf(fp0, "   file2 ....... Output Raw Data Filename. This must be the last argument.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	long int *arg1;
#else
char splitArg(const char *argV, char *arg0, long int *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atol(chPtr1+1);
	return (2);
}
