/* Program to convert IP data file */
#define ProgName "ipresize"
#define ProgInfo "Resize Pixel Size"
/*                                                        12/16/1999  */
/*         for command-line argument                      12/20/1999  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.21_1/15/2007"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#ifdef _HASUTIME
#	include <sys/stat.h>
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef _HASUTIME
#	define MAXARG 5
#else
#	define MAXARG 4
#endif
#define MINARG 2
#define MAXMAG 20.0
#define MAXWIDTH 10000
#define MAXHEIGHT 10000

#ifdef _OLDHP
	long int (*readDat)();
	short int (*writeDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	short int (*writeDat)(const double *x, const double *rato, const unsigned long int *maxInt, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1, double *arg2);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr1, *datPtr2;
	long int i, j, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2, recordLength,
	     ipHeaderSizeMin, recordLengthCreate, xpxlOld,
	     imax, imin, itmp, jmax, jmin, jtmp, readByte, filesize,
	     flushWidth, flushWidth0, xpxlM, jmaxL, dblBufSize;
	unsigned long int maxInt;
	char file1[FNAMELEN], file2[FNAMELEN], cpuIntelFlag, ipHeaderSwapFlag,
	     shiftBit, strangeShiftBitFlag, ipDataSwapFlag, byte, createDosFlag,
	     argNum, arg0[4], verboseFlag, xsizDoneFlag, zsizDoneFlag,
	     confirmBuffer[8], inputBuffer[20], replaceFlag, setTimeFlag;
#ifdef _HAS_STRANGE_RENAME
	char fnameTmp[FNAMELEN];
#endif
	double rato, xsiz, zsiz, magX, magZ, *datPtrDouble, arg1, arg2;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	verboseFlag = 1;
	readByte = 0;
	ipDataSwapFlag = xsizDoneFlag = zsizDoneFlag = replaceFlag = setTimeFlag = 0;
	magX = magZ = 1.0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Argument 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
		if (!strcmp(argv[argc-1], "-replace")) replaceFlag = 1;
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
		createDosFlag = 1;
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
		createDosFlag = 0;
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (createDosFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) {
		ipHeader.zpxl = ipHeader.rnum;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height has modified.\n");
	}
	recordLength = ipHeader.rlng;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) {
		ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width has modified.\n");
	}
	xpxlOld = ipHeader.xpxl;
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	xsiz = ipHeader.xsiz;
	zsiz = ipHeader.zsiz;
	rato = ipHeader.rato;
	filesize = recordLength * (ipHeader.rnum + 1);
	if (ipHeader.cyld >= 256) {
		if (verboseFlag) fprintf(stdout, " Camera Type Code has changed from %ld to 0.\n", ipHeader.cyld);
		ipHeader.cyld = 0;
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
	}
	
/* Clean Header */
	cleanIpHeader(&ipHeader);
	createDosFlag ? cleanIpHeader2VER1(&ipHeader2VER1) : cleanIpHeader2VER2(&ipHeader2VER2);
	
/* Logging */
	if (logWrite(&ipHeader, ProgName)) {
		if (verboseFlag) fprintf(stdout, " ipHeader.memo is full, and No Logging.\n");
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop. \n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
		writeDat = ipDtoShortBit0;
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
				writeDat = ipDtoShort;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
			writeDat = ipDtoShort;
		}
	}
	
/* Get New Pixel Size */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "PixelSize in X is : %7.5f . Is it correct ? [Y/N] : ", ipHeader.xsiz);
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'N') {
			do {
				fprintf(stdout, "Enter Correct Value for Pixel Size in X : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				xsiz = atof(inputBuffer);
				if (xsiz <= 0.0) continue;
				xsizDoneFlag = 1;
			} while (!xsizDoneFlag);
			xsizDoneFlag = 0;
		}
		do {
			fprintf(stdout, "PixelSize in Z is : %7.5f . Is it correct ? [Y/N] : ", ipHeader.zsiz);
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'N') {
			do {
				fprintf(stdout, "Enter Correct Value for PixelSize in Z : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				zsiz = atof(inputBuffer);
				if (zsiz <= 0.0) continue;
				zsizDoneFlag = 1;
			} while (!zsizDoneFlag);
			zsizDoneFlag = 0;
		}
		do {
			fprintf(stdout, "Enter New Value for PixelSize in X : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.xsiz = atof(inputBuffer);
			magX = ipHeader.xsiz / xsiz;
			if ((magX < 1.0) || (magX > MAXMAG)) continue;
			xsizDoneFlag = 1;
		} while (!xsizDoneFlag);
		do {
			fprintf(stdout, "Enter New Value for PixelSize in Z : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.zsiz = atof(inputBuffer);
			magZ = ipHeader.zsiz / zsiz;
			if ((magZ < 1.0) || (magZ > MAXMAG)) continue;
			zsizDoneFlag = 1;
		} while (!zsizDoneFlag);
		do {
			fprintf(stdout, "Replace Original R-AXIS File with Output File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') replaceFlag = 1;
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
	else {
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1, &arg2);
			if (argNum == 1) {
#ifdef _HASUTIME
				if (!strcmp(arg0, "-t")) setTimeFlag++;
				else break;
#else
				break;
#endif
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-x")) {
					xsiz = ipHeader.xsiz;
					ipHeader.xsiz = arg1;
					if ((xsiz <= 0.0) || (ipHeader.xsiz <= 0.0)) break;
					magX = ipHeader.xsiz / xsiz;
					if ((magX < 1.0) || (magX > MAXMAG)) break;
					xsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-z")) {
					zsiz = ipHeader.zsiz;
					ipHeader.zsiz = arg1;
					if ((zsiz <= 0.0) || (ipHeader.zsiz <= 0.0)) break;
					magZ = ipHeader.zsiz / zsiz;
					if ((magZ < 1.0) || (magZ > MAXMAG)) break;
					zsizDoneFlag++;
				}
				else break;
			}
			else if (argNum == 3) {
				if (!strcmp(arg0, "-x")) {
					xsiz = arg1;
					ipHeader.xsiz = arg2;
					if ((xsiz <= 0.0) || (ipHeader.xsiz <= 0.0)) break;
					magX = ipHeader.xsiz / xsiz;
					if ((magX < 1.0) || (magX > MAXMAG)) break;
					xsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-z")) {
					zsiz = arg1;
					ipHeader.zsiz = arg2;
					if ((zsiz <= 0.0) || (ipHeader.zsiz <= 0.0)) break;
					magZ = ipHeader.zsiz / zsiz;
					if ((magZ < 1.0) || (magZ > MAXMAG)) break;
					zsizDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((xsizDoneFlag > 1) || (zsizDoneFlag > 1) || (setTimeFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	
/* Set Parameters to Create */
	ipHeader.xpxl = (long)(ceil(ipHeader.xpxl/magX));
	ipHeader.zpxl = (long)(ceil(ipHeader.zpxl/magZ));
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(ipHeaderSizeMin, flushWidth);
	if (createDosFlag) recordLengthCreate = ((long)(ceil(recordLengthCreate/512.0)))*512;
	ipHeader.drtx /= magX;
	ipHeader.drtz /= magZ;
	if (ipHeader.drtx >= ipHeader.xpxl) ipHeader.drtx = ipHeader.xpxl - 1.0;
	else if (ipHeader.drtx < 0.0) ipHeader.drtx = 0.0;
	if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
	else if (ipHeader.drtz < 0.0) ipHeader.drtz = 0.0;
	ipHeader.rnum = ipHeader.zpxl;
	ipHeader.rlng = recordLengthCreate;
	flushWidth0 = recordLengthCreate - flushWidth;
	dblBufSize = recordLengthCreate*(sizeof(double)/RAXDATAWIDTH);
	xpxlM = ipHeader.xpxl - 1;
	jmaxL = (long)(floor(ipHeader.xpxl*magX));
	if (verboseFlag >= 2) {
		fprintf(stdout, " PixelSize is changed from (%7.5f mm, %7.5f mm) to (%7.5f mm, %7.5f mm).\n", xsiz, zsiz, ipHeader.xsiz, ipHeader.zsiz);
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		if (replaceFlag) fprintf(stdout, " Original R-AXIS Image File will be replaced !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Create Output File */
	if (replaceFlag) {
		if (mkstempEXT(file1, file2, &fp2, 0)) {
			fprintf(stderr, " Temporary File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		if (verboseFlag == 3) {
			fprintf(stdout, "Enter  Output  Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		while (createFileEXT(file2, &fp2, 0)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
	}
	
/* Restore endian on ipHeader */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	createDosFlag ? fwrite(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp2) : fwrite(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSizeMin, 1, fp2);
	if (fseek(fp1, recordLength, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	if (flushWidth0 == 0) free(datPtr0);
	
/* Swap Header again */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	imax = 0;
	if ((datPtr1 = malloc(recordLength)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		if (flushWidth0 > 0) free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	fread(datPtr1, recordLength, 1, fp1);
	readByte += recordLength;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
		free(datPtr1);
		if (flushWidth0 > 0) free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	for (i=0; i<ipHeader.rnum; i++) {
		if ((datPtr2 = malloc(recordLengthCreate)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtr1);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			if (i) {
				ipHeader.zpxl = ipHeader.rnum = i;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
					}
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		if ((datPtrDouble = malloc(dblBufSize)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtr1);
			free(datPtr2);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp1);
			if (i) {
				ipHeader.zpxl = ipHeader.rnum = i;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (replaceFlag) {
						fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
						fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
					}
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		imin = imax;
		imax = (long)(floor((i+1)*magZ));
	/* Lowest Line(original) contribute i-th line(new) */
		if (!i) for (j=0; j<ipHeader.xpxl; j++) datPtrDouble[j] = 0.0;
		else {
			jmax = 0;
			for (j=0; j<xpxlM; j++) {
				jmin = jmax;
				jmax = (long)(floor((j+1)*magX));
				datPtrDouble[j] = (double)((*readDat)(&datPtr1[jmin], &rato, &shiftBit, &ipDataSwapFlag))*((imin+1)-i*magZ)*((jmin+1)-j*magX);
				for (jtmp=jmin+1; jtmp<jmax; jtmp++) datPtrDouble[j] += (*readDat)(&datPtr1[jtmp], &rato, &shiftBit, &ipDataSwapFlag)*((imin+1)-i*magZ);
				datPtrDouble[j] += (*readDat)(&datPtr1[jmax], &rato, &shiftBit, &ipDataSwapFlag)*((imin+1)-i*magZ)*((j+1)*magX-jmax);
			}
			jmin = jmax;
			jmax = jmaxL;
			datPtrDouble[xpxlM] = (double)((*readDat)(&datPtr1[jmin], &rato, &shiftBit, &ipDataSwapFlag))*((imin+1)-i*magZ)*((jmin+1)-j*magX);
			for (jtmp=jmin+1; jtmp<xpxlOld; jtmp++) datPtrDouble[xpxlM] += (*readDat)(&datPtr1[jtmp], &rato, &shiftBit, &ipDataSwapFlag)*((imin+1)-i*magZ);
			free(datPtr1);
		}
	/* Middle Line(original) contribute i-th line(new) */
		for (itmp=imin+1; itmp<imax; itmp++) {
			if ((datPtr1 = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtr2);
				free(datPtrDouble);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (replaceFlag) {
							fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
							fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
						}
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			if (readByte == filesize) break;
			fread(datPtr1, recordLength, 1, fp1);
			readByte += recordLength;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtr1);
				free(datPtr2);
				free(datPtrDouble);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (replaceFlag) {
							fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
							fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
						}
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			jmax = 0;
			for (j=0; j<xpxlM; j++) {
				jmin = jmax;
				jmax = (long)(floor((j+1)*magX));
				datPtrDouble[j] += (double)((*readDat)(&datPtr1[jmin], &rato, &shiftBit, &ipDataSwapFlag))*((jmin+1)-j*magX);
				for (jtmp=jmin+1; jtmp<jmax; jtmp++) datPtrDouble[j] += (*readDat)(&datPtr1[jtmp], &rato, &shiftBit, &ipDataSwapFlag);
				datPtrDouble[j] += (*readDat)(&datPtr1[jmax], &rato, &shiftBit, &ipDataSwapFlag)*((j+1)*magX-jmax);
			}
			jmin = jmax;
			jmax = jmaxL;
			datPtrDouble[xpxlM] += (double)((*readDat)(&datPtr1[jmin], &rato, &shiftBit, &ipDataSwapFlag))*((jmin+1)-j*magX);
			for (jtmp=jmin+1; jtmp<xpxlOld; jtmp++) datPtrDouble[xpxlM] += (*readDat)(&datPtr1[jtmp], &rato, &shiftBit, &ipDataSwapFlag);
			free(datPtr1);
		}
	/* Highest Line(original) contribute i-th line(new) */
		if (i == ipHeader.rnum - 1) {
			for (j=0; j<ipHeader.xpxl; j++) datPtr2[j] = (*writeDat)(&datPtrDouble[j], &rato, &maxInt, &ipDataSwapFlag);
			fwrite(datPtr2, flushWidth, 1, fp2);
			if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
			free(datPtrDouble);
			free(datPtr2);
		}
		else {
			if ((datPtr1 = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtr2);
				free(datPtrDouble);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (replaceFlag) {
							fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
							fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
						}
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			fread(datPtr1, recordLength, 1, fp1);
			readByte += recordLength;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtr1);
				free(datPtr2);
				free(datPtrDouble);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				if (i) {
					ipHeader.zpxl = ipHeader.rnum = i;
					if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					fseek(fp2, 0, SEEK_SET);
					fwrite(&ipHeader, ipHeaderSize, 1, fp2);
					fprintf(stdout, " Output File is saved using available data.\n");
					if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
					if (verboseFlag) {
						fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
						fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
						if (replaceFlag) {
							fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
							fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
						}
					}
					fclose(fp2);
#ifdef _HASUTIME
					if (setTimeFlag) {
						if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
					}
					if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
				}
				else {
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
				}
				exit(EXIT_FAILURE);
			}
			jmax = 0;
			for (j=0; j<xpxlM; j++) {
				jmin = jmax;
				jmax = (long)(floor((j+1)*magX));
				datPtrDouble[j] += (double)((*readDat)(&datPtr1[jmin], &rato, &shiftBit, &ipDataSwapFlag))*((i+1)*magZ-imax)*((jmin+1)-j*magX);
				for (jtmp=jmin+1; jtmp<jmax; jtmp++) datPtrDouble[j] += (*readDat)(&datPtr1[jtmp], &rato, &shiftBit, &ipDataSwapFlag)*((i+1)*magZ-imax);
				datPtrDouble[j] += (*readDat)(&datPtr1[jmax], &rato, &shiftBit, &ipDataSwapFlag)*((i+1)*magZ-imax)*((j+1)*magX-jmax);
				datPtr2[j] = (*writeDat)(&datPtrDouble[j], &rato, &maxInt, &ipDataSwapFlag);
			}
			jmin = jmax;
			jmax = jmaxL;
			datPtrDouble[ipHeader.xpxl-1] += (double)((*readDat)(&datPtr1[jmin], &rato, &shiftBit, &ipDataSwapFlag))*((i+1)*magZ-imax)*((jmin+1)-j*magX);
			for (jtmp=jmin+1; jtmp<xpxlOld; jtmp++) datPtrDouble[xpxlM] += (*readDat)(&datPtr1[jtmp], &rato, &shiftBit, &ipDataSwapFlag)*((i+1)*magZ-imax);
			datPtr2[xpxlM] = (*writeDat)(&datPtrDouble[xpxlM], &rato, &maxInt, &ipDataSwapFlag);
			fwrite(datPtr2, flushWidth, 1, fp2);
			if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
			free(datPtrDouble);
			free(datPtr2);
		}
	}
	
/* End */
	if (flushWidth0 > 0) free(datPtr0);
	fclose(fp1);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
#ifndef _HAS_STRANGE_RENAME
	if (replaceFlag) {
		if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			fprintf(stderr, " Original R-AXIS Image File might be lost... Check It !!\n");
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#else
	if (replaceFlag) {
		if ((fp1 = fopen(file1, "rb+")) == NULL) {
			fprintf(stderr, "Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			replaceFlag = 0;
		}
		else fclose(fp1);
	}
	if (replaceFlag) {
		if (tmpNameEXT(file1, fnameTmp)) strcpy(fnameTmp, tmpnam(NULL));
		if (rename(file1, fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
		}
		else if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (remove(fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#endif
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-t] -x[:VALUE1]:VALUE2\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 -x[:VALUE1]:VALUE2\n", ProgName);
#endif
	fprintf(fp0, "           -z[:VALUE1]:VALUE2 {file2|-replace}\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Change Pixel Size of R-AXIS Image File.\n");
	fprintf(fp0, "   -h ...... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i ...... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q ...... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v ...... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -x:VALUE1:VALUE2 ...\n");
	fprintf(fp0, "             Change PixelSize of X-direction from \"VALUE1\" to \"VALUE2\".\n");
	fprintf(fp0, "             If \"VALUE1\" is absent, Old PixelSize is assumed to be\n");
	fprintf(fp0, "             that recorded in R-AXIS File. Both Value is in [mm] unit.\n");
	fprintf(fp0, "             PixelSize change must be in a range of x1.0 - x%4.1f.\n", MAXMAG);
	fprintf(fp0, "   -z:VALUE1:VALUE2 ...\n");
	fprintf(fp0, "             Change PixelSize of Z-direction from \"VALUE1\" to \"VALUE2\".\n");
	fprintf(fp0, "             If \"VALUE1\" is absent, Old PixelSize is assumed to be\n");
	fprintf(fp0, "             that recorded in R-AXIS File. Both Value is in [mm] unit.\n");
	fprintf(fp0, "             PixelSize change must be in a range of x1.0 - x%4.1f.\n", MAXMAG);
#ifdef _HASUTIME
	fprintf(fp0, "   -t ...... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ... Filename of Original R-AXIS Image File.\n");
	fprintf(fp0, "   file2 ... Output R-AXIS Image Filename. This must be the last argument.\n");
	fprintf(fp0, "             If last argument is \"-replace\", Original R-AXIS Image File is\n");
	fprintf(fp0, "             replaced by Output File. Be careful using this option.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1, arg2)
	char *argV, *arg0;
	double *arg1, *arg2;
#else
char splitArg(const char *argV, char *arg0, double *arg1, double *arg2)
#endif
{
	char *chPtr1, *chPtr2;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 2) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 2) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	if ((chPtr2 = strchr(chPtr1+1, ':')) == NULL) return (2);
	if (!isdigitEXT(chPtr2+1)) return (0);
	*arg1 = atof(chPtr1+1);
	*arg2 = atof(chPtr2+1);
	return (3);
}
