/* Program to convert IP data file */
#define ProgName "iprot"
#define ProgInfo "Image rotation"
/*                                                         1/18/2002  */
/*         for ANSI C                                                 */

/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v0.18_1/15/2007"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#ifdef _HASUTIME
#	include <sys/stat.h>
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef _HASUTIME
#	define MAXARG 8
#else
#	define MAXARG 7
#endif
#define MINARG 3
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define DIVDEFAULT 4
#define DIV 1
#define INT 2
#define SIMPLE 3
#define R_RIGHT 11
#define R_LEFT 12
#define ROTATE 13
#define MirrorVert 14
#define MirrorHoriz 15

#ifdef _OLDHP
	long int (*readDat)();
	short int (*writeDat)();
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	long int (*readDat)(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag);
	short int (*writeDat)(const double *x, const double *rato, const unsigned long int *maxInt, const char *ipDataSwapFlag);
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr, *datPtrRes;
	long int i, j, ipHeaderSize, ipHeaderSize2VER1, ipHeaderSize2VER2, recordLength,
	     recordLengthCreate, ipHeaderSizeMin, readWidth,
	     readByte, divNum, done, flushWidth, ii, jj, oldx, oldz, *datPtrLong, intBuf,
	     zpxlOld, xpxlOld, readRes, newWidth, newHeight, offX, offZ, flushWidth0;
	unsigned long int maxInt;
	char file1[FNAMELEN], file2[FNAMELEN], inputBuffer[20], argNum,
	     confirmBuffer[8], cpuIntelFlag, ipHeaderSwapFlag, ipDataSwapFlag,
	     shiftBit, strangeShiftBitFlag, byte, createDosFlag, runMode,
	     arg0[4], verboseFlag, angDoneFlag, replaceFlag, setTimeFlag,
	     xsizDoneFlag, zsizDoneFlag, divDoneFlag, changeValueTypeIndex,
	     maxFlag;
#ifdef _HAS_STRANGE_RENAME
	char fnameTmp[FNAMELEN];
#endif
	double rato, arg1, divStep2, angle, cosPhi, sinPhi, relx, relz, getax, getaz,
	     centrex, centrez, divStepx, divStepz, intensity, centrexNew, centrezNew;
	struct ipHead ipHeader;
	struct ipHead2VER1 ipHeader2VER1;
	struct ipHead2VER2 ipHeader2VER2;
	FILE *fp1, *fp2;
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER1 = SIZE_OF_IPHEADER2VER1;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	verboseFlag = 1;
	readByte = 0;
	angle = sinPhi = 0.0;
	cosPhi = 1.0;
	ipDataSwapFlag = angDoneFlag = replaceFlag = setTimeFlag = divDoneFlag
	 = xsizDoneFlag = zsizDoneFlag = maxFlag = 0;
	divNum = DIVDEFAULT;
	done = 0;
	runMode = DIV;
	datPtr = datPtrRes = (short*)NULL;
	divStep2 = divStepx = divStepz = 0.0;
	offX = offZ = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check Arguments 1 */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
		if (!strcmp(argv[argc-1], "-replace")) replaceFlag = 1;
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((fp1 = fopen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	fread(&ipHeader, ipHeaderSize, 1, fp1);
	readByte += ipHeaderSize;
	if (ftell(fp1) != readByte) {
		fprintf(stderr, " Input File Read Error.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	
/* File Format Check */
	ipHeaderSwapFlag = (ipHeader.rlng & 0x0000ffff) ? 0 : 1;
	/* ipHeader.rlng is assumed to be less than 65536;
		if each data has 16-bits, width of IP should less than 32768 pixels. */
	if (XOREXT(cpuIntelFlag, ipHeaderSwapFlag)) {
		if (verboseFlag) {
			fprintf(stdout, " This File is probably created with OLD MD-DOS(R) Machine.\n");
			do {
				fprintf(stdout, "  Is it true ? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'N') {
				fprintf(stderr, " Unable to continue. Stop.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, "  Output File is for OLD MD-DOS(R) Machine.\n");
		}
		fread(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp1);
		readByte += ipHeaderSize2VER1;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ((long)(ceil((ipHeaderSize + ipHeaderSize2VER1)/512.0)))*512;
		createDosFlag = 1;
	}
	else {
		fread(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp1);
		readByte += ipHeaderSize2VER2;
		if (ftell(fp1) != readByte) {
			fprintf(stderr, " Input File Read Error.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
		createDosFlag = 0;
	}
	
/* Swap Header */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		if (!cpuIntelFlag) swapIpHeader2VER1(&ipHeader2VER1);
		else swapIpHeader2VER2(&ipHeader2VER2);
	}
	byte = XOREXT(cpuIntelFlag, ipHeaderSwapFlag) ? ipHeader2VER1.byte : ipHeader2VER2.byte;
	
/* Get ipHeader Information and Check */
	if (XOREXT(byte, ipHeaderSwapFlag)) ipDataSwapFlag = 1;
	if (ipHeader.zpxl != ipHeader.rnum) {
		ipHeader.zpxl = ipHeader.rnum;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Height has modified.\n");
	}
	zpxlOld = ipHeader.rnum;
	recordLength = ipHeader.rlng;
	if (ipHeader.xpxl*RAXDATAWIDTH > ipHeader.rlng) {
		ipHeader.xpxl = ipHeader.rlng/RAXDATAWIDTH;
		if (verboseFlag >= 2) fprintf(stdout, " Incorrect Image Width has modified.\n");
	}
	xpxlOld = ipHeader.xpxl;
	if ((ipHeader.xpxl <= 0) || (ipHeader.xpxl > MAXWIDTH) || (ipHeader.zpxl <= 0) || (ipHeader.zpxl > MAXHEIGHT)) {
		fprintf(stderr, " File Format is Incorrect.\n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	if (ipHeader.xsiz <= 0.0) {
		ipHeader.xsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in X has modified to 0.05 [mm].\n");
	}
	if (ipHeader.zsiz <= 0.0) {
		ipHeader.zsiz = 0.05;
		if (verboseFlag) fprintf(stdout, " Incorrect PixelSize in Z has modified to 0.05 [mm].\n");
	}
	rato = ipHeader.rato;
	if (ipHeader.cyld >= 256) {
		if (verboseFlag) fprintf(stdout, " Camera Type Code has changed from %ld to 0.\n", ipHeader.cyld);
		ipHeader.cyld = 0;
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ipHeader.rlng*(ipHeader.rnum+1));
	}
	
/* Clean Header */
	cleanIpHeader(&ipHeader);
	createDosFlag ? cleanIpHeader2VER1(&ipHeader2VER1) : cleanIpHeader2VER2(&ipHeader2VER2);
	
/* Logging */
	if (logWrite(&ipHeader, ProgName)) {
		if (verboseFlag) fprintf(stdout, " ipHeader.memo is full, and No Logging.\n");
	}
	
/* Check bits to shift to gain dynamic-range */
	/* ipHeader.rato is probably an integer of 2^n type (typically 8.0, i.e. 8-bits shift).
	    But for future change of this rule, it may be a value larger than 1.0 and
	    less than 2^15. */
	strangeShiftBitFlag = 1;
	if ((rato < MINRATO) || (rato > MAXRATO)) {
		fprintf(stderr, " Input File Format Error : ipHeader.rato. Stop. \n");
		fclose(fp1);
		exit(EXIT_FAILURE);
	}
	else if (rato == 1.0) {
		maxInt = 0x7fff;
		readDat = ipStoLongBit0;
		writeDat = ipDtoShortBit0;
		shiftBit = 0;
	}
	else {
		for (shiftBit=1; shiftBit<16; shiftBit++) {
			if ((double)(0x0001 << shiftBit) == rato) {
				strangeShiftBitFlag = 0;
				maxInt = ((long)(0x7fff)) << shiftBit;
				readDat = ipStoLongBit;
				writeDat = ipDtoShort;
				break;
			}
		}
		if (strangeShiftBitFlag) {
			maxInt = dToLong((0x7fff) * rato);
			readDat = ipStoLongRato;
			writeDat = ipDtoShort;
		}
	}
	
/* Get Constant to Add */
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, " Select Operation : \n");
			fprintf(stdout, "   (0) Rotate , (1) Rotate +90 , (2) Rotate -90 , (3) Rotate 180 ,\n");
			fprintf(stdout, "   (4) Mirror about vertical line , (5) Mirror about horizontal line : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while (inputBuffer[0]-'0' > 5 || inputBuffer[0]-'0' < 0);
		if (inputBuffer[0]-'0' != 0) {
			runMode = SIMPLE;
			divNum = inputBuffer[0]-'0'+R_LEFT-1;
		}
		else {
			do {
				fprintf(stdout, "Enter Rotate Angle (degrees) : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while (!isdigitEXT(inputBuffer));
			angle = atof(inputBuffer);
			do {
				fprintf(stdout, "Enter Division Number of each pixel. Default is %2d.", DIVDEFAULT);
				fprintf(stdout, " Type 'D' if use default value.\n");
				fprintf(stdout, " Type 'X' if work use interpolation method instead : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
				if (toupper(inputBuffer[0]) == 'X') {
					divDoneFlag = 1;
					runMode = INT;
					break;
				}
				if (toupper(inputBuffer[0]) == 'D') {
					divDoneFlag = 1;
					break;
				}
				if (!isdigitEXT(inputBuffer)) continue;
				divNum = atol(inputBuffer);
				if (divNum < 1) continue;
				divDoneFlag = 1;
			} while (!divDoneFlag);
			angle = atof(inputBuffer);
			fprintf(stdout, " Centre is : %6.1f, %6.1f.\n", ipHeader.drtx, ipHeader.drtz);
			fprintf(stdout, " PixelSize is : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
			do {
				fprintf(stdout, "Do you want to change the value shown above? [Y/N] : ");
				if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					fclose(fp1);
					exit(EXIT_FAILURE);
				}
			} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
			if (toupper(confirmBuffer[0]) == 'Y') {
				do {
					fprintf(stdout, "  Choice in (1) Pixel Size x , (2) Pixel Size z , (0) Return to Previous : ");
					if (fscanf(stdin, "%s", inputBuffer) == EOF) {
						fprintf(stderr, "\nError in getting input. Exit.\n");
						fclose(fp1);
						exit(EXIT_FAILURE);
					}
					changeValueTypeIndex = inputBuffer[0]-'0';
					switch (changeValueTypeIndex) {
						case 0:
							break;
						case 1:
							xsizDoneFlag = 0;
							do {
								fprintf(stdout, "    Enter Value for Pixel Size x [mm] : ");
								if (fscanf(stdin, "%s", inputBuffer) == EOF) {
									fprintf(stderr, "\nError in getting input. Exit.\n");
									fclose(fp1);
									exit(EXIT_FAILURE);
								}
								if (!isdigitEXT(inputBuffer)) continue;
								ipHeader.xsiz = atof(inputBuffer);
								if (ipHeader.xsiz <= 0.0) continue;
								xsizDoneFlag = 1;
							} while (!xsizDoneFlag);
							fprintf(stdout, "Pixel Size x is changed to %7.5f mm.\n", ipHeader.xsiz);
							break;
						case 2:
							zsizDoneFlag = 0;
							do {
								fprintf(stdout, "    Enter Value for Pixel Size z [mm] : ");
								if (fscanf(stdin, "%s", inputBuffer) == EOF) {
									fprintf(stderr, "\nError in getting input. Exit.\n");
									fclose(fp1);
									exit(EXIT_FAILURE);
								}
								if (!isdigitEXT(inputBuffer)) continue;
								ipHeader.zsiz = atof(inputBuffer);
								if (ipHeader.zsiz <= 0.0) continue;
								zsizDoneFlag = 1;
							} while (!zsizDoneFlag);
							fprintf(stdout, "Pixel Size z is changed to %7.5f mm.\n", ipHeader.zsiz);
							break;
						default:
							fprintf(stdout, "  Enter appropriate value.\n");
					}
				} while (changeValueTypeIndex);
			}
		}
		do {
			fprintf(stdout, "Replace Original R-AXIS File with Output File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') replaceFlag = 1;
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
	else {
		if (verboseFlag >= 2) {
			fprintf(stdout, " Centre is : %6.1f, %6.1f.\n", ipHeader.drtx, ipHeader.drtz);
			fprintf(stdout, " PixelSize is : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
		}
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-c")) {
					runMode = INT;
					divDoneFlag++;
				}
				else if (!strcmp(arg0, "-mx")) maxFlag++;
				else if (!strcmp(arg0, "-sl")) {
					runMode = SIMPLE;
					angDoneFlag++;
					divDoneFlag++;
					divNum = R_LEFT;
				}
				else if (!strcmp(arg0, "-sr")) {
					runMode = SIMPLE;
					angDoneFlag++;
					divDoneFlag++;
					divNum = R_RIGHT;
				}
				else if (!strcmp(arg0, "-su")) {
					runMode = SIMPLE;
					angDoneFlag++;
					divDoneFlag++;
					divNum = ROTATE;
				}
				else if (!strcmp(arg0, "-mv")) {
					runMode = SIMPLE;
					angDoneFlag++;
					divDoneFlag++;
					divNum = MirrorVert;
				}
				else if (!strcmp(arg0, "-mh")) {
					runMode = SIMPLE;
					angDoneFlag++;
					divDoneFlag++;
					divNum = MirrorHoriz;
				}
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-r")) {
					angle = arg1;
					angDoneFlag++;
				}
				else if (!strcmp(arg0, "-n")) {
					divNum = dToLong(arg1);
					if (divNum < 1) break;
					divDoneFlag++;
				}
				else if (!strcmp(arg0, "-dx")) {
					ipHeader.xsiz = arg1;
					if (ipHeader.xsiz <= 0) break;
					xsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-dz")) {
					ipHeader.zsiz = arg1;
					if (ipHeader.zsiz <= 0) break;
					zsizDoneFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if ((angDoneFlag > 1) || (setTimeFlag > 1) || (divDoneFlag > 1)
		 || (xsizDoneFlag > 1) || (zsizDoneFlag > 1) || (maxFlag > 1)
		 || (runMode == SIMPLE && ((xsizDoneFlag > 1) || (zsizDoneFlag > 1)))) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
		if (!angDoneFlag) {
			fprintf(stderr, " Lack of Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	centrex = centrexNew = ipHeader.xpxl / 2.0;
	centrez = centrezNew = ipHeader.zpxl / 2.0;
	newWidth = ipHeader.xpxl;
	newHeight = ipHeader.zpxl;
	if (runMode == SIMPLE) {
		switch (divNum) {
			case R_LEFT:
				arg1 = ipHeader.zpxl - ipHeader.drtz;
				ipHeader.drtz = ipHeader.drtx;
				ipHeader.drtx = arg1;
				arg1 = ipHeader.zsiz;
				ipHeader.zsiz = ipHeader.xsiz;
				ipHeader.xsiz = arg1;
				newWidth = ipHeader.zpxl;
				newHeight = ipHeader.xpxl;
				break;
			case R_RIGHT:
				arg1 = ipHeader.drtz;
				ipHeader.drtz = ipHeader.xpxl - ipHeader.drtx;
				ipHeader.drtx = arg1;
				arg1 = ipHeader.zsiz;
				ipHeader.zsiz = ipHeader.xsiz;
				ipHeader.xsiz = arg1;
				newWidth = ipHeader.zpxl;
				newHeight = ipHeader.xpxl;
				break;
			case ROTATE:
				ipHeader.drtz = ipHeader.zpxl - ipHeader.drtz;
				ipHeader.drtx = ipHeader.xpxl - ipHeader.drtx;
				break;
			case MirrorVert:
				ipHeader.drtx = ipHeader.xpxl - ipHeader.drtx;
				break;
			case MirrorHoriz:
				ipHeader.drtz = ipHeader.zpxl - ipHeader.drtz;
				break;
			default:
				/* NOT REACHED */
				break;
		}
	}
	else {
		angle = dToLong(-angle/360.0)*360.0+angle;
		cosPhi = cos(radEXT(angle));
		sinPhi = sin(radEXT(angle));
		if (maxFlag) {
			if (angle < -90.0) {
				offX = (long)ceil(- ((ipHeader.xpxl - centrex) * ipHeader.xsiz * cosPhi
				                     - (- centrez) * ipHeader.zsiz * sinPhi) / ipHeader.xsiz - centrex);
				offZ = (long)ceil(- ((ipHeader.xpxl - centrex) * ipHeader.xsiz * sinPhi
				                     + (ipHeader.zpxl - centrez) * ipHeader.zsiz * cosPhi) / ipHeader.zsiz - centrez);
				newWidth = (long)floor(((- centrex) * ipHeader.xsiz * cosPhi
				                        - (ipHeader.zpxl - centrez) * ipHeader.zsiz * sinPhi) / ipHeader.xsiz + centrex)
				           + offX;
				newHeight = (long)floor(((- centrex) * ipHeader.xsiz * sinPhi
				                         + (- centrez) * ipHeader.zsiz * cosPhi) / ipHeader.zsiz + centrez)
				            + offZ;
			}
			else if (angle < 0.0) {
				offX = (long)ceil(- ((- centrex) * ipHeader.xsiz * cosPhi
				                     - (- centrez) * ipHeader.zsiz * sinPhi) / ipHeader.xsiz - centrex);
				offZ = (long)ceil(- ((ipHeader.xpxl - centrex) * ipHeader.xsiz * sinPhi
				                     + (- centrez) * ipHeader.zsiz * cosPhi) / ipHeader.zsiz - centrez);
				newWidth = (long)ceil(((ipHeader.xpxl - centrex) * ipHeader.xsiz * cosPhi
				                       - (ipHeader.zpxl - centrez) * ipHeader.zsiz * sinPhi) / ipHeader.xsiz + centrex)
				           + offX;
				newHeight = (long)ceil(((- centrex) * ipHeader.xsiz * sinPhi
				                        + (ipHeader.zpxl - centrez) * ipHeader.zsiz * cosPhi) / ipHeader.zsiz + centrez)
				           + offZ;
			}
			else if (angle < 90.0) {
				offX = (long)ceil(- ((- centrex) * ipHeader.xsiz * cosPhi
				                     - (ipHeader.zpxl - centrez) * ipHeader.zsiz * sinPhi) / ipHeader.xsiz - centrex);
				offZ = (long)ceil(- ((- centrex) * ipHeader.xsiz * sinPhi
				                     + (- centrez) * ipHeader.zsiz * cosPhi) / ipHeader.zsiz - centrez);
				newWidth = (long)ceil(((ipHeader.xpxl - centrex) * ipHeader.xsiz * cosPhi
				                       - (- centrez) * ipHeader.zsiz * sinPhi) / ipHeader.xsiz + centrex)
				           + offX;
				newHeight = (long)ceil(((ipHeader.xpxl - centrex) * ipHeader.xsiz * sinPhi
				                        + (ipHeader.zpxl - centrez) * ipHeader.zsiz * cosPhi) / ipHeader.zsiz + centrez)
				            + offZ;
			}
			else {
				offX = (long)ceil(- ((ipHeader.xpxl - centrex) * ipHeader.xsiz * cosPhi
				                     - (ipHeader.zpxl - centrez) * ipHeader.zsiz * sinPhi) / ipHeader.xsiz - centrex);
				offZ = (long)ceil(- ((- centrex) * ipHeader.xsiz * sinPhi
				                     + (ipHeader.zpxl - centrez) * ipHeader.zsiz * cosPhi) / ipHeader.zsiz - centrez);
				newWidth = (long)ceil(((- centrex) * ipHeader.xsiz * cosPhi
				                       - (- centrez) * ipHeader.zsiz * sinPhi) / ipHeader.xsiz + centrex)
				           + offX;
				newHeight = (long)ceil(((ipHeader.xpxl - centrex) * ipHeader.xsiz * sinPhi
				                        + (- centrez) * ipHeader.zsiz * cosPhi) / ipHeader.zsiz + centrez)
				            + offZ;
			}
		}
		centrexNew = centrex + offX;
		centrezNew = centrez + offZ;
		arg1 = ((ipHeader.drtx - centrex) * ipHeader.xsiz * cosPhi
		       - (ipHeader.drtz - centrez) * ipHeader.zsiz * sinPhi) / ipHeader.xsiz + centrexNew;
		ipHeader.drtz = ((ipHeader.drtx - centrex) * ipHeader.xsiz * sinPhi
		                + (ipHeader.drtz - centrez) * ipHeader.zsiz * cosPhi) / ipHeader.zsiz + centrezNew;
		ipHeader.drtx = arg1;
		divStepx = ipHeader.xsiz / divNum;
		divStepz = ipHeader.zsiz / divNum;
		divStep2 = 1.0 / divNum / divNum;
	}
	ipHeader.xpxl = newWidth;
	ipHeader.zpxl = newHeight;
	if (verboseFlag >= 2) {
		if (runMode == SIMPLE) {
			switch (divNum) {
				case R_LEFT:
					fprintf(stdout, " Image is rotated by +90 degrees.\n");
					break;
				case R_RIGHT:
					fprintf(stdout, " Image is rotated by -90 degrees.\n");
					break;
				case ROTATE:
					fprintf(stdout, " Image is rotated by +180 degrees.\n");
					break;
				case MirrorVert:
					fprintf(stdout, " Image is mirrored about vertical axis.\n");
					break;
				case MirrorHoriz:
					fprintf(stdout, " Image is mirrored about horizontal axis.\n");
					break;
				default:
					/* NOT REACHED */
					break;
			}
		}
		else {
			fprintf(stdout, " Image is rotated by %4.1f degrees.\n", angle);
			if (xsizDoneFlag || zsizDoneFlag) fprintf(stdout, " Pixelsize has changed to : %7.5f mm x %7.5f mm.\n", ipHeader.xsiz, ipHeader.zsiz);
		}
		if (replaceFlag) fprintf(stdout, " Original R-AXIS Image File will be replaced !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Set Parameters to Create */
	readWidth = xpxlOld*RAXDATAWIDTH;
	readRes = recordLength - readWidth;
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate = MAXEXT(ipHeaderSizeMin, flushWidth);
	if (createDosFlag) recordLengthCreate = ((long)(ceil(recordLengthCreate/512.0)))*512;
	if (ipHeader.drtx >= ipHeader.xpxl) ipHeader.drtx = ipHeader.xpxl - 1.0;
	else if (ipHeader.drtx < 0.0) ipHeader.drtx = 0.0;
	if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
	else if (ipHeader.drtz < 0.0) ipHeader.drtz = 0.0;
	ipHeader.rlng = recordLengthCreate;
	ipHeader.rnum = ipHeader.zpxl;
	flushWidth0 = recordLengthCreate - flushWidth;
	
/* Create Output File */
	if (replaceFlag) {
		if (mkstempEXT(file1, file2, &fp2, 0)) {
			fprintf(stderr, " Temporary File Creation Failure. Exit.\n");
			fclose(fp1);
			exit(EXIT_FAILURE);
		}
	}
	else {
		if (verboseFlag == 3) {
			fprintf(stdout, "Enter  Output  Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
		while (createFileEXT(file2, &fp2, 0)) {
			if (!verboseFlag) {
				fprintf(stderr, " Output File Creation Failure. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
			fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
			if (fscanf(stdin, "%s", file2) == EOF) {
				fprintf(stderr, "\nError in getting filename. Exit.\n");
				fclose(fp1);
				exit(EXIT_FAILURE);
			}
		}
	}
	
/* Restore endian on ipHeader */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, RAXDATAWIDTH)) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	createDosFlag ? fwrite(&ipHeader2VER1, ipHeaderSize2VER1, 1, fp2) : fwrite(&ipHeader2VER2, ipHeaderSize2VER2, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSizeMin, 1, fp2);
	if (fseek(fp1, recordLength, SEEK_SET)) {
		fprintf(stderr, " Input File Read Error.\n");
		free(datPtr0);
		fclose(fp1);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	readByte = ftell(fp1);
	if (flushWidth0 == 0) free(datPtr0);
	
/* Swap Header again */
	if (ipHeaderSwapFlag) {
		swapIpHeader(&ipHeader);
		createDosFlag ? swapIpHeader2VER1(&ipHeader2VER1) : swapIpHeader2VER2(&ipHeader2VER2);
	}
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if (runMode == SIMPLE) {
		switch (divNum) {
			case R_LEFT:
				if ((datPtrRes = malloc(zpxlOld*readWidth)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					fclose(fp1);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
					exit(EXIT_FAILURE);
				}
				for (i=0; i<zpxlOld; i++) {
					fread(datPtrRes+done, readWidth, 1, fp1);
					readByte += readWidth;
					if (ftell(fp1) != readByte) {
						fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
						free(datPtrRes);
						if (flushWidth0 > 0) free(datPtr0);
						fclose(fp1);
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
						exit(EXIT_FAILURE);
					}
					if (fseek(fp1, readRes, SEEK_CUR)) {
						if (i != zpxlOld-1) {
							fprintf(stderr, " Input File Read Error.\n");
							free(datPtrRes);
							if (flushWidth0 > 0) free(datPtr0);
							fclose(fp1);
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
							exit(EXIT_FAILURE);
						}
					}
					done += xpxlOld;
					readByte += readRes;
				}
				fclose(fp1);
				for (i=0; i<ipHeader.zpxl; i++) {
					if ((datPtr = malloc(flushWidth)) == NULL) {
						fprintf(stderr, " Failed to allocate memory.\n");
						free(datPtrRes);
						if (flushWidth0 > 0) free(datPtr0);
						if (i) {
							ipHeader.zpxl = ipHeader.rnum = i;
							if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							fseek(fp2, 0, SEEK_SET);
							fwrite(&ipHeader, ipHeaderSize, 1, fp2);
							fprintf(stdout, " Output File is saved using available data.\n");
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							if (verboseFlag) {
								fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
								fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
								if (replaceFlag) {
									fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
									fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
								}
							}
							fclose(fp2);
#ifdef _HASUTIME
							if (setTimeFlag) {
								if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
							}
							if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
						}
						else {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					for (j=0; j<ipHeader.xpxl; j++) datPtr[j] = datPtrRes[(zpxlOld-j-1)*xpxlOld+i];
					fwrite(datPtr, flushWidth, 1, fp2);
					if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
					free(datPtr);
				}
				free(datPtrRes);
				break;
			case R_RIGHT:
				if ((datPtrRes = malloc(zpxlOld*readWidth)) == NULL) {
					fprintf(stderr, " Failed to allocate memory.\n");
					fclose(fp1);
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
					exit(EXIT_FAILURE);
				}
				for (i=0; i<zpxlOld; i++) {
					fread(datPtrRes+done, readWidth, 1, fp1);
					readByte += readWidth;
					if (ftell(fp1) != readByte) {
						fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
						free(datPtrRes);
						if (flushWidth0 > 0) free(datPtr0);
						fclose(fp1);
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
						exit(EXIT_FAILURE);
					}
					if (fseek(fp1, readRes, SEEK_CUR)) {
						if (i != zpxlOld-1) {
							fprintf(stderr, " Input File Read Error.\n");
							free(datPtrRes);
							if (flushWidth0 > 0) free(datPtr0);
							fclose(fp1);
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
							exit(EXIT_FAILURE);
						}
					}
					done += xpxlOld;
					readByte += readRes;
				}
				fclose(fp1);
				for (i=0; i<ipHeader.zpxl; i++) {
					if ((datPtr = malloc(flushWidth)) == NULL) {
						fprintf(stderr, " Failed to allocate memory.\n");
						free(datPtrRes);
						if (flushWidth0 > 0) free(datPtr0);
						if (i) {
							ipHeader.zpxl = ipHeader.rnum = i;
							if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							fseek(fp2, 0, SEEK_SET);
							fwrite(&ipHeader, ipHeaderSize, 1, fp2);
							fprintf(stdout, " Output File is saved using available data.\n");
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							if (verboseFlag) {
								fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
								fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
								if (replaceFlag) {
									fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
									fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
								}
							}
							fclose(fp2);
#ifdef _HASUTIME
							if (setTimeFlag) {
								if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
							}
							if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
						}
						else {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					for (j=0; j<ipHeader.xpxl; j++) datPtr[j] = datPtrRes[(j+1)*xpxlOld-i-1];
					fwrite(datPtr, flushWidth, 1, fp2);
					if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
					free(datPtr);
				}
				free(datPtrRes);
				break;
			case ROTATE:
				readWidth = recordLength*(zpxlOld+3);
				if (fseek(fp1, recordLength*zpxlOld, SEEK_SET)) {
					fprintf(stderr, " Input File Read Error.\n");
					if (flushWidth0 > 0) free(datPtr0);
					fclose(fp1);
					fclose(fp2);
					remove(file2);
					fprintf(stderr, " Output File is not created or removed.\n");
					exit(EXIT_FAILURE);
				}
				for (i=0; i<zpxlOld; i++) {
					if ((datPtrRes = malloc(recordLength)) == NULL) {
						fprintf(stderr, " Failed to allocate memory.\n");
						if (flushWidth0 > 0) free(datPtr0);
						fclose(fp1);
						if (i) {
							ipHeader.zpxl = ipHeader.rnum = i;
							if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							fseek(fp2, 0, SEEK_SET);
							fwrite(&ipHeader, ipHeaderSize, 1, fp2);
							fprintf(stdout, " Output File is saved using available data.\n");
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							if (verboseFlag) {
								fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
								fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
								if (replaceFlag) {
									fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
									fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
								}
							}
							fclose(fp2);
#ifdef _HASUTIME
							if (setTimeFlag) {
								if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
							}
							if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
						}
						else {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					if ((datPtr = malloc(flushWidth)) == NULL) {
						fprintf(stderr, " Failed to allocate memory.\n");
						free(datPtrRes);
						if (flushWidth0 > 0) free(datPtr0);
						fclose(fp1);
						if (i) {
							ipHeader.zpxl = ipHeader.rnum = i;
							if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							fseek(fp2, 0, SEEK_SET);
							fwrite(&ipHeader, ipHeaderSize, 1, fp2);
							fprintf(stdout, " Output File is saved using available data.\n");
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							if (verboseFlag) {
								fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
								fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
								if (replaceFlag) {
									fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
									fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
								}
							}
							fclose(fp2);
#ifdef _HASUTIME
							if (setTimeFlag) {
								if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
							}
							if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
						}
						else {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					fread(datPtrRes, recordLength, 1, fp1);
					readByte += recordLength;
					if (ftell(fp1) != readWidth - readByte) {
						fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
						free(datPtrRes);
						free(datPtr);
						if (flushWidth0 > 0) free(datPtr0);
						fclose(fp1);
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
						exit(EXIT_FAILURE);
					}
					for (j=0; j<ipHeader.xpxl; j++) datPtr[j] = datPtrRes[ipHeader.xpxl-j-1];
					fwrite(datPtr, flushWidth, 1, fp2);
					if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
					free(datPtr);
					free(datPtrRes);
					fseek(fp1, recordLength*(-2), SEEK_CUR);
				}
				fclose(fp1);
				break;
			case MirrorVert:
				for (i=0; i<zpxlOld; i++) {
					if ((datPtrRes = malloc(recordLength)) == NULL) {
						fprintf(stderr, " Failed to allocate memory.\n");
						if (flushWidth0 > 0) free(datPtr0);
						fclose(fp1);
						if (i) {
							ipHeader.zpxl = ipHeader.rnum = i;
							if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							fseek(fp2, 0, SEEK_SET);
							fwrite(&ipHeader, ipHeaderSize, 1, fp2);
							fprintf(stdout, " Output File is saved using available data.\n");
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							if (verboseFlag) {
								fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
								fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
								if (replaceFlag) {
									fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
									fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
								}
							}
							fclose(fp2);
#ifdef _HASUTIME
							if (setTimeFlag) {
								if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
							}
							if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
						}
						else {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					if ((datPtr = malloc(flushWidth)) == NULL) {
						fprintf(stderr, " Failed to allocate memory.\n");
						free(datPtrRes);
						if (flushWidth0 > 0) free(datPtr0);
						fclose(fp1);
						if (i) {
							ipHeader.zpxl = ipHeader.rnum = i;
							if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							fseek(fp2, 0, SEEK_SET);
							fwrite(&ipHeader, ipHeaderSize, 1, fp2);
							fprintf(stdout, " Output File is saved using available data.\n");
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							if (verboseFlag) {
								fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
								fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
								if (replaceFlag) {
									fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
									fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
								}
							}
							fclose(fp2);
#ifdef _HASUTIME
							if (setTimeFlag) {
								if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
							}
							if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
						}
						else {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					fread(datPtrRes, recordLength, 1, fp1);
					readByte += recordLength;
					if (ftell(fp1) != readByte) {
						fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
						free(datPtrRes);
						free(datPtr);
						if (flushWidth0 > 0) free(datPtr0);
						fclose(fp1);
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
						exit(EXIT_FAILURE);
					}
					for (j=0; j<ipHeader.xpxl; j++) datPtr[j] = datPtrRes[ipHeader.xpxl-j-1];
					fwrite(datPtr, flushWidth, 1, fp2);
					if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
					free(datPtr);
					free(datPtrRes);
				}
				fclose(fp1);
				break;
			case MirrorHoriz:
				readWidth = recordLength*(zpxlOld+3);
				fseek(fp1, recordLength*zpxlOld, SEEK_SET);
				for (i=0; i<zpxlOld; i++) {
					if ((datPtr = malloc(recordLength)) == NULL) {
						fprintf(stderr, " Failed to allocate memory.\n");
						if (flushWidth0 > 0) free(datPtr0);
						fclose(fp1);
						if (i) {
							ipHeader.zpxl = ipHeader.rnum = i;
							if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							fseek(fp2, 0, SEEK_SET);
							fwrite(&ipHeader, ipHeaderSize, 1, fp2);
							fprintf(stdout, " Output File is saved using available data.\n");
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							if (verboseFlag) {
								fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
								fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
								if (replaceFlag) {
									fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
									fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
								}
							}
							fclose(fp2);
#ifdef _HASUTIME
							if (setTimeFlag) {
								if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
							}
							if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
						}
						else {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					fread(datPtr, recordLength, 1, fp1);
					readByte += recordLength;
					if (ftell(fp1) != readWidth - readByte) {
						fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
						free(datPtr);
						if (flushWidth0 > 0) free(datPtr0);
						fclose(fp1);
						fclose(fp2);
						remove(file2);
						fprintf(stderr, " Output File is not created or removed.\n");
						exit(EXIT_FAILURE);
					}
					fwrite(datPtr, flushWidth, 1, fp2);
					if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
					free(datPtr);
					fseek(fp1, recordLength*(-2), SEEK_CUR);
				}
				fclose(fp1);
				break;
			default:
				/* NOT REACHED */
				break;
		}
	}
	else {
		if ((datPtrLong = malloc(xpxlOld*zpxlOld*sizeof(long))) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			fclose(fp1);
			if (flushWidth0 > 0) free(datPtr0);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		for (i=0; i<zpxlOld; i++) {
			if ((datPtr = malloc(recordLength)) == NULL) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtrLong);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			fread(datPtr, recordLength, 1, fp1);
			readByte += recordLength;
			if (ftell(fp1) != readByte) {
				fprintf(stderr, " Input File Read Error : Input File is Too Short.\n");
				free(datPtr);
				free(datPtrLong);
				if (flushWidth0 > 0) free(datPtr0);
				fclose(fp1);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			for (j=0; j<xpxlOld; j++)
			    datPtrLong[done+j] = (*readDat)(&datPtr[j], &rato, &shiftBit, &ipDataSwapFlag);
			free(datPtr);
			done += xpxlOld;
		}
		fclose(fp1);
		switch (runMode) {
			case DIV:
				for (i=0; i<ipHeader.zpxl; i++) {
					if ((datPtr = malloc(flushWidth)) == NULL) {
						fprintf(stderr, " Failed to allocate memory.\n");
						free(datPtrLong);
						if (flushWidth0 > 0) free(datPtr0);
						if (i) {
							ipHeader.zpxl = ipHeader.rnum = i;
							if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							fseek(fp2, 0, SEEK_SET);
							fwrite(&ipHeader, ipHeaderSize, 1, fp2);
							fprintf(stdout, " Output File is saved using available data.\n");
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							if (verboseFlag) {
								fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
								fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
								if (replaceFlag) {
									fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
									fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
								}
							}
							fclose(fp2);
#ifdef _HASUTIME
							if (setTimeFlag) {
								if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
							}
							if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
						}
						else {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					getaz = (i - centrezNew) * ipHeader.zsiz + 0.5*divStepz;
					for (j=0; j<ipHeader.xpxl; j++) {
						getax = (j - centrexNew) * ipHeader.xsiz + 0.5*divStepx;
						intBuf = 0;
						for (ii=0; ii<divNum; ii++) {
							relz = getaz + ii * divStepz;
							for (jj=0; jj<divNum; jj++) {
								relx = getax + jj * divStepx;
								oldx = (long)floor((relx * cosPhi + relz * sinPhi) / ipHeader.xsiz + centrex);
								oldz = (long)floor((-relx * sinPhi + relz * cosPhi) / ipHeader.zsiz + centrez);
								if (oldx >= 0 && oldx < xpxlOld && oldz >= 0 && oldz < zpxlOld)
								    intBuf += datPtrLong[ oldz*xpxlOld + oldx ];
							}
						}
						intensity = dToLong(intBuf * divStep2);
						datPtr[j] = (*writeDat)(&intensity, &rato, &maxInt, &ipDataSwapFlag);
					}
					fwrite(datPtr, flushWidth, 1, fp2);
					if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
					free(datPtr);
				}
				break;
			case INT:
				for (i=0; i<ipHeader.zpxl; i++) {
					if ((datPtr = malloc(flushWidth)) == NULL) {
						fprintf(stderr, " Failed to allocate memory.\n");
						free(datPtrLong);
						if (flushWidth0 > 0) free(datPtr0);
						if (i) {
							ipHeader.zpxl = ipHeader.rnum = i;
							if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							fseek(fp2, 0, SEEK_SET);
							fwrite(&ipHeader, ipHeaderSize, 1, fp2);
							fprintf(stdout, " Output File is saved using available data.\n");
							if (ipHeaderSwapFlag) swapIpHeader(&ipHeader);
							if (verboseFlag) {
								fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
								fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
								if (replaceFlag) {
									fprintf(stdout, " Original R-AXIS Image File is not replaced.\n");
									fprintf(stdout, " Output R-AXIS Image File is saved as \"%s\".\n", file2);
								}
							}
							fclose(fp2);
#ifdef _HASUTIME
							if (setTimeFlag) {
								if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
							}
							if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
						}
						else {
							fclose(fp2);
							remove(file2);
							fprintf(stderr, " Output File is not created or removed.\n");
						}
						exit(EXIT_FAILURE);
					}
					relz = (i - centrezNew + 0.5) * ipHeader.zsiz;
					for (j=0; j<ipHeader.xpxl; j++) {
						relx = (j - centrexNew + 0.5) * ipHeader.xsiz;
						getax = (relx * cosPhi + relz * sinPhi) / ipHeader.xsiz + centrex;
						getaz = (-relx * sinPhi + relz * cosPhi) / ipHeader.zsiz + centrez;
						oldx = dToLong(getax);
						oldz = dToLong(getaz);
						if (oldx > 0 && oldx < xpxlOld && oldz > 0 && oldz < zpxlOld) {
							divStepx = oldx - getax + 0.5;
							intensity = (datPtrLong[ xpxlOld*(oldz-1) +oldx-1 ] * divStepx
							            + datPtrLong[ xpxlOld*(oldz-1) +oldx   ] * (1.0-divStepx)) * (oldz-getaz+0.5)
							          + (datPtrLong[ xpxlOld* oldz    +oldx-1 ] * divStepx
							            + datPtrLong[ xpxlOld* oldz    +oldx   ] * (1.0-divStepx)) * (getaz-oldz+0.5);
						}
						else intensity = 0.0;
						datPtr[j] = (*writeDat)(&intensity, &rato, &maxInt, &ipDataSwapFlag);
					}
					fwrite(datPtr, flushWidth, 1, fp2);
					if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
					free(datPtr);
				}
				break;
			default:
				/* NOT REACHED */
				break;
		}
		free(datPtrLong);
	}
/* End */
	if (flushWidth0 > 0) free(datPtr0);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
#ifndef _HAS_STRANGE_RENAME
	if (replaceFlag) {
		if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			fprintf(stderr, " Original R-AXIS Image File might be lost... Check It !!\n");
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#else
	if (replaceFlag) {
		if ((fp1 = fopen(file1, "rb+")) == NULL) {
			fprintf(stderr, "Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			replaceFlag = 0;
		}
		else fclose(fp1);
	}
	if (replaceFlag) {
		if (tmpNameEXT(file1, fnameTmp)) strcpy(fnameTmp, tmpnam(NULL));
		if (rename(file1, fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
		}
		else if (rename(file2, file1)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " New File is saved as \"%s\"\n", file2);
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (remove(fnameTmp)) {
			fprintf(stderr, " Unable to replace Original R-AXIS Image File.\n");
			fprintf(stderr, " Original File is saved as \"%s\"\n", fnameTmp);
		}
		else if (verboseFlag) fprintf(stdout, " Original R-AXIS Image File has Replaced.\n");
	}
#endif
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
	fprintf(fp0, "        %s [-q|-v] file1 -r:VALUE [-n:VALUE|-c] [-mx]\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "       [-dx:VALUE] [-dz:VALUE] [-t] {file2|-replace}\n");
	fprintf(fp0, "        %s [-q|-v] file1 {-sr|-sl|-su|-mh|-mv} [-t] {file2|-replace}\n", ProgName);
#else
	fprintf(fp0, "       [-dx:VALUE] [-dz:VALUE] {file2|-replace}\n");
	fprintf(fp0, "        %s [-q|-v] file1 {-sr|-sl|-su|-mh|-mv} {file2|-replace}\n", ProgName);
#endif
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
	fprintf(fp0, " This Program Rotate R-AXIS Image File.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -r:VALUE .... Rotate \"VALUE\" degrees.\n");
	fprintf(fp0, "   -n:VALUE .... Set division number of each pixel to \"VALUE\"^2.\n");
	fprintf(fp0, "                 Larger value makes more accurate result, but takes\n");
	fprintf(fp0, "                 long time. If absent, set to %2d.\n", DIVDEFAULT);
	fprintf(fp0, "                 With this option, work with division mode.\n");
	fprintf(fp0, "   -c .......... Instead of above, work with interpolation mode.\n");
	fprintf(fp0, "                 I do not know which of above 2 options and values are good.\n");
	fprintf(fp0, "                 Please choice by yourself.\n");
	fprintf(fp0, "   -mx ......... Create Image without any lost region.\n");
	fprintf(fp0, "   -dx:VALUE ... Change PixelSize in X to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -dz:VALUE ... Change PixelSize in Z to \"VALUE\" [mm].\n");
	fprintf(fp0, "   -sl ......... Rotate 90 degrees to left. With this option, direction of\n");
	fprintf(fp0, "                 array of pixel is changed, and pixel size in X and Z\n");
	fprintf(fp0, "                 is swapped, with no error.\n");
	fprintf(fp0, "   -sr ......... Rotate 90 degrees to right. Same as above.\n");
	fprintf(fp0, "   -su ......... Rotate 180 degrees.\n");
	fprintf(fp0, "   -mv ......... Mirror operation about vertical line.\n");
	fprintf(fp0, "   -mh ......... Mirror operation about horizontal line.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t .......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ....... Filename of Original R-AXIS Image File.\n");
	fprintf(fp0, "   file2 ....... Output R-AXIS Image Filename. This must be the last argument.\n");
	fprintf(fp0, "                 If last argument is \"-replace\", Original R-AXIS Image File is\n");
	fprintf(fp0, "                 replaced by Output File. Be careful using this option.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	double *arg1;
#else
char splitArg(const char *argV, char *arg0, double *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	return (2);
}
