/* Program to generate IP data file */
#define ProgName "iptiff2rax"
#define ProgInfo "Convert TIFF File to R-AXIS Image File"
/*                                                         11/5/2010  */
/*         for ANSI C                                      11/5/2010  */

/* This program depends on TIFF library.
   Install it from somewhere else, if you want to compile this program. */
/* If the system has "utime()" function and "stat()" system call,
   compile with "_HASUTIME" Macro Definition. This enables to use
   the time and mode of the Original File as the ones of New File. */
/* only 8 or 16 bits grayscale image is supported for the time being.
                                                           11/5/2010 */

#define COPYRIGHT "(C) Tsurutani Naoki"
#define VERSION "v1.00_11/5/2010"

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <time.h>
#ifdef _HASTIFF
#	include <tiffio.h>
#endif
#ifdef _HASUTIME
#	include <sys/stat.h>
#	include <utime.h>
#	include <sys/types.h>
#endif
#include "libm2.h"
#include "ipheader.h"
#include "libip.h"

#ifdef _HASUTIME
#	define MAXARG 9
#else
#	define MAXARG 8
#endif
#define MINARG 2
#define MAXWIDTH 10000
#define MAXHEIGHT 10000
#define DEFAULT_SIZE 0.05
#define DEFAULT_LENG 100.0

#ifdef _OLDHP
	char showMessage();
	char showUsage();
	char showHelp();
	char splitArg();
#else
	char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag);
	char showUsage(FILE *fp0);
	char showHelp(FILE *fp0);
	char splitArg(const char *argV, char *arg0, double *arg1);
#endif

#ifdef _OLDHP
int main(argc, argv)
	int argc;
	char *argv[];
#else
int main(int argc, char *argv[])
#endif
{
	short int *datPtr0, *datPtr2;
	long int i, j, k, ipHeaderSize, ipHeaderSize2VER2, width, height,
	     imageSize, recordLengthCreate, ipHeaderSizeMin,
	     flushWidth, flushWidth0;
	unsigned long int maxInt;
	unsigned short int resUnit, bitsPerSample, samplesPerPixel, *buf, *lineBufS;
	char file1[FNAMELEN], file2[FNAMELEN], cpuIntelFlag, reverseIntFlag, magIntFlag,
	     confirmBuffer[8], inputBuffer[20], argNum, arg0[4],
	     verboseFlag, shiftBit, xsizDoneFlag, zsizDoneFlag,
	     removeFlag, setTimeFlag, camrDoneFlag;
	unsigned char *lineBufC;
	double rato, arg1, intensityDbl, xRes, zRes,
	       maxDblInt, mag;
	struct ipHead ipHeader;
	time_t timeNow;
	FILE *fp2;
#ifdef _HASTIFF
	TIFF *tiff;
#endif
#ifdef _HASUTIME
	struct stat st1;
	struct utimbuf utime1;
#endif
	
/* Initialize and Definition of several critical values */
	ipHeaderSize = SIZE_OF_IPHEADER;
	ipHeaderSize2VER2 = SIZE_OF_IPHEADER2VER2;
	xsizDoneFlag = zsizDoneFlag = reverseIntFlag = magIntFlag
	 = removeFlag = setTimeFlag = camrDoneFlag = 0;
	verboseFlag = 1;
	mag = 1.0;
	rato = 8.0;
	shiftBit = 3;
	maxInt = ((long)(0x7fff)) << shiftBit;
	ipHeaderSizeMin = ipHeaderSize + ipHeaderSize2VER2;
	flushWidth0 = 0;
	
/* Check the size of each type and CPU type */
	cpuIntelFlag = cpuCheck();
	if (compilerCheck() && (compilerCheck() != 2)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (compilerCheckIp()) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of IP Header mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	if (cpuIntelFlag && (cpuIntelFlag != 1)) {
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Unable to recognize this Platform.\n");
		exit(EXIT_FAILURE);
	}
	if (sizeof(long) != 4 || sizeof(short) != 2) {	/* for TIFF */
		showMessage(stdout, 0, 0);
		fprintf(stderr, " Size of data mismatch. Stop.\n");
		exit(EXIT_FAILURE);
	}
	
/* Check zlib */
#ifndef _HASTIFF
	showMessage(stdout, cpuIntelFlag, 2);
	showUsage(stderr);
	fprintf(stderr, "This program does not work without \"tiff\" library.\n");
	fprintf(stderr, " Re-compile after install tiff and define \"_HASTIFF\".\n");
	exit(EXIT_FAILURE);
#else
	
/* Check Argument */
	if (argc == 1) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stderr);
		fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
		exit(EXIT_FAILURE);
	}
	if (!strcmp(argv[1], "-h")) {
		showMessage(stdout, cpuIntelFlag, 2);
		showUsage(stdout);
		showHelp(stdout);
		exit(EXIT_SUCCESS);
	}
	if (!strcmp(argv[1], "-i")) verboseFlag = 3;
	else {
		if (!strcmp(argv[1], "-q")) {
			verboseFlag = 0;
			argv += 2;
			argc -= 2;
		}
		else if (!strcmp(argv[1], "-v")) {
			verboseFlag = 2;
			argv += 2;
			argc -= 2;
		}
		else {
			argv++;
			argc--;
		}
		if (argc > MAXARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Too Many Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		if (argc < MINARG) {
			showMessage(stdout, cpuIntelFlag, verboseFlag);
			fprintf(stderr, " Invalid Arguments.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			exit(EXIT_FAILURE);
		}
		strcpy(file1, argv[0]);
		strcpy(file2, argv[argc-1]);
	}
	showMessage(stdout, cpuIntelFlag, verboseFlag);
	
/* File Read */
	if (verboseFlag == 3) {
		fprintf(stdout, "OK, proceed interactively.\n");
		fprintf(stdout, "Enter Original Filename : ");
		if (fscanf(stdin, "%s", file1) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			exit(EXIT_FAILURE);
		}
	}
	if ((tiff = TIFFOpen(file1, "rb")) == NULL) {
		fprintf(stderr, " Unable to Open File.\n");
		exit(EXIT_FAILURE);
	}
#ifdef _HASUTIME
	stat(file1, &st1);
	utime1.actime = st1.st_atime;
	utime1.modtime = st1.st_mtime;
#endif
	
/* Get TIFF File Information */
	TIFFGetField(tiff, TIFFTAG_IMAGEWIDTH, &width);
	TIFFGetField(tiff, TIFFTAG_IMAGELENGTH, &height);
	TIFFGetField(tiff, TIFFTAG_BITSPERSAMPLE, &bitsPerSample);
	TIFFGetField(tiff, TIFFTAG_SAMPLESPERPIXEL, &samplesPerPixel);
	TIFFGetField(tiff, TIFFTAG_RESOLUTIONUNIT, &resUnit);
	TIFFGetField(tiff, TIFFTAG_XRESOLUTION, &xRes);
	TIFFGetField(tiff, TIFFTAG_YRESOLUTION, &zRes);
	if ((width > MAXWIDTH) || (width <= 0)
	 || (height > MAXHEIGHT) || (height <= 0)
	 || (samplesPerPixel != 1)
	 || (
#if 0
	     (bitsPerSample != 1) && (bitsPerSample != 4) &&
#endif
	     (bitsPerSample != 8) && (bitsPerSample != 16))) {
		fprintf(stderr, " File Format is Incorrect.\n");
		if (verboseFlag >= 2) {
			fprintf(stderr, " width : %ld , height : %ld\n", width, height);
			fprintf(stderr, " samplesPerPixel : %d , bitsPerSample : %d .\n", samplesPerPixel, bitsPerSample);
		}
		TIFFClose(tiff);
		exit(EXIT_FAILURE);
	}
//	biCount = (0x0001 << bmpInfoHeader.biBitCount); /* = maximum colour depth */
	
/* Define ipHeader and Initialize */
	memset(&ipHeader, 0, ipHeaderSize);
	strcpy(ipHeader.name, "R-AXIS-DS"); /* Name of the Device */
	strcpy(ipHeader.vers, "1.0-2.0"); /* Version Number */
	strcpy(ipHeader.crnm, "N/A"); /* Crystal Name */
	strcpy(ipHeader.crsy, "cubic"); /* Crystal System */
	ipHeader.alng = 10.0; /* a-axis length in A */
	ipHeader.blng = 10.0; /* b-axis length in A */
	ipHeader.clng = 10.0; /* c-axis length in A */
	ipHeader.alfa = 90.0; /* angle alpha */
	ipHeader.beta = 90.0; /* angle beta */
	ipHeader.gamm = 90.0; /* angle gamma */
	strcpy(ipHeader.spgr, "P23"); /* Space Group */
	ipHeader.mosc = 0.3; /* Mozaic Spread */
	strcpy(ipHeader.opnm, "rigaku"); /* Operator Name */
	strcpy(ipHeader.trgt, "Cu"); /* X-ray Target */
	ipHeader.wlng = 1.5418; /* Wave Length of the x-ray */
	strcpy(ipHeader.mcro, "None"); /* Monochromator Material */
	ipHeader.m2ta = 0.0; /* Angle of the monochromator in 2-theta */
	strcpy(ipHeader.colm, "0.05-0.10"); /* Collimator */
	strcpy(ipHeader.filt, "Ni"); /* Kb filter */
	ipHeader.vltg = 40.0; /* X-ray tube voltage */
	ipHeader.crnt = 100.0; /* X-ray tube current */
	strcpy(ipHeader.focs, "0.3x0.3"); /* X-ray focus */
	strcpy(ipHeader.optc, "Ni-filter"); /* X-ray optics */
	ipHeader.cyld = 0; /* Camera type */
	ipHeader.weis = 0.0; /* weisenberg oscillation */
	strcpy(ipHeader.mnax, "-b*"); /* Mount Axis */
	strcpy(ipHeader.bmax, "-a"); /* Beam Axis */
	ipHeader.phi0 = 0.0; /* phi0 */
	ipHeader.phis = 0.0; /* phi start */
	ipHeader.phie = 0.0; /* phi end */
	ipHeader.oscn = 0; /* Frequency of Oscillation */
	ipHeader.fext = 20.0; /* Exposure Time in min. */
	ipHeader.omga = 0.0; /* omega in theta */
	ipHeader.fkai = 0.0; /* xai */
	ipHeader.thta = 0.000; /* 2-theta */
	ipHeader.mu = 0.000; /* mu */
	ipHeader.ipst = 0; /* Line of start reading */
	ipHeader.ipnm = 1; /* Number of IP */
	ipHeader.rato = rato; /* Ratio of IP in High/Low */
	ipHeader.ft_1 = 0.0; /* fading time 1 */
	ipHeader.ft_2 = 0.0; /* fading time 2 */
	strcpy(ipHeader.host, "IRIS"); /* Type of Host Computer */
	strcpy(ipHeader.ip, "HR-3"); /* Type of IP */
	ipHeader.dr_x = 0; /* Data direction in horizontal */
	ipHeader.dr_z = 0; /* Data direction in vertical */
	ipHeader.drxz = 0; /* Data direction face/back */
	timeNow = time(NULL);
	strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&timeNow));
	strcat(ipHeader.memo, ProgName);
	strcat(ipHeader.memo, "/");
	strcat(ipHeader.memo, ipHeader.date);
#ifdef _HASUTIME
	strftime(ipHeader.date, 12, "%Y-%m-%d", localtime(&utime1.modtime));
#endif
	
/* Set ipHeader Information */
	ipHeader.xpxl = width;
	ipHeader.zpxl = height;
	ipHeader.camr = DEFAULT_LENG;
	if (xRes <= 0.0 || zRes <= 0.0 || (resUnit != 2 && resUnit != 3)) {
		xRes = 0.0;
		zRes = 0.0;
		resUnit = 1;
	}
	else {
		if (resUnit == 2) {
			xRes = 25.4/xRes;
			zRes = 25.4/zRes;
		}
		else if (resUnit == 3) {
			xRes = 10.0/xRes;
			zRes = 10.0/zRes;
		}
	}
	
/* Conversion Parameter */
	if (verboseFlag >= 2) {
		fprintf(stdout, " Input File has ( %ld x %ld ) pixels, each pixel has %d bits.\n", width, height, bitsPerSample);
	}
	if (verboseFlag == 3) {
		do {
			fprintf(stdout, "Reverse intensity (i.e. \"White\" corresponds to \"0\") ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				TIFFClose(tiff);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') reverseIntFlag = 1;
		do {
			fprintf(stdout, "Do you want to Multiply some Constants to Intensity [Y/N] ? : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				TIFFClose(tiff);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') {
			do {
				fprintf(stdout, "Enter Magnification Value : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					TIFFClose(tiff);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				mag = atof(inputBuffer);
				if (mag < 0.0) continue;
				magIntFlag = 1;
			} while (!magIntFlag);
		}
		if (resUnit != 1) ipHeader.xsiz = xRes;
		else ipHeader.xsiz = DEFAULT_SIZE;
		do {
			fprintf(stdout, "PixelSize in X-direction is %.5f [mm]. OK ? [Y/N] : ", ipHeader.xsiz);
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				TIFFClose(tiff);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'N') {
			do {
				fprintf(stdout, "  Enter PixelSize in X-direction [mm] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					TIFFClose(tiff);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				ipHeader.xsiz = atof(inputBuffer);
				if (ipHeader.xsiz <= 0.0) continue;
				xsizDoneFlag = 1;
			} while (!xsizDoneFlag);
		}
		if (resUnit != 1) ipHeader.zsiz = zRes;
		else ipHeader.zsiz = DEFAULT_SIZE;
		do {
			fprintf(stdout, "PixelSize in Z-direction is %.5f [mm]. OK ? [Y/N] : ", ipHeader.zsiz);
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				TIFFClose(tiff);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'N') {
			do {
				fprintf(stdout, "  Enter PixelSize in Z-direction [mm] : ");
				if (fscanf(stdin, "%s", inputBuffer) == EOF) {
					fprintf(stderr, "\nError in getting input. Exit.\n");
					TIFFClose(tiff);
					exit(EXIT_FAILURE);
				}
				if (!isdigitEXT(inputBuffer)) continue;
				ipHeader.zsiz = atof(inputBuffer);
				if (ipHeader.zsiz <= 0.0) continue;
				zsizDoneFlag = 1;
			} while (!zsizDoneFlag);
		}
		do {
			fprintf(stdout, "Enter Camera Length [mm] : ");
			if (fscanf(stdin, "%s", inputBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				TIFFClose(tiff);
				exit(EXIT_FAILURE);
			}
			if (!isdigitEXT(inputBuffer)) continue;
			ipHeader.camr = atof(inputBuffer);
			if (ipHeader.camr <= 0.0) continue;
			camrDoneFlag = 1;
		} while (!camrDoneFlag);
		do {
			fprintf(stdout, "Delete Original TIFF File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				TIFFClose(tiff);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') removeFlag = 1;
#ifdef _HASUTIME
		do {
			fprintf(stdout, "Set Modification time of Output File as Original File ? [Y/N] : ");
			if (fscanf(stdin, "%s", confirmBuffer) == EOF) {
				fprintf(stderr, "\nError in getting input. Exit.\n");
				TIFFClose(tiff);
				exit(EXIT_FAILURE);
			}
		} while ((toupper(confirmBuffer[0]) != 'Y') && (toupper(confirmBuffer[0]) != 'N'));
		if (toupper(confirmBuffer[0]) == 'Y') setTimeFlag = 1;
#endif
	}
	else {
		if (resUnit != 1) {
			ipHeader.xsiz = xRes;
			ipHeader.zsiz = zRes;
		}
		else {
			ipHeader.xsiz = DEFAULT_SIZE;
			ipHeader.zsiz = DEFAULT_SIZE;
		}
		for (i=1; i<argc-1; i++) {
			argNum = splitArg(argv[i], arg0, &arg1);
			if (argNum == 1) {
				if (!strcmp(arg0, "-r")) reverseIntFlag++;
				else if (!strcmp(arg0, "-rm")) removeFlag++;
#ifdef _HASUTIME
				else if (!strcmp(arg0, "-t")) setTimeFlag++;
#endif
				else break;
			}
			else if (argNum == 2) {
				if (!strcmp(arg0, "-dx")) {
					ipHeader.xsiz = arg1;
					if (ipHeader.xsiz <= 0.0) break;
					xsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-dz")) {
					ipHeader.zsiz = arg1;
					if (ipHeader.zsiz <= 0.0) break;
					zsizDoneFlag++;
				}
				else if (!strcmp(arg0, "-cl")) {
					ipHeader.camr = arg1;
					if (ipHeader.camr <= 0.0) break;
					camrDoneFlag++;
				}
				else if (!strcmp(arg0, "-m")) {
					mag = arg1;
					if (mag < 0.0) break;
					magIntFlag++;
				}
				else break;
			}
			else break;
		}
		if (i != argc-1) {
			fprintf(stderr, " Invalid Arguments : \"%s\"\n", argv[i]);
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			TIFFClose(tiff);
			exit(EXIT_FAILURE);
		}
		if ((reverseIntFlag > 1) || (magIntFlag > 1) || (xsizDoneFlag > 1)
		 || (zsizDoneFlag > 1) || (removeFlag > 1) || (setTimeFlag > 1)
		 || (camrDoneFlag > 1)) {
			fprintf(stderr, " Arguments Duplicated or Conflicts.\n");
			showUsage(stderr);
			fprintf(stderr, " Try `%s -h\' to get help.\n", ProgName);
			TIFFClose(tiff);
			exit(EXIT_FAILURE);
		}
	}
	if (verboseFlag >= 2) {
		fprintf(stdout, " PixelSize is %8.5f [mm] x %8.5f [mm], Camera Length is %8.2f [mm]\n", ipHeader.xsiz, ipHeader.zsiz, ipHeader.camr);
		fprintf(stdout, " Data Conversion is ");
		if (reverseIntFlag) fprintf(stdout, "reverse, ");
		if (magIntFlag) fprintf(stdout, "magnified by %8.5f.\n", mag);
		else fprintf(stdout, "linear.\n");
		if (removeFlag) fprintf(stdout, " Original TIFF File will be removed !\n");
#ifdef _HASUTIME
		if (setTimeFlag) fprintf(stdout, " Modification time of Output File is set as Original File.\n");
#endif
	}
	
/* Set Parameters to Create */
	flushWidth = ipHeader.xpxl*RAXDATAWIDTH;
	recordLengthCreate =  MAXEXT(ipHeaderSizeMin, flushWidth);
	ipHeader.drtx = ipHeader.xpxl/2.0;
	ipHeader.drtz = ipHeader.zpxl/2.0;
	ipHeader.rlng = recordLengthCreate;
	ipHeader.rnum = ipHeader.zpxl;
	flushWidth0 = recordLengthCreate - flushWidth;
	maxDblInt = ((0x01)<<(bitsPerSample-2))*(double)2.0;
	imageSize = width * height * sizeof(unsigned short);
	
/* Create Output File */
	if (verboseFlag == 3) {
		fprintf(stdout, "Enter  Output  Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			TIFFClose(tiff);
			exit(EXIT_FAILURE);
		}
	}
	while (createFileEXT(file2, &fp2, 0)) {
		if (!verboseFlag) {
			fprintf(stderr, " Output File Creation Failure. Exit.\n");
			TIFFClose(tiff);
			exit(EXIT_FAILURE);
		}
		fprintf(stdout, " Cannot Create Output File. Enter Another Filename : ");
		if (fscanf(stdin, "%s", file2) == EOF) {
			fprintf(stderr, "\nError in getting filename. Exit.\n");
			TIFFClose(tiff);
			exit(EXIT_FAILURE);
		}
	}
	
/* Swap Header */
	if (cpuIntelFlag) swapIpHeader(&ipHeader);
	
/* Null arrangement */
	if ((datPtr0 = calloc(recordLengthCreate/RAXDATAWIDTH, sizeof(short))) == NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		TIFFClose(tiff);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	
/* Write Header */
	fwrite(&ipHeader, ipHeaderSize, 1, fp2);
	fwrite(datPtr0, recordLengthCreate-ipHeaderSize, 1, fp2);
	if (flushWidth0 == 0) free(datPtr0);
	
/* Swap Header again */
	if (cpuIntelFlag) swapIpHeader(&ipHeader);
	
/* Start Calculation */
	if (verboseFlag) fprintf(stdout, " Now, in progress...\n");
	if ((buf = (unsigned short*)malloc(imageSize))== NULL) {
		fprintf(stderr, " Failed to allocate memory.\n");
		free(datPtr0);
		TIFFClose(tiff);
		fclose(fp2);
		remove(file2);
		fprintf(stderr, " Output File is not created or removed.\n");
		exit(EXIT_FAILURE);
	}
	if (bitsPerSample == 16) {
		if ((lineBufS = (unsigned short*)_TIFFmalloc(TIFFScanlineSize(tiff))) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtr0);
			free(buf);
			TIFFClose(tiff);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		k = 0;
		for (i=0; i<height; i++) {
			if (TIFFReadScanline(tiff, lineBufS, i, 0) < 0) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtr0);
				free(buf);
				_TIFFfree(lineBufS);
				TIFFClose(tiff);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			for (j=0; j<width; j++) buf[k++] = lineBufS[j];
		}
		_TIFFfree(lineBufS);
	}
	else {
		if ((lineBufC = (unsigned char*)_TIFFmalloc(TIFFScanlineSize(tiff))) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			free(datPtr0);
			free(buf);
			TIFFClose(tiff);
			fclose(fp2);
			remove(file2);
			fprintf(stderr, " Output File is not created or removed.\n");
			exit(EXIT_FAILURE);
		}
		k = 0;
		for (i=0; i<height; i++) {
			if (TIFFReadScanline(tiff, lineBufC, i, 0) < 0) {
				fprintf(stderr, " Failed to allocate memory.\n");
				free(datPtr0);
				free(buf);
				_TIFFfree(lineBufC);
				TIFFClose(tiff);
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
				exit(EXIT_FAILURE);
			}
			for (j=0; j<width; j++) buf[k++] = lineBufC[j];
		}
		_TIFFfree(lineBufC);
	}
	TIFFClose(tiff);
	for (i=0; i<ipHeader.rnum; i++) {
		k = (ipHeader.rnum-i-1)*width;	/* TIFF starts with the top of the Image */
		if ((datPtr2 = malloc(flushWidth)) == NULL) {
			fprintf(stderr, " Failed to allocate memory.\n");
			if (flushWidth0 > 0) free(datPtr0);
			if (i) {
				ipHeader.zpxl = ipHeader.rnum = i;
				if (ipHeader.drtz >= ipHeader.zpxl) ipHeader.drtz = ipHeader.zpxl - 1.0;
				if (cpuIntelFlag) swapIpHeader(&ipHeader);
				fseek(fp2, 0, SEEK_SET);
				fwrite(&ipHeader, ipHeaderSize, 1, fp2);
				fprintf(stdout, " Output File is saved using available data.\n");
				if (cpuIntelFlag) swapIpHeader(&ipHeader);
				if (verboseFlag) {
					fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
					fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
					if (removeFlag) fprintf(stdout, " Original TIFF File is not removed.\n");
				}
				fclose(fp2);
#ifdef _HASUTIME
				if (setTimeFlag) {
					if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
				}
				if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
			}
			else {
				fclose(fp2);
				remove(file2);
				fprintf(stderr, " Output File is not created or removed.\n");
			}
			exit(EXIT_FAILURE);
		}
		if (bitsPerSample == 16) {
			for (j=0; j<ipHeader.xpxl; j++) {
				intensityDbl = buf[k++];
				if (reverseIntFlag) intensityDbl = maxDblInt - intensityDbl;
				if (magIntFlag) intensityDbl *= mag;
				datPtr2[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
			}
		}
		else {
			for (j=0; j<ipHeader.xpxl; j++) {
				intensityDbl = buf[k++];
				if (reverseIntFlag) intensityDbl = maxDblInt - intensityDbl;
				if (magIntFlag) intensityDbl *= mag;
				datPtr2[j] = ipDtoShort(&intensityDbl, &rato, &maxInt, &cpuIntelFlag);
			}
		}
		fwrite(datPtr2, flushWidth, 1, fp2);
		if (flushWidth0 > 0) fwrite(datPtr0, flushWidth0, 1, fp2);
		free(datPtr2);
	}
	
/* End */
	free(buf);
	if (flushWidth0 > 0) free(datPtr0);
	if (verboseFlag >= 2) {
		fprintf(stdout, " Output File has ( %ld x %ld ) pixels,\n", ipHeader.xpxl, ipHeader.zpxl);
		fprintf(stdout, "  %ld bytes/line and %ld lines, total %ld bytes.\n", ipHeader.rlng, ipHeader.rnum, ftell(fp2));
	}
	fclose(fp2);
#ifdef _HASUTIME
	if (setTimeFlag) {
		if (utime(file2, &utime1) && verboseFlag) fprintf(stderr, " Unable to set access time.\n");
	}
	if (chmod(file2, st1.st_mode) && verboseFlag) fprintf(stderr, " Unable to set file mode.\n");
#endif
	if (removeFlag) {
		if (remove(file1)) fprintf(stderr, " Unable to remove Original TIFF File.\n");
		else if (verboseFlag) fprintf(stdout, " Original TIFF File has Removed.\n");
	}
	if (verboseFlag) fprintf(stdout, "     ... Finished.\n");
	return 0;
#endif /* _HASTIFF */
}

#ifdef _OLDHP
char showMessage(fp0, cpuIntelFlag, verboseFlag)
	FILE *fp0;
	char cpuIntelFlag, verboseFlag;
#else
char showMessage(FILE *fp0, char cpuIntelFlag, char verboseFlag)
#endif
{
	fprintf(fp0, "%s : %s\n", ProgName, ProgInfo);
	fprintf(fp0, "    %s  %s\n", VERSION, COPYRIGHT);
#ifdef __STDC__
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System, compiled on %s at %s. For ANSI C.\n", cpuIntelFlag?"Little":"Big", __DATE__, __TIME__);
#else
	if (verboseFlag >= 2) fprintf(fp0, "    for %sEndian System. For non-ANSI C.\n", cpuIntelFlag?"Little":"Big");
	fprintf(fp0, "  Warning : Compiled with Non- ANSI C compiler might generate wrong result.\n");
#endif
#ifdef _HASUTIME
	if (verboseFlag >= 2) fprintf(fp0, "    Compiled with \"_HASUTIME\".\n");
#endif
	return (0);
}

#ifdef _OLDHP
char showUsage(fp0)
	FILE *fp0;
#else
char showUsage(FILE *fp0)
#endif
{
	fprintf(fp0, " Usage: %s {-h|-i}\n", ProgName);
#ifdef _HASUTIME
	fprintf(fp0, "        %s [-q|-v] file1 [-r] [-m:VALUE] [-rm] [-t]\n", ProgName);
#else
	fprintf(fp0, "        %s [-q|-v] file1 [-r] [-m:VALUE] [-rm]\n", ProgName);
#endif
	fprintf(fp0, "          [-dx:VALUE] [-dz:VALUE] [-cl:VALUE] file2\n");
	return (0);
}

#ifdef _OLDHP
char showHelp(fp0)
	FILE *fp0;
#else
char showHelp(FILE *fp0)
#endif
{
#if 0
	fprintf(fp0, " This Program Convert 1, 4, 8, or 16-bits gray-scale TIFF File\n");
#else
	fprintf(fp0, " This Program Convert 8, or 16-bits gray-scale TIFF File\n");
#endif
	fprintf(fp0, "  into R-AXIS Image File. Colour TIFF etc. may cause error.\n");
	fprintf(fp0, "   -h .......... Show This Help. Other Arguments are ignored.\n");
	fprintf(fp0, "   -i .......... Interactive Mode. Other Arguments are ignored.\n");
	fprintf(fp0, "   -q .......... Quiet Mode. Only important messages are shown.\n");
	fprintf(fp0, "   -v .......... Verbose Mode. Many messages are shown.\n");
	fprintf(fp0, "   -r .......... Reverse Intensity. i.e. \"0\" corresponds to White.\n");
	fprintf(fp0, "   -m:VALUE .... Intensity is multiplied by \"VALUE\".\n");
	fprintf(fp0, "   -dx:VALUE ... Set PixelSize in X-direction to \"VALUE\" [mm].\n");
	fprintf(fp0, "                 If absent, try to get infomation from TIFF File,\n");
	fprintf(fp0, "                 and if failed, set to %.5f.\n", DEFAULT_SIZE);
	fprintf(fp0, "   -dz:VALUE ... Set PixelSize in Z-direction to \"VALUE\".\n");
	fprintf(fp0, "                 If absent, same as above.\n");
	fprintf(fp0, "   -cl:VALUE ... Change Camera Langth to \"VALUE\" [mm].\n");
	fprintf(fp0, "                 If absent, set to %.5f.\n", DEFAULT_LENG);
	fprintf(fp0, "   -rm ......... Remove Original TIFF File after conversion.\n");
#ifdef _HASUTIME
	fprintf(fp0, "   -t .......... Set Modification time of Output File as Original File.\n");
#endif
	fprintf(fp0, "   file1 ....... Filename of Original TIFF File.\n");
	fprintf(fp0, "   file2 ....... Output R-AXIS Image Filename. This must be the last argument.\n");
	return (0);
}

#ifdef _OLDHP
char splitArg(argV, arg0, arg1)
	char *argV, *arg0;
	double *arg1;
#else
char splitArg(const char *argV, char *arg0, double *arg1)
#endif
{
	char *chPtr1;
	if (argV[0] != '-') return (0);
	if ((chPtr1 = strchr(argV, ':')) == NULL) {
		if (strlen(argV) > 3) return (0);
		strcpy(arg0, argV);
		return (1);
	}
	if (chPtr1-argV > 3) return (0);
	strncpy(arg0, argV, chPtr1-argV);
	arg0[chPtr1-argV] = 0;
	if (!isdigitEXT(chPtr1+1)) return (0);
	*arg1 = atof(chPtr1+1);
	return (2);
}
