/* Libraries fot FFT                                       3/19/2002  */
/*         for ANSI C                                                 */
/*                              v0.03d                     3/29/2005  */
/*                                               (C) Tsurutani Naoki  */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "libm2.h"
#include "libfft.h"

#ifdef _OLDHP
char createCosTable(table, fft_dim)
	fft_float *table;
	struct fft_dim *fft_dim;
#else
char createCosTable(fft_float *table, struct fft_dim *fft_dim)
#endif
{
	long int i;
	double phi, step;
	
	step = (fft_dim->dir & FFT_DIR_IDFT) ? M_2PI / (double)fft_dim->len : -M_2PI / (double)fft_dim->len;
	for (i=0; i<fft_dim->len; i++) {
		phi = i * step;
		table[               i] = cos(phi);
		table[fft_dim->len + i] = sin(phi);
	}
	return (0);
}

#ifdef _OLDHP
char fft_calc(data_r, data_i, fft_dim, table, buf)
	fft_float *data_r, *data_i, *table, *buf;
	struct fft_dim *fft_dim;
#else
char fft_calc(fft_float *data_r, fft_float *data_i, struct fft_dim *fft_dim, fft_float *table, fft_float *buf)
#endif
{
	long int i, j, k, bit, phi, m1, m2, n, len0, geta;
	double x1_r, x1_i, x2_r, x2_i;
	
	if (fft_dim->dir & FFT_DIR_IMG) {
		for (i=1; i<fft_dim->len; i+=2) {
			data_r[i] *= -1.0;
			data_i[i] *= -1.0;
		}
	}
	n = 1;
	len0 = fft_dim->len;
	for (i=0; i<fft_dim->num; i++) {
		len0 >>= 1;
		geta = 0;
		for (j=0; j<n; j++) {
			phi = 0;
			for (k=0; k<len0; k++) {
				m1 = geta + k;
				m2 = m1 + len0;
				x1_r = data_r[m1];
				x1_i = data_i[m1];
				x2_r = data_r[m2];
				x2_i = data_i[m2];
				data_r[m1] = x1_r + x2_r;
				data_i[m1] = x1_i + x2_i;
				x1_r -= x2_r;
				x1_i -= x2_i;
				data_r[m2] = x1_r * table[               phi] - x1_i * table[fft_dim->len + phi];
				data_i[m2] = x1_r * table[fft_dim->len + phi] + x1_i * table[               phi];
				phi += n;
			}
			geta += (len0<<1);
		}
		n <<= 1;
	}
	/* bit reversal */
	for (i=0; i<fft_dim->len; i++) {
		j = i;
		bit = 0;
		for (k=1;; k++) {
			bit |= (j&1);
			if (k == fft_dim->num) break;
			bit <<= 1;
			j >>= 1;
		}
		buf[i] = data_r[bit] * fft_dim->rev_len_sq;
		buf[fft_dim->len + i] = data_i[bit] * fft_dim->rev_len_sq;
	}
	for (i=0; i<fft_dim->len; i++) {
		data_r[i] = buf[i];
		data_i[i] = buf[fft_dim->len + i];
	}
	if (fft_dim->dir & FFT_DIR_IMG) {
		for (i=1; i<fft_dim->len; i+=2) {
			data_r[i] *= -1.0;
			data_i[i] *= -1.0;
		}
	}
	return (0);
}

#ifdef _OLDHP
char fft_calc_off(data_r, data_i, fft_dim, table, buf, offset)
	fft_float *data_r, *data_i, *table, *buf;
	struct fft_dim *fft_dim;
	long int offset;
#else
char fft_calc_off(fft_float *data_r, fft_float *data_i, struct fft_dim *fft_dim, fft_float *table, fft_float *buf, long int offset)
#endif
{
	long int i, j, k, bit, phi, m1, m2, n, len0, geta, off2;
	double x1_r, x1_i, x2_r, x2_i;
	
	if (fft_dim->dir & FFT_DIR_IMG) {
		for (i=1, j=offset; i<fft_dim->len; i+=2, j+=(offset * 2)) {
			data_r[j] *= -1.0;
			data_i[j] *= -1.0;
		}
	}
	n = 1;
	off2 = offset * 2;
	len0 = fft_dim->len * offset;
	for (i=0; i<fft_dim->num; i++) {
		len0 >>= 1;
		geta = 0;
		for (j=0; j<n; j++) {
			phi = 0;
			for (k=0; k<len0; k+=offset) {
				m1 = geta + k;
				m2 = m1 + len0;
				x1_r = data_r[m1];
				x1_i = data_i[m1];
				x2_r = data_r[m2];
				x2_i = data_i[m2];
				data_r[m1] = x1_r + x2_r;
				data_i[m1] = x1_i + x2_i;
				x1_r -= x2_r;
				x1_i -= x2_i;
				data_r[m2] = x1_r * table[               phi] - x1_i * table[fft_dim->len + phi];
				data_i[m2] = x1_r * table[fft_dim->len + phi] + x1_i * table[               phi];
				phi += n;
			}
			geta += (len0<<1);
		}
		n <<= 1;
	}
	/* bit reversal */
	for (i=0; i<fft_dim->len; i++) {
		j = i;
		bit = 0;
		for (k=1;; k++) {
			bit |= (j&1);
			if (k == fft_dim->num) break;
			bit <<= 1;
			j >>= 1;
		}
		buf[i] = data_r[bit * offset];
		buf[fft_dim->len + i] = data_i[bit * offset];
	}
	for (i=0, j=0; i<fft_dim->len; i++, j+=offset) {
		data_r[j] = buf[i] * fft_dim->rev_len_sq;
		data_i[j] = buf[fft_dim->len + i] * fft_dim->rev_len_sq;
	}
	if (fft_dim->dir & FFT_DIR_IMG) {
		for (i=1, j=offset; i<fft_dim->len; i+=2, j+=off2) {
			data_r[j] *= -1.0;
			data_i[j] *= -1.0;
		}
	}
	return (0);
}

#ifdef _OLDHP
char fft1d(data_r, data_i, fft_dim)
	fft_float *data_r, *data_i;
	struct fft_dim *fft_dim;
#else
char fft1d(fft_float *data_r, fft_float *data_i, struct fft_dim *fft_dim)
#endif
{
	fft_float *table, *buf;
	
	if ((table = malloc(fft_dim->len * sizeof(fft_float) * 2)) == NULL) {
		return (1);
	}
	if ((buf = malloc(fft_dim->len * sizeof(fft_float) * 2)) == NULL) {
		free(table);
		return (1);
	}
	createCosTable(table, fft_dim);
	fft_calc(data_r, data_i, fft_dim, table, buf);
	free(table);
	free(buf);
	return (0);
}

#ifdef _OLDHP
char fft2d(data_r, data_i, fft_dim_h, fft_dim_v)
	fft_float *data_r, *data_i;
	struct fft_dim *fft_dim_h, *fft_dim_v;
#else
char fft2d(fft_float *data_r, fft_float *data_i, struct fft_dim *fft_dim_h, struct fft_dim *fft_dim_v)
#endif
{
	long int i;
	fft_float *table_h, *table_v, *buf_h, *buf_v, *cur_r, *cur_i;
	
	/* FFT for horizontal */
	if ((table_h = malloc(fft_dim_h->len * sizeof(fft_float) * 2)) == NULL) {
		return (1);
	}
	createCosTable(table_h, fft_dim_h);
	if ((buf_h = malloc(fft_dim_h->len * sizeof(fft_float) * 2)) == NULL) {
		free(table_h);
		return (1);
	}
	cur_r = data_r;
	cur_i = data_i;
	for (i=0; i<fft_dim_v->len; i++) {
		fft_calc(cur_r, cur_i, fft_dim_h, table_h, buf_h);
		cur_r += fft_dim_h->len;
		cur_i += fft_dim_h->len;
	}
	free(buf_h);
	free(table_h);
	
	/* FFT for vertical */
	if ((table_v = malloc(fft_dim_v->len * sizeof(fft_float) * 2)) == NULL) {
		return (1);
	}
	if ((buf_v = malloc(fft_dim_v->len * sizeof(fft_float) * 2)) == NULL) {
		free(table_v);
		return (1);
	}
	createCosTable(table_v, fft_dim_v);
	cur_r = data_r;
	cur_i = data_i;
	for (i=0; i<fft_dim_h->len; i++) {
		fft_calc_off(cur_r, cur_i, fft_dim_v, table_v, buf_v, fft_dim_h->len);
		cur_r++;
		cur_i++;
	}
	free(buf_v);
	free(table_v);
	return (0);
}
