/* Libraries to treat IP data file                        11/12/1999  */
/*         for ANSI C                                     11/12/1999  */
/*                              v2.14                     12/25/1999  */
/*   Separated some common functions                        1/4/2000  */
/*                              v2.29                      1/15/2007  */
/*                                               (C) Tsurutani Naoki  */

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include "ipheader.h"
#include "libm2.h"
#include "libip.h"

#ifdef _OLDHP
char compilerCheckIp()
#else
char compilerCheckIp(void)
#endif
{
	if (SIZE_OF_IPHEADER != SIZE_OF_IPHEADER_EACH) return (1);
	if (SIZE_OF_IPHEADER2VER1 != SIZE_OF_IPHEADER2VER1_EACH) return (1);
	if (SIZE_OF_IPHEADER2VER2 != SIZE_OF_IPHEADER2VER2_EACH) return (1);
	return (0);
}

#ifdef _OLDHP
char ipMagicCheck(ipHeader)
	struct ipHead *ipHeader;
#else
char ipMagicCheck(struct ipHead *ipHeader)
#endif
{
	if (strncmp(ipHeader->name, "R-AXIS", 6) && strncmp(ipHeader->name, "RAXIS", 5)) return (1);
	return (0);
}

#ifdef _OLDHP
char swapIpHeader(ipHeader)
	struct ipHead *ipHeader;
#else
char swapIpHeader(struct ipHead *ipHeader)
#endif
{
	ipHeader->alng = swapFloat(ipHeader->alng);
	ipHeader->blng = swapFloat(ipHeader->blng);
	ipHeader->clng = swapFloat(ipHeader->clng);
	ipHeader->alfa = swapFloat(ipHeader->alfa);
	ipHeader->beta = swapFloat(ipHeader->beta);
	ipHeader->gamm = swapFloat(ipHeader->gamm);
	ipHeader->mosc = swapFloat(ipHeader->mosc);
	ipHeader->wlng = swapFloat(ipHeader->wlng);
	ipHeader->m2ta = swapFloat(ipHeader->m2ta);
	ipHeader->camr = swapFloat(ipHeader->camr);
	ipHeader->vltg = swapFloat(ipHeader->vltg);
	ipHeader->crnt = swapFloat(ipHeader->crnt);
	ipHeader->cyld = swapLong(ipHeader->cyld);
	ipHeader->weis = swapFloat(ipHeader->weis);
	ipHeader->phi0 = swapFloat(ipHeader->phi0);
	ipHeader->phis = swapFloat(ipHeader->phis);
	ipHeader->phie = swapFloat(ipHeader->phie);
	ipHeader->oscn = swapLong(ipHeader->oscn);
	ipHeader->fext = swapFloat(ipHeader->fext);
	ipHeader->drtx = swapFloat(ipHeader->drtx);
	ipHeader->drtz = swapFloat(ipHeader->drtz);
	ipHeader->omga = swapFloat(ipHeader->omga);
	ipHeader->fkai = swapFloat(ipHeader->fkai);
	ipHeader->thta = swapFloat(ipHeader->thta);
	ipHeader->mu = swapFloat(ipHeader->mu);
	ipHeader->xpxl = swapLong(ipHeader->xpxl);
	ipHeader->zpxl = swapLong(ipHeader->zpxl);
	ipHeader->xsiz = swapFloat(ipHeader->xsiz);
	ipHeader->zsiz = swapFloat(ipHeader->zsiz);
	ipHeader->rlng = swapLong(ipHeader->rlng);
	ipHeader->rnum = swapLong(ipHeader->rnum);
	ipHeader->ipst = swapLong(ipHeader->ipst);
	ipHeader->ipnm = swapLong(ipHeader->ipnm);
	ipHeader->rato = swapFloat(ipHeader->rato);
	ipHeader->ft_1 = swapFloat(ipHeader->ft_1);
	ipHeader->ft_2 = swapFloat(ipHeader->ft_2);
	ipHeader->dr_x = swapLong(ipHeader->dr_x);
	ipHeader->dr_z = swapLong(ipHeader->dr_z);
	ipHeader->drxz = swapLong(ipHeader->drxz);
	return (0);
}

#ifdef _OLDHP
char swapIpHeader2VER1(ipHeader2)
	struct ipHead2VER1 *ipHeader2;
#else
char swapIpHeader2VER1(struct ipHead2VER1 *ipHeader2)
#endif
{
	ipHeader2->iext = swapShort(ipHeader2->iext);
	ipHeader2->reso = swapShort(ipHeader2->reso);
	ipHeader2->save = swapShort(ipHeader2->save);
/*	ipHeader2->dint = swapLong(ipHeader2->dint);*/
	ipHeader2->dint1 = swapShort(ipHeader2->dint2);
	ipHeader2->dint2 = swapShort(ipHeader2->dint1);
	ipHeader2->byte = swapShort(ipHeader2->byte);
	return (0);
}

#ifdef _OLDHP
char swapIpHeader2VER2(ipHeader2)
	struct ipHead2VER2 *ipHeader2;
#else
char swapIpHeader2VER2(struct ipHead2VER2 *ipHeader2)
#endif
{
	int i;
	ipHeader2->iext = swapLong(ipHeader2->iext);
	ipHeader2->reso = swapLong(ipHeader2->reso);
	ipHeader2->save = swapLong(ipHeader2->save);
	ipHeader2->dint = swapLong(ipHeader2->dint);
	ipHeader2->byte = swapLong(ipHeader2->byte);
	ipHeader2->init = swapLong(ipHeader2->init);
	ipHeader2->ipus = swapLong(ipHeader2->ipus);
	ipHeader2->dexp = swapLong(ipHeader2->dexp);
	ipHeader2->expn = swapLong(ipHeader2->expn);
	for (i=0; i<20; i++) {
		ipHeader2->posx[i] = swapLong(ipHeader2->posx[i]);
		ipHeader2->posy[i] = swapLong(ipHeader2->posy[i]);
	}
	ipHeader2->xray = swapLong(ipHeader2->xray);
	return (0);
}

#ifdef _OLDHP
char cleanIpHeader(ipHeader)
	struct ipHead *ipHeader;
#else
char cleanIpHeader(struct ipHead *ipHeader)
#endif
{
	short int i;
	for (i=1; i<=10; i++) {
		if (ipHeader->name[10-i] == 0x00 || ipHeader->name[10-i] == 0x20) ipHeader->name[10-i] = 0x00;
		else break;
	}
	for (i=1; i<=10; i++) {
		if (ipHeader->vers[10-i] == 0x00 || ipHeader->vers[10-i] == 0x20) ipHeader->vers[10-i] = 0x00;
		else break;
	}
	for (i=1; i<=20; i++) {
		if (ipHeader->crnm[20-i] == 0x00 || ipHeader->crnm[20-i] == 0x20) ipHeader->crnm[20-i] = 0x00;
		else break;
	}
	for (i=1; i<=12; i++) {
		if (ipHeader->crsy[12-i] == 0x00 || ipHeader->crsy[12-i] == 0x20) ipHeader->crsy[12-i] = 0x00;
		else break;
	}
	for (i=1; i<=12; i++) {
		if (ipHeader->spgr[12-i] == 0x00 || ipHeader->spgr[12-i] == 0x20) ipHeader->spgr[12-i] = 0x00;
		else break;
	}
	for (i=1; i<=80; i++) {
		if (ipHeader->memo[80-i] == 0x00 || ipHeader->memo[80-i] == 0x20) ipHeader->memo[80-i] = 0x00;
		else break;
	}
	for (i=1; i<=84; i++) {
		if (ipHeader->res1[84-i] == 0x00 || ipHeader->res1[84-i] == 0x20) ipHeader->res1[84-i] = 0x00;
		else break;
	}
	for (i=1; i<=12; i++) {
		if (ipHeader->date[12-i] == 0x00 || ipHeader->date[12-i] == 0x20) ipHeader->date[12-i] = 0x00;
		else break;
	}
	for (i=1; i<=20; i++) {
		if (ipHeader->opnm[20-i] == 0x00 || ipHeader->opnm[20-i] == 0x20) ipHeader->opnm[20-i] = 0x00;
		else break;
	}
	for (i=1; i<=4; i++) {
		if (ipHeader->trgt[4-i] == 0x00 || ipHeader->trgt[4-i] == 0x20) ipHeader->trgt[4-i] = 0x00;
		else break;
	}
	for (i=1; i<=20; i++) {
		if (ipHeader->mcro[20-i] == 0x00 || ipHeader->mcro[20-i] == 0x20) ipHeader->mcro[20-i] = 0x00;
		else break;
	}
	for (i=1; i<=20; i++) {
		if (ipHeader->colm[20-i] == 0x00 || ipHeader->colm[20-i] == 0x20) ipHeader->colm[20-i] = 0x00;
		else break;
	}
	for (i=1; i<=4; i++) {
		if (ipHeader->filt[4-i] == 0x00 || ipHeader->filt[4-i] == 0x20) ipHeader->filt[4-i] = 0x00;
		else break;
	}
	for (i=1; i<=12; i++) {
		if (ipHeader->focs[12-i] == 0x00 || ipHeader->focs[12-i] == 0x20) ipHeader->focs[12-i] = 0x00;
		else break;
	}
	for (i=1; i<=80; i++) {
		if (ipHeader->optc[80-i] == 0x00 || ipHeader->optc[80-i] == 0x20) ipHeader->optc[80-i] = 0x00;
		else break;
	}
	for (i=1; i<=56; i++) {
		if (ipHeader->res2[56-i] == 0x00 || ipHeader->res2[56-i] == 0x20) ipHeader->res2[56-i] = 0x00;
		else break;
	}
	for (i=1; i<=4; i++) {
		if (ipHeader->mnax[4-i] == 0x00 || ipHeader->mnax[4-i] == 0x20) ipHeader->mnax[4-i] = 0x00;
		else break;
	}
	for (i=1; i<=4; i++) {
		if (ipHeader->bmax[4-i] == 0x00 || ipHeader->bmax[4-i] == 0x20) ipHeader->bmax[4-i] = 0x00;
		else break;
	}
	for (i=1; i<=204; i++) {
		if (ipHeader->res3[204-i] == 0x00 || ipHeader->res3[204-i] == 0x20) ipHeader->res3[204-i] = 0x00;
		else break;
	}
	for (i=1; i<=10; i++) {
		if (ipHeader->host[10-i] == 0x00 || ipHeader->host[10-i] == 0x20) ipHeader->host[10-i] = 0x00;
		else break;
	}
	for (i=1; i<=10; i++) {
		if (ipHeader->ip[10-i] == 0x00 || ipHeader->ip[10-i] == 0x20) ipHeader->ip[10-i] = 0x00;
		else break;
	}
	for (i=1; i<=180; i++) {
		if (ipHeader->res4[180-i] == 0x00 || ipHeader->res4[180-i] == 0x20) ipHeader->res4[180-i] = 0x00;
		else break;
	}
	return (0);
}

#ifdef _OLDHP
char cleanIpHeader2VER1(ipHeader2)
	struct ipHead2VER1 *ipHeader2;
#else
char cleanIpHeader2VER1(struct ipHead2VER1 *ipHeader2)
#endif
{
	short int i;
	for (i=1; i<=16; i++) {
		if (ipHeader2->file[16-i] == 0x00 || ipHeader2->file[16-i] == 0x20) ipHeader2->file[16-i] = 0x00;
		else break;
	}
	for (i=1; i<=996; i++) {
		if (ipHeader2->res5[996-i] == 0x00 || ipHeader2->res5[996-i] == 0x20) ipHeader2->res5[996-i] = 0x00;
		else break;
	}
	return (0);
}

#ifdef _OLDHP
char cleanIpHeader2VER2(ipHeader2)
	struct ipHead2VER2 *ipHeader2;
#else
char cleanIpHeader2VER2(struct ipHead2VER2 *ipHeader2)
#endif
{
	short int i;
	for (i=1; i<=16; i++) {
		if (ipHeader2->file[16-i] == 0x00 || ipHeader2->file[16-i] == 0x20) ipHeader2->file[16-i] = 0x00;
		else break;
	}
	for (i=1; i<=20; i++) {
		if (ipHeader2->cmnt[20-i] == 0x00 || ipHeader2->cmnt[20-i] == 0x20) ipHeader2->cmnt[20-i] = 0x00;
		else break;
	}
	for (i=1; i<=20; i++) {
		if (ipHeader2->smpl[20-i] == 0x00 || ipHeader2->smpl[20-i] == 0x20) ipHeader2->smpl[20-i] = 0x00;
		else break;
	}
	for (i=1; i<=768; i++) {
		if (ipHeader2->res5[768-i] == 0x00 || ipHeader2->res5[768-i] == 0x20) ipHeader2->res5[768-i] = 0x00;
		else break;
	}
	return (0);
}

#ifdef _OLDHP
char logWrite(ipHeader, str)
	struct ipHead *ipHeader;
	char *str;
#else
char logWrite(struct ipHead *ipHeader, const char *str)
#endif
{
	time_t timeNow;
	char dateBuf[12], message[20], messageLength;
	timeNow = time(NULL);
	strftime(dateBuf, 12, "%Y-%m-%d", localtime(&timeNow));
	messageLength = strlen(str) + 12;
	strcpy(message, str);
	strcat(message, "/");
	if (strlen(ipHeader->memo) == 0) {
		strcat(ipHeader->memo, message);
		strcat(ipHeader->memo, dateBuf);
		return (0);
	}
	if (strlen(ipHeader->memo)+messageLength < sizeof(ipHeader->memo)) {
		strcat(ipHeader->memo, ", ");
		strcat(ipHeader->memo, message);
		strcat(ipHeader->memo, dateBuf);
		return (0);
	}
	return (1);
}

#if 0
#ifdef _OLDHP
char y2k(ipHeader)
	struct ipHead *ipHeader;
#else
char y2k(struct ipHead *ipHeader)
#endif
{
	char yearBuf[8];
	long int year;
	strcpy(yearBuf, "xxxxxxx");
	strncpy(yearBuf, ipHeader->date, 4);
	year = atol(yearBuf);
	if ((year < 1969) && (year > 1899)) {
		sprintf(yearBuf, "%.4d", year+100);
		strncpy(ipHeader->date, yearBuf, 4);
		return (1);
	}
	return (0);
}
#endif

#ifdef _OLDHP
long int ipStoLongBit(x, rato, shiftBit, ipDataSwapFlag)
	short int *x;
	double *rato;
	char *shiftBit, *ipDataSwapFlag;
#else
long int ipStoLongBit(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag)
#endif
{
	if (*ipDataSwapFlag) {
		if (!((*x) & 0x0080)) return ((long)(swapShort(*x)));
		return ((long)((swapShort(*x)) & 0x7fff) << (*shiftBit));
	}
	else {
		if (!((*x) & 0x8000)) return ((long)(*x));
		return ((long)((*x) & 0x7fff) << (*shiftBit));
	}
}

#ifdef _OLDHP
long int ipStoLongRato(x, rato, shiftBit, ipDataSwapFlag)
	short int *x;
	double *rato;
	char *shiftBit, *ipDataSwapFlag;
#else
long int ipStoLongRato(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag)
#endif
{
	if (*ipDataSwapFlag) {
		if (!((*x) & 0x0080)) return ((long)(swapShort(*x)));
		return (dToLong((long)((swapShort(*x)) & 0x7fff) * (*rato)));
	}
	else {
		if (!((*x) & 0x8000)) return ((long)(*x));
		return (dToLong((long)((*x) & 0x7fff) * (*rato)));
	}
}

#ifdef _OLDHP
long int ipStoLongBit0(x, rato, shiftBit, ipDataSwapFlag)
	short int *x;
	double *rato;
	char *shiftBit, *ipDataSwapFlag;
#else
long int ipStoLongBit0(const short int *x, const double *rato, const char *shiftBit, const char *ipDataSwapFlag)
#endif
{
	if (*ipDataSwapFlag) {
		if (!((*x) & 0x0080)) return ((long)(swapShort(*x)));
		return (0);
	}
	else {
		if (!((*x) & 0x8000)) return ((long)(*x));
		return (0);
	}
}

#ifdef _OLDHP
short int ipLtoShortBit(x, rato, shiftBit, maxInt, ipDataSwapFlag)
	long int *x;
	unsigned long int *maxInt;
	double *rato;
	char *shiftBit, *ipDataSwapFlag;
#else
short int ipLtoShortBit(const long int *x, const double *rato, const char *shiftBit, const unsigned long int *maxInt, const char *ipDataSwapFlag)
#endif
{
	if (*ipDataSwapFlag) {
		if ((*x) < 0) return (0);
		if ((unsigned long)(*x) >= (*maxInt)) return (0xffff);
		if (!((*x) & 0xffff8000)) return (swapShort((short)(*x)));
		if (((*x) >> ((*shiftBit) - 1)) & 0x00000001) return (swapShort((short)(((*x) >> (*shiftBit)) + 1) | 0x8000));
		return (swapShort((short)((*x) >> (*shiftBit)) | 0x8000));
	}
	else {
		if ((*x) < 0) return (0);
		if ((unsigned long)(*x) >= (*maxInt)) return (0xffff);
		if (!((*x) & 0xffff8000)) return ((short)(*x));
		if (((*x) >> ((*shiftBit) - 1)) & 0x00000001) return ((short)(((*x) >> (*shiftBit)) + 1) | 0x8000);
		return ((short)((*x) >> (*shiftBit)) | 0x8000);
	}
}

#ifdef _OLDHP
short int ipULtoShortBit(x, rato, shiftBit, maxInt, ipDataSwapFlag)
	unsigned long int *x, *maxInt;
	double *rato;
	char *shiftBit, *ipDataSwapFlag;
#else
short int ipULtoShortBit(const unsigned long int *x, const double *rato, const char *shiftBit, const unsigned long int *maxInt, const char *ipDataSwapFlag)
#endif
{
	if (*ipDataSwapFlag) {
		if ((*x) >= (*maxInt)) return (0xffff);
		if (!((*x) & 0xffff8000)) return (swapShort((short)(*x)));
		if (((*x) >> ((*shiftBit) - 1)) & 0x00000001) return (swapShort((short)(((*x) >> (*shiftBit)) + 1) | 0x8000));
		return (swapShort((short)((*x) >> (*shiftBit)) | 0x8000));
	}
	else {
		if ((*x) >= (*maxInt)) return (0xffff);
		if (!((*x) & 0xffff8000)) return ((short)(*x));
		if (((*x) >> ((*shiftBit) - 1)) & 0x00000001) return ((short)(((*x) >> (*shiftBit)) + 1) | 0x8000);
		return ((short)((*x) >> (*shiftBit)) | 0x8000);
	}
}

#ifdef _OLDHP
short int ipLtoShortRato(x, rato, shiftBit, maxInt, ipDataSwapFlag)
	long int *x;
	unsigned long int *maxInt;
	double *rato;
	char *shiftBit, *ipDataSwapFlag;
#else
short int ipLtoShortRato(const long int *x, const double *rato, const char *shiftBit, const unsigned long int *maxInt, const char *ipDataSwapFlag)
#endif
{
	if (*ipDataSwapFlag) {
		if ((*x) < 0) return (0);
		if ((unsigned long)(*x) >= (*maxInt)) return (0xffff);
		if (!((*x) & 0xffff8000)) return (swapShort((short)(*x)));
		return (swapShort((dToShort((*x) / (*rato))) | 0x8000));
	}
	else {
		if ((*x) < 0) return (0);
		if ((unsigned long)(*x) >= (*maxInt)) return (0xffff);
		if (!((*x) & 0xffff8000)) return ((short)(*x));
		return ((dToShort((*x) / (*rato))) | 0x8000);
	}
}

#ifdef _OLDHP
short int ipULtoShortRato(x, rato, shiftBit, maxInt, ipDataSwapFlag)
	unsigned long int *x, *maxInt;
	double *rato;
	char *shiftBit, *ipDataSwapFlag;
#else
short int ipULtoShortRato(const unsigned long int *x, const double *rato, const char *shiftBit, const unsigned long int *maxInt, const char *ipDataSwapFlag)
#endif
{
	if (*ipDataSwapFlag) {
		if ((*x) >= (*maxInt)) return (0xffff);
		if (!((*x) & 0xffff8000)) return (swapShort((short)(*x)));
		return (swapShort((dToShort((*x) / (*rato))) | 0x8000));
	}
	else {
		if ((*x) >= (*maxInt)) return (0xffff);
		if (!((*x) & 0xffff8000)) return ((short)(*x));
		return ((dToShort((*x) / (*rato))) | 0x8000);
	}
}

#ifdef _OLDHP
short int ipLtoShortBit0(x, rato, shiftBit, maxInt, ipDataSwapFlag)
	unsigned long int *x, *maxInt;
	double *rato;
	char *shiftBit, *ipDataSwapFlag;
#else
short int ipLtoShortBit0(const long int *x, const double *rato, const char *shiftBit, const unsigned long int *maxInt, const char *ipDataSwapFlag)
#endif
{
	if (*ipDataSwapFlag) {
		if ((*x) < 0) return (0);
		if ((unsigned long)(*x) >= (*maxInt)) return (0xffff);
		return (swapShort((short)(*x)));
	}
	else {
		if ((*x) < 0) return (0);
		if ((unsigned long)(*x) >= (*maxInt)) return (0xffff);
		return ((short)(*x));
	}
}

#ifdef _OLDHP
short int ipULtoShortBit0(x, rato, shiftBit, maxInt, ipDataSwapFlag)
	unsigned long int *x, *maxInt;
	double *rato;
	char *shiftBit, *ipDataSwapFlag;
#else
short int ipULtoShortBit0(const unsigned long int *x, const double *rato, const char *shiftBit, const unsigned long int *maxInt, const char *ipDataSwapFlag)
#endif
{
	if (*ipDataSwapFlag) {
		if ((*x) >= (*maxInt)) return (0xffff);
		return (swapShort((short)(*x)));
	}
	else {
		if ((*x) >= (*maxInt)) return (0xffff);
		return ((short)(*x));
	}
}

#ifdef _OLDHP
short int ipDtoShort(x, rato, maxInt, ipDataSwapFlag)
	double *x, *rato;
	unsigned long int *maxInt;
	char *ipDataSwapFlag;
#else
short int ipDtoShort(const double *x, const double *rato, const unsigned long int *maxInt, const char *ipDataSwapFlag)
#endif
{
	if (*ipDataSwapFlag) {
		if ((*x) <= 0.0) return (0);
		if ((*x) >= (*maxInt)) return (0xffff);
		if (dToLong(*x) <= 0x00007fff) return (swapShort(dToShort(*x)));
		return (swapShort((dToShort((*x) / (*rato))) | 0x8000));
	}
	else {
		if ((*x) <= 0.0) return (0);
		if ((*x) >= (*maxInt)) return (0xffff);
		if (dToLong(*x) <= 0x00007fff) return (dToShort(*x));
		return ((dToShort((*x) / (*rato))) | 0x8000);
	}
}

#ifdef _OLDHP
short int ipDtoShortBit0(x, rato, maxInt, ipDataSwapFlag)
	double *x, *rato;
	unsigned long int *maxInt;
	char *ipDataSwapFlag;
#else
short int ipDtoShortBit0(const double *x, const double *rato, const unsigned long int *maxInt, const char *ipDataSwapFlag)
#endif
{
	if (*ipDataSwapFlag) {
		if ((*x) <= 0.0) return (0);
		if ((*x) >= (*maxInt)) return (0xffff);
		return (swapShort(dToShort(*x)));
	}
	else {
		if ((*x) <= 0.0) return (0);
		if ((*x) >= (*maxInt)) return (0xffff);
		return (dToShort(*x));
	}
}

#ifdef _OLDHP
short int ipFtoShort(x, rato, maxInt, ipDataSwapFlag)
	float *x;
	double *rato;
	unsigned long int *maxInt;
	char *ipDataSwapFlag;
#else
short int ipFtoShort(const float *x, const double *rato, const unsigned long int *maxInt, const char *ipDataSwapFlag)
#endif
{
	if (*ipDataSwapFlag) {
		if ((*x) <= 0.0) return (0);
		if ((*x) >= (*maxInt)) return (0xffff);
		if (dToLong((double)(*x)) <= 0x00007fff) return (swapShort(dToShort((double)(*x))));
		return (swapShort((dToShort((*x) / (*rato))) | 0x8000));
	}
	else {
		if ((*x) <= 0.0) return (0);
		if ((*x) >= (*maxInt)) return (0xffff);
		if (dToLong((double)(*x)) <= 0x00007fff) return (dToShort((double)(*x)));
		return ((dToShort((*x) / (*rato))) | 0x8000);
	}
}

#ifdef _OLDHP
short int ipFtoShortBit0(x, rato, maxInt, ipDataSwapFlag)
	float *x;
	double *rato;
	unsigned long int *maxInt;
	char *ipDataSwapFlag;
#else
short int ipFtoShortBit0(const float *x, const double *rato, const unsigned long int *maxInt, const char *ipDataSwapFlag)
#endif
{
	if (*ipDataSwapFlag) {
		if ((*x) <= 0.0) return (0);
		if ((*x) >= (*maxInt)) return (0xffff);
		return (swapShort(dToShort((double)(*x))));
	}
	else {
		if ((*x) <= 0.0) return (0);
		if ((*x) >= (*maxInt)) return (0xffff);
		return (dToShort((double)(*x)));
	}
}
