/* Libraries to treat data                                11/12/1999  */
/*         for ANSI C                                     11/12/1999  */
/*                              v2.14                     12/25/1999  */
/*   Separated from libip.c                                 1/4/2000  */
/*                              v2.47                       3/7/2009  */
/*                                               (C) Tsurutani Naoki  */

/* Programs using this code assumes to be compiled with ANSI C compiler.
   But C compiler on standard HP-UX 9.0x operating system is not based on
   ANSI C nor has ANSI C mode, so if you want to compile with HP-UX cc,
   set Macro Definition "_OLDHP" as
     cc -D_OLDHP source.c ...
   This definition is probably also valid with ANSI C compiler...
   If _OLDHP is not defined and non-ANSI C compiler (__STDC__ is not defined),
   programs does not work.
   _OLDHP macro mode might cause another problem, but I do not know...
   Only HP-UX 9.03 C compiler is taken into account. Other K&R C based may
   need Low Level I/O functions such as open, write ...
                                                          8 Jan, 2000 */
/* If failed to judge "endian" of the system, define "_ENDIAN_B" or "_ENDIAN_L"
   for Big-Endian and Little-Endian system respectively, when you compile.
                                                          7 Feb, 2000 */

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <ctype.h>
#include <math.h>
#include <string.h>
#include <limits.h>
#include <float.h>
#ifdef _HAS_MKSTEMP
#	include <unistd.h>
#endif
#if defined(_HAS_OPEN) || defined(_HAS__OPEN)
#	include <fcntl.h>
#	ifdef _HAS_IOH
#		include <io.h>
#	endif
#endif
#include "libm2.h"

#ifdef _OLDHP
double dist(x, y)
	double x, y;
#else
double dist(double x, double y)
#endif
{
	return (sqrt(x*x + y*y));
}

#ifdef _OLDHP
float swapFloat(x)
	float x;
#else
float swapFloat(float x)
#endif
{
	union {
		char c_tmp[4];
		float f_tmp;
	} swap4;
	char s_tmp;
	swap4.f_tmp = x;
	s_tmp = swap4.c_tmp[0];
	swap4.c_tmp[0] = swap4.c_tmp[3];
	swap4.c_tmp[3] = s_tmp;
	s_tmp = swap4.c_tmp[1];
	swap4.c_tmp[1] = swap4.c_tmp[2];
	swap4.c_tmp[2] = s_tmp;
	return (swap4.f_tmp);
}

#ifdef _OLDHP
long int swapLong(x)
	long int x;
#else
long int swapLong(long int x)
#endif
{
	union {
		char c_tmp[4];
		long int i_tmp;
	} swap4;
	char s_tmp;
	swap4.i_tmp = x;
	s_tmp = swap4.c_tmp[0];
	swap4.c_tmp[0] = swap4.c_tmp[3];
	swap4.c_tmp[3] = s_tmp;
	s_tmp = swap4.c_tmp[1];
	swap4.c_tmp[1] = swap4.c_tmp[2];
	swap4.c_tmp[2] = s_tmp;
	return (swap4.i_tmp);
}

#ifdef _OLDHP
unsigned long int swapULong(x)
	unsigned long int x;
#else
unsigned long int swapULong(unsigned long int x)
#endif
{
	union {
		char c_tmp[4];
		unsigned long int i_tmp;
	} swap4;
	char s_tmp;
	swap4.i_tmp = x;
	s_tmp = swap4.c_tmp[0];
	swap4.c_tmp[0] = swap4.c_tmp[3];
	swap4.c_tmp[3] = s_tmp;
	s_tmp = swap4.c_tmp[1];
	swap4.c_tmp[1] = swap4.c_tmp[2];
	swap4.c_tmp[2] = s_tmp;
	return (swap4.i_tmp);
}

#ifdef _OLDHP
short int swapShort(x)
	short int x;
#else
short int swapShort(short int x)
#endif
{
	union {
		char c_tmp[2];
		short int i_tmp;
	} swap2;
	char s_tmp;
	swap2.i_tmp = x;
	s_tmp = swap2.c_tmp[0];
	swap2.c_tmp[0] = swap2.c_tmp[1];
	swap2.c_tmp[1] = s_tmp;
	return (swap2.i_tmp);
}

#ifdef _OLDHP
unsigned short int swapUShort(x)
	unsigned short int x;
#else
unsigned short int swapUShort(unsigned short int x)
#endif
{
	union {
		char c_tmp[2];
		unsigned short int i_tmp;
	} swap2;
	char s_tmp;
	swap2.i_tmp = x;
	s_tmp = swap2.c_tmp[0];
	swap2.c_tmp[0] = swap2.c_tmp[1];
	swap2.c_tmp[1] = s_tmp;
	return (swap2.i_tmp);
}

#ifdef _OLDHP
long int dToLong(x)
	double x;
#else
long int dToLong(double x)
#endif
{
	return ((long)(floor(x + 0.5)));
}

#ifdef _OLDHP
unsigned long int dToULong(x)
	double x;
#else
unsigned long int dToULong(double x)
#endif
{
	return ((unsigned long)(floor(x + 0.5)));
}

#ifdef _OLDHP
short int dToShort(x)
	double x;
#else
short int dToShort(double x)
#endif
{
	return ((short)(floor(x + 0.5)));
}

#ifdef _OLDHP
unsigned short int dToUShort(x)
	double x;
#else
unsigned short int dToUShort(double x)
#endif
{
	return ((unsigned short)(floor(x + 0.5)));
}

#ifdef _OLDHP
char cpuCheck()
#else
char cpuCheck(void)
#endif
{
/* CPU Check routine */
#ifdef _ENDIAN_L
	return (1);
#endif
#ifdef _ENDIAN_B
	return (0);
#endif
	char int_a='a';
	union {
		short int sint;
		char characters[2];
	} endians;
	strcpy(endians.characters , "a");
	if (endians.sint == (short) int_a) return (1); /* Intel, or Little Endian */
	if (endians.sint == ((short) int_a) << 8) return (0); /* Non-Intel, or Big Endian */
	return (2); /* Error */
}

#ifdef _OLDHP
char compilerCheck()
#else
char compilerCheck(void)
#endif
{
/* Chack the size of each type */
	if ((sizeof(INT32) != 4)  || (sizeof(INT16) != 2)
	 || (sizeof(UINT32) != 4) || (sizeof(UINT16) != 2)
	 || (sizeof(float) != 4)) return (1);
	/* Currently, gcc for 64-bit system such as Alpha or SPARC processor,
	    short int is 2 bytes and float is 4 bytes, but
	    long int is 8 bytes, which is not match the size expected.
	    Change "long int" or "long" to simple "int" in all source and header files.
	                                                                11 Jan, 2000 */
	/*  Now, work in progress ... 7 Mar, 2009 */
#ifdef _OLDHP
	return (2);
#else
#	ifndef __STDC__
		return (3);
#	endif
#endif
	return (0);
}

#ifdef _OLDHP
char isdigitEXT(str)
	char *str;
#else
char isdigitEXT(const char *str)
#endif
{
	unsigned long int i;
	for (i=0; i<strlen(str); i++) {
		if (isdigit((int)str[i]) || (str[i] == '-') || (str[i] == '+') || (str[i] == '.')) return (1);
		if (str[i] == ' ') continue;
		else return (0);
	}
	return (0);
}

#ifdef _OLDHP
char createFileEXT(str, fp, tflag)
	char *str, tflag;
	FILE **fp;
#else
char createFileEXT(char *str, FILE **fp, char tflag)
#endif
{
	/*
	 * if tflag is not 0, open with text mode.
	 * make no means on UNIX systems.
	 */
#ifdef _HAS_OPEN
	/* Edit the below, if complie error occurs ! */
#	define FMODE 0644
/*#	define FMODE (S_IREAD | S_IWRITE)*/
	long int fd;
	if (tflag) {
		if ((fd = open(str, O_RDWR|O_CREAT|O_EXCL|O_TEXT, FMODE)) < 0) return (1);
		if ((*fp = fdopen(fd, "w+")) == NULL) return (1);	/* something strange */
	}
	else {
		if ((fd = open(str, O_RDWR|O_CREAT|O_EXCL|O_BINARY, FMODE)) < 0) return (1);
		if ((*fp = fdopen(fd, "wb+")) == NULL) return (1);	/* something strange */
	}
#elif defined(_HAS__OPEN)
#include <sys/stat.h>
	/* Edit the below, if complie error occurs ! */
#	define FMODE (_S_IREAD | _S_IWRITE)
	long int fd;
	if (tflag) {
		if ((fd = _open(str, _O_RDWR|_O_CREAT|_O_EXCL|_O_TEXT, FMODE)) < 0) return (1);
		if ((*fp = _fdopen(fd, "w+")) == NULL) return (1);	/* something strange */
	}
	else {
		if ((fd = _open(str, _O_RDWR|_O_CREAT|_O_EXCL|_O_BINARY, FMODE)) < 0) return (1);
		if ((*fp = _fdopen(fd, "wb+")) == NULL) return (1);	/* something strange */
	}
#else
	char mode[4];
	strcpy(mode, "r");
	if ((*fp = fopen(str, mode)) != NULL) {
		fclose(*fp);
		return (1);
	}
	strncpy(mode, "w+", 3);
	if (!tflag) strcat(mode, "b");
	if ((*fp = fopen(str, mode)) == NULL) return (1);
#endif
	return (0);
}

#ifdef _OLDHP
char tmpNameEXT(str1, str2)
	char *str1, *str2;
#else
char tmpNameEXT(const char *str1, char *str2)
#endif
{
	char *rep;
	long int j, pos, plen;
	FILE *fp;
	rep = "0123456789_$~@+-ABCDEFHIJKLMNOPQRSTUVWXYZabcdefhijklmnopqrstuvwxyz";
	plen = strlen(rep);
	strcpy(str2, str1);
	pos = strlen(str2) - 1;
	for (j=0; j<plen; j++) {
		str2[pos] = rep[j];
		if ((fp = fopen(str2, "rb")) != NULL) fclose(fp);
		else return (0);
	}
	return (1);
}

#ifdef _OLDHP
char mkstempEXT(str1, str2, fp, tflag)
	char *str1, *str2, tflag;
	FILE **fp;
#else
char mkstempEXT(const char *str1, char *str2, FILE **fp, char tflag)
#endif
{
#ifdef _HAS_MKSTEMP
	long int j, fd, len;
	char mode[3];
	strcpy(mode, "w");
	if (!tflag) strcat(mode, "b");
	strcpy(str2, str1);
	if ((len = strlen(str2)) < 1) return (1);
	if (len < FNAMELEN-7) strncat(str2, ".XXXXXX", 7);
	else {
		fd = MINEXT(len, 6);
		len--;
		for (j=0; j<fd; j++) str2[len-j] = 'X';
	}
	if ((fd = mkstemp(str2)) < 0) return (1);
	if ((*fp = fdopen(fd, mode)) == NULL) return (1);	/* something strange */
	return (0);
#else
	char *rep;
	long int j, pos, plen;
	rep = "0123456789ABCDEFHIJKLMNOPQRSTUVWXYZabcdefhijklmnopqrstuvwxyz";
	plen = strlen(rep);
	strcpy(str2, str1);
	pos = strlen(str1) - 1;
	for (j=0; j<plen; j++) {
		str2[pos] = rep[j];
		if (createFileEXT(str2, fp, tflag) == 0) return (0);
	}
	strcpy(str2, tmpnam(NULL));
	if (createFileEXT(str2, fp, tflag) == 0) return (0);
	return (1);
#endif
}

#ifdef _OLDHP
long int getValueFromStream(fp, value, num, length, flag)
	FILE *fp;
	double *value;
	long int num, length;
	char flag;
#else
long int getValueFromStream(FILE *fp, double *value, long int num, long int length, char flag)
#endif
{
	long int i, j, k, longBuf, dumBuf;
	char str[MAXCHAR], fileMark, *str0, *strNew;
	double tmpValue;
	
	if (num <= 0 || length <= 0 || length > MAXCHAR) return (-1);
	i = 0;
	for (;;) {
		fileMark = 0;
		for (j=0; j<length; j++) {
			longBuf = fgetc(fp);
			str[j] = longBuf;
			if (longBuf == EOF) {
				str[j] = 0;
				fileMark = 2;
			}
			else if ((str[j] >= '0' && str[j] <= '9') || str[j] == '+' || str[j] == '-' || str[j] == '.' || toupper(str[j]) == 'E') continue;
			else if (str[j] == '#') { /* treatment of comments */
				do {
					dumBuf = fgetc(fp);
					if (dumBuf == EOF) break;
				} while ((dumBuf != 0x0A) && (dumBuf != 0x0D));
				str[j] = 0;
				fileMark = (dumBuf == EOF) ? 2 : 1;
			}
			else if ((str[j] == 0x0A) || (str[j] == 0x0D)) {
				str[j] = 0;
				fileMark = 1;
			}
			else str[j] = 0x20;
			if (fileMark) break;
		}
		if (j >= length) return (num+2); /* Input File is too Long */
		for (k=j-1; k>=0; k--) {
			if (str[k] != 0x20) break;
			else {
				str[k] = 0;
				j--;
			}
		}
		if (j == 0) {
			if (fileMark == 2) break;
			else continue;
		}
		str0 = str;
		for (;;) {
			tmpValue = strtod(str0, &strNew);
			if (strNew == str0) {
				if (strNew[0] != 0) return (-2); /* File Format Error */
				break;
			}
			if (i >= num) return (num+1);
			value[i] = tmpValue;
			i++;
			if (flag && i == num) return (i);
			str0 = strNew;
		}
		if (fileMark == 2) break;
	}
	return (i);
}

#ifdef _OLDHP
char base64en(charPtr, resBuf, len)
	char *charPtr, *resBuf;
	long int len;
#else
char base64en(const char *charPtr, char *resBuf, long int len)
#endif
{
	unsigned char b64char[64];
	long int i, j, k;
	
/* Make Base64 Alphabet */
	if (len <= 0) return (1);
	for (i=0; i<26; i++) b64char[i] = 'A' + i;
	for (i=26; i<52; i++) b64char[i] = i-26 + 'a';
	for (i=52; i<62; i++) b64char[i] = i-52 + '0';
	b64char[62] = '+';
	b64char[63] = '/';
	
	for (i=0; i<len/3; i++) {
		j = 3 * i;
		k = 4 * i;
		resBuf[k]   = b64char[(charPtr[j] >> 2) & 0x3F];
		resBuf[k+1] = b64char[(((charPtr[j] << 6) >> 2) & 0x3F) + ((charPtr[j+1] >> 4) & 0x0F)];
		resBuf[k+2] = b64char[(((charPtr[j+1] << 4) >> 2) & 0x3F) + ((charPtr[j+2] >> 6) & 0x03)];
		resBuf[k+3] = b64char[charPtr[j+2] & 0x3F];
	}
	if (len % 3 == 1) {
		j = 3 * i;
		k = 4 * i;
		resBuf[k]   = b64char[(charPtr[j] >> 2) & 0x3F];
		resBuf[k+1] = b64char[((charPtr[j] << 6) >> 2) & 0x3F];
		resBuf[k+2] = resBuf[k+3] = '=';
	}
	else if (len % 3 == 2) {
		j = 3 * i;
		k = 4 * i;
		resBuf[k]   = b64char[(charPtr[j] >> 2) & 0x3F];
		resBuf[k+1] = b64char[(((charPtr[j] << 6) >> 2) & 0x3F) + ((charPtr[j+1] >> 4) & 0x0F)];
		resBuf[k+2] = b64char[((charPtr[j+1] << 4) >> 2) & 0x3F];
		resBuf[k+3] = '=';
	}
	return (0);
}

#ifdef _OLDHP
char base64de(charPtr, resBuf, len, newLen)
	char *charPtr, *resBuf;
	long int len, *newLen;
#else
char base64de(const char *charPtr, char *resBuf, long int len, long int *newLen)
#endif
{
	unsigned char b64charInv[128];
	long int i, j, k;
	
/* Make Base64 Alphabet */
	if (len <= 0 || len % 4 != 0) return (1);
	for (i=0; i<128; i++) b64charInv[i] = 0;
	for (i='A'; i<'A'+26; i++) b64charInv[i] = i - 'A';
	for (i='a'; i<'a'+26; i++) b64charInv[i] = i - 'a' + 26;
	for (i='0'; i<'0'+10; i++) b64charInv[i] = i - '0' + 52;
	b64charInv['+'] = 62;
	b64charInv['/'] = 63;
	
	for (i=0; i<len/4 - 1; i++) {
		j = 3*i;
		k = 4*i;
		resBuf[j]   = (b64charInv[(int)charPtr[k]] << 2) + ((b64charInv[(int)charPtr[k+1]] >> 4) & 0x0F);
		resBuf[j+1] = (b64charInv[(int)charPtr[k+1]] << 4) + ((b64charInv[(int)charPtr[k+2]] >> 2) & 0x3F);
		resBuf[j+2] = (b64charInv[(int)charPtr[k+2]] << 6) + b64charInv[(int)charPtr[k+3]];
	}
	j = 3 * i;
	k = 4 * i;
	resBuf[j]   = (b64charInv[(int)charPtr[k]] << 2) + ((b64charInv[(int)charPtr[k+1]] >> 4) & 0x0F);
	*newLen = len * 3 / 4 - 2;
	if (charPtr[len-2] != '=') {
		resBuf[j+1] = (b64charInv[(int)charPtr[k+1]] << 4) + ((b64charInv[(int)charPtr[k+2]] >> 2) & 0x3F);
		(*newLen)++;
	}
	if (charPtr[len-1] != '=') {
		resBuf[j+2] = (b64charInv[(int)charPtr[k+2]] << 6) + b64charInv[(int)charPtr[k+3]];
		(*newLen)++;
	}
	return (0);
}

#ifdef _OLDHP
SUM_DOUBLE care_sum(n, data)
	long int n;
	SUM_DOUBLE *data;
#else
SUM_DOUBLE care_sum(long int n, SUM_DOUBLE *data)
#endif
{
	/* Sum up values carefully, with few cancellation of significant digits and round-off error */
	
	long int i, c;
	SUM_DOUBLE sum_m, sum_p;
	
	qsort(data, n, sizeof(SUM_DOUBLE), compareSumDouble);
	for (c=0; c<n; c++) {
		if (data[c] > SDBL0) break;
	}
	sum_m = sum_p = SDBL0;
	for (i=c; i<n; i++) sum_p += data[i];
	for (i=c-1; i>-1; i--) sum_m += data[i];
	return (sum_p+sum_m);
}

#ifdef _OLDHP
double care_sumDbl(n, data)
	long int n;
	double *data;
#else
double care_sumDbl(long int n, double *data)
#endif
{
	/* Sum up values carefully, with few cancellation of significant digits and round-off error */
	
	long int i, c;
	double sum_m, sum_p;
	
	qsort(data, n, sizeof(double), compareDouble);
	for (c=0; c<n; c++) {
		if (data[c] > 0.0) break;
	}
	sum_m = sum_p = 0.0;
	for (i=c; i<n; i++) sum_p += data[i];
	for (i=c-1; i>-1; i--) sum_m += data[i];
	return (sum_p+sum_m);
}

#ifdef _OLDHP
int compareSumDouble(r1, r2)
	void *r1, *r2;
#else
int compareSumDouble(const void *r1, const void *r2)
#endif
{
	SUM_DOUBLE v1, v2;
	
	v1 = *(SUM_DOUBLE *)r1;
	v2 = *(SUM_DOUBLE *)r2;
	if (v1 < v2) return (-1);
	if (v1 == v2) return (0);
	return (1);
}

#ifdef _OLDHP
int compareDouble(r1, r2)
	void *r1, *r2;
#else
int compareDouble(const void *r1, const void *r2)
#endif
{
	double v1, v2;
	
	v1 = *(double *)r1;
	v2 = *(double *)r2;
	if (v1 < v2) return (-1);
	if (v1 == v2) return (0);
	return (1);
}

#ifdef _OLDHP
SUM_DOUBLE care_sum2(n, data)
	long int n;
	SUM_DOUBLE *data;
#else
SUM_DOUBLE care_sum2(long int n, SUM_DOUBLE *data)
#endif
{
	/* Sum up values carefully, with round-off error.
	   This would be much faster than care_sum. */
	
	long int i;
	SUM_DOUBLE sum_m, sum_p;
	
	sum_m = sum_p = SDBL0;
	for (i=0; i<n; i++) {
		if (data[i] < SDBL0) sum_m += data[i];
		else sum_p += data[i];
	}
	return (sum_p+sum_m);
}

#ifdef _OLDHP
double care_sumDbl2(n, data)
	long int n;
	double *data;
#else
double care_sumDbl2(long int n, double *data)
#endif
{
	/* Sum up values carefully, with round-off error.
	   This would be much faster than care_sum. */
	
	long int i;
	double sum_m, sum_p;
	
	sum_m = sum_p = 0.0;
	for (i=0; i<n; i++) {
		if (data[i] < 0.0) sum_m += data[i];
		else sum_p += data[i];
	}
	return (sum_p+sum_m);
}

#ifdef USE_QSORT
#ifdef _OLDHP
void quickSort(data, n)
	long int *data;
	long int n;
#else
void quickSort(long int *data, long int n)
#endif
{
	quick_sort(data, 0, n-1);
}

#ifdef _OLDHP
void quick_sort(data, r1, r2)
	long int *data;
	long int r1, r2;
#else
void quick_sort(long int *data, long int r1, long int r2)
#endif
{
	long int p;
	
	if (r1 >= r2) return;
	p = get_part(data, r1, r2);
	quick_sort(data, r1, p-1);
	quick_sort(data, p+1, r2);
}

#ifdef _OLDHP
long int get_part(data, r1, r2)
	long int *data;
	long int r1, r2;
#else
long int get_part(long int *data, long int r1, long int r2)
#endif
{
	long int i, j;
	long int pv, tmp;
	
	i = r1-1;
	j = r2;
	pv = data[r2];
	
	for (;;) {
		while (data[++i] < pv);
		while (i < --j && pv < data[j]);
		if (i==j) break;
		tmp = data[i];
		data[i] = data[j];
		data[j] = tmp;
	}
	tmp = data[i];
	data[i] = data[r2];
	data[r2] = tmp;
	
	return (i);
}
#endif /* USE_QSORT */

#ifdef _OLDHP
char solve2Deq(args, res)
	double *args, *res;
#else
char solve2Deq(double *args, double *res)
#endif
{
/* Solve 2D-equation with all the coefficients are real.
 * Returns number of real root. This takes only 0 or 2.
 *  2 : All the "res[]"s are roots.
 *  0 : res[0] and res[1] are real and imaginary part of root, respectively.
 * args[0] and args[1] are the coefficients of x^0 and x^1, respectively.
 */
	double a1, det;
	
	if (args[0] == 0.0 && args[1] == 0.0) {
		res[0] = res[1] = 0.0;
		return (2);
	}
	a1 = args[1]/2.0;
	if ((det = a1 * a1 - args[0]) < 0.0) {
		res[0] = - a1;
		res[1] = sqrt(- det);
		return (0);
	}
	if (det == 0.0) {
		res[0] = res[1] = - a1;
		return (2);
	}
	res[0] = -1.0 * (a1 + signEXT(a1) * sqrt(det));
	res[1] = args[0] / res[0];
	return (2);
}

#ifdef _OLDHP
char solve3Deq(args, res)
	double *args, *res;
#else
char solve3Deq(double *args, double *res)
#endif
{
/* Solve 3D-equation with all the coefficients are real.
 * Returns number of real root. This takes only 1 or 3.
 *  3 : All the "res[]"s are roots.
 *  1 : res[0] is real root. res[1] and res[2] are real and imaginary part of root, respectively.
 * args[n] is the coefficient of x^n [n=0, 1, 2].
 */
	double buf1, buf2, buf13, len[2], a2;
	
	a2 = args[2]/3.0;
	buf1 = (args[2] * args[2] - 3.0 * args[1]) / 9.0;
	buf2 = (2.0 * args[2] * args[2] * args[2] - 9.0 * args[2] * args[1] + 27.0 * args[0]) / 54.0;
	if ((buf13 = buf1 * buf1 * buf1 - buf2 * buf2) < 0.0) {
		len[0] = pow(sqrt(- buf13) + fabs(buf2), 1.0 / 3.0);
		if ((res[0] = - signEXT(buf2) * (len[0] + buf1 / len[0]) - a2) == 0.0) {
			return (solve2Deq(args + 1, res + 1) + 1);
		}
		else {
			len[0] = - args[0] / res[0];
			len[1] =   args[2] + res[0];
			return (solve2Deq(len, res + 1) + 1);
		}
	}
	if (buf1 == 0.0) res[0] = res[1] = res[2] = -a2;
	else {
		len[0] = sqrt(buf1);
		len[1] = acos(buf2 / buf1 / len[0]) / 3.0;
		res[0] = - 2.0*len[0]*cos(len[1]              ) - a2;
		res[1] = - 2.0*len[0]*cos(len[1] + M_2PI / 3.0) - a2;
		res[2] = - 2.0*len[0]*cos(len[1] - M_2PI / 3.0) - a2;
	}
	return (3);
}

#ifdef _OLDHP
double bisect(rfunc, r0, r1, delta)
	double (*rfunc)();
	double r0, r1, delta;
#else
double bisect(double (*rfunc)(double x), double r0, double r1, double delta)
#endif
{
	double mid, res0, res1, resm, width;
	mid = r0;
	
	if (delta <= DBL_MIN) return (0.0);
	res0 = (*rfunc)(r0);
	res1 = (*rfunc)(r1);
	if (res0 * res1 > 0.0) return (0.0);
	if (res0 == 0.0) return (r0);
	if (res1 == 0.0) return (r1);
	if (r0 > r1) {
		mid = r0;
		r0 = r1;
		r1 = mid;
		mid = res0;
		res0 = res1;
		res1 = mid;
	}
	width = r1 - r0;
	while (width > delta) {
		width *= 0.5;
		mid = r0 + width;
		resm = (*rfunc)(mid);
		if (resm == 0.0) break;
		if (resm*res0 > 0.0) {
			r0 = mid;
			res0 = (*rfunc)(r0);
		}
		else {
			r1 = mid;
			res1 = (*rfunc)(r1);
		}
	}
	return (mid);
}
